/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

/*  
 *
 *                              
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *                                  
 *  Copyright, The Regents of the University of California.
 *                                  
 *           
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 *	505 665 1831
 */

#ifndef autoPtrDestroyh
#define autoPtrDestroyh

// o more flexible than auto_ptr because, when the target is 
// destroyed, it calls a member function instead of delete
// o works only with a class (does not work with atomic or 
// array data)
// o unlike auto_ptr ownership does not transfer during 
// assignment, and in fact the code will not compile if
// assignment between two autoDestroyPtr objects is attempted.
// For many situations that behavior avoids bugs.
// o autoDestroyPtr is more efficent compared to auto_ptr 
// because the target is a reference (isnt expected to be nill). 
template < class T, 
    void ( T :: * PM = & T :: destroy ) ( void ) >
class autoDestroyPtr {
public:
    autoDestroyPtr ( T & );
    ~autoDestroyPtr ();
    T & operator * () const;
    T * operator -> () const;
private:
    T & _target;
    // not implemented
    autoDestroyPtr < T > & operator = ( const autoDestroyPtr < T > & );
    autoDestroyPtr ( const autoDestroyPtr < T > & );
};

template < class T >
inline autoDestroyPtr < T > :: 
    autoDestroyPtr ( T & target ) :
        _target ( target ) 
{
}

template < class T >
inline autoDestroyPtr < T > :: 
    ~autoDestroyPtr ()
{
    ( _target.* PM )();
}

template < class T >
inline T & autoDestroyPtr < T > :: operator * () const
{
    return _target;
}

template < class T >
inline T * autoDestroyPtr < T > :: operator -> () const
{
    return & _target;
}

#endif // #ifdef autoPtrDestroyh
