/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

/*  
 *
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Copyright, 1986, The Regents of the University of California.
 *
 *  Author: Jeff Hill
 */

#ifndef bheh
#define bheh

#ifdef epicsExportSharedSymbols
#   define bhehEpicsExportSharedSymbols
#   undef epicsExportSharedSymbols
#endif

#include "tsDLList.h"
#include "tsFreeList.h"
#include "epicsTime.h"
#include "epicsMutex.h"
#include "compilerDependencies.h"

#ifdef bhehEpicsExportSharedSymbols
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "inetAddrID.h"
#include "caProto.h"

class tcpiiu;

typedef epics :: Mutex :: Guard Guard;

class epicsShareClass bhe : 
    public tsSLNode < bhe >, 
    public inetAddrID {
public:
    bhe ( 
        epics :: Mutex &, const epicsTime & initialTimeStamp, 
        unsigned initialBeaconNumber, const inetAddrID & addr );
    ~bhe (); 
    bool updatePeriod ( 
        Guard &,
        const epicsTime & programBeginTime, 
        const epicsTime & currentTime, ca_uint32_t beaconNumber, 
        unsigned protocolRevision );
    double period ( Guard & ) const;
    epicsTime updateTime ( Guard & ) const;
    void show ( unsigned level ) const;
    void show ( Guard &, unsigned /* level */ ) const;
    void registerIIU ( Guard &, tcpiiu & );
    void unregisterIIU ( Guard &, tcpiiu & );
private:
    epicsTime timeStamp;
    double averagePeriod;
    epics :: Mutex & mutex;
    tcpiiu * pIIU;
    ca_uint32_t lastBeaconNumber;
    void beaconAnomalyNotify ( Guard & );
    void logBeacon ( const char * pDiagnostic, 
                     const double & currentPeriod,
                     const epicsTime & currentTime );
    void logBeaconDiscard ( unsigned beaconAdvance,
                     const epicsTime & currentTime );
	bhe ( const bhe & );
	bhe & operator = ( const bhe & );
};

#endif // ifdef bheh


