/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*  
 *
 *                              
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *                                  
 *  Copyright, 1986, The Regents of the University of California.
 *                                  
 *           
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#include <string>
#include <stdexcept>

#define epicsAssertAuthor "Jeff Hill johill@lanl.gov"

#include "epicsThrowTrace.h"

#include "comBuf.h"
#include "errlog.h"

bool comBuf :: flushToWire ( 
    wireSendAdapter & wire, 
    const epicsTime & currentTime )
{
    unsigned index = _nextReadIndex;
    unsigned finalIndex = _commitIndex;
    while ( index < finalIndex ) {
        unsigned nBytes = wire.sendBytes ( 
            &_buf[index], finalIndex - index, currentTime );
        if ( nBytes == 0u ) {
            _nextReadIndex = index;
            return false;
        }
        index += nBytes;
    }
    _nextReadIndex = index; 
    return true;
}

// throwing the exception from a function that isnt inline 
// shrinks the GNU compiled object code
void comBuf :: throwInsufficentBytesException () 
{
    epicsThrowTrace std :: logic_error ( 
        "comBuf: insufficent bytes available" );
}



