/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

/*  
 *
 *                              
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *                                  
 *  Copyright, 1986, The Regents of the University of California.
 *                                  
 *           
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 *	505 665 1831
 */

#ifndef comQueRecvh  
#define comQueRecvh

#include "comBuf.h"

typedef epics :: Mutex :: Guard Guard;

class comQueRecv {
public:
    comQueRecv ( epics :: Mutex &, comBufMemoryManager & );
    ~comQueRecv ();
    size_t occupiedBytes ( Guard & ) const;
    size_t copyOutBytes ( 
        Guard &, epicsInt8 *pBuf, unsigned nBytes );
    size_t removeBytes ( Guard &, unsigned nBytes );
    void pushLastComBufReceived ( Guard &, comBuf & );
    void clear ( Guard & );
    bool popOldMsgHeader ( Guard &, struct caHdrLargeArray & );
    epicsInt8 popInt8 ( Guard & );
    epicsUInt8 popUInt8 ( Guard & );
    epicsInt16 popInt16 ( Guard & );
    epicsUInt16 popUInt16 ( Guard & );
    epicsInt32 popInt32 ( Guard & );
    epicsUInt32 popUInt32 ( Guard & );
    epicsFloat32 popFloat32 ( Guard & );
    epicsFloat64 popFloat64 ( Guard & );
    void popString ( Guard &, epicsOldString * );
private:
    epics :: Mutex & _mutex;
    tsDLList < comBuf > _bufs;
    comBufMemoryManager & _comBufMemMgr;
    size_t _nBytesPending;
    epicsUInt16 _multiBufferPopUInt16 ( Guard & );
    epicsUInt32 _multiBufferPopUInt32 ( Guard & );
    void _removeAndDestroyBuf ( Guard &, comBuf & );
	comQueRecv ( const comQueRecv & );
	comQueRecv & operator = ( const comQueRecv & );
};

inline size_t comQueRecv :: occupiedBytes ( Guard & guard ) const
{
    guard.assertIdenticalMutex ( _mutex );
    return _nBytesPending;
}

inline epicsInt8 comQueRecv :: popInt8 ( Guard & guard )
{
    return static_cast < epicsInt8 > ( this->popUInt8 ( guard ) );
}

inline epicsInt16 comQueRecv :: popInt16 ( Guard & guard )
{
    return static_cast < epicsInt16 > ( this->popUInt16 ( guard ) );
}

inline epicsInt32 comQueRecv :: popInt32 ( Guard & guard )
{
    return static_cast < epicsInt32 > ( this->popUInt32 ( guard ) );
}

// this has been optimized to aligned convert, maybe more could be done,
// but since it is currently not used ...
inline epicsFloat32 comQueRecv :: popFloat32 ( Guard & guard )
{
    union {
        epicsUInt8 _wire[ sizeof ( epicsFloat32 ) ];
        epicsFloat32 _fp;
    } tmp;
    // optimizer will unroll this loop
    for ( unsigned i = 0u; i < sizeof ( tmp._wire ); i++ ) {
        tmp._wire[i] = this->popUInt8 ( guard );
    }
    return AlignedWireRef < epicsFloat32 > ( tmp._fp );
}

// this has been optimized to aligned convert, maybe more could be done,
// but since it is currently not used ...
inline epicsFloat64 comQueRecv::popFloat64 ( Guard & guard )
{
    union {
        epicsUInt8 _wire[ sizeof ( epicsFloat64 ) ];
        epicsFloat64 _fp;
    } tmp;
    // optimizer will unroll this loop
    for ( unsigned i = 0u; i < sizeof ( tmp._wire ); i++ ) {
        tmp._wire[i] = this->popUInt8 ( guard );
    }
    return AlignedWireRef < epicsFloat64 > ( tmp._fp );
}

#endif // ifndef comQueRecvh
