/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/


/*  
 *
 *                              
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *                                  
 *  Copyright, 1986, The Regents of the University of California.
 *                                  
 *           
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 *	505 665 1831
 */

#ifndef comQueSendh  
#define comQueSendh

#include <new> 

#ifdef epicsExportSharedSymbols
#   define comQueSendh_epicsExportSharedSymbols
#   undef epicsExportSharedSymbols
#endif

#include "tsDLList.h"

#ifdef comQueSendh_epicsExportSharedSymbols
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "comBuf.h"

#define comQueSendCopyDispatchSize 39

using epics :: Mutex;
typedef epics :: Mutex :: Guard Guard;

class comQueSendMsgMinder {
public:
    comQueSendMsgMinder ( 
        Guard &, class comQueSend & );
    ~comQueSendMsgMinder ();
    void commit ( Guard & );
private:
    class comQueSend * _pSendQue;
    Guard & _guard;
};

//
// Notes.
//
class comQueSend {
public:
    comQueSend ( 
        Mutex &, wireSendAdapter &, comBufMemoryManager & );
    ~comQueSend ();
    void clear ( Guard & );
    size_t occupiedBytes ( Guard & ) const; 
    bool flushEarlyThreshold ( Guard &, size_t nBytesThisMsg ) const;
    bool flushBlockThreshold ( Guard & ) const; 
    void pushUInt16 ( Guard &, const ca_uint16_t value );
    void pushUInt32 ( Guard &, const ca_uint32_t value );
    void pushFloat32 ( Guard &, const ca_float32_t value );
    void pushString ( Guard &, const char *pVal, size_t nChar );
    void insertRequestHeader ( 
        Guard &, ca_uint16_t request, ca_uint32_t payloadSize, 
        ca_uint16_t dataType, ca_uint32_t nElem, ca_uint32_t cid, 
        ca_uint32_t requestDependent, bool v49Ok );
    void insertRequestWithPayLoad (
        Guard &, ca_uint16_t request, unsigned dataType, arrayElementCount nElem, 
        ca_uint32_t cid, ca_uint32_t requestDependent, 
        const void * pPayload, bool v49Ok );
    comBuf * popNextComBufToSend ( Guard & );
private:
    Mutex & _mutex;
    tsFreeList < comBuf, 0x10 > & _comBufMemMgr;
    tsDLList < comBuf > _bufs;
    tsDLIter < comBuf > _pFirstUncommited;
    wireSendAdapter & _wire;
    size_t _nBytesPending;

    typedef void ( comQueSend :: * copyScalarFunc_t ) 
                        ( Guard &, const void * pValue );
    static const copyScalarFunc_t 
        dbrCopyScalar [comQueSendCopyDispatchSize];
    void copy_dbr_string ( Guard &, const void * pValue );
    void copy_dbr_short ( Guard &, const void * pValue ); 
    void copy_dbr_float ( Guard &, const void * pValue ); 
    void copy_dbr_char ( Guard &, const void * pValue ); 
    void copy_dbr_long ( Guard &, const void * pValue ); 
    void copy_dbr_double ( Guard &, const void * pValue ); 
    void copy_dbr_invalid ( Guard &, const void * pValue );

    typedef void ( comQueSend::*copyVectorFunc_t ) 
        ( Guard &, const void * pValue, size_t nElem );
    static const copyVectorFunc_t 
        dbrCopyVector [comQueSendCopyDispatchSize];
    void copy_dbr_string ( Guard &, const void *pValue, size_t nElem );
    void copy_dbr_short ( Guard &, const void *pValue, size_t nElem ); 
    void copy_dbr_float ( Guard &, const void *pValue, size_t nElem ); 
    void copy_dbr_char ( Guard &, const void *pValue, size_t nElem ); 
    void copy_dbr_long ( Guard &, const void *pValue, size_t nElem ); 
    void copy_dbr_double ( Guard &, const void *pValue, size_t nElem ); 
    void copy_dbr_invalid ( Guard &, const void * pValue, size_t nElem );

    void _pushComBuf ( Guard &, comBuf & ); 
    comBuf * _newComBuf ( Guard & ); 

    void _beginMsg ( Guard & ); 
    void _commitMsg ( Guard & ); 
    void _clearUncommitedMsg ( Guard & );

    friend class comQueSendMsgMinder;

    //
    // visual C++ versions 6 & 7 do not allow out of 
    // class member template function definition
    //
    template < class T >
    inline void _push ( 
        Guard & guard, const T *pVal, const size_t nElem ) 
    {
        guard.assertIdenticalMutex ( _mutex );
        comBuf * pLastBuf = _bufs.last ();
        size_t nCopied;
        if ( pLastBuf ) {
            nCopied = pLastBuf->push ( pVal, nElem );
        }
        else {
            nCopied = 0u;
        }
        while ( nElem > nCopied ) {
            comBuf * pComBuf = _newComBuf ( guard );
            nCopied += pComBuf->push 
                        ( &pVal[nCopied], nElem - nCopied );
            _pushComBuf ( guard, *pComBuf );
        }
    }

    //
    // visual C++ versions 6 and 7 do not allow out of 
    // class member template function definition
    //
    template < class T >
    inline void _push ( Guard & guard, const T & val ) 
    {
        guard.assertIdenticalMutex ( _mutex );
        comBuf * pComBuf = _bufs.last ();
        if ( pComBuf && pComBuf->push ( val ) ) {
            return;
        }
        pComBuf = _newComBuf ( guard );
        assert ( pComBuf->push ( val ) );
        _pushComBuf ( guard, *pComBuf );
    }

    template < class T >
    inline void push ( const T * ); // disabled

    comQueSend ( const comQueSend & );
    comQueSend & operator = ( const comQueSend & );
};

extern const char cacNillBytes[];

inline comQueSendMsgMinder :: comQueSendMsgMinder ( 
    Guard & guard, comQueSend & sendQueIn ) : 
        _pSendQue ( & sendQueIn ),
        _guard ( guard )
{
    sendQueIn._beginMsg ( guard );
}

inline comQueSendMsgMinder :: ~comQueSendMsgMinder ()
{
    if ( _pSendQue ) {
        _pSendQue->_clearUncommitedMsg ( _guard );
    }
}

inline void comQueSendMsgMinder :: commit (
    Guard & guard )
{
    assert ( & guard == & _guard );
    if ( _pSendQue ) {
        _pSendQue->_commitMsg ( _guard );
        _pSendQue = 0;
    }
}

inline void comQueSend :: _beginMsg ( Guard & ) 
{
    _pFirstUncommited = _bufs.lastIter ();
}

inline void comQueSend :: pushUInt16 ( 
    Guard & guard, const ca_uint16_t value ) 
{
    _push ( guard, value );
}

inline void comQueSend :: pushUInt32 ( 
    Guard & guard, const ca_uint32_t value ) 
{
    _push ( guard, value );
}

inline void comQueSend :: pushFloat32 ( 
    Guard & guard, const ca_float32_t value ) 
{
    _push ( guard, value );
}

inline void comQueSend :: pushString ( 
    Guard & guard, const char *pVal, size_t nChar ) 
{
    _push ( guard, pVal, nChar );
}

inline void comQueSend :: _pushComBuf ( 
    Guard & guard, comBuf & cb ) 
{
    guard.assertIdenticalMutex ( _mutex );
    _bufs.add ( cb );
    if ( ! _pFirstUncommited.valid() ) {
        _pFirstUncommited = _bufs.lastIter ();
    }
}

inline size_t comQueSend :: occupiedBytes ( Guard & ) const 
{
    return _nBytesPending;
}

inline bool comQueSend :: flushBlockThreshold ( Guard & ) const
{
    return ( _nBytesPending > 16 * comBuf::capacityBytes () );
}

inline bool comQueSend :: flushEarlyThreshold ( 
    Guard &, size_t nBytesThisMsg ) const
{
    return ( _nBytesPending + nBytesThisMsg > 4 * comBuf::capacityBytes () );
}

// wrapping this with a function avoids WRS T2.2 Cygnus GNU compiler bugs
inline comBuf * comQueSend :: _newComBuf ( Guard & guard )
{
    return new ( _comBufMemMgr ) comBuf;
}

#endif // ifndef comQueSendh
