/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*  
 *
 *                              
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *                                  
 *  Copyright, 1986, The Regents of the University of California.
 *                                  
 *           
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 *	505 665 1831
 */

#ifndef nciuh  
#define nciuh

#ifdef epicsExportSharedSymbols
#   define nciuh_restore_epicsExportSharedSymbols
#   undef epicsExportSharedSymbols
#endif

#include "resourceLib.h"
#include "tsDLList.h"
#include "tsFreeList.h"
#include "epicsMutex.h"
#include "compilerDependencies.h"

#ifdef nciuh_restore_epicsExportSharedSymbols
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#define CA_MINOR_PROTOCOL_REVISION 12
#include "caProto.h"

#include "cacIO.h"

class cac;
class netiiu;

// The node and the state which tracks the list membership
// are in the channel, but belong to the circuit.
// Protected by the callback mutex
class channelNode : public tsDLNode < class nciu >
{
protected:
    channelNode ();
    bool isInstalledInServer ( Guard & ) const;
    bool isConnected ( Guard & ) const;
    static unsigned getMaxSearchTimerCount ();
private:
    enum channelState {
        cs_none,
        cs_disconnGov,
        // note: indexing is used here
        // so these must be contiguous
        cs_searchReqPending0,
        cs_searchReqPending1,
        cs_searchReqPending2,
        cs_searchReqPending3,
        cs_searchReqPending4,
        cs_searchReqPending5,
        cs_searchReqPending6,
        cs_searchReqPending7,
        cs_searchReqPending8,
        cs_searchReqPending9,
        cs_searchReqPending10,
        cs_searchReqPending11,
        cs_searchReqPending12,
        cs_searchReqPending13,
        cs_searchReqPending14,
        cs_searchReqPending15,
        cs_searchReqPending16,
        cs_searchReqPending17,
        // note: indexing is used here
        // so these must be contiguous
        cs_searchRespPending0,
        cs_searchRespPending1,
        cs_searchRespPending2,
        cs_searchRespPending3,
        cs_searchRespPending4,
        cs_searchRespPending5,
        cs_searchRespPending6,
        cs_searchRespPending7,
        cs_searchRespPending8,
        cs_searchRespPending9,
        cs_searchRespPending10,
        cs_searchRespPending11,
        cs_searchRespPending12,
        cs_searchRespPending13,
        cs_searchRespPending14,
        cs_searchRespPending15,
        cs_searchRespPending16,
        cs_searchRespPending17,
        cs_createReqPend,
        cs_createRespPend,
        cs_v42ConnCallbackPend,
        cs_subscripReqPend,
        cs_connected,
        cs_unrespCircuit,
        cs_subscripUpdateReqPend
    } listMember;
    void setRespPendingState ( Guard &, unsigned index );
    void setReqPendingState ( Guard &, unsigned index );
    unsigned getSearchTimerIndex ( Guard & );
    friend class tcpiiu;
    friend class udpiiu;
    friend class tcpSendThread;
    friend class searchTimer;
    friend class disconnectGovernorTimer;
};

class privateInterfaceForIO { // X aCC 655
public:
    virtual void ioCompletionNotify ( 
        Guard &, class baseNMIU & ) = 0;
    virtual arrayElementCount nativeElementCount ( 
        Guard & ) const = 0;
    virtual bool connected ( Guard & ) const = 0;
protected:
    virtual ~privateInterfaceForIO() {}
};

class nciu : 
    public cacChannel,
    public chronIntIdRes < nciu >, 
    public channelNode,
    private privateInterfaceForIO {
public:
    nciu ( Guard &, cac &, netiiu &, cacChannelNotify &, 
        const char * pNameIn, cacChannel::priLev );
    ~nciu ();
    void cleanup ( Guard & );
    void connect ( unsigned nativeType, 
        size_t nativeCount, unsigned sid, 
        Guard & cbGuard, 
        Guard & guard );
    void connect ( Guard & cbGuard, 
        Guard & guard );
    void unresponsiveCircuitNotify ( 
        Guard & cbGuard, 
        Guard & guard );
    void circuitHangupNotify ( class udpiiu &, 
        Guard & cbGuard, 
        Guard & guard );
    void setServerAddressUnknown ( 
        netiiu & newiiu, Guard & guard );
    bool searchMsg ( Guard & );
    void serviceShutdownNotify (
        Guard & callbackControlGuard, 
        Guard & mutualExclusionGuard );
    void accessRightsStateChange ( 
            const caAccessRights &, Guard & cbGuard, 
            Guard & guard );
    ca_uint32_t getSID ( Guard & ) const;
    ca_uint32_t getCID ( Guard & ) const;
    netiiu * getPIIU ( Guard & );
    const netiiu * getConstPIIU ( Guard & ) const;
    cac & getClient ();
    void searchReplySetUp ( netiiu &iiu, unsigned sidIn, 
        ca_uint16_t typeIn, arrayElementCount countIn,
        Guard & );
    void show ( unsigned level ) const;
    void show ( Guard &, unsigned level ) const;
    unsigned getName ( Guard &,
            char * pBuf, unsigned bufLen ) const throw ();
    const char * pName ( Guard & ) const throw ();
    size_t nameLen ( Guard & ) const;
    size_t getHostName (
        Guard &,
        char * pBuf, size_t bufLen ) const throw (); 
    void writeException ( 
        Guard &, Guard &,
        int status, const char *pContext, unsigned type, arrayElementCount count );
    cacChannel::priLev getPriority (
        Guard & ) const;
    //arrayElementCount nativeElementCount ( Guard & ) const;
    void resubscribe ( Guard & );
    void sendSubscriptionUpdateRequests ( Guard & );
    void disconnectAllIO ( Guard &, Guard & );
    bool connected ( Guard & ) const; 

private:
    tsDLList < class baseNMIU > eventq;
    caAccessRights accessRightState;
    epics :: Mutex _mutex;
    cac & cacCtx;
    char * pNameStr;
    netiiu * piiu;
    ca_uint32_t sid; // server id
    size_t count;
    size_t retry; // search retry number
    unsigned short nameLength; // channel name length
    ca_uint16_t typeCode;
    ca_uint8_t priority; 
    bool _cleanupHasRun;
    void initiateConnect (
        Guard & );
    unsigned requestMessageBytesPending ( 
        Guard & mutualExclusionGuard );
    void flush ( 
        Guard & mutualExclusionGuard );
    ioStatus read ( 
        Guard &,
        unsigned type, arrayElementCount count, 
        cacReadNotify &, ioid * );
    void write ( 
        Guard &,
        unsigned type, arrayElementCount count, 
        const void *pValue );
    ioStatus write ( 
        Guard &,
        unsigned type, arrayElementCount count, 
        const void *pValue, cacWriteNotify &, ioid * );
    void subscribe ( 
        Guard & guard, 
        unsigned type, arrayElementCount nElem, 
        unsigned mask, cacStateNotify &notify, ioid * );
    virtual void ioCancel ( 
        Guard & mutualExclusionGuard,
        const ioid & );
    void ioShow ( 
        Guard &,
        const ioid &, unsigned level ) const;
    short nativeType ( 
        Guard & ) const;
    caAccessRights accessRights (
        Guard & ) const;
    unsigned searchAttempts (
        Guard & ) const;
    double beaconPeriod (
        Guard & ) const;
    double receiveWatchdogDelay (
        Guard & ) const;
    bool ca_v42_ok (
        Guard & ) const;
    arrayElementCount nativeElementCount (
        Guard & ) const;
    static void stringVerify ( const char *pStr, const unsigned count );
    void ioCompletionNotify ( 
        Guard &, class baseNMIU & );
    const char * pHostName (
        Guard & guard ) const throw ();
	nciu ( const nciu & );
	nciu & operator = ( const nciu & );
};

inline ca_uint32_t nciu::getSID ( Guard & ) const
{
    return this->sid;
}

inline ca_uint32_t nciu::getCID ( Guard & ) const
{
    return this->id;
}

// this is to only be used by early protocol revisions
inline void nciu::connect ( Guard & cbGuard, Guard & guard )
{
    this->connect ( this->typeCode, this->count, 
        this->sid, cbGuard, guard );
}

inline void nciu::searchReplySetUp ( netiiu &iiu, unsigned sidIn, 
    ca_uint16_t typeIn, arrayElementCount countIn, Guard & )
{
    this->piiu = & iiu;
    this->typeCode = typeIn;      
    this->count = countIn;
    this->sid = sidIn;
}

inline netiiu * nciu::getPIIU ( Guard & )
{
    return this->piiu;
}

inline void nciu::writeException ( 
    Guard & /* cbGuard */, 
    Guard & guard, 
    int status, const char * pContext, 
    unsigned typeIn, arrayElementCount countIn )
{
    this->notify().writeException ( guard, 
        status, pContext, typeIn, countIn );
}

inline const netiiu * nciu::getConstPIIU ( Guard & ) const
{
    return this->piiu;
}

inline cac & nciu::getClient ()
{
    return this->cacCtx;
}

inline cacChannel::priLev nciu::getPriority ( Guard & ) const
{
    return this->priority;
}

inline channelNode::channelNode () :
    listMember ( cs_none )
{
}

inline bool channelNode::isConnected ( Guard & ) const
{
    return 
        this->listMember == cs_connected || 
        this->listMember == cs_subscripReqPend ||
        this->listMember == cs_subscripUpdateReqPend;
}

inline bool channelNode::isInstalledInServer ( Guard & ) const
{
    return 
        this->listMember == cs_connected ||
        this->listMember == cs_subscripReqPend ||
        this->listMember == cs_unrespCircuit ||
        this->listMember == cs_subscripUpdateReqPend;
}

#endif // ifdef nciuh
