/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*  
 *
 *                              
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *                                  
 *  Copyright, 1986, The Regents of the University of California.
 *                                  
 *           
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 *	505 665 1831
 */

#ifndef netIOh  
#define netIOh

#include "nciu.h"
#include "compilerDependencies.h"

// SUN PRO generates multiply defined symbols if the baseNMIU
// destructor is virtual (therefore it is protected). 
// I assume that SUNPRO will fix this in future versions.
// With other compilers we get warnings (and 
// potential problems) if we dont make the baseNMIU
// destructor virtual.
#if defined ( __SUNPRO_CC ) && ( __SUNPRO_CC <= 0x540 )
#   define NETIO_VIRTUAL_DESTRUCTOR
#else
#   define NETIO_VIRTUAL_DESTRUCTOR virtual
#endif

class privateInterfaceForIO;

class baseNMIU : public tsDLNode < baseNMIU >, // X aCC 655
        public chronIntIdRes < baseNMIU > {
public:
    virtual void destroy ( 
        Guard &, class cacRecycle & ) = 0; // only called by cac
    virtual void completion ( 
        Guard &, cacRecycle & ) = 0;
    virtual void exception ( 
        Guard &, cacRecycle &, 
        int status, const char * pContext ) = 0;
    virtual void exception ( 
        Guard &, cacRecycle &,
        int status, const char * pContext, unsigned type, 
        arrayElementCount count ) = 0;
    virtual void completion ( 
        Guard &, cacRecycle &,
        unsigned type, arrayElementCount count, 
        const void * pData ) = 0;
    virtual void forceSubscriptionUpdate (
        Guard & guard, nciu & chan ) = 0;
    virtual class netSubscription * isSubscription () = 0;
    virtual void show ( 
        unsigned level ) const = 0;
    virtual void show ( 
        Guard &, 
        unsigned level ) const = 0;
protected:
    NETIO_VIRTUAL_DESTRUCTOR ~baseNMIU (); 
};

class netSubscription : public baseNMIU  {
public:
    netSubscription ( 
        class privateInterfaceForIO &, unsigned type, 
        arrayElementCount count, 
        unsigned mask, cacStateNotify & );
    void show ( 
        unsigned level ) const;
    void show ( 
        Guard &, unsigned level ) const;
    arrayElementCount getCount (
        Guard & ) const;
    unsigned getType (
        Guard & ) const;
    unsigned getMask (
        Guard & ) const;
    void subscribeIfRequired (
        Guard & guard, nciu & chan );
    void unsubscribeIfRequired ( 
        Guard & guard, nciu & chan );
private:
    const arrayElementCount count;
    class privateInterfaceForIO & privateChanForIO;
    cacStateNotify & notify;
    const unsigned type;
    const unsigned mask;
    bool subscribed;
    class netSubscription * isSubscription ();
    void destroy ( 
        Guard &, class cacRecycle & );
    void completion (
        Guard &, cacRecycle & );
    void exception ( 
        Guard &, cacRecycle &,
        int status, const char * pContext );
    void completion ( 
        Guard &, cacRecycle &,
        unsigned type, arrayElementCount count, const void * pData );
    void exception ( 
        Guard &, cacRecycle &,
        int status, const char * pContext, unsigned type, 
        arrayElementCount count );
    void forceSubscriptionUpdate (
        Guard & guard, nciu & chan );
    netSubscription ( const netSubscription & );
    netSubscription & operator = ( const netSubscription & );
};

class netReadNotifyIO : public baseNMIU {
public:
    netReadNotifyIO ( privateInterfaceForIO &, cacReadNotify & );
    void show ( 
        unsigned level ) const;
    void show ( 
        Guard &, unsigned level ) const;
private:
    cacReadNotify & notify;
    class privateInterfaceForIO & privateChanForIO;
    void destroy ( 
        Guard &, class cacRecycle & );
    void completion (
        Guard &, cacRecycle & );
    void exception ( 
        Guard &, cacRecycle &,
        int status, const char * pContext );
    void completion ( 
        Guard &, cacRecycle &,
        unsigned type, arrayElementCount count,
        const void * pData );
    void exception ( 
        Guard &, cacRecycle &,
        int status, const char * pContext, 
        unsigned type, arrayElementCount count );
    class netSubscription * isSubscription ();
    void forceSubscriptionUpdate (
        Guard & guard, nciu & chan );
    netReadNotifyIO ( const netReadNotifyIO & );
    netReadNotifyIO & operator = ( const netReadNotifyIO & );
};

class netWriteNotifyIO : public baseNMIU {
public:
    netWriteNotifyIO ( privateInterfaceForIO &, cacWriteNotify & );
    void show ( 
        unsigned level ) const;
    void show ( 
        Guard &, unsigned level ) const;
private:
    cacWriteNotify & notify;
    privateInterfaceForIO & privateChanForIO;
    class netSubscription * isSubscription ();
    void destroy ( 
        Guard &, class cacRecycle & );
    void completion (
        Guard &, cacRecycle & );
    void exception ( 
        Guard &, cacRecycle &,
        int status, const char * pContext );
    void completion ( 
        Guard &, cacRecycle &,
        unsigned type, arrayElementCount count,
        const void * pData );
    void exception ( 
        Guard &, cacRecycle &,
        int status, const char * pContext, unsigned type, 
        arrayElementCount count );
    void forceSubscriptionUpdate (
        Guard & guard, nciu & chan );
    netWriteNotifyIO ( const netWriteNotifyIO & );
    netWriteNotifyIO & operator = ( const netWriteNotifyIO & );
};

inline arrayElementCount netSubscription :: getCount (
    Guard & guard ) const // X aCC 361
{
    //guard.assertIdenticalMutex ( this->mutex );
    arrayElementCount nativeCount = 
        this->privateChanForIO.nativeElementCount ( guard );
    if ( this->count == 0u || this->count > nativeCount ) {
        return nativeCount;
    }
    else {
        return this->count;
    }
}

inline unsigned netSubscription::getType ( Guard & ) const
{
    return this->type;
}

inline unsigned netSubscription::getMask ( Guard & ) const
{
    return this->mask;
}

#endif // ifdef netIOh
