/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Copyright, 1986, The Regents of the University of California.
 *
 *  Author: Jeff Hill
 */

#include <string>
#include <stdexcept>

#include "errlog.h"

#define epicsAssertAuthor "Jeff Hill johill@lanl.gov"

#include "epicsThrowTrace.h"

#define epicsExportSharedSymbols
#include "iocinf.h"
#include "nciu.h"
#include "cac.h"

netReadNotifyIO::netReadNotifyIO ( 
    privateInterfaceForIO & ioComplIntfIn, 
        cacReadNotify & notify ) :
    notify ( notify ), privateChanForIO ( ioComplIntfIn )
{
}

void netReadNotifyIO::show ( unsigned /* level */ ) const
{
    ::printf ( "netReadNotifyIO at %p\n", 
        static_cast < const void * > ( this ) );
}

void netReadNotifyIO::show ( 
    Guard &, unsigned level ) const
{
    this->show ( level );
}

void netReadNotifyIO::destroy ( 
    Guard & guard, cacRecycle & recycle )
{
    recycle.destroyReadNotifyIO ( guard, *this );
}

void netReadNotifyIO::completion ( 
    Guard & guard, 
    cacRecycle & recycle, unsigned type, 
    arrayElementCount count, const void * pData )
{
    //guard.assertIdenticalMutex ( this->mutex );
    this->privateChanForIO.ioCompletionNotify ( guard, *this );
    this->notify.completion ( guard, type, count, pData );
    recycle.destroyReadNotifyIO ( guard, *this );
}

void netReadNotifyIO::completion (
    Guard & guard,
    cacRecycle & recycle )
{
    //guard.assertIdenticalMutex ( this->mutex );
    //this->chan.getClient().printf ( "Read response w/o data ?\n" );
    this->privateChanForIO.ioCompletionNotify ( guard, *this );
    recycle.destroyReadNotifyIO ( guard, *this );
}

void netReadNotifyIO::exception ( 
    Guard & guard, 
    cacRecycle & recycle,
    int status, const char *pContext )
{
    //guard.assertIdenticalMutex ( this->mutex );
    this->privateChanForIO.ioCompletionNotify ( guard, *this );
    this->notify.exception ( 
        guard, status, pContext, UINT_MAX, 0u );
    recycle.destroyReadNotifyIO ( guard, *this );
}

void netReadNotifyIO::exception ( 
    Guard & guard, 
    cacRecycle & recycle,
    int status, const char *pContext, 
    unsigned type, arrayElementCount count )
{
    //guard.assertIdenticalMutex ( this->mutex )
    this->privateChanForIO.ioCompletionNotify ( guard, *this );
    this->notify.exception ( 
        guard, status, pContext, type, count );
    recycle.destroyReadNotifyIO ( guard, *this );
}

class netSubscription * netReadNotifyIO::isSubscription ()
{
    return 0;
}

void netReadNotifyIO::forceSubscriptionUpdate (
    Guard &, nciu & )
{
}



