/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Copyright, 1986, The Regents of the University of California.
 *
 *  Author: Jeff Hill
 */

#include <string>
#include <stdexcept>

#include "epicsThrowTrace.h"
#include "errlog.h"

#define epicsAssertAuthor "Jeff Hill johill@lanl.gov"

#include "iocinf.h"
#include "nciu.h"
#include "cac.h"

netWriteNotifyIO::netWriteNotifyIO ( 
    privateInterfaceForIO & ioComplIntf, cacWriteNotify & notifyIn ) :
    notify ( notifyIn ), privateChanForIO ( ioComplIntf )
{
}

void netWriteNotifyIO::show ( unsigned /* level */ ) const
{
    ::printf ( "read write notify IO at %p\n", 
        static_cast < const void * > ( this ) );
}

void netWriteNotifyIO::show ( 
    Guard &,  
    unsigned level ) const
{
    this->show ( level );
}

void netWriteNotifyIO::destroy ( 
    Guard & guard, 
    cacRecycle & recycle )
{
    recycle.destroyWriteNotifyIO ( guard, *this );
}

void netWriteNotifyIO::completion (
    Guard & guard,
    cacRecycle & recycle )
{
    this->privateChanForIO.ioCompletionNotify ( guard, *this );
    this->notify.completion ( guard );
    recycle.destroyWriteNotifyIO ( guard, *this );
}

void netWriteNotifyIO::completion ( 
    Guard & guard, 
    cacRecycle & recycle,
    unsigned /* type */, arrayElementCount /* count */, 
    const void * /* pData */ )
{
    //this->chan.getClient().printf ( "Write response with data ?\n" );
    this->privateChanForIO.ioCompletionNotify ( guard, *this );
    recycle.destroyWriteNotifyIO ( guard, *this );
}

void netWriteNotifyIO::exception ( 
    Guard & guard,
    cacRecycle & recycle,
    int status, const char * pContext )
{
    this->privateChanForIO.ioCompletionNotify ( guard, *this );
    this->notify.exception ( 
        guard, status, pContext, UINT_MAX, 0u );
    recycle.destroyWriteNotifyIO ( guard, *this );
}

void netWriteNotifyIO::exception ( 
    Guard & guard, 
    cacRecycle & recycle,
    int status, const char *pContext, 
    unsigned type, arrayElementCount count )
{
    this->privateChanForIO.ioCompletionNotify ( guard, *this );
    this->notify.exception ( 
        guard, status, pContext, type, count );
    recycle.destroyWriteNotifyIO ( guard, *this );
}

class netSubscription * netWriteNotifyIO::isSubscription ()
{
    return 0;
}

void netWriteNotifyIO::forceSubscriptionUpdate (
    Guard &, nciu & )
{
}

