/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

/*  
 *
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Copyright, 1986, The Regents of the University of California.
 *
 *  Author: Jeff Hill
 */

#include <stdexcept>
#include <string> // vxWorks 6.0 requires this include 

#include <limits.h>
#include <float.h>

#define epicsAssertAuthor "Jeff Hill johill@lanl.gov"

#include "epicsThrowTrace.h"

#define epicsExportSharedSymbols
#include "iocinf.h"
#include "cac.h"
#include "netiiu.h"

netiiu::~netiiu ()
{
}

bool netiiu::ca_v42_ok (
    Guard & ) const
{
    return false;
}

bool netiiu::ca_v41_ok (
    Guard & ) const
{
    return false;
}

void netiiu::writeRequest ( 
    Guard &, nciu &, 
    unsigned, arrayElementCount, const void * )
{
}

void netiiu::writeNotifyRequest ( 
    Guard &, 
    nciu &, netWriteNotifyIO &, unsigned, 
    arrayElementCount, const void * )
{
}

void netiiu::readNotifyRequest ( 
    Guard &, 
    nciu &, netReadNotifyIO &, unsigned, arrayElementCount )
{
}

void netiiu::clearChannelRequest ( 
    Guard &, ca_uint32_t, ca_uint32_t )
{
}

void netiiu::subscriptionRequest ( 
    Guard &, nciu &, netSubscription & )
{
}

void netiiu::subscriptionCancelRequest ( 
    Guard &, nciu &, netSubscription & )
{
}

void netiiu::subscriptionUpdateRequest ( 
    Guard &, nciu &, netSubscription & )
{
}

static const char * const pHostNameNetIIU = "<disconnected>";

unsigned netiiu::getHostName ( 
    Guard & guard,
    char * pBuf, unsigned bufLen ) const throw ()
{
    if ( bufLen ) {
        unsigned len = strlen ( pHostNameNetIIU );
        strncpy ( pBuf, pHostNameNetIIU, bufLen );
        if ( len < bufLen ) {
            return len;
        }
        else {
            unsigned reducedSize = bufLen - 1u;
            pBuf[reducedSize] = '\0';
            return reducedSize;
        }
    }
    return 0u;
}

const char * netiiu::pHostName (
    Guard & ) const throw ()
{
    return pHostNameNetIIU;
}

osiSockAddr netiiu::getNetworkAddress (
    Guard & ) const
{
    osiSockAddr addr;
    addr.sa.sa_family = AF_UNSPEC;
    return addr;
}

void netiiu::flushRequest ( 
    Guard & )
{
}

size_t netiiu::requestMessageBytesPending ( 
    Guard & )
{
    return 0u;
}

void netiiu::flush ( 
    Guard & )
{
}

void netiiu::requestRecvProcessPostponedFlush (
    Guard & )
{
}

void netiiu::uninstallChan ( 
    Guard &, nciu & )
{
    throw cacChannel::notConnected();
}

double netiiu::receiveWatchdogDelay (
    Guard & guard ) const
{
    return - DBL_MAX;
}

void netiiu::uninstallChanDueToSuccessfulSearchResponse ( 
    Guard &, nciu &, const epicsTime & )
{
    throw std::runtime_error ( 
        "search response occured when not attached to udpiiu?" );
}

bool netiiu::searchMsg (
    Guard &, ca_uint32_t id, 
        const char * pName, unsigned nameLength )
{
    return false;
}


