/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

/*  
 *
 *                              
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *                                  
 *  Copyright, The Regents of the University of California.
 *                                  
 *           
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 *	505 665 1831
 */

#ifndef oldAccessh
#define oldAccessh

#ifdef epicsExportSharedSymbols
#   define oldAccessh_restore_epicsExportSharedSymbols
#   undef epicsExportSharedSymbols
#endif

#include "tsFreeList.h"
#include "epicsMemory.h"
#include "compilerDependencies.h"
#include "osiSock.h"

#ifdef oldAccessh_restore_epicsExportSharedSymbols
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "caProto.h"
#include "cacIO.h"
#include "cadef.h"
#include "syncGroup.h"

struct oldChannelNotify : 
    private cacChannelNotify {
public:
    oldChannelNotify ( 
        Guard &, struct ca_client_context &, 
        const char * pName, caCh * pConnCallBackIn, 
        void * pPrivateIn, capri priority );
    void cleanup ( 
        Guard & guard );
    ~oldChannelNotify ();

    // legacy C API
    friend unsigned epicsShareAPI ca_get_host_name ( 
        chid pChan, char * pBuf, unsigned bufLength );
    friend const char * epicsShareAPI ca_host_name ( 
        chid pChan );
    friend const char * epicsShareAPI ca_name ( 
        chid pChan );
    friend void epicsShareAPI ca_set_puser ( 
        chid pChan, void * puser );
    friend void * epicsShareAPI ca_puser ( 
        chid pChan );
    friend int epicsShareAPI ca_change_connection_event ( 
        chid pChan, caCh * pfunc );
    friend int epicsShareAPI ca_replace_access_rights_event ( 
        chid pChan, caArh *pfunc );
    friend int epicsShareAPI ca_array_get ( chtype type, 
        arrayElementCount count, chid pChan, void * pValue );
    friend int epicsShareAPI ca_array_get_callback ( chtype type, 
        arrayElementCount count, chid pChan,
        caEventCallBackFunc *pfunc, void *arg );
    friend int epicsShareAPI ca_array_put ( 
        chtype type, arrayElementCount count, 
        chid pChan, const void * pValue );
    friend int epicsShareAPI ca_array_put_callback ( 
        chtype type, arrayElementCount count, 
        chid pChan, const void *pValue, 
        caEventCallBackFunc *pfunc, void *usrarg );
    friend double epicsShareAPI ca_beacon_period ( 
        chid pChan );
    friend unsigned epicsShareAPI ca_search_attempts ( 
        chid pChan );
    friend unsigned epicsShareAPI ca_write_access ( 
        chid pChan );
    friend unsigned epicsShareAPI ca_read_access ( 
        chid pChan );
    friend short epicsShareAPI ca_field_type ( 
        chid pChan );
    friend arrayElementCount epicsShareAPI ca_element_count ( 
        chid pChan );
    friend int epicsShareAPI ca_v42_ok ( 
        chid pChan );
    friend int epicsShareAPI ca_create_subscription ( 
        chtype type, arrayElementCount count, chid pChan, 
        long mask, caEventCallBackFunc * pCallBack, 
        void * pCallBackArg, evid * monixptr );
    friend enum channel_state epicsShareAPI ca_state ( 
        chid pChan ); 
    friend double epicsShareAPI ca_receive_watchdog_delay ( 
        chid pChan );

    unsigned getName (
        Guard &,
        char * pBuf, unsigned bufLen ) const throw ();
    void show ( 
        Guard &,
        unsigned level ) const;
    void initiateConnect (
        Guard & );
    void read ( 
        Guard &,
        unsigned type, arrayElementCount count, 
        cacReadNotify &notify, cacChannel::ioid *pId = 0 );
    void write ( 
        Guard &,
        unsigned type, arrayElementCount count, const void *pValue, 
        cacWriteNotify &, cacChannel::ioid *pId = 0 );
    void ioCancel ( 
        Guard & mutualExclusionGuard, 
        const cacChannel::ioid & );
    void ioShow ( 
        Guard & guard,
        const cacChannel::ioid &, unsigned level ) const;
    ca_client_context & getClientCtx ();
    void eliminateExcessiveSendBacklog ( 
        Guard & );

private:
    ca_client_context & cacCtx;
    cacChannel & io;
    caCh * pConnCallBack;
    void * pPrivate;
    caArh * pAccessRightsFunc;
    unsigned ioSeqNo;
    bool currentlyConnected;
    bool prevConnected;
    bool _cleanupCompleted;
    void connectNotify ( Guard & );
    void disconnectNotify ( Guard & );
    void serviceShutdownNotify (
        Guard & mutualExclusionGuard );
    void accessRightsNotify (  
        Guard &, const caAccessRights & );
    void exception ( Guard &,
        int status, const char * pContext );
    void readException ( Guard &,
        int status, const char * pContext,
        unsigned type, arrayElementCount count, void *pValue );
    void writeException ( Guard &,
        int status, const char * pContext,
        unsigned type, arrayElementCount count );
	oldChannelNotify ( const oldChannelNotify & );
	oldChannelNotify & operator = ( const oldChannelNotify & );
};

class getCopy : 
    public cacReadNotify {
public:
    getCopy ( 
        Guard & guard, 
        ca_client_context & cacCtx, 
        oldChannelNotify &, unsigned type, 
        arrayElementCount count, void *pValue );
    void show ( unsigned level ) const;
    void cancel ();
private:
    arrayElementCount count;
    ca_client_context & cacCtx;
    oldChannelNotify & chan;
    void * pValue;
    unsigned ioSeqNo;
    unsigned type;
    void completion (
        Guard &, unsigned type, 
        arrayElementCount count, const void *pData );
    void exception ( 
        Guard &, int status, 
        const char *pContext, unsigned type, 
        arrayElementCount count );
	getCopy ( const getCopy & );
	getCopy & operator = ( const getCopy & );
};

class getCallback : public cacReadNotify {
public:
    getCallback ( 
        oldChannelNotify & chanIn, 
        caEventCallBackFunc *pFunc, void *pPrivate );
private:
    oldChannelNotify & chan;
    caEventCallBackFunc * pFunc;
    void * pPrivate;
    void completion (
        Guard &, unsigned type, 
        arrayElementCount count, const void *pData);
    void exception ( 
        Guard &, int status, 
        const char * pContext, unsigned type, 
        arrayElementCount count );
	getCallback ( const getCallback & );
	getCallback & operator = ( const getCallback & );
};

class putCallback : public cacWriteNotify {
public:
    putCallback ( 
        oldChannelNotify &, 
        caEventCallBackFunc *pFunc, void *pPrivate );
private:
    oldChannelNotify & chan;
    caEventCallBackFunc * pFunc;
    void *pPrivate;
    void completion ( Guard & );
    void exception ( 
        Guard &, int status, const char *pContext, 
        unsigned type, arrayElementCount count );
	putCallback ( const putCallback & );
	putCallback & operator = ( const putCallback & );
};

struct oldSubscription : private cacStateNotify {
public:
    oldSubscription (
        Guard & guard,
        oldChannelNotify & chanIn, cacChannel & io, 
        unsigned type, arrayElementCount nElem, unsigned mask,
        caEventCallBackFunc * pFuncIn, void * pPrivateIn,
        evid * );
    oldChannelNotify & channel () const;
    void cancel ( Guard & guard );
private:
    oldChannelNotify & chan;
    cacChannel::ioid id;
    caEventCallBackFunc * pFunc;
    void * pPrivate;
    void current (
        Guard &, unsigned type, 
        arrayElementCount count, const void *pData );
    void exception ( 
        Guard &, int status, 
        const char *pContext, unsigned type, 
        arrayElementCount count );
	oldSubscription ( const oldSubscription & );
	oldSubscription & operator = ( const oldSubscription & );
};

extern "C" void cacOnceFunc ( void * );
extern "C" void cacExitHandler ( void *);

struct ca_client_context : public cacContextNotify
{
public:
    ca_client_context ( bool enablePreemptiveCallback = false );
    virtual ~ca_client_context ();
    void changeExceptionEvent ( 
        caExceptionHandler * pfunc, void * arg );
    void registerForFileDescriptorCallBack ( 
        CAFDHANDLER * pFunc, void * pArg );
    void replaceErrLogHandler ( caPrintfFunc * ca_printf_func );
    cacChannel & createChannel ( 
        Guard &, const char * pChannelName, 
        cacChannelNotify &, cacChannel::priLev pri );
    void destroyChannel ( Guard &, cacChannel & );
    void flush ( Guard & );
    void eliminateExcessiveSendBacklog ( 
        Guard &, cacChannel & );
    int pendIO ( const double & timeout );
    int pendEvent ( const double & timeout );
    bool ioComplete () const;
    void show ( unsigned level ) const;
    unsigned circuitCount () const;
    unsigned sequenceNumberOfOutstandingIO (
        Guard & ) const;
    unsigned beaconAnomaliesSinceProgramStart () const;
    void incrementOutstandingIO ( 
        Guard &, unsigned ioSeqNo );
    void decrementOutstandingIO ( 
        Guard &, unsigned ioSeqNo );
    void exception ( 
        Guard &, int status, const char * pContext, 
        const char * pFileName, unsigned lineNo );
    void exception ( 
        Guard &, int status, const char * pContext,
        const char * pFileName, unsigned lineNo, oldChannelNotify & chan, 
        unsigned type, arrayElementCount count, unsigned op );
    void blockForEventAndEnableCallbacks ( 
        epicsEvent & event, const double & timeout );
    CASG * lookupCASG ( Guard &, unsigned id );
    static void installDefaultService ( cacService & );
    void installCASG ( Guard &, CASG & );
    void uninstallCASG ( Guard &, CASG & );
    void selfTest () const;
// perhaps these should be eliminated in deference to the exception mechanism
    int printFormated ( const char * pformat, ... ) const;
    int varArgsPrintFormated ( const char * pformat, va_list args ) const;
    void signal ( int ca_status, const char * pfilenm, 
                     int lineno, const char * pFormat, ... );
    void vSignal ( int ca_status, const char * pfilenm, 
                     int lineno, const char  *pFormat, va_list args );
    bool preemptiveCallbakIsEnabled () const;
    void destroyGetCopy ( Guard &, getCopy & );
    void destroyGetCallback ( Guard &, getCallback & );
    void destroyPutCallback ( Guard &, putCallback & );
    void destroySubscription ( Guard &, oldSubscription & );
    epics :: Mutex & mutexRef () const;

    // legacy C API    
    friend int epicsShareAPI ca_create_channel (
        const char * name_str, caCh * conn_func, void * puser,
        capri priority, chid * chanptr );
    friend int epicsShareAPI ca_clear_channel ( chid pChan );
    friend int epicsShareAPI ca_array_get ( chtype type, 
        arrayElementCount count, chid pChan, void * pValue );
    friend int epicsShareAPI ca_array_get_callback ( chtype type, 
        arrayElementCount count, chid pChan,
        caEventCallBackFunc *pfunc, void *arg );
    friend int epicsShareAPI ca_array_put ( chtype type, 
        arrayElementCount count, chid pChan, const void * pValue );
    friend int epicsShareAPI ca_array_put_callback ( chtype type, 
        arrayElementCount count, chid pChan, const void * pValue, 
        caEventCallBackFunc *pfunc, void *usrarg );
    friend int epicsShareAPI ca_create_subscription ( 
        chtype type, arrayElementCount count, chid pChan, 
        long mask, caEventCallBackFunc * pCallBack, void * pCallBackArg, 
        evid *monixptr );
    friend int epicsShareAPI ca_flush_io ();
    friend int epicsShareAPI ca_clear_subscription ( evid pMon );
    friend int epicsShareAPI ca_sg_create ( CA_SYNC_GID * pgid );
    friend int epicsShareAPI ca_sg_delete ( const CA_SYNC_GID gid );
    friend int epicsShareAPI ca_sg_block ( const CA_SYNC_GID gid, ca_real timeout );
    friend int epicsShareAPI ca_sg_reset ( const CA_SYNC_GID gid );
    friend int epicsShareAPI ca_sg_test ( const CA_SYNC_GID gid );

    // exceptions
    class noSocket {};
private:
    mutable epics :: Mutex mutex;
    mutable epics :: Mutex cbMutex; 
    chronIntIdResTable < CASG > sgTable;
    typedef tsFreeList < struct oldChannelNotify, 1024 > 
            FreeListOfOldChannelNotify;
    typedef tsFreeList < class getCopy, 1024 > 
            FreeListOfGetCopy;
    typedef tsFreeList < class getCallback, 1024 > 
            FreeListOfGetCallback;
    typedef tsFreeList < class putCallback, 1024 > 
            FreeListOfPutCallback;
    typedef tsFreeList < struct oldSubscription, 1024 > 
            FreeListOfOldSubscription;
    typedef tsFreeList < struct CASG, 128 > 
            FreeListOfCASG;
    FreeListOfOldChannelNotify oldChannelNotifyFreeList;
    FreeListOfGetCopy getCopyFreeList;
    FreeListOfGetCallback getCallbackFreeList;
    FreeListOfPutCallback putCallbackFreeList;
    FreeListOfOldSubscription subscriptionFreeList;
    FreeListOfCASG casgFreeList;
    epicsEvent ioDone;
    epicsEvent callbackThreadActivityComplete;
    epicsThreadId createdByThread;
    epics_auto_ptr < Guard > pCallbackGuard;
    epics_auto_ptr < cacContext > pServiceContext;
    caExceptionHandler * ca_exception_func;
    void * ca_exception_arg;
    caPrintfFunc * pVPrintfFunc;
    CAFDHANDLER * fdRegFunc;
    void * fdRegArg;
    SOCKET sock;
    unsigned pndRecvCnt;
    unsigned ioSeqNo;
    unsigned callbackThreadsPending;
    ca_uint16_t localPort;
    bool fdRegFuncNeedsToBeCalled;
    bool noWakeupSincePend;

    void attachToClientCtx ();
    void callbackProcessingInitiateNotify ();
    void callbackProcessingCompleteNotify ();
    cacContext & createNetworkContext ( 
        epics :: Mutex & mutualExclusion, 
        epics :: Mutex & callbackControl );

	ca_client_context ( const ca_client_context & );
	ca_client_context & operator = ( const ca_client_context & );
	
	friend void cacOnceFunc ( void * );
	friend void cacExitHandler ( void *);
    static cacService * pDefaultService;
    static epics :: Mutex * pDefaultServiceInstallMutex;
    static const unsigned flushBlockThreshold;
};

int fetchClientContext ( ca_client_context * * ppcac );

inline ca_client_context & oldChannelNotify::getClientCtx ()
{
    return this->cacCtx;
}

inline unsigned oldChannelNotify::getName (
    Guard & guard,
    char * pBuf, unsigned bufLen ) const throw ()
{
    return this->io.getName ( guard, pBuf, bufLen );
}

inline void oldChannelNotify::show ( 
    Guard & guard,
    unsigned level ) const
{
    this->io.show ( guard, level );
}

inline void oldChannelNotify::initiateConnect (
    Guard & guard )
{
    this->io.initiateConnect ( guard );
}

inline void oldChannelNotify::ioCancel (    
    Guard & guard, 
    const cacChannel::ioid & id )
{
    this->io.ioCancel ( guard, id );
}

inline void oldChannelNotify::ioShow ( 
    Guard & guard,
    const cacChannel::ioid & id, unsigned level ) const
{
    this->io.ioShow ( guard, id, level );
}

inline void oldChannelNotify::eliminateExcessiveSendBacklog ( 
    Guard & guard )
{
    this->cacCtx.eliminateExcessiveSendBacklog ( guard, this->io );
}

inline void oldSubscription :: cancel ( Guard & guard ) 
{
    this->chan.ioCancel ( guard, this->id );
}

inline oldChannelNotify & oldSubscription::channel () const
{
    return this->chan;
}

inline bool ca_client_context::preemptiveCallbakIsEnabled () const
{
    return this->pCallbackGuard.get () == 0;
}

inline bool ca_client_context::ioComplete () const
{
    return ( this->pndRecvCnt == 0u );
}

inline unsigned ca_client_context::sequenceNumberOfOutstandingIO (
    Guard & ) const
{
    // perhaps on SMP systems THERE should be lock/unlock around this
    return this->ioSeqNo;
}
        
#endif // ifndef oldAccessh
