/*************************************************************************\
 * Copyright (c) 2002 The University of Chicago, as Operator of Argonne
 *     National Laboratory.
 * Copyright (c) 2002 The Regents of the University of California, as
 *     Operator of Los Alamos National Laboratory.
 * EPICS BASE Versions 3.13.7
 * and higher are distributed subject to a Software License Agreement found
 * in file LICENSE that is included with this distribution. 
\*************************************************************************/

/*  
 *
 *                              
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *                                  
 *  Copyright, 1986, The Regents of the University of California.
 *                                  
 *           
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 *	505 665 1831
 */

#ifndef virtualCircuith  
#define virtualCircuith

#ifdef epicsExportSharedSymbols
#   define virtualCircuith_epicsExportSharedSymbols
#   undef epicsExportSharedSymbols
#endif

#include "epicsMemory.h"
#include "tsDLList.h"
#include "compilerDependencies.h"

#ifdef virtualCircuith_epicsExportSharedSymbols
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "netiiu.h"
#include "comBuf.h"
#include "SearchDest.h"
#include "caServerID.h"
#include "comQueSend.h"
#include "comQueRecv.h"
#include "hostNameCache.h"
#include "tcpRecvWatchdog.h"
#include "tcpSendWatchdog.h"
#include "compilerDependencies.h"

class callbackManager;

// a modified ca header with capacity for large arrays
struct caHdrLargeArray {
    ca_uint32_t m_postsize;     // size of message extension 
    ca_uint32_t m_count;        // operation data count 
    ca_uint32_t m_cid;          // channel identifier 
    ca_uint32_t m_available;    // protocol stub dependent
    ca_uint16_t m_dataType;     // operation data type 
    ca_uint16_t m_cmmd;         // operation to be performed 
};

class ipAddrToAsciiEngine;

class tcpRecvThread : 
    private epicsThreadRunable {
public:
    tcpRecvThread ( 
        class tcpiiu & iiuIn, cacContextNotify &, 
        const char * pName, unsigned int stackSize, 
        unsigned int priority );
    virtual ~tcpRecvThread ();
    void start ();
    void exitWait ();
    bool exitWait ( double delay );
    void show ( Guard &, unsigned level ) const;
private:
    Mutex _recvQueMutex;
    Guard _recvQueGuard; // only this thread can use recv queue
    epicsThread _thread;
    comBufMemoryManager _memMgr;
    comQueRecv _recvQue;
    caHdrLargeArray _curMsg;
    class tcpiiu & _iiu;
    cacContextNotify & _ctxNotify;
    char * _pCurData;
    arrayElementCount _curDataMax;
    arrayElementCount _curDataBytes;
    unsigned _contigRecvMsgCount;
    bool _msgHeaderAvailable;
    bool _oldMsgHeaderAvailable;
    void run ();
    void _connect (
        Guard & guard );
    bool _validFillStatus ( 
        Guard & guard, 
        const statusWireIO & stat );
    bool _processIncoming ( 
        const epicsTime & currentTime, callbackManager & );
};

class tcpSendThread : private epicsThreadRunable {
public:
    tcpSendThread ( 
        class tcpiiu & iiuIn, const char * pName, 
        unsigned int stackSize, unsigned int priority );
    virtual ~tcpSendThread ();
    void start ();
    void exitWait ();
    void show ( unsigned level ) const;
private:
    epicsThread thread;
    class tcpiiu & iiu;
    void run ();
    bool flush ( Guard & );
};

class tcpiiu : 
        public netiiu, public tsDLNode < tcpiiu >,
        public tsSLNode < tcpiiu >, public caServerID, 
        private wireSendAdapter, private wireRecvAdapter {
public:
    tcpiiu ( 
        cac & cac, epics :: Mutex & mutualExclusion, 
        epics :: Mutex & callbackControl, 
        cacContextNotify &, double connectionTimeout, 
        epicsTimerQueue & timerQueue, 
        const osiSockAddr & addrIn, 
        unsigned minorVersion, 
        ipAddrToAsciiEngine & engineIn, 
        const cacChannel::priLev & priorityIn,
        const bool nameService );
    void cleanup ( Guard & );
    ~tcpiiu ();
    void start (
        Guard & );
    void responsiveCircuitNotify ( 
        Guard & cbGuard,
        Guard & guard );
    void sendTimeoutNotify ( 
        callbackManager & cbMgr,
        Guard & guard );
    // dont call deferToRecvBacklog() while holding the callback lock.
    void deferToRecvBacklog ();
    void receiveTimeoutNotify( 
        callbackManager &,
        Guard & );
    void beaconAnomalyNotify ( 
        Guard & );
    void beaconArrivalNotify ( 
        Guard & );
    void probeResponseNotify ( 
        Guard & );

    void flushRequest ( 
        Guard & );
    size_t requestMessageBytesPending ( 
        Guard & mutualExclusionGuard );
    void flush ( 
        Guard & mutualExclusionGuard );

    void show ( unsigned level ) const;
    bool setEchoRequestPending ( 
        Guard & );
    void requestRecvProcessPostponedFlush (
        Guard & );
    void clearChannelRequest ( 
        Guard &, 
        ca_uint32_t sid, ca_uint32_t cid );

    bool ca_v41_ok (
        Guard & ) const;
    bool ca_v42_ok (
        Guard & ) const;
    bool ca_v44_ok (
        Guard & ) const;
    bool ca_v49_ok (
        Guard & ) const;

    unsigned getHostName ( 
        Guard &,
        char *pBuf, unsigned bufLength ) const throw ();
    bool alive (
        Guard & ) const;
    bool connecting (
        Guard & ) const;
    bool receiveThreadIsBusy ( 
        Guard & );
    osiSockAddr getNetworkAddress (
        Guard & ) const;
    int printFormated ( 
        Guard & cbGuard, 
        const char *pformat, ... );
    unsigned channelCount ( 
        Guard & );
    void disconnectAllChannels (
        Guard & cbGuard, 
        Guard & guard, class udpiiu & );
    void unlinkAllChannels (
        Guard & cbGuard, 
        Guard & guard );
    void installChannel ( 
        Guard &, nciu & chan, 
        unsigned sidIn, ca_uint16_t typeIn, arrayElementCount countIn );
    void uninstallChan ( 
        Guard &, nciu & );
    bool connectNotify ( 
        Guard &, nciu & );
#if 0
    void nameResolutionMsgEndNotify ();
#endif    
    void searchRespNotify ( 
        const epicsTime &, const caHdrLargeArray & );
    void versionRespNotify ( const caHdrLargeArray & );

private:
    class SearchDestTCP :
        public SearchDest {
    public:
        SearchDestTCP ( tcpiiu *, cac &, const osiSockAddr & );
        void searchRequest ( Guard & guard,
             const char * pbuf, size_t len );
        void show ( Guard & guard, unsigned level ) const;
        void disable ();
    private:
        tcpiiu * _ptcpiiu;
        cac & _cac;
        const osiSockAddr & _addr;
        bool _active;
    };

    cac & cacRef;
    epics :: Mutex & mutex;
    epics :: Mutex & cbMutex;
    SearchDestTCP * pSearchDest;
    hostNameCache hostNameCacheInstance;
    tcpRecvThread recvThread;
    tcpSendThread sendThread;
    tcpRecvWatchdog recvDog;
    tcpSendWatchdog sendDog;
    comQueSend sendQue;
    comBufMemoryManager _comBufMemMgr;
    // nciu state field tells us which list
    // protected by the callback mutex
    tsDLList < nciu > createReqPend;
    tsDLList < nciu > createRespPend;
    tsDLList < nciu > v42ConnCallbackPend;
    tsDLList < nciu > subscripReqPend;
    tsDLList < nciu > connectedList;
    tsDLList < nciu > unrespCircuit;
    tsDLList < nciu > subscripUpdateReqPend;
    unsigned minorProtocolVersion;
    enum iiu_conn_state { 
        iiucs_connecting, // pending circuit connect
        iiucs_connected, // live circuit
        iiucs_clean_shutdown, // live circuit will shutdown when flush completes
        iiucs_disconnected, // socket informed us of disconnect
        iiucs_abort_shutdown // socket has been closed
                } state;
    epicsEvent sendThreadFlushEvent;
    epicsEvent flushBlockEvent;
    SOCKET sock;
    size_t contigRecvMsgCount;
    size_t blockingForFlush;
    size_t socketLibrarySendBufferSize;
    size_t unacknowledgedSendBytes;
    size_t channelCountTot;
    bool _receiveThreadIsBusy;
    bool _nameService;
    bool busyStateDetected; // only modified by the recv thread
    bool flowControlActive; // only modified by the send process thread
    bool echoRequestPending; 
    bool earlyFlush;
    bool recvProcessPostponedFlush;
    bool discardingPendingData;
    bool socketHasBeenClosed;
    bool unresponsiveCircuit;
    bool _cleanupCompleted;

    size_t sendBytes ( const void *pBuf, 
        size_t nBytesInBuf, const epicsTime & currentTime );
    void recvBytes ( 
        void * pBuf, size_t nBytesInBuf, statusWireIO & );
    const char * pHostName (
        Guard & ) const throw ();
    double receiveWatchdogDelay (
        Guard & ) const;
    void unresponsiveCircuitNotify ( 
        Guard & cbGuard, 
        Guard & guard );
    void initiateCleanShutdown ( 
        Guard & );
    void initiateAbortShutdown ( 
        Guard & ); 
    void disconnectNotify (
        Guard & );
    bool bytesArePendingInOS () const;
    void decrementBlockingForFlushCount ( 
        Guard & guard );

    // send protocol stubs
    void echoRequest ( 
        Guard & );
    void versionMessage ( 
        Guard &, const cacChannel::priLev & priority );
    void disableFlowControlRequest (
        Guard & );
    void enableFlowControlRequest (
        Guard & );
    void hostNameSetRequest ( 
        Guard & );
    void userNameSetRequest ( 
        Guard & );
    void createChannelRequest ( 
        nciu &, Guard & );
    void writeRequest ( 
        Guard &, nciu &, 
        unsigned type, arrayElementCount nElem, const void *pValue );
    void writeNotifyRequest ( 
        Guard &, nciu &, 
        netWriteNotifyIO &, unsigned type, 
        arrayElementCount nElem, const void *pValue );
    void readNotifyRequest ( 
        Guard &, nciu &, 
        netReadNotifyIO &, unsigned type, 
        arrayElementCount nElem );
    void subscriptionRequest ( 
        Guard &, 
        nciu &, netSubscription & subscr );
    void subscriptionUpdateRequest ( 
        Guard &, 
        nciu & chan, netSubscription & subscr );
    void subscriptionCancelRequest ( 
        Guard &, 
        nciu & chan, netSubscription & subscr );
    void flushIfRecvProcessRequested (
        Guard & );

    // netiiu stubs
    void uninstallChanDueToSuccessfulSearchResponse ( 
        Guard &, nciu &, const class epicsTime & );
    bool searchMsg (
        Guard &, ca_uint32_t id, 
            const char * pName, unsigned nameLength );

    friend class tcpRecvThread;
    friend class tcpSendThread;

    tcpiiu ( const tcpiiu & );
    tcpiiu & operator = ( const tcpiiu & );
#   if __GNUC__ == 2 && _GNUC_MINOR__ <= 96
        friend class SearchDestTCP;
#   endif
};

inline bool tcpiiu::ca_v41_ok (
    Guard & ) const
{
    return CA_V41 ( this->minorProtocolVersion );
}

inline bool tcpiiu::ca_v44_ok (
    Guard & ) const
{
    return CA_V44 ( this->minorProtocolVersion );
}

inline bool tcpiiu::ca_v49_ok (
    Guard & ) const
{
    return CA_V49 ( this->minorProtocolVersion );
}

inline bool tcpiiu::alive (
    Guard & ) const // X aCC 361
{
    return ( this->state == iiucs_connecting || 
        this->state == iiucs_connected );
}

inline bool tcpiiu::connecting (
    Guard & ) const
{
    return ( this->state == iiucs_connecting );
}

inline bool tcpiiu::receiveThreadIsBusy ( 
    Guard & guard )
{
    guard.assertIdenticalMutex ( this->mutex );
    return this->_receiveThreadIsBusy;
}

inline void tcpiiu::beaconAnomalyNotify ( 
    Guard & guard )
{
    //guard.assertIdenticalMutex ( this->cacRef.mutexRef () );
    this->recvDog.beaconAnomalyNotify ( guard );
}

inline void tcpiiu::beaconArrivalNotify (
    Guard & guard )
{
    //guard.assertIdenticalMutex ( this->cacRef.mutexRef () );
    this->recvDog.beaconArrivalNotify ( guard );
}

inline void tcpiiu::probeResponseNotify (
    Guard & cbGuard )
{
    this->recvDog.probeResponseNotify ( cbGuard );
}

#endif // ifdef virtualCircuith

