
// Author: Jeff HIll

#error defunct

#define epicsExportSharedSymbols
#include "ByteSeqIn.h"

namespace ca {

ByteSeqIn :: 
    ByteSeqIn ( Guard & guard, 
        MemMgrListNodeComBuf & memMgrListNodeComBuf,
            Reader & reader ) :
    m_mutex ( guard ),
    m_memMgrListNodeComBuf ( memMgrListNodeComBuf ),
    m_reader ( reader ),
    m_nextElem ( 0 ), 
    //m_nBytesTotal ( 0 ),
    m_cleanupHasCompleted ( false )
{
}

ByteSeqIn :: 
    ~ByteSeqIn ()
{
    if ( ! m_cleanupHasCompleted ) {
        Guard guard ( m_mutex );
        this->cleanup ( guard );
    }
}

void ByteSeqIn :: 
    cleanup ( Guard & guard )
{
    guard.assertIdenticalMutex ( m_mutex );
    if ( ! m_cleanupHasCompleted ) {
        while ( ListNodeComBuf * pVec = m_bufList.get () ) {
            pVec->cleanup ( guard );
            m_byteVecInMemoryManager.deleteProduct ( *pVec );
        }
        m_mutex.cleanup ( guard );
        m_cleanupHasCompleted = true;
    }
}

#if 0
size_t ByteSeqIn :: 
    m_nBytesBuffered ( Guard & guard ) const
{
    guard.assertIdenticalMutex ( m_mutex );
    size_t nBytesTotal = 0u;
    size_t nBufs = m_bufList.count ();
    if ( nBufs >= 1 ) {
        nBytesTotal = ( nBufs - 1 ) * ComBuf :: size ()
        nBytesTotal += m_bufList.pLast ()->occupiedBytes ();
    }
}

void ByteSeqIn :: 
    extend ( Guard & guard )
{
    ListNodeComBuf * pListNode = m_bufList.pLast ();
    if ( ! pListNode || pListNode->unoccupiedBytes () == 0 ) {
        pListNode = new ( m_memMgrListNodeComBuf ) 
                    ListNodeComBuf ( guard ) );
        m_bufList.add ( *pListNode );
    }
    size_t nBytesActual =
        pListNode->fill ( guard, m_reader );
    if ( nBytesActual == 0 ) {
        throw std :: runtime_error ( 
            "ByteSeqIn :: extend (): "
            "end of stream detected" );
    }
}

void ByteSeqIn :: 
    extend ( Guard & guard, size_t nBytesRequired )
{
    {
        size_t nBytesPending = m_nBytesBuffered ( guard );
        if ( nBytesPending >= nBytesRequired ) {
            return;
        }
        nBytesRequired -= nBytesPending;
    }
    while ( true ) {
        ListNodeComBuf * pListNode = m_bufList.pLast ();
        if ( ! pListNode || pListNode->unoccupiedBytes () == 0 ) {
            pListNode = new ( m_memMgrListNodeComBuf ) 
                        ListNodeComBuf ( guard ) );
            m_bufList.add ( *pListNode );
        }
        size_t nBytesActual =
            pListNode->fill ( guard, m_reader, nBytesRequired );
        if ( nBytesActual == 0 ) {
            throw std :: runtime_error ( 
                "ByteSeqIn :: extend (): "
                "end of stream detected" );
        }
        if ( nBytesActual >= nBytesRequired ) {
            return;
        }
        nBytesRequired -= nBytesActual;
    }
}
#endif

ByteSeqIn :: Reader ::
    Reader ( ByteSeqIn & byteSeqIn ) :
    m_byteSeqIn ( byteSeqIn )
{
}

size_t ByteSeqIn :: Reader ::
    read ( Guard &, ByteVecPtrConst & )
{
    ListNodeComBuf * pSrcNode = m_byteSeqIn.m_bufList.pFirst ();
    if ( pSrcNode == 0 ) {
        m_byteSeqIn.extend ( guard );
        pSrcNode = m_byteSeqIn.m_bufList.pFirst ();
        if ( pSrcNode == 0 ) {
            throw std :: runtime_error ( 
                "ByteSeqIn :: extend () needs to "
                "throw an exception when it hits "
                "end of stream, but it didnt" );
        }
    }
    if ( pSrcNode->size() == 0 ) {
        m_byteSeqIn.m_bufList.remove ( *pSrcNode );
        m_byteSeqIn.m_memMgrListNodeComBuf.deleteProduct ( *pSrcNode );
        continue;
    }
    FreeListAutoPtr < MemMgrListNodeComBuf >
        pListNode ( m_memMgrListNodeComBuf,
            new ( m_memMgrListNodeComBuf ) 
                ListNodeComBuf ( guard ) );
    ListNodeComBuf :: Reader reader ( *pSrcNode );
    size_t nRemoved = pListNode->fill ( guard, reader );
    if ( nRemoved == 0 ) {
        throw std :: runtime_error ( 
            "ListNodeComBuf :: Reader returned zero "
            "bytes from list node that wasnt empty?" );
    }
    if ( pSrcNode->size() == 0 ) {
        m_byteSeqIn.m_bufList.remove ( *pSrcNode );
        m_byteSeqIn.m_memMgrListNodeComBuf.deleteProduct ( *pSrcNode );
    }
    m_bufList.add ( *pListNode );
    pListNode.release ();
    return nRemoved;
}

size_t ByteSeqIn :: Reader ::
    read ( Guard &, ByteVecPtrConst &, size_t nBytesRequired )
{
    while ( nBytesRequired ) {
        ListNodeComBuf * pSrcNode = m_byteSeqIn.m_bufList.pFirst ();
        if ( pSrcNode == 0 ) {
            m_byteSeqIn.extend ( guard, nBytesRequired );
            pSrcNode = m_byteSeqIn.m_bufList.pFirst ();
            if ( pSrcNode == 0 ) {
                throw std :: runtime_error ( 
                    "ByteSeqIn :: extend () needs to "
                    "throw an exception when it hits "
                    "end of stream, but it didnt" );
            }
        }
        if ( pSrcNode->size() == 0 ) {
            m_byteSeqIn.m_bufList.remove ( *pSrcNode );
            m_byteSeqIn.m_memMgrListNodeComBuf.deleteProduct ( *pSrcNode );
            continue;
        }
        FreeListAutoPtr < MemMgrListNodeComBuf >
            pListNode ( m_memMgrListNodeComBuf,
                new ( m_memMgrListNodeComBuf ) 
                    ListNodeComBuf ( guard ) );
        ListNodeComBuf :: Reader reader ( *pSrcNode );
        size_t nRemoved = pListNode->fill ( guard, reader );
        if ( nRemoved == 0 ) {
            throw std :: runtime_error ( 
                "ListNodeComBuf :: Reader returned zero "
                "bytes from list node that wasnt empty?" );
        }
        if ( pSrcNode->size() == 0 ) {
            m_byteSeqIn.m_bufList.remove ( *pSrcNode );
            m_byteSeqIn.m_memMgrListNodeComBuf.deleteProduct ( *pSrcNode );
        }
        m_bufList.add ( *pListNode );
        pListNode.release ();
        nBytesRequired -= nRemoved;
    }
}

ByteSeqIn :: ListNodeComBuf :: Reader ::
    Reader ( ListNodeComBuf & listNodeComBuf ) :
    m_listNodeComBuf ( listNodeComBuf )
{
}

void ByteSeqIn :: ListNodeComBuf :: Reader :: 
    read ( Guard & guard, ByteVecPtrConst & byteVecPtrConst )
{ 
    byteVecPtrConst.assign ( guard, *m_pVec );
    m_pVec.release ( guard );
}

void ByteSeqIn :: ListNodeComBuf :: Reader :: 
    read ( Guard &, ByteVecPtrConst & dst, size_t nBytesMax )
{
    m_pVec.remove ( guard, dst, nBytesMax );
}

}; // end of namespace ca
