
/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *
 *
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Copyright, 1986, The Regents of the University of California.
 *
 *
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#error defunct

#ifndef ByteSeqInh
#define ByteSeqInh

class StringEditor;

namespace ca {

struct HdrLargeArray;

class ByteSeqIn {
public:
    class IterBuf;
    ByteSeqIn ( Guard &, Reader & );
    void cleanup ( Guard & );
    ~ByteSeqIn ();
	void show ( Guard &, unsigned level ) const;
private:
    class ListNodeComBuf;
    typedef FreeList < ListNodeComBuf, 0x100 > MemMgrListNodeComBuf;
    tsDLList < ListNodeComBuf > m_bufList;
    Mutex m_mutex;
    // 1) sharing these between the different byte sequences is
    // problematic because they are invariably in different 
    // lock domains (even in and out). Performance is severely 
    // impacted if we have a common free list with a global lock.
    // 2) Modern best fit memory generalized memory allocators
    // are very good about not fragmenting memory so perhaps
    // we need not be concerned about returning the free list 
    // to pool when a client goes away.
    MemMgrListNodeComBuf m_memMgrListNodeComBuf;
    Reader & m_reader;
    size_t m_nextElem;
    bool m_cleanupHasCompleted;
    //size_t m_nBytesBuffered ( Guard & ) const;
    ByteSeqIn ( const ByteSeqIn & ); // disabled
    ByteSeqIn & operator = ( const ByteSeqIn & ); // disabled
};

class ByteSeqIn :: ListNodeComBuf : 
    public tsDLNode < ListNodeComBuf > {
public:
    class Reader;
    ListNodeComBuf ( Guard & );
    void cleanup ( Guard & );
    ~ListNodeComBuf ();
    size_t fill ( Guard &, Reader & );
    size_t fill ( Guard &, Reader &, size_t nBytesRequired );
    size_t occupiedBytes ();
    size_t unoccupiedBytes ();
    static size_t maxSize ();
    void show ( Guard &, unsigned level ) const;
    void assignPtr ( Guard &, VecGuard < const epicsUInt8 > & );
private:
    volatile Mutex m_mutex;
    ByteVecPtrConst m_pVec;
};

class ByteSeqIn :: IterBuf {
public:
    IterBuf ( Guard &, ByteSeqIn & );
    // there can be only one iterator outstanding at a time
    // so the copy constructor disconnects the source
    // iterator from the stream
    IterBuf ( IterBuf & );
    ~IterBuf ();
    void next ( ByteVecPtrConst &, size_t nBytesToSkip = 0 );
    void pushBack ( ByteVecPtrConst & );
private:
    // guarantees that the ByteSeqIn is protected 
    // for the full duration of this object's lifespan
    Guard m_guard;
    ByteSeqIn * m_pByteSeqIn;
    IterBuf & operator = ( const IterBuf & );
};

class ByteSeqIn :: Reader : 
    public Reader {
public: 
    Reader ( ByteSeqIn & byteSeqIn );
    void read ( Guard &, ByteVecPtrConst & );
    void read ( Guard &, ByteVecPtrConst &, size_t nBytesRequired );
private:
    ByteSeqIn & m_byteSeqIn;
};

class ByteSeqIn :: ListNodeComBuf :: Reader : 
    public Reader {
public: 
    Reader ( ListNodeComBuf & listNodeComBuf );
    void read ( Guard &, ByteVecPtrConst & );
    void read ( Guard &, ByteVecPtrConst &, size_t nBytesRequired );
private:
    ListNodeComBuf & m_listNodeComBuf;
};

} // end of namespace ca

#endif // ByteSeqInh

