
/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *
 *
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Copyright, 1986, The Regents of the University of California.
 *
 *
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#ifndef CatalogIterDBRh
#define CatalogIterDBRh

#include <cstdlib>

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_CatalogIterDBRh
#   undef epicsExportSharedSymbols
#endif

#include "epicsMutex.h"
#include "daCatalog.h"
#include "tsFreeList.h"
#include "EnclLnk.h"

#ifdef epicsExportSharedSymbols_CatalogIterDBRh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "caPtr.h"
#include "caRefCtr.h"
#include "IterInReader.h"

namespace ca {

using std :: size_t;
using namespace da;

class CatalogIterDBR {
public:
    static Ptr < const Catalog > factory ( unsigned dbrType, IterIn & iterIn, 
        size_t nElem, size_t payloadSize );
    CatalogIterDBR ( unsigned dbrType, IterIn & iterIn, 
        size_t nElem, size_t payloadSize );
private:
    class M_Handle : 
        public HandleIntf < const Catalog > {
    private:
        TargetMutexPair m_targetMutexPair ();
        HandleIntf < const Catalog > & m_clone ();
        void m_release ();
    private:
        RefCtr m_refCtr;
        typedef const epics :: EnclLnk < 
            CatalogIterDBR, M_Handle > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
        static epics :: Mutex m_mutexNoop;
    };
    // Using a common base, and a sans data member
    // deriving template makes it hopefully possible 
    // to compute the maximum size of any of the template 
    // objects. See the failsafe assert in m_catFactory
    // which detects when this presumption might turn
    // out to be wrong.
    class CatalogIterBase : public Catalog {
    public:
        CatalogIterBase ( CatalogIterDBR & );
    protected:
        CatalogIterDBR & m_catalogIterDBR;
    };
    // template < unsigned BEGTYPE, unsigned ENDTYPE >
    // static size_t m_maxCatSize ();
    typedef Catalog & ( CatalogIterDBR :: * PCatFactory )();
    static const PCatFactory m_pCatFactoryDBR [];
    template < class T > class M_CatalogAtomic;
    template < class F, class T, class PAD0 = void, 
            class PAD1 = void > class M_CatalogCompound;
    char m_catBuf [ sizeof ( CatalogIterBase ) ];
    mutable IterInReader m_iterInReader;
    const Catalog & m_catalog;
    M_Handle m_handle;
    size_t m_nElem;
    template < class T > Catalog & m_catFactory ();
    Catalog & m_catFactory ( unsigned type );
    static tsFreeList < CatalogIterDBR > m_freeList;
    static epics :: Mutex m_mutex;
};

#if 0
template < unsigned BEGTYPE, unsigned ENDTYPE >
size_t CatalogIterDBR :: m_maxCatSize ()
{
    assert ( BEGTYPE <= ENDTYPE );
    size_t maxSize = sizeof ( typename M_CatalogMap < BEGTYPE > :: Cat );
    if ( BEGTYPE < ENDTYPE ) {
        size_t maxOthers = m_maxCatSize < BEGTYPE+1u, ENDTYPE > ();
        if ( maxSize < maxOthers ) {
            maxSize = maxOthers;
        }
    }
    return maxSize;
}
#endif

inline CatalogIterDBR :: 
    CatalogIterDBR ( unsigned dbrType, IterIn & iterIn, 
        size_t nElem, size_t payloadSize ) :
    m_iterInReader ( iterIn, payloadSize ),
    m_catalog ( this->m_catFactory ( dbrType ) ),
    m_nElem ( nElem )
{
}

template < class T >
inline Catalog & CatalogIterDBR :: m_catFactory ()
{
    assert ( sizeof ( T ) <= sizeof ( m_catBuf ) );
    return * new ( & m_catBuf ) T ( *this );
}

} // end of namespace ca

#endif // ifndef CatalogIterDBRh

