
/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *
 *
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Copyright, 1986, The Regents of the University of California.
 *
 *
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#error defunct

#ifndef CatalogIterDBRAtomich
#define CatalogIterDBRAtomich

#include <cstdlib>

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_CatalogIterDBRAtomich
#   undef epicsExportSharedSymbols
#endif

#include "dadbrIdent.h"
#include "daCatalog.h"

#ifdef epicsExportSharedSymbols_CatalogIterDBRAtomich
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "caPtr.h"
#include "IterInReader.h"

namespace ca {

using std :: size_t;

class CatalogIterDBR : 
    public Catalog {
public:
    Ptr < Catalog > factory ( IterIn & iterIn, 
                    size_t nElem, size_t payloadSize );
    template < unsigned TYPE >
    CatalogIterDBR ( IterIn & iterIn, 
        size_t nElem, size_t payloadSize );
private:
    class M_Handle : 
        public HandleIntf < Catalog > {
    public:
        typedef M_Handle CatalogIterDBR :: * PMemSelf;
        static PMemSelf pMemSelf ();
    private:
        TargetMutexPair m_targetMutexPair ();
        HandleIntf < Catalog > & m_clone ();
        void m_release ();
    };
    template < unsigned BEGTYPE, unsigned ENDTYPE >
    static size_t m_maxCatSize ();
    template < unsigned TYPE >
    class M_CatalogMap;
    template < class T >
    class M_CatalogAtomic : 
        public Catalog {
    public:
        M_CatalogAtomic ( CatalogIterDBR & );
        void traverse ( Introspector & ) const;
    private:
        CatalogIterDBR & m_catalogIterDBR;
    };
    char m_catBuf [ sizeof ( M_CatalogAtomic < char > ) ];
    mutable IterInReader m_iterInReader;
    const Catalog & m_catalog;
    size_t m_nElem;
};

template <>
class CatalogIterDBR :: M_CatalogMap < DBR_SHORT > {
    typedef M_CatalogAtomic < dbr_short_t > Cat;
};

template < unsigned BEGTYPE, unsigned ENDTYPE >
size_t CatalogIterDBR :: m_maxCatSize ()
{
    assert ( BEGTYPE <= ENDTYPE );
    size_t maxSize = sizeof ( typename M_CatalogMap < TYPE >:: Cat );
    if ( BEGTYPE < ENDTYPE ) {
        size_t maxOthers = m_maxCatSize < BEGTYPE+1u, ENDTYPE > ();
        if ( maxSize < maxOthers ) {
            maxSize = maxOthers;
        }
    }
    return maxSize;
}

template < unsigned TYPE >
inline CatalogIterDBR :: 
    CatalogIterDBR ( IterIn & iterIn, 
        size_t nElem, size_t payloadSize ) :
    m_iterInReader ( iterIn, payloadSize ),
    m_catalog ( * new ( m_catBuf ) typename M_CatalogMap < TYPE >:: Cat () ),
    m_nElem ( nElem )
{
    assert ( sizeof ( typename M_CatalogMap < TYPE >:: Cat ) <= sizeof ( m_catBuf ) );
}

template < class T >
inline CatalogIterDBR :: M_CatalogAtomic < T > :: 
    M_CatalogAtomic ( CatalogIterDBR & catalogIterDBR ) :
    m_catalogIterDBR ( catalogIterDBR )
{
}

class FrameSeq

template < class T >
void CatalogIterDBR :: M_CatalogAtomic < T > :: 
    traverse ( Introspector & introspector ) const
{
    m_catalogIterDBR.m_iterInReader.rewind ();
    IterIn iterIn ( m_catalogIterDBR.m_iterInReader );
@@@@
template < class T >
class FrameSeq {
public:
    virtual ~FrameSeq () {}
    virtual void rewind () = 0;
    // a capability for skipping over some number 
    // of elements (the nElemToBypass arg) improves 
    // performance when iterating an array slice
    virtual Frame < T > next ( size_t nElemToBypass ) = 0;
};

typedef epicsUInt8 Octet;
typedef FrameSeq < const Octet > FrameSeqNet;
@@@@
    for ( size_t i = 0u; i < m_catalogIterDBR.m_nElem; i++ ) {
#error no
        introspector.reveal ( da :: dbr :: pi_value, iterIn.pop < T > () );
    }
}

} // end of namespace ca

#endif // ifndef CatalogIterDBRAtomich

