
/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *
 *
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Copyright, 1986, The Regents of the University of California.
 *
 *
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#error defunct

#ifndef CatalogIterDBRStsh
#define CatalogIterDBRStsh

#include <cstdlib>

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_CatalogIterDBRStsh
#   undef epicsExportSharedSymbols
#endif

#include "EnclBinding.h"
#include "dadbrIdent.h"
#include "daCatalog.h"

#ifdef epicsExportSharedSymbols_CatalogIterDBRStsh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "IterInReader.h"

namespace ca {

using std :: size_t;





template < class T >
CatalogIterDBRSts < T > :: 
    CatalogIterDBRSts ( IterIn & iterIn, 
        size_t nElem, size_t payloadSize ) :
    m_iterInReader ( iterIn, payloadSize ), 
    m_nElem ( nElem )
{
}

template < class T >
void CatalogIterDBRSts < T > :: 
    traverse ( Introspector & introspector ) const
{
    m_iterInReader.rewind ();
    IterIn iterIn ( m_iterInReader );
    CatalogIterAlarm catalogIterAlarm ( iterIn, 2 * sizeof ( dbr_short_t ) );
    introspector.reveal ( da :: dbr :: pi_alarm, catalogIterAlarm );
    for ( size_t i = 0u; i < m_nElem; i++ ) {
        introspector.reveal ( da :: dbr :: pi_value, iterIn.pop < T > () );
    }
}

template <>
void CatalogIterDBRSts < dbr_char_t > :: 
    traverse ( Introspector & introspector ) const
{
    m_iterInReader.rewind ();
    IterIn iterIn ( m_iterInReader );
    CatalogIterAlarm catalogIterAlarm ( iterIn, 2 * sizeof ( dbr_short_t ) );
    introspector.reveal ( da :: dbr :: pi_alarm, catalogIterAlarm );
    iterIn.pop < dbr_char_t > ();
    for ( size_t i = 0u; i < m_nElem; i++ ) {
        introspector.reveal ( da :: dbr :: pi_value, iterIn.pop < dbr_char_t > () );
    }
}

template <>
void CatalogIterDBRSts < dbr_double_t > :: 
    traverse ( Introspector & introspector ) const
{
    m_iterInReader.rewind ();
    IterIn iterIn ( m_iterInReader );
    CatalogIterAlarm catalogIterAlarm ( iterIn, 2 * sizeof ( dbr_short_t ) );
    introspector.reveal ( da :: dbr :: pi_alarm, catalogIterAlarm );
    iterIn.pop < dbr_long_t > ();
    for ( size_t i = 0u; i < m_nElem; i++ ) {
        introspector.reveal ( da :: dbr :: pi_value, iterIn.pop < dbr_double_t > () );
    }
}

} // end of namespace ca

#endif // ifndef CatalogIterDBRStsh

