/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *
 *
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Copyright, 1986, The Regents of the University of California.
 *
 *
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#include "ComBuf.h"

namespace ca {

//
// ComBuf :: M_HandleVec
//
ComBuf :: M_HandleVec :: M_EnclLnk
    ComBuf :: M_HandleVec ::   
        m_enclLnk ( & ComBuf :: m_handleVec );
HandleVec &
    ComBuf :: M_HandleVec :: 
        clone ()
{
    ComBuf & comBuf ( m_enclLnk.enclosure ( *this ) );
    comBuf.m_refCtr.reference ();
    return *this;
}

void ComBuf :: M_HandleVec :: 
    release ()
{
    ComBuf & comBuf ( m_enclLnk.enclosure ( *this ) );
    if ( comBuf.m_refCtr.unreference () ) {
        comBuf.m_destructor.destroy ( comBuf );
    }
}

} // end of namespace ca
#if 0
void ComBuf :: ByteSeq :: 
    m_lastChanceCleanup ()
{
#   ifndef NDEBUG
        fprintf ( stderr, 
            "Performance enhancement tip: "
            "call ByteSeq :: clear() "
            "explicitly prior to ~ByteSeq" );
#   endif
    Guard guard ( m_mutex );
    this->clearAll ( guard );
}

void ComBuf :: ByteSeq :: flush ( 
    Guard & guard, FlushAgent & agent )
{
    guard.assertIdenticalMutex ( m_mutex );
    tsDLIter < ComBuf > iter = m_bufList.firstIter ();
    while ( iter.valid () ) {
        size_t totalBytes = iter->m_nBytesInUse;
        size_t amountFlushed = 0;
        for ( 
            size_t index = 0; 
            index < iter->m_nBytesInUse; 
            index += amountFlushed ) {
            amountFlushed = agent.flush ( 
                    & iter->m_buf [ index ], 
                    iter->m_nBytesInUse - index );
        }
        iter->m_ref.unreference ( guard );
        iter++;
    }
}

void ComBuf :: ByteSeq :: add ( 
    Guard & guard, ComBuf & buf )
{
    guard.assertIdenticalMutex ( m_mutex );
    buf.m_ref.reference ( guard );
    m_bufList.add ( buf );
}

void ComBuf :: ByteSeq :: clearAll ( 
    Guard & guard )
{
    guard.assertIdenticalMutex ( m_mutex );
    tsDLIter < ComBuf > iter = m_bufList.firstIter ();
    while ( iter.valid () ) {
        iter->m_ref.unreference ( guard );
    }
}

void ComBuf :: ByteSeq :: pop ( 
    Guard & guard, HdrLargeArray & destHdr )
{
    guard.assertIdenticalMutex ( m_mutex );


    // try first for all in one buffer efficent version
    comBuf * pComBuf = m_bufs.first ();
    if ( ! pComBuf ) {
        return false;
    }
    unsigned avail = pComBuf->occupiedBytes ();
    if ( avail >= sizeof ( caHdr ) ) {
        pComBuf->pop ( destHdr.m_cmmd );
        ca_uint16_t smallPostsize;
        pComBuf->pop ( smallPostsize );
        msg.m_postsize = smallPostsize;
        pComBuf->pop ( destHdr.m_dataType );
        ca_uint16_t smallCount;
        pComBuf->pop ( smallCount );
        msg.m_count = smallCount;
        pComBuf->pop ( destHdr.m_cid );
        pComBuf->pop ( destHdr.m_available );
        m_nBytesPending -= sizeof ( caHdr );
        if ( avail == sizeof ( caHdr ) ) {
            m_removeAndDestroyBuf ( guard, *pComBuf );
        }
        return true;
    }
    else if ( this->occupiedBytes ( guard ) >= sizeof ( caHdr ) ) {
        destHdr.m_cmmd = this->popUInt16 ( guard );
        destHdr.m_postsize = this->popUInt16 ( guard );
        destHdr.m_dataType = this->popUInt16 ( guard );
        destHdr.m_count = this->popUInt16 ( guard );
        destHdr.m_cid = this->popUInt32 ( guard );
        destHdr.m_available = this->popUInt32 ( guard );
        return true;
    }
    else {
        return false;
    }

@@@@@@
    if ( m_curMsg.m_postsize == 0xffff ) {
        static const size_t annexSize = 
            sizeof ( m_curMsg.m_postsize ) + 
            sizeof ( m_curMsg.m_count );
        if ( m_recvQue.occupiedBytes ( m_recvQueGuard ) < annexSize ) {
            Guard guard ( m_iiu.mutex );
            m_iiu.flushIfRecvProcessRequested ( guard );
            return true;
        }
        m_curMsg.m_postsize = m_recvQue.popUInt32 ( m_recvQueGuard );
        m_curMsg.m_count = m_recvQue.popUInt32 ( m_recvQueGuard );
    }
@@@@@@

    unsigned msgSize;
    ca_uint32_t hdrSize;
    char * rawMP;
    {
        //
        // copy as raw bytes in order to avoid
        // alignment problems
        //
        caHdr smallHdr;
        if ( bytesLeft < sizeof ( smallHdr ) ) {
            break;
        }

        rawMP = m_in.msgPtr ( inBufGuard );
        memcpy ( & smallHdr, rawMP, sizeof ( smallHdr ) );

        ca_uint32_t payloadSize = AlignedWireRef < epicsUInt16 > ( smallHdr.m_postsize );
        ca_uint32_t nElem = AlignedWireRef < epicsUInt16 > ( smallHdr.m_count );
        if ( payloadSize != 0xffff && nElem != 0xffff ) {
            hdrSize = sizeof ( smallHdr );
        }
        else {
            ca_uint32_t LWA[2];
            hdrSize = sizeof ( smallHdr ) + sizeof ( LWA );
            if ( bytesLeft < hdrSize ) {
                break;
            }
            //
            // copy as raw bytes in order to avoid
            // alignment problems
            //
            memcpy ( LWA, rawMP + sizeof ( caHdr ), sizeof( LWA ) );
            payloadSize = AlignedWireRef < epicsUInt32 > ( LWA[0] );
            nElem = AlignedWireRef < epicsUInt32 > ( LWA[1] );
        }

        msgTmp.m_cmmd = AlignedWireRef < epicsUInt16 > ( smallHdr.m_cmmd );
        msgTmp.m_postsize = payloadSize;
        msgTmp.m_dataType = AlignedWireRef < epicsUInt16 > ( smallHdr.m_dataType );
        msgTmp.m_count = nElem;
        msgTmp.m_cid = AlignedWireRef < epicsUInt32 > ( smallHdr.m_cid );
        msgTmp.m_available = AlignedWireRef < epicsUInt32 > ( smallHdr.m_available );

        // disconnect clients that dont send 8 byte aligned payloads
        if ( payloadSize & 0x7 ) {
            Server :: dumpMsg ( 
                m_client.m_hostName.c_str (), m_client.m_userName.c_str (), 
                msgTmp, "CAS: Stream request wasn't 8 byte aligned\n" );
            throw InvalidProtocol ( "input stream request wasn't 8 byte aligned" );
        }

        msgSize = hdrSize + payloadSize;
        if ( bytesLeft < msgSize ) {
            if ( msgSize > m_in.bufferSize ( inBufGuard ) ) {
                m_in.expandBuffer ( inBufGuard );
                // detect msg was too large
                if ( msgSize > m_in.bufferSize ( inBufGuard ) ) {
                    // set up diagnostic response
                    new ( m_client.m_badMsgLenEventFreeList )
                        BadMsgLen ( guard, m_client, msgSize );
                    m_byteSeqIn.drain ( inBufGuard, msgSize );
                }
            }
            break;
        }

        if ( m_client.m_cas.getDebugLevel () > 2u ) {
	        Server :: dumpMsg ( 
                m_client.m_hostName.c_str (), m_client.m_userName.c_str (), 
                msgTmp, 0 );
        }
    }
@@@@@@
}

// dbrPushSize () - a static method
size_t ByteSeqOut :: 
    dbrPushSize ( unsigned dbr, size_t nElem, const Catalog & )
{
    return dbr_size_n ( dbr, nElem );
#error calculate different size if scalar strings are variable sized in push
}



void ByteSeqOut :: pushErrorResponse ( 
    Guard & guard, const HdrLargeArray & msg, 
    ca_uint32_t cid, const int reportedStatus, 
    const char *pformat, ... )
{
#error old code copied in here needs work
    guard.assertIdenticalMutex ( m_mutex );
	unsigned stringSize;
	char msgBuf[1024]; /* allocate plenty of space for the message string */
	if ( pformat ) {
	    va_list args;
		va_start ( args, pformat );
		int status = vsprintf ( msgBuf, pformat, args );
		if ( status < 0 ) {
			errPrintf ( S_cas_internal, __FILE__, __LINE__,
				"bad sendErr(%s)", pformat);
			stringSize = 0u;
		}
		else {
			stringSize = static_cast < unsigned > ( status );
			stringSize += 1;
		}
	}
	else {
		stringSize = 0u;
	}
	ConstString diag ( msgBuf, stringSize );
	sendErr ( guard, msg, cid, reportedStatus, diag );
}

void ByteSeqOut :: pushErrorResponse ( 
    Guard & guard, 
    const HdrLargeArray & msg,
	ca_uint32_t cid, const int reportedStatus, 
    const Catalog & diag )
{
#error old code copied in here needs work
    {
        Property < int > dst ( 
            FailureResponse :: pi_ecaLegacyErrorCode, reportedStatus );
        Property < const int > dflt ( 
            FailureResponse :: pi_ecaLegacyErrorCode, reportedStatus );
        prop.value ().extract ( dst, & dflt );
    }

    // allocate plenty of space for the message string
    FixedString < 1024 > fs;
    Property < StringEditor > sp ( pi_context, fs );
    diag.extract ( sp, & sp );

    sendErr ( guard, msg, cid, reportedStatus, fs );
}

void ByteSeqOut :: pushErrorResponse ( 
    Guard & guard, 
    const HdrLargeArray & msg, 
    ca_uint32_t cid, const int reportedStatus, 
    const StringReader & diag )
{
#error old code copied in here needs work
	unsigned stringSize = diag.size ();
    if ( stringSize > 0 ) {
        stringSize++;
    }

    unsigned hdrSize = sizeof ( caHdr );
    if ( ( msg.m_postsize >= 0xffff || msg.m_count >= 0xffff ) && 
            CA_V49( m_minor_version_number ) ) {
        hdrSize += 2 * sizeof ( ca_uint32_t );
    }

    caHdr * pReqOut = reinterpret_cast < caHdr * > (
        copyInHeader ( guard, CA_PROTO_ERROR, hdrSize + stringSize, 
                0, 0, cid, reportedStatus ) );

    // copy back the request protocol (in network byte order)
    char * pMsgString;
    if ( ( msg.m_postsize >= 0xffff || msg.m_count >= 0xffff ) && 
            CA_V49( m_minor_version_number ) ) {
        ca_uint32_t *pLW = ( ca_uint32_t * ) ( pReqOut + 1 );
        pReqOut->m_cmmd = htons ( msg.m_cmmd );
        pReqOut->m_postsize = htons ( 0xffff );
        pReqOut->m_dataType = htons ( msg.m_dataType );
        pReqOut->m_count = htons ( 0u );
        pReqOut->m_cid = htonl ( msg.m_cid );
        pReqOut->m_available = htonl ( msg.m_available );
        pLW[0] = htonl ( msg.m_postsize );
        pLW[1] = htonl ( msg.m_count );
        pMsgString = ( char * ) ( pLW + 2 );
    }
    else {
        pReqOut->m_cmmd = htons (msg.m_cmmd);
        pReqOut->m_postsize = htons ( ( (ca_uint16_t) msg.m_postsize ) );
        pReqOut->m_dataType = htons (msg.m_dataType);
        pReqOut->m_count = htons ( ( (ca_uint16_t) msg.m_count ) );
        pReqOut->m_cid = htonl (msg.m_cid);
        pReqOut->m_available = htonl (msg.m_available);
        pMsgString = ( char * ) ( pReqOut + 1 );
     }

    // add their context string into the protocol
    if ( stringSize ) {
	    VarLenStrRefEditor payload ( pMsgString, 0, stringSize );
	    payload = diag;
    }

    commitMsg ( guard );
}

#endif


