/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *
 *
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Copyright, 1986, The Regents of the University of California.
 *
 *
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#ifndef ComBufh
#define ComBufh

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_ComBufh
#   undef epicsExportSharedSymbols
#endif

#include "epicsAssert.h"
#include "epicsTypes.h"
#include "tsFreeList.h"
#include "tsDLList.h"
#include "daCatalog.h"
#include "EnclLnk.h"

#ifdef epicsExportSharedSymbols_ComBufh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

class StringReader;

#include "caVec.h"
#include "caRefCtr.h"

namespace ca {

class ComBuf {
public:
    struct Destructor {
        virtual void destroy ( ComBuf & ) = 0;
    };
    ComBuf ( Destructor & );
    Vec < epicsUInt8 > bufPtr ();
    Vec < const epicsUInt8 > bufPtr () const;
    static size_t size ();
private:
    class M_HandleVec : 
        public HandleVec {
    public:
        HandleVec & clone ();
        void release ();
    private:
        typedef const epics :: EnclLnk < ComBuf, 
                                    M_HandleVec > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    static const size_t m_size = 0x4000;
    epicsUInt8 m_buf [ m_size ];
    mutable M_HandleVec m_handleVec;
    RefCtr m_refCtr;
    Destructor & m_destructor;
};

inline ComBuf :: 
    ComBuf ( Destructor & destructor ) :
    m_destructor ( destructor )
{
}

inline size_t ComBuf :: size ()
{
    return m_size;
}

inline Vec < epicsUInt8 > ComBuf :: 
    bufPtr ()
{
    return Vec < epicsUInt8 > ( m_handleVec, m_buf, m_size );
}

inline Vec < const epicsUInt8 > ComBuf :: 
    bufPtr () const
{
    return Vec < const epicsUInt8 > ( m_handleVec, m_buf, m_size );
}

} // end of namespace ca

#endif // ifndef ComBufh
