
/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *      Author  Jeffrey O. Hill
 *              johill@lanl.gov
 *              505 665 1831
 */

#include <cstdio>

#define epicsExportSharedSymbols
#include "DGParm.h"

namespace ca {
namespace Svr {
namespace DG {

DGParm :: DGParm () :
    m_sequenceNumber ( 0u ),
    m_protocolRevision ( 0u )
{
}

DGParm :: DGParm ( const NetAddr & srcAddr,
                        const NetAddr & intfAddr ) :
    m_srcAddr ( srcAddr ),
    m_intfAddress ( intfAddr ),
    m_sequenceNumber ( 0u ),
    m_protocolRevision ( 0u )
{
}

bool DGParm :: operator == ( const DGParm & lhs ) const
{
    return m_srcAddr == lhs.m_srcAddr && 
        m_intfAddress == lhs.m_intfAddress &&
        m_sequenceNumber == lhs.m_sequenceNumber &&
        m_protocolRevision == lhs.m_protocolRevision;
}

//
// DGParm :: hostName()
//
void DGParm :: hostName ( char * pBufIn, unsigned bufSizeIn ) const
{
    m_srcAddr.stringConvert ( pBufIn, bufSizeIn );
}

void DGParm :: show ( unsigned level ) const
{
    printf ( "DGParm:\n" );
    if ( level > 0 ) {
        printf ( "DGParm: m_srcAddr\n" );
        m_srcAddr.show ( level - 1 );
        printf ( "DGParm: m_intfAddress\n" );
        m_intfAddress.show ( level - 1 );
        printf ( "DGParm: m_sequenceNumber=%u, m_protocolRevision=%u\n", 
            m_sequenceNumber, m_protocolRevision );
    }
}

} // end of namespace DG
} // end of namespace Svr
} // end of namespace ca
