
/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

#ifndef DGParmh
#define DGParmh

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_DGParmh
#   undef epicsExportSharedSymbols
#endif

#include "caProto.h"

#ifdef epicsExportSharedSymbols_DGParmh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#define caNetAddrSock
#include "caNetAddr.h"

namespace ca {
namespace Svr {
namespace DG {

class DGParm {
public:
    DGParm ();
    DGParm ( const NetAddr & srcAddr,
        const NetAddr & intfAddr );
    bool operator == ( const DGParm & ) const;
    bool operator != ( const DGParm & ) const;
    ca_uint16_t protocolRevision () const;
    void hostName ( char * pBuf, unsigned bufSize ) const;
    NetAddr srcAddr () const;
    NetAddr intfAddr () const;
    void setHdrParms ( ca_uint32_t sequenceNumber, 
                    ca_uint32_t protocolRevision);
    void show ( unsigned level ) const;
private:
    NetAddr m_srcAddr;
    // m_intfAddress is the destination address to be used 
    // in the client when creating a virtual circuit to 
    // the network interface the datagram was received on
    NetAddr m_intfAddress;
    ca_uint32_t m_sequenceNumber;
    ca_uint16_t m_protocolRevision;
};

inline ca_uint16_t DGParm :: 
    protocolRevision () const
{
    return m_protocolRevision;
}

inline void DGParm :: 
    setHdrParms ( ca_uint32_t sequenceNumber, 
            ca_uint32_t protocolRevision)
{
    m_sequenceNumber = sequenceNumber;
    m_protocolRevision = protocolRevision;
}

inline bool DGParm :: 
    operator != ( const DGParm & dgParm ) const
{
    return ! operator == ( dgParm );
}

inline NetAddr DGParm :: 
    srcAddr () const
{
    return m_srcAddr;
}

inline NetAddr DGParm :: 
    intfAddr () const
{
    return m_intfAddress;
}

} // end of namespace DG
} // end of namespace Svr
} // end of namespace ca

#endif // ifndef DGParm

