/*
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#error probably defunct

#define epicsExportSharedSymbols
#include "caSvrEvQue.h"

namespace ca {

inline void EvQue :: Reservation ::
    m_sanityCheck () const
{
#   ifndef NDEBUG
        bool slotsAreReserved = m_nSlots != 0;
        bool queueIsAttached = m_pEventQueue != 0;
        assert ( slotsAreReserved == queueIsAttached );
#   endif
}

// transfer ownership of the source Reservation
// to this class
EvQue :: Reservation :: 
    Reservation ( Reservation & reservation ) :
    m_pEventQueue ( reservation.m_pEventQueue ), 
    m_nSlots ( reservation.m_nSlots )
{
    m_sanityCheck ();
    // release ownership in the source reserveation
    reservation.m_nSlots = 0;
    reservation.m_pEventQueue = 0;
}

EvQue :: Reservation :: 
    ~Reservation ()
{
    if ( m_nSlots ) {
        assert ( m_pEventQueue );
        m_pEventQueue->m_release ( m_nSlots );
    }
}

void EvQue :: Reservation :: 
    failResponse ( 
        Responder & responder,
        const char * pContext )
{
    if ( ! m_pEventQueue ) {
        throw std :: runtime_error (
            "EvQue :: Reservation :: post: "
            "an event was posted when the Reservation "
            "wasnt attached to an event queue" );
    }
    m_pEventQueue->m_failResponse ( guard, responder, pContext );
}

EvQue :: Reservation & 
    EvQue :: Reservation :: 
        operator = ( Reservation & reservation )
{
    if ( this != & reservation ) {
        if ( m_nSlots ) {
            assert ( m_pEventQueue );
            // implicit clear
            // if you end up here its less efficent because
            // an explicit clear didnt run
            if ( m_pEventQueue == reservation.m_pEventQueue ) {
                if ( m_nSlots > reservation.m_nSlots ) {
                    Guard guard ( m_pEventQueue->m_interfaceMutex );
                    m_pEventQueue->m_release ( 
                        guard, m_nSlots - reservation.m_nSlots );
                }
                else if ( m_nSlots < reservation.m_nSlots ) {
                    Guard guard ( m_pEventQueue->m_interfaceMutex );
                    m_pEventQueue->m_reserve ( 
                        guard, reservation.m_nSlots - m_nSlots );
                }
            }
            else {
                Guard guard ( m_pEventQueue->m_interfaceMutex );
                m_pEventQueue->m_release ( guard, m_nSlots );
            }
        }
        m_pEventQueue = reservation.m_pEventQueue;
        m_nSlots = reservation.m_nSlots;
        reservation.m_pEventQueue = 0;
        reservation.m_nSlots = 0;
    }
    return *this;
}

void EvQue :: Reservation :: 
    reserve ( Guard & guard, EvQue & eventQueue, 
        size_t size )
{
    if ( m_pEventQueue == & eventQueue ) {
        if ( size > m_nSlots ) {
            assert ( m_pEventQueue );
            m_pEventQueue->m_reserve ( guard, size - m_nSlots );
            m_nSlots = size;
        }
    }
    else {
        if ( m_nSlots ) {
            assert ( m_pEventQueue );
            m_pEventQueue->m_release ( guard, m_nSlots );
        }
        if ( size ) {
            m_pEventQueue = & eventQueue;
        }
        else {
            m_pEventQueue = 0;
        }
        m_nSlots = size;
    }
}

// it is more efficent in terms of Guard reuse
// to explicitly clear the reservation before 
// the destructor runs 
void EvQue :: Reservation :: 
    clear ( Guard & guard )
{
    if ( m_nSlots ) {
        assert ( m_pEventQueue );
        m_pEventQueue->m_release ( guard, m_nSlots );
        m_pEventQueue = 0;
        m_nSlots = 0;
    }
}

} // end of namespace ca