

#define epicsExportSharedSymbols
#include "IterInReader.h"


namespace ca {

IterInReader :: VecNode ::
    VecNode ( IterIn & iterIn, size_t nBytes ) :
    m_vec ( iterIn.popVec ( nBytes ) )
{
    const size_t nBytesCur = m_vec.nElem ();
    if ( nBytesCur > nBytes ) {
        throw std :: runtime_error ( 
            "iterIn.popVec: intf violation - "
            "too many bytes read" );
    }
    size_t nBytesLeft = nBytes - nBytesCur;
    if ( nBytesLeft ) {
        if ( nBytesCur == 0u ) {
            throw std :: runtime_error ( 
                "IterInReader :: VecNode ctor: end of stream detected" );
        }
        m_pNext.reset ( new VecNode ( iterIn, nBytesLeft ) );
    }
}

IterInReader :: VecNode :: 
    VecNode ( const VecNode & vecNode ) :
    m_vec ( vecNode.m_vec ),
    m_pNext ( vecNode.m_pNext.get () ? 
        new VecNode ( *vecNode.m_pNext ) : 0 )
{
    if ( m_vec.nElem () == 0u ) {
        throw std :: runtime_error ( 
            "IterInReader :: VecNode copy ctor: empty source" );
    }
}

IterInReader :: VecNode :: 
    VecNode ( const VecNode & vecNode, size_t nBytes ) :
    m_vec ( vecNode.m_vec )
{
    if ( nBytes != 0u && m_vec.nElem () == 0u ) {
        throw std :: runtime_error ( 
            "IterInReader :: VecNode copy ctor: empty source" );
    }
    size_t nCur  = m_vec.nElem ();
    if ( nCur >= nBytes ) {
        m_vec.trimBack ( nCur - nBytes );
    }
    else {
        m_pNext.reset ( new VecNode ( *vecNode.m_pNext,  nBytes - nCur ) );
    }
}

IterInReader :: VecNode & 
    IterInReader :: VecNode :: 
        operator = ( const VecNode & vecNode )
{
    if ( vecNode.m_vec.nElem () == 0u ) {
        throw std :: runtime_error ( 
            "IterInReader :: VecNode copy ctor: empty source" );
    }
    m_vec = vecNode.m_vec;
    m_pNext.reset ( vecNode.m_pNext.get () ? 
        new VecNode ( *vecNode.m_pNext ) : 0 );
    return *this;
}

inline Vec < const epicsUInt8 >
    IterInReader :: VecNode :: 
        vector () const
{
    return m_vec;
}

inline IterInReader :: VecNode * 
    IterInReader :: VecNode :: pNext () const
{
    return m_pNext.get ();
}

inline size_t IterInReader :: VecNode :: 
	size () const
{
    return m_vec.nElem ();
}

size_t IterInReader :: VecNode :: 
	sizeAll () const
{
    size_t nSelf = m_vec.nElem ();
    if ( m_pNext.get () == 0 ) {
        return nSelf;
    }
    else {
        return nSelf + m_pNext->sizeAll ();
    }
}

IterInReader ::
    IterInReader ( const IterInReader & iterInReader ) :
    m_firstVecNode ( iterInReader.m_firstVecNode ),
    m_pCur ( & m_firstVecNode ),
    m_offsetCur ( 0u )
{
}

IterInReader ::
    IterInReader ( const IterInReader & iterInReader, size_t nBytes ) :
    m_firstVecNode ( iterInReader.m_firstVecNode, nBytes ),
    m_pCur ( & m_firstVecNode ),
    m_offsetCur ( 0u )
{
}

IterInReader & IterInReader ::
    operator = ( const IterInReader & iterInReader )
{
    m_firstVecNode = iterInReader.m_firstVecNode;
    m_pCur = & m_firstVecNode;
    m_offsetCur = 0u;
    return *this;
}

Vec < const epicsUInt8 > IterInReader :: 
    read ( size_t nBytesMax )
{
    if ( m_pCur == 0 ) {
        throw std :: range_error ( 
            "IterInReader :: attempt to read "
            "past the end of the message payload" );
    }
    size_t nCur = m_pCur->size ();
    assert ( m_offsetCur <= nCur );
    nCur -= m_offsetCur;
    assert ( nCur > 0u );
    Vec < const epicsUInt8 > vec = m_pCur->vector ();
    if ( nBytesMax < nCur ) {
        vec.trimBack ( nCur - nBytesMax );
        m_offsetCur += nBytesMax;
    }
    else {
        m_offsetCur = 0u;
        m_pCur = m_pCur->pNext ();
    }
    return vec;
}

void IterInReader :: 
    moveForward ( size_t nBytes )
{
    while ( nBytes ) {
        if ( m_pCur == 0 ) {
            throw std :: range_error ( 
                "IterInReader :: attempt to read "
                "past the end of the message payload" );
        }
        size_t nCur = m_pCur->size ();
        assert ( m_offsetCur <= nCur );
        nCur -= m_offsetCur;
        assert ( nCur > 0u );
        Vec < const epicsUInt8 > vec = m_pCur->vector ();
        if ( nBytes < nCur ) {
            m_offsetCur += nBytes;
            nBytes = 0;
        }
        else {
            m_offsetCur = 0u;
            m_pCur = m_pCur->pNext ();
            nBytes -= nCur;
        }
    }
}

void IterInReader :: 
    rewind ()
{
    m_pCur = & m_firstVecNode;
}

}

