
/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *
 *
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Copyright, 1986, The Regents of the University of California.
 *
 *
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#ifndef IterInReaderh
#define IterInReaderh

#include <memory>

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_IterInReaderh
#   undef epicsExportSharedSymbols
#endif

#include <daFrameSeq.h>

#ifdef epicsExportSharedSymbols_IterInReaderh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "IterIn.h"

namespace ca {

//
// The purpose of this class is to maintain a list of Vec < T >
// read only references to the payload so that two goals are 
// accomplished.
// 1) The number of bytes the paylod iterator will process
// is constrained.
// 2) The input buffers can be interfaced with a Catalog.
// This requires a rewind capability.
// 3) The IO operations can maintain references to the multiple
// input stream buffers which make up an aray.
//
// The ByteVecPtrConst passed back uses the same lock as 
// the Reader in the IterIn.
//
class IterInReader : 
    public Reader {
public:
    IterInReader ( IterIn &, size_t nBytes );
    // assignment and copy construction cause
    // the list of references to the vectors 
    // to be duplicated.
    IterInReader ( const IterInReader & ); 
    IterInReader ( const IterInReader &, size_t nBytes ); 
    IterInReader & operator = ( const IterInReader & ); 
    Vec < const epicsUInt8 > read ( size_t nBytesMax );
    void moveForward ( size_t );
    void rewind ();
    size_t size () const;
private:
    class VecNode {
    public:
        VecNode ( IterIn &, size_t nBytes );
        VecNode ( const VecNode & );
        VecNode ( const VecNode &, size_t nBytes );
        VecNode & operator = ( const VecNode & ); 
        Vec < const epicsUInt8 > vector () const;
        VecNode * pNext () const;
        size_t size () const;
        size_t sizeAll () const;
    private:
        Vec < const epicsUInt8 > m_vec;
        std :: auto_ptr < VecNode > m_pNext;
    };
    VecNode m_firstVecNode;
    VecNode * m_pCur;
    size_t m_offsetCur;
};

inline IterInReader :: 
    IterInReader ( IterIn & iterIn, size_t nBytes ) :
    m_firstVecNode ( iterIn, nBytes ),
    m_pCur ( & m_firstVecNode ),
    m_offsetCur ( 0u )
{
}

inline size_t IterInReader ::
	size () const
{
    return m_firstVecNode.sizeAll ();
}

} // end of namespace ca

#endif /// ifndef IterInReaderh

