
/*
 *      OnceReq.cpp,v 1.6 2009/11/24 01:34:21 jhill Exp
 *
 *      Author  Jeffrey O. Hill
 *              johill@lanl.gov
 *              505 665 1831
 */

#error defunct?

#include "errlog.h"

#define epicsExportSharedSymbols
#include "OnceReq.h"

namespace ca {

OnceReq ::
    OnceReq ( Guard & guard, EvQue & eventQueue ) :
    m_responderService ( guard, eventQueue )
{
}

void OnceReq ::
    initiate ( PReq & pReq, PResp & pResp, PCat & pReqParms )
{
    try {
        m_responderEventQueue.setResp ( pResp );
        m_responderService.initiate ( pReq, pReqParms );
    }
    catch ( std :: exception & except ) {
        m_initiateFailHandler ( guard, except.what () );
    }
    catch ( ... ) {
        m_initiateFailHandler ( guard, 
            "non-standard c++ exception" );
    }
}

void OnceReq :: 
    m_initiateFailHandler ( Guard & guard, const char * pContext )
{
    m_responderEventQueue.clrResp ();
    m_pReq.clear ();
    m_event.postFailure ( guard, Xxxxxx, pContext );
}

OnceReq :: 
    ~OnceReq ()
{
    m_requestCleanup ();
}

// exceptions are not allowed to propagate
// outside of this function
void OnceReq :: 
    m_requestCleanup ()
{
    if ( ! m_pReq.isNill () ) {
        try {
            m_pReq->cancel ();
        }
        catch ( std :: exception & except ) {
            errlogPrintf (  
                "OnceReq :: m_requestCleanup: "
                "unexpected exception from "
                "Request :: cancel or destroy was \"%s\"\n",
                except.what () );
        }
        catch ( ... ) {
            errlogPrintf (  
                "OnceReq :: m_requestCleanup: "
                "nonstandard exception from "
                "Request :: cancel or destroy\n" );
        }
        m_pReq.clear ();
    }
}

OnceReq :: ResponderService :: 
    ResponderService (  Guard & guard, EvQue & eventQueue ) :
    m_eventQueue ( eventQueue ), m_mutex ( guard )
{
}

#if 0
void OnceReq :: ResponderService :: 
    initiate ()
{
#error
}
#endif

void OnceReq :: ResponderService :: 
    initiate ( PReq & pReq, PCat & pReqParms )
{
    if ( !m_pReq.isNill () ) {
        throw std :: invalid_argument ( 
            "OnceReq :: initiate: attempt to initaiate "
            "the OnceReq more than once?" );
    }
    if ( pReq.isNill () ) {
        throw std :: invalid_argument ( 
            "OnceReq :: initiate: invalid nill request "
            "pointer" );
    }
#error exception catching
    m_pReq = pReq;
    pReq->initiate ( PResp ( & m_handle ), pReqParms );
}

void OnceReq :: ResponderService :: 
    m_responseNotify ( const PCat & pRespParms )
{
    EnclBinding :: Ptr < OnceReq > pOnceReq ( *this );
    if ( m_pReq.isNill () ) {
        // We have already released the request so we dont
        // have much information about where this came from
        PCat :: Locker pCatLocked = pResp;
        errlogPrintf (
            "OnceReq :: Responder ::m_responseNotify (): "
            "redundant completion notify is an "
            "interface violation - payload type \"%s\"\n",
            typeid ( *pCatLocked ).name () );
    }
    else {
        m_pReq.clear ();
        PResp pResp ( pOnceReq->m_handleResponderEventQueue );
        m_eventQueue.postSuccess ( m_pResp, pResp );
    }
}

void OnceReq :: ResponderEventQueue :: 
    setResp ( PResp & pResp )
{
    if ( !m_pResp.isNill () ) {
        throw std :: invalid_argument ( 
            "OnceReq :: initiate: attempt to initaiate "
            "the OnceReq more than once?" );
    }
    if ( pResp.isNill () ) {
        throw std :: invalid_argument ( 
            "OnceReq :: initiate: invalid nill response "
            "pointer" );
    }
    m_pResp = pResp;
}

void OnceReq :: ResponderEventQueue :: 
    m_responseNotify ( const PCat & pRespParms )
{
    assert ( ! m_pResp.isNill () );
    m_pResp->responseNotify ( pRespParms );
    m_pResp.clear ();
}

void OnceReq :: 
    cancel ( Guard & guard, const char * pContext )
{
    if ( m_pRequest ) {
        m_requestCleanup ( guard );
        assert ( m_pCallback );
        m_event.postFailure ( guard, *m_pCallback, pContext );
        m_pCallback = 0;
    }
}

void OnceReq :: 
    show ( const Guard & guard, unsigned level ) const
{
    guard.assertIdenticalMutex ( m_mutex );
    printf ( "OnceReq:\n" );
    if ( level > 0 ) {
        m_event.show ( guard, level - 1 );
        m_mutex.show ( level - 1 );
        if ( ! m_pReq.isNill () ) {
            m_pReq->show ( level - 1 );
        }
        if ( m_pCallback ) {
             printf ( "OnceReq: m_pCallback = \"%s\"\n",
                 typeid ( m_pCallback ).name () );
        }
    }
}

Responder & OnceReq :: ResponderHandler :: 
    lock ()
{
    EnclBinding :: Ptr < OnceReq > pOnceReq ( *this );
    pOnceReq->m_mutex.lock ();
    return pOnceReq->m_responder;
}

Responder & OnceReq :: ResponderHandler :: 
    lockRecursive ()
{
    EnclBinding :: Ptr < OnceReq > pOnceReq ( *this );
    pOnceReq->m_mutex.lockRecursive ();
    return pOnceReq->m_responder;
}

void OnceReq :: ResponderHandler :: 
    unlock ()
{
    EnclBinding :: Ptr < OnceReq > pOnceReq ( *this );
    pOnceReq->m_mutex.unlock ();
}

HandleIntf & OnceReq :: ResponderHandler :: 
    clone ()
{
    EnclBinding :: Ptr < OnceReq > pOnceReq ( *this );
    pOnceReq->m_refCtr.reference ();
    return *this;
}

void OnceReq :: ResponderHandler :: 
    release ()
{
    EnclBinding :: Ptr < OnceReq > pOnceReq ( *this );
    m_responderRefIsActive = ! pOnceReq->m_refCtr.unreference ();
}

} // end of name space ca

