
#error defunct

#define epicsExportSharedSymbols
#include "caPtr.h"


namespace ca {

class RefAny {
public:
    RefAny ( Mutex &, const void * pTarget, ReleaseNotify & );
    RefAny ( Guard &, const void * pTarget, ReleaseNotify & );
    void cleanup ( Guard & );
    ~RefAny ();
    const void * pTarget () const;
    void reference ( Guard & );
    void unreference ( Guard & );
    void unreference ();
    void assertIdenticalMutex ( Guard & );
private:
    const void * const m_pTarget;
    Mutex m_mutex;
    ReleaseNotify & m_releaseNotify;
    size_t m_refCount;
    bool m_cleanupCompeted;
};

class RefAnyVec : 
    public RefAny {
public:
    RefAnyVec ( Mutex &, const void * pTarget, 
        size_t nElem, ReleaseNotify & );
    RefAnyVec ( Guard &, const void * pTarget, 
        size_t nElem, ReleaseNotify & );
    size_t nElem () const;
private:
    size_t m_nElem;
};

RefAny :: RefAny ( 
    Mutex & mutex, const void * pTarget, 
    ReleaseNotify & releaseNotify ) :
    m_pTarget ( pTarget ), m_mutex ( mutex ),
    m_releaseNotify ( releaseNotify ), 
    m_refCount ( 0 )
{
}

RefAny :: RefAny ( 
    Guard & guard, const void * pTarget, 
    ReleaseNotify & releaseNotify ) :
    m_pTarget ( pTarget ), m_mutex ( guard ), 
    m_releaseNotify ( releaseNotify ),
    m_refCount ( 0 )
{
}

void RefAny :: cleanup ( Guard & guard )
{
    guard.assertIdenticalMutex ( m_mutex );
    if ( ! m_cleanupCompeted ) {
        assert ( m_refCount == 0 ); 
        m_mutex.cleanup ( guard );
    }
}

RefAny :: ~RefAny ()
{
    if ( ! m_cleanupCompeted ) {
#       ifndef NDEBUG
            fprintf ( stderr, 
                "Tip: call Ref < T > :: cleanup() "
                "explicitly prior to ~Ref<T> "
                "for better efficency on ordinary, "
                "no exception thrown, code paths" );
#       endif
        Guard guard ( m_mutex );
        this->cleanup ( guard );
    }
}

void RefAny :: reference ( Guard & guard )
{
    guard.assertIdenticalMutex ( m_mutex );
    assert ( m_refCount < UINT_MAX );
    m_refCount++;
}

void RefAny :: unreference ( Guard & guard )
{
    guard.assertIdenticalMutex ( m_mutex );
    assert ( m_refCount > 0 );
    m_refCount--;
    if ( m_refCount == 0 ) {
        m_releaseNotify.release ( guard /*, m_pTarget */ );
    }
}

void RefAny :: unreference ()
{
#   ifndef NDEBUG
        fprintf ( stderr, 
            "Tip: call Ptr < T > :: release() "
            "explicitly prior to ~Ptr<T> "
            "for better efficency on ordinary, "
            "no exception thrown, code paths" );
#   endif
    // this codes presumes that, on exception paths, the
    // mutex might be taken recursively
    Guard guard ( m_mutex );
    this->unreference ( guard );
}

RefAnyVec :: 
    RefAnyVec ( Mutex & mutex, const void * pTarget, 
        size_t nElem, ReleaseNotify & destroyNotify ) :
    RefAny ( mutex, pTarget, destroyNotify ),
    m_nElem ( nElem )
{
}

RefAnyVec :: 
    RefAnyVec ( Guard & guard, const void * pTarget, 
        size_t nElem, ReleaseNotify & destroyNotify ) :
    RefAny ( guard, pTarget, destroyNotify ),
    m_nElem ( nElem )
{
}

inline size_t RefAnyVec :: nElem () const
{
    return m_nElem; 
}

inline const void * RefAny :: pTarget () const
{
    return m_pTarget;
}

inline void RefAny :: assertIdenticalMutex ( Guard & guard )
{
    guard.assertIdenticalMutex ( m_mutex );
}

} // end of namespace ca
