
#error defunct?

#ifndef VecPtrh
#define VecPtrh

#ifndef assert // allow use of epicsAssert.h
#   include <cassert>
#endif
#include <cstdio>
#include <climits>
#include <exception>
#include <stdexcept>

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_caPtrh
#   undef epicsExportSharedSymbols
#endif

#include "epicsMutex.h"
#include "epicsTypes.h"
#include "osiWireFormat.h"
#include "BaseString.h"

#ifdef epicsExportSharedSymbols_caPtrh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "caRef.h"

namespace ca {

using std :: exception;

typedef epics :: Mutex Mutex;
typedef Mutex :: Guard Guard;
template < class > class Vec;

template < class T >
class VecPtr {
public:
    VecPtr ( const Guard & );
    VecPtr ( const Guard &, Ref < T > &, const Vec < T > & );
    explicit VecPtr ( const VecPtr & );
    VecPtr & operator = ( const VecPtr & );
    ~VecPtr ();
    void release ( Guard & );
    bool isEmpty ();
    size_t nElem () const;
    T & operator [] ( size_t ) const;
    T * pTarget () const;
    void remove ( VecPtr & dst, size_t nElemMax );
    void advance ( size_t nElem );
    operator Guard & ();
protected:
    Guard m_guard;
    Ref < T > m_ref;
    T * m_pTarget;
    size_t m_nElem;
};

#if 0
// Visual c++ 2008 seems to require this
// specialization while both g++ and comeau dont?
// This is related to the compiler's choosing
// what type const T will be, when T is 
// "char [N]". Will it be a const array, or an 
// array of const elements. The standard says
// "const char [N]". Presumable, vc++ decides to
// strip const from the array applying it instead
// to the element leading to a conclusion that 
// VecPtr < T >, and not VecPtr < const T >, is a base 
// class for VecPtr < T >. That conclusion could 
// lead to vc++'s decision not to compile the 
// code, but perhaps that behavior is incorrect.
template < size_t N >
class VecPtr < const char [ N ] > {
public:
    typedef const char FixedStrConst [ N ];
    VecPtr ();
    explicit VecPtr ( RefVoid & );
    VecPtr ( const Guard &, RefVoid & );
    VecPtr ( const Guard &, const VecPtr & );
    size_t nElem () const;
    const FixedStrConst & operator [] ( size_t ) const;
    const FixedStrConst * pTarget () const;
    void advance ( size_t nElem );
protected:
    Guard m_guard;
    FixedStrConst * m_pTarget;
    size_t m_nElem;
    VecPtr ( const VecPtr & ); // disabled
    VecPtr & operator = ( const VecPtr & ); // disabled
};

template < class T >
class VecPtr : 
    public VecPtr < const T > {
public:
    VecPtr ( const Guard & );
    template < class Ref >
    VecPtr ( const Guard &, Ref & );
    VecPtr ( const Guard &, const VecPtr & );
    using VecPtr < const T > :: assign;
    using VecPtr < const T > :: release;
    using VecPtr < const T > :: nElem;
    using VecPtr < const T > :: remove;
    using VecPtr < const T > :: advance;
    using VecPtr < const T > :: isEmpty;
    using VecPtr < const T > :: operator =;
    T & operator [] ( size_t ) const;
    T * pTarget () const;
private:
    using VecPtr < const T > :: m_pTarget;
    VecPtr ( const VecPtr & ); // disabled
    VecPtr & operator = ( const VecPtr & ); // disabled
};
#endif

class ByteVecPtrConst : 
    public VecPtr < const epicsUInt8 > {
public:
    ByteVecPtrConst ( const Guard & );
    ByteVecPtrConst ( const Guard &, 
        Ref < const epicsUInt8 > &, const Vec < const epicsUInt8 > & );
    ByteVecPtrConst ( const ByteVecPtrConst & );
#if 0
    using VecPtr < const epicsUInt8 > :: assign;
#endif
    using VecPtr < const epicsUInt8 > :: release;
    using VecPtr < const epicsUInt8 > :: nElem;
    using VecPtr < const epicsUInt8 > :: remove;
    using VecPtr < const epicsUInt8 > :: advance;
	using VecPtr < const epicsUInt8 > :: operator [];
	using VecPtr < const epicsUInt8 > :: pTarget;
    using VecPtr < const epicsUInt8 > :: isEmpty;
    using VecPtr < const epicsUInt8 > :: operator =;
    template < class V >
    bool pop ( V & );
    // appends to the StringEditor
    size_t popString ( size_t maxElems, StringEditor & );

    struct Writer { 
        virtual ~Writer () {}
        // The write function is permitted to block until 
        // a sufficent number of bytes have been written. 
        // It throws an exception if there is a circuit 
        // disconnect or some other form of communication 
        // failure.
        //
        // When the server calls this function, the 
        // ByteVecPtr passed here is locked with a noop
        // mutex.
        //
        virtual void write ( ByteVecPtrConst & ) = 0;
    };

private:
    template < class V >
    bool pop ( const V & val ); // disabled
};

class ByteVecPtr : 
    public ByteVecPtrConst {
public:
    ByteVecPtr ( const Guard & );
    ByteVecPtr ( const Guard &, Ref < epicsUInt8 > &, const Vec < epicsUInt8 > & );
    ByteVecPtr ( const ByteVecPtr & );
#if 0
    using ByteVecPtrConst :: assign;
#endif
    using ByteVecPtrConst :: release;
    using ByteVecPtrConst :: nElem;
    using ByteVecPtrConst :: remove;
    using ByteVecPtrConst :: advance;
    using ByteVecPtrConst :: isEmpty;
    using ByteVecPtrConst :: operator =;
    epicsUInt8 & operator [] ( size_t ) const;
    epicsUInt8 * pTarget () const;
    template < class V >
    bool push ( const V & val );
    // returns the number of characters pushed
    size_t pushString ( size_t pos, const StringReader & );
    // returns the number of characters pushed
    size_t pushZeros ( size_t len );
private:
};


template < class T >
inline VecPtr < T > :: 
    VecPtr ( const Guard & guard ) :
    m_guard ( guard ),
    m_pTarget ( 0 ),
    m_nElem ( 0 )
{
}

template < class T >
inline VecPtr < T > :: 
    VecPtr ( const Guard & guard, 
        Ref < T > & ref, const Vec < T > & vec ) :
    m_guard ( guard ),
    m_ref( guard, ref ),
    m_pTarget ( vec.pTarget () ),
    m_nElem ( vec.nElem () )
{
}

template < class T >
inline VecPtr < T > :: 
    VecPtr ( const VecPtr & vecPtr ) :
    m_guard ( vecPtr.m_guard ),
    m_ref( vecPtr.m_guard, vecPtr.m_ref ),
    m_pTarget ( vecPtr.m_pTarget ),
    m_nElem ( vecPtr.nElem () )
{
}

template < class T >
inline VecPtr < T > :: 
    ~VecPtr ()
{
    m_ref.release ( m_guard );
}

template < class T >
void VecPtr < T > :: 
    remove ( VecPtr & dst, size_t nElemMax )
{
    size_t nElem = std :: min ( m_nElem, nElemMax );
    if ( nElem ) {
        dst.m_ref.assign ( m_guard, m_ref );
        dst.m_pTarget = m_pTarget;
        dst.m_nElem = nElem;
        this->advance ( nElem );
    }
    else {
        dst.m_ref.release ( m_guard );
        dst.m_pTarget = 0;
        dst.m_nElem = 0u;
    }
}

template < class T >
void VecPtr < T > :: 
    advance ( size_t nElem )
{
    if ( nElem < m_nElem ) {
        m_pTarget += nElem;
        m_nElem -= nElem;
    }
    else {
        if ( nElem > m_nElem ) {
            throw std :: range_error ( 
                "VecPtr :: advance:: nElem arg is "
                "larger than the vector size" );
        }
        m_pTarget = 0;
        m_nElem = 0;
        m_ref.release ( m_guard );
    }
}

template < class T >
inline size_t VecPtr < T > :: 
    nElem () const
{
    return m_nElem;
}

template < class T >
inline T & 
    VecPtr < T > :: 
        operator [] ( size_t idx ) const
{
    return m_pTarget [ idx ];
}

template < class T >
inline T * VecPtr < T > :: 
    pTarget () const
{
    return m_pTarget;
}

#if 0

template < size_t N >
inline VecPtr < const char [ N ] > :: 
    VecPtr () :
    m_nElem ( 0 )
{
}

template < size_t N >
inline VecPtr < const char [ N ] > :: 
    VecPtr ( const Guard & guard, RefMgr & refMgr, 
        FixedStrConst * pTarget, size_t nElem ) :
    m_ref ( guard, refMgr ),
    m_pTarget ( pTarget ),
    m_nElem ( nElem )
{
}

template < size_t N >
inline VecPtr < const char [ N ] > :: 
    VecPtr ( const Guard & guard, VecPtr < FixedStrConst > & ptr ) :
    m_ref ( guard, ptr.m_ref ), 
    m_pTarget ( ptr.m_pTarget ),
    m_nElem ( ptr.m_nElem )
{
}

template < size_t N >
inline void VecPtr < const char [ N ] > :: 
    assign ( const Guard & guard, RefMgr & refMgr, 
            FixedStrConst * pTarget, size_t nElem )
{
    m_ref.assign ( guard, refMgr );
    m_pTarget = pTarget;
    m_nElem = nElem;
}

template < size_t N >
inline void VecPtr < const char [ N ] > :: 
    assign ( const Guard & guard, VecPtr < FixedStrConst > & ptr )
{
     m_ref.assign ( guard, ptr.m_ref );
     m_pTarget = ptr.m_pTarget;
     m_nElem = ptr.m_nElem;
}

template < size_t N >
void VecPtr < const char [ N ] > :: 
    remove ( const Guard & guard, VecPtr < FixedStrConst > & dst, 
        size_t nElemMax )
{
    size_t nElem = std :: min ( m_nElem, nElemMax );
    if ( nElem ) {
        dst.m_ref.assign ( guard, m_ref );
        dst.m_pTarget = m_pTarget;
        dst.m_nElem = nElem;
        this->advance ( guard, nElem );
    }
    else {
        dst.m_ref.release ( guard );
        dst.m_pTarget = 0;
        dst.m_nElem = 0u;
    }
}

template < size_t N >
inline void VecPtr < const char [ N ] > :: 
    release ( const Guard & guard )
{
     m_ref.release ( guard );
}

template < size_t N >
void VecPtr < const char [ N ] > :: 
    advance ( const Guard & guard, size_t nElem )
{
    if ( nElem < m_nElem ) {
        m_pTarget += nElem;
        m_nElem -= nElem;
    }
    else {
        if ( nElem > m_nElem ) {
            throw std :: range_error ( 
                "VecPtr :: advance:: nElem arg is "
                "larger than the vector size" );
        }
        m_pTarget = 0;
        m_nElem = 0;
        m_ref.release ( guard );
    }
}

template < size_t N >
inline bool VecPtr < const char [ N ] > :: 
    isNill () const
{
    return m_pTarget == 0;
}

template < size_t N >
inline bool VecPtr < const char [ N ] > :: 
    isValid () const
{
    return m_pTarget != 0;
}

template < size_t N >
inline size_t VecPtr < const char [ N ] > :: 
    nElem () const
{
    return m_nElem;
}

template < size_t N >
inline typename VecPtr < const char [ N ] > :: FixedStrConst & 
    VecPtr < const char [ N ] > :: 
        operator [] ( size_t idx ) const
{
    return m_pTarget [ idx ];
}

template < size_t N >
inline typename VecPtr < const char [ N ] > :: FixedStrConst * 
    VecPtr < const char [ N ] > :: 
        pTarget () const
{
    return m_pTarget;
}

#endif

inline ByteVecPtrConst :: 
    ByteVecPtrConst ( const Guard & guard ) :
    VecPtr < const epicsUInt8 > ( guard )
{
}

inline ByteVecPtrConst :: 
    ByteVecPtrConst ( const Guard & guard, 
        Ref < const epicsUInt8 > & ref, 
        const Vec < const epicsUInt8 > & vec ) :
    VecPtr < const epicsUInt8 > ( guard, ref, vec )
{
}

inline ByteVecPtrConst :: 
    ByteVecPtrConst ( const ByteVecPtrConst & bvpc ) :
    VecPtr < const epicsUInt8 > ( bvpc )
{
}

template < class V >
inline bool ByteVecPtrConst :: 
    pop ( V & val )
{
    const size_t vSize = sizeof ( V );
    bool success = ( m_nElem >= vSize );
    if ( success ) {
        WireGet ( m_pTarget, val );
        m_pTarget += vSize;
        m_nElem -= vSize;
    }
    return success;
}

inline ByteVecPtr :: 
ByteVecPtr ( const Guard & guard ) :
    ByteVecPtrConst ( guard )
{
}

inline ByteVecPtr :: 
    ByteVecPtr ( const Guard & guard, 
        Ref < epicsUInt8 > & ref, 
        const Vec < epicsUInt8 > & vec ) :
    ByteVecPtrConst ( guard, ref, vec )
{
}

inline ByteVecPtr :: 
    ByteVecPtr ( const ByteVecPtr & byteVecPtr ) :
    ByteVecPtrConst ( byteVecPtr )
{
}

template < class V >
inline bool ByteVecPtr :: 
    push ( const V & val )
{
    const size_t vSize = sizeof ( V );
    bool success = ( m_nElem >= vSize );
    if ( success ) {
        epicsUInt8 * pTarget = 
            const_cast < epicsUInt8 * > ( m_pTarget );
        WireSet ( val, pTarget );
        m_pTarget += vSize;
        m_nElem -= vSize;
    }
    return success;
}

} // end of namespace ca

#endif // ifndef VecPtrh
