

/*
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#ifndef caAccessRightsh
#define caAccessRightsh

#ifdef epicsExportSharedSymbols
#   define caAccessRightsh_restore_epicsExportSharedSymbols
#   undef epicsExportSharedSymbols
#endif

#include "da.h"

#ifdef caAccessRightsh_restore_epicsExportSharedSymbols
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

namespace ca {

using namespace da;

// a parallel of this class appears to be in the ca client library
class AccessRights {
public:
    AccessRights ( bool readable, bool writeable );
    bool readable () const;
    bool writeable () const;
private:
    bool m_readable;
    bool m_writeable;
};

// Implementation Notes
// --------------------
// o this needs to supply default values if the requested properties 
// are not present
// o access rights might vary depending on where we are 
// in the property hierarchy so the access rights returned here
// must be the access rights of the value - of the PV
epicsShareExtern Ident pi_readable;
epicsShareExtern Ident pi_writable;
extern AccessRights extractAccessRights ( const Catalog & );

inline AccessRights :: 
    AccessRights ( bool readable, bool writeable ) : 
    m_readable ( readable ), m_writeable ( writeable ) 
{
}

inline bool AccessRights :: readable () const 
{ 
    return m_readable; 
}

inline bool AccessRights :: writeable () const 
{ 
    return m_writeable; 
}

} // end of namespace ca

#endif // ifndef caAccessRightsh
