/*
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#include "ConstString.h"
#include "StandardString.h"

#define epicsExportSharedSymbols
#include "IterIn.h"
#include "casdef2.h"
#include "caChannelDescription.h"

namespace ca {

PCat ChannelDescription :: 
    factory ( const StringReader & name )
{
    ChannelDescription * pChanDescr = 
	    new ChannelDescription ( name );
    return PCat ( pChanDescr->m_catHandle );
}

ChannelDescription :: 
    ChannelDescription () :
    m_mutex ( epics :: Mutex :: factoryNOOP )
{
}

ChannelDescription :: 
    ChannelDescription ( const StringReader & channelName ) :
    m_channelName ( channelName ),
    m_mutex ( epics :: Mutex :: factoryNOOP )
{
}

ChannelDescription :: 
    ChannelDescription ( const std :: string & str ) :
    m_mutex ( epics :: Mutex :: factoryNOOP )
{
    ConstString constString ( str.c_str (), str.size () );
    m_channelName = constString;
}

void ChannelDescription :: 
    traverse ( Introspector & introspector ) const
{
    introspector.reveal ( Channel :: name, m_channelName );
}

void ChannelDescription :: 
    find ( const Ident & id, Viewer & viewer ) const
{
    if ( id  == Channel :: name ) {
        viewer.reveal ( m_channelName );
    }
}

size_t ChannelDescription :: nameLength () const
{
    return m_channelName.size ();
}

void ChannelDescription :: show ( unsigned level ) const
{
    printf ( "ChannelDescription\n" );
    if ( level > 0u ) {
        StandardString nameTmp ( m_channelName );
        printf ( "name = %s\n", nameTmp.c_str () );
    }
}

//////////////////////////////////////////////////////
// M_CatHandle 
//////////////////////////////////////////////////////
ChannelDescription :: M_CatHandle :: M_EnclLnk
    ChannelDescription :: M_CatHandle :: 
        m_enclLnk ( & ChannelDescription :: m_catHandle );
ChannelDescription :: M_CatHandle :: TargetMutexPair 
    ChannelDescription :: M_CatHandle :: m_targetMutexPair ()
{
    ChannelDescription & chanDescr ( m_enclLnk.enclosure ( *this ) );
    return TargetMutexPair ( chanDescr, chanDescr.m_mutex );
}
HandleIntf < const Catalog > & 
    ChannelDescription :: M_CatHandle :: m_clone ()
{
    ChannelDescription & chanDescr ( m_enclLnk.enclosure ( *this ) );
    chanDescr.m_refCtr.reference ();
    return *this;
}
void ChannelDescription :: M_CatHandle :: m_release ()
{
    ChannelDescription & chanDescr ( m_enclLnk.enclosure ( *this ) );
    if ( chanDescr.m_refCtr.unreference () ) {
        delete this;
        // no code after above delete
    }
    // no code after above delete
}

} // end of namespace ca

