
/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

#ifndef caChannelDescriptionh
#define caChannelDescriptionh

#include <string>

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_caSvrDGMgrh
#   undef epicsExportSharedSymbols
#endif

#include "tsFreeList.h"
#include "ChunkedString.h"
#include "EnclLnk.h"

#ifdef epicsExportSharedSymbols_caSvrDGMgrh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "casdef2.h"
#include "caPtr.h"
#include "caRefCtr.h"

namespace ca {

using namespace da;

class ChannelDescription : 
        public Catalog {
public:
    static PCat factory ( const StringReader & );
    ChannelDescription ();
    ChannelDescription ( const StringReader & );
    ChannelDescription ( const std :: string & );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
    size_t nameLength () const;
    void show ( unsigned level ) const;
private:
    class M_CatHandle : 
        public HandleIntf < const Catalog> {
    private:
        TargetMutexPair m_targetMutexPair ();
        HandleIntf < const Catalog > & m_clone ();
        void m_release ();
        typedef const epics :: EnclLnk < ChannelDescription, 
                                M_CatHandle > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    ChunkedString m_channelName;
    M_CatHandle m_catHandle;
    epics :: Mutex m_mutex;
    RefCtr m_refCtr;
};  

#if 0
class ChannelDescriptionFactory {
    typedef tsFreeList < ChannelDescription > FreeList;
    static PCat factory ( FreeList &, size_t maxStrSize, IterIn & );
    static PCat factory ( FreeList &, const std :: string & );
    class M_Handle : 
        public HandleIntf < Catalog > {
    private:
		TargetMutexPair m_targetMutexPair ();
		HandleIntf < Catalog > & m_clone ();
		void m_release ();
    };
    RefCtr m_refCtr;
}
HandleIntf < Catalog > :: TargetMutexPair 
    ChannelDescription :: M_Handle ::
    m_targetMutexPair () 
{
    return TargetMutexPair ( m_catalog, xxxMutex );
}

HandleIntf < Catalog > & 
    ChannelDescription :: M_Handle ::m_clone ()
{
    m_refCtr.reference ();
    return m_handle;
}

void ChannelDescription :: M_Handle ::m_release ()
{
    if ( m_refCtr.unreference () ) {
#error lock
    }
}

PCat factory ( FreeList & freeList, size_t maxStrSize, IterIn & iterIn )
{
}

PCat factory ( FreeList & freeList, const std :: string & str )
{
}
#endif

} // end of namespace ca

#endif // ifndef caChannelDescriptionh
