
#ifndef caMethod_h
#define caMethod_h

namespace ca {

template < class P >
class Method
{
public:
	template < class T >
	struct Class {
		typedef void ( T :: * PFunc )( P & );
		template < PFunc PMF >
		static void m_functor ( void *, P & );
	};
	struct Func {
		typedef void ( * PFunc )( P & );
		template < PFunc PF >
		static void m_functor ( void *, P & );
	};
    Method ();
    Method ( typename Func :: PFunc );
#if 0
    template < class T >
    Method ( T &, typename Class < T > :: PFunc );
#else
    template < class T, typename Class < T > :: PFunc PMF >
    Method ( T & );
#endif

    void invoke ( P & parm ) const;

    class Invoker {
    public:
        Invoker ();
        Invoker ( Method < P > & method, P & parm );
        Invoker ( Invoker & );
        ~Invoker ();
        Invoker & operator = ( Invoker & );
    private :
        Method < P > m_method;
        P * m_pParm;
    };

private:
	typedef void ( * PFunctor ) ( void * pVoidTarget, P & parm );
	void * m_pTarget;
	PFunctor m_pFunctor;
    static void m_noopFunc ( P & ) {}
};

template < class P >
template < class T >
template < void ( T :: * PMF )( P & ) >
void Method < P > :: Class < T > :: m_functor ( 
	void * pVoidTarget, P & parm )
{
	// since P, T, and PMF are known at compile time here 
	// there is potential for the target member function to 
    // be inlined into this functor by the optimizer
	T & target ( * reinterpret_cast < T * > ( pVoidTarget ) );
	( target .* PMF ) ( parm );
}

template < class P >
template < void ( * PF )( P & ) >
void Method < P > :: Func :: m_functor ( void *, P & parm )
{
	// since P, T, and PMF are known at compile time here 
	// there is potential for the target function to be 
    // inlined into this functor by the optimizer
	( *PF ) ( parm );
}

template < class P >
inline Method < P > :: Method () :
	m_pTarget ( 0 ), 
    m_pFunctor ( Func :: template m_functor < m_noopFunc > )
{
}

template < class P >
inline Method < P > :: Method ( typename Func :: PFunc pF ) :
	m_pTarget ( 0 ), 
    m_pFunctor ( Func :: template m_functor < pF > )
{
}

#if 0
template < class P >
template < class T >
inline Method < P > :: Method ( 
	T & target, typename Class < T > :: PFunc pMF ) :
	m_pTarget ( & target ), 
    m_pFunctor ( & Class < T > :: template m_functor < pMF > )
{
}
#else
template < class P >
template < class T, typename Method < P > :: template Class < T > :: PFunc PMF >
inline Method < P > :: Method ( T & target ):
	m_pTarget ( & target ), 
    m_pFunctor ( & Class < T > :: template m_functor < PMF > )
{
}
#endif

template < class P >
inline void Method < P > :: invoke ( P & parm ) const
{
	( *m_pFunctor ) ( m_pTarget, parm );
}

template < class P >
inline Method < P > :: Invoker :: Invoker () : 
    m_method ( Method < P > () ), 
    m_pParm ( 0 )
{
}

template < class P >
inline Method < P > :: Invoker :: Invoker ( 
    Method < P > & method, P & parm ) : 
    m_method ( method ), 
    m_pParm ( & parm )
{
}

template < class P >
inline Method < P > :: Invoker :: Invoker ( 
    Invoker & invoker ) : 
    m_method ( invoker.m_method ),
    m_pParm ( invoker.m_pParm )
{
    invoker.m_method = Method < P > ();
    invoker.m_pParm = 0;
}

template < class P >
inline Method < P > :: Invoker :: ~Invoker ()
{
    if ( m_pParm ) {
        m_method.invoke ( *m_pParm );
    }
}

template < class P >
inline typename Method < P > :: Invoker & 
    Method < P > :: Invoker ::
    operator = ( Invoker & invoker )
{
    m_method = invoker.m_method;
    m_pParm = invoker.m_pParm;
    invoker.m_method = Method < P > ();
    invoker.m_pParm = 0;
}

} // end of namespace ca

#endif // ifndef caMethod_h

