/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

//
// Netaddr
//
// special cas specific network address class so:
// 1) we dont drag BSD socket headers into the server tool
// 2) we are not prevented from using other networking services
//      besides sockets in the future
//

#ifndef caNetAddrH
#define caNetAddrH

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_caNetAddrH
#   undef epicsExportSharedSymbols
#endif

#if defined(caNetAddrSock) 
#   include "osiSock.h"
#endif

#include "daIdent.h"
#include "daCatalog.h"

#ifdef epicsExportSharedSymbols_caNetAddrH
#   define epicsExportSharedSymbols
#endif

#include "shareLib.h"

namespace ca {

using namespace da;

class epicsShareClass NetAddr {
public:
    NetAddr ();
    void clear ();
    bool isValid () const;
    bool operator == ( const NetAddr & rhs ) const;
    bool operator != ( const NetAddr & rhs) const;
    NetAddr & operator = ( const NetAddr & );
    void stringConvert ( char * pString, unsigned stringLength ) const;
    void selfTest ();

    // support for socket addresses
    // (other address formats may be supported in the future)
    bool isInet () const;
    bool isMulticast () const; // true also if its a broadcast
    void setSockIP ( unsigned long inaIn, unsigned short portIn );
    void setSockIP ( const struct sockaddr_in & sockIPIn );
    void setSock ( const struct sockaddr & sock );
    NetAddr ( const struct sockaddr_in & sockIPIn );
    NetAddr & operator = ( const struct sockaddr & sockIn );
    NetAddr & operator = ( const struct sockaddr_in & sockIPIn );
    struct sockaddr getSock() const;
    struct sockaddr_in getSockIP() const;
    operator struct sockaddr () const;
    operator struct sockaddr_in () const;
    void show ( unsigned level ) const;
private:
    enum NetAddrType { casnaUDF, casnaInet } type;
    union { 
        unsigned char pad[16];
#       ifdef caNetAddrSock     
            struct sockaddr sa;
            struct sockaddr_in ip;
#       endif
    } addr;
};

extern const char pn_networkAddress[];
extern const char pn_internetV4[];
extern const char pn_address[];
extern const char pn_port[];

typedef IdentNamed < pn_networkAddress > ID_networkAddress;
typedef IdentNamed < pn_internetV4 > ID_internetV4;
typedef IdentNamed < pn_address > ID_address;
typedef IdentNamed < pn_port > ID_port;

extern ID_networkAddress pi_networkAddress;
extern ID_internetV4 pi_internetV4;
extern ID_address pi_address;
extern ID_port pi_port;

NetAddr fetchNetworkAddress ( const Catalog &, 
                               const NetAddr * pDflt );

class InetAddrV4Catalog : 
    public da :: Catalog {
public:
    InetAddrV4Catalog ();
    InetAddrV4Catalog ( unsigned address, unsigned port );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
private:
    unsigned m_address;
    unsigned m_port;
};

} // end of namespace ca

#endif // ifndef caNetAddrH

