
#error defunct

#ifndef caRefh
#define caRefh

// so ... I could change this to not allow access excpet
// through a guarded pointer object.
//
// there are some advantages
// o No need to pass (and verify) a guard in every interface 
// o objects that are shared about are likely to be shared by
// many threads and they should have interlocked access.
//
// there are some disadvantages
// o There isnt any way that the internal function can 
// release the Guard. If the Guard is explicitly passed
// then the function can temporarily release the Guard
// although admitedly this original design can cause trouble 
// if they create a Guard :: Release and dont destroy it before 
// returning from the Guard protected function. PERHAPS
// THE BETTER ALTERNATIVE IS TO PROVIDE AN ALTERNATIVE 
// INTERFACE THAT ISNT PROTECTED BY A LOCK.
// o there isnt any way to explicitly verify a guard but
// perhaps this is nolonger necessary.
//
// ISSUES WITH PTR INTERFACE
// o the ptr can be nill (which requires frequent testing 
// for nillness, but this isnt easy to fix.
//

#ifndef assert // allow use of epicsAssert.h
#   include <cassert>
#endif
#include <cstdio>
#include <exception>
#include <climits>
#include <stdexcept>
#include <typeinfo>

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_caPtrh
#   undef epicsExportSharedSymbols
#endif

#include "epicsMutex.h"
#include "epicsTypes.h"
#include "errlog.h"

#ifdef epicsExportSharedSymbols_caPtrh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#ifndef SIZE_MAX 
#   define SIZE_MAX ( static_cast < size_t > ( -1 ) )
#endif

namespace ca {

using std :: exception;

typedef epics :: Mutex Mutex;
typedef Mutex :: Guard Guard;
template < class T > class RefMgr;
template < class T > class Ref;
template < class T > class Ptr;

// Notes:
//
// o It is much more efficent to explicitly call 
// release ( Guard & ) prior to when ~Ref runs. 
// If not, then ~Ref will call unreference (), and 
// unreference () will create a Guard. That could cause 
// a recursive lock invocation, but that possibility can 
// never be completely eliminated if we are cleaning up 
// when an exception has been thrown.
//

template < class T >
class Ref < const T > {
public:
    typedef typename T :: Handle Handle;
    //
    // class T :: Handle {
    // public:
    //     Mutex & mutex () const;
    //     T * pTarget () const;
    //     size_t nElems () const;
    //     void reference ( Guard & );
    //     void unreference ( Guard & );
    //     void lockNotify ( Guard & );
    //     void unlockNotify ( Guard & );
    // };
    //
    Ref ( const Guard &, Handle & ); 
    Ref ( const Guard &, const Ref & ); 
    void release ( Guard & );
    ~Ref (); 
protected:
    Handle * m_pHandle;
    Ref ( const Ref & ); // disabled
    Ref & operator = ( const Ref & ); // disabled
};

template < class T >
class Ref : 
    public Ref < const T > {
public:
    Ref ( const Guard &, RefMgr < T > & ); 
    Ref ( const Guard &, const Ref & ); 
    ~Ref (); 
    using Ref < const T > :: assign;
private:
    Ref ( const Ref & ); // disabled
    Ref & operator = ( const Ref & ); // disabled
};

template < class T >
Ref < const T > :: 
    Ref ( const Guard & guard, Handle & handle ) :
    m_pHandle ( & handle )
{
    handle.reference ( guard );
}

template < class T >
Ref < const T > :: 
    Ref ( const Guard & guard, const Ref & refConst ) :
    m_pHandle ( refConst.m_pHandle )
{
	if ( m_pHandle ) {
        m_pHandle->reference ( guard );
	}
}

template < class T >
inline void Ref < const T > :: 
    release ( Guard & guard ) 
{
    if ( m_pHandle ) {
        m_pHandle->unreference ( guard );
        m_pHandle = 0;
    }
}

template < class T >
Ref < const T > :: 
    ~Ref ()
{
	if ( m_pHandle ) {
#       ifndef NDEBUG
            fprintf ( stderr, 
                "Tip: call Ref < T > :: cleanup() "
                "explicitly prior to ~Ref < T > "
                "for better efficency on ordinary "
                "(no exception thrown) code paths" );
#       endif
        Guard guard ( m_pHandle->mutex () );
        m_pHandle->unreference ( guard );
	}
}

} // end of namespace ca

#endif // ifndef caRefh
