
#error defunct

/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

#ifndef caServerIh
#define caServerIh

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_caServerIh
#   undef epicsExportSharedSymbols
#endif

// external headers included here
#include "tsFreeList.h"
#include "caProto.h"

#ifdef epicsExportSharedSymbols_caServerIh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "casdef2.h"
#include "clientBufMemoryManager.h"
#include "EventRegistry.h"
#include "ioBlocked.h"
#include "caServerDefs.h"
#include "CirSvr.h"

template < class T > class epicsGuard;

#if ! defined ( __GNUC__ ) || __GNUC__ > 2 || ( __GNUC__ == 2 && __GNUC_MINOR__ >= 95 )
namespace ca {
#endif

class CirSvr;
class casIntfOS;
//class casMonitor;
class casChannelI;
struct ServerIO;

// caStatus convertContainerMemberToAtomic ( class gdd & dd, 
//          aitUint32 appType, aitUint32 elemCount );

class InvalidProtocol : 
    public std :: exception {
public:
    const char * what () throw ();
};

struct IntfIO;

class caServerI : 
	public EventRegistry {
public:
    caServerI ();
	~caServerI ();
	unsigned getDebugLevel() const;
	inline void setDebugLevel ( unsigned debugLevelIn );
	void show ( unsigned level ) const;
    //void destroyMonitor ( casMonitor & );
	EventMask valueEventMask () const; // DBE_VALUE registerEvent("value")
	EventMask logEventMask () const; // DBE_LOG registerEvent("log") 
	EventMask alarmEventMask () const; // DBE_ALARM registerEvent("alarm") 
    unsigned subscriptionEventsProcessed () const;
    void incrEventsProcessedCounter ();
    unsigned subscriptionEventsPosted () const;
    void updateEventsPostedCounter ( unsigned nNewPosts );
    void generateBeaconAnomaly ();
    //casMonitor & casMonitorFactory ( casChannelI &, 
    //    caResId clientId, const unsigned long count, 
    //    const unsigned type, const EventMask &, 
    //    class casMonitorCallbackInterface & );
    //void casMonitorDestroy ( casMonitor & );
    //void destroyClient ( CirSvr & );
    static void dumpMsg ( 
        class StringReader & hostName, 
        class StringReader & userName,
        const struct HdrLargeArray &, 
        const char * pFormat, ... );
    static void dumpMsg ( 
        class StringReader & hostName, 
        class StringReader & userName,
        const struct HdrLargeArray &, 
        const char * pFormat, 
        va_list & vaList );
    static void dumpMsg ( 
        const caNetAddr &,
        const struct HdrLargeArray &, 
        const char * pFormat, ... );
    static void dumpMsg ( 
        const caNetAddr &,
        const struct HdrLargeArray &, 
        const char * pFormat, 
        va_list & vaList );
private:
    //tsFreeList < casMonitor, 1024 > m_casMonitorFreeList;
    tsFreeList < CirSvr > m_clientFreeList;
	tsDLList < CirSvr > m_cirMgrList;
	mutable Mutex m_mutex;
	mutable Mutex m_diagnosticCountersMutex;
    std :: auto_ptr < beaconAnomalyGovernor > m_pBeaconAnomalyGov;
    std :: auto_ptr < ServerIO > m_pIO;
	unsigned m_debugLevel;
    unsigned m_nEventsProcessed; 
    unsigned m_nEventsPosted; 
	bool m_newCircuitThreadShutdown;

    EventMask m_valueEvent; // DBE_VALUE registerEvent("value")
	EventMask m_logEvent; // DBE_LOG registerEvent("log") 
	EventMask m_alarmEvent; // DBE_ALARM registerEvent("alarm")

    void m_sendBeacon ( ca_uint32_t beaconNo );
	void m_newCircuitThread ( IntfIO & );

	caServerI ( const caServerI & );
	caServerI & operator = ( const caServerI & );
};

inline unsigned caServerI :: getDebugLevel () const 
{ 
    return m_debugLevel; 
}

inline void caServerI :: setDebugLevel (
    unsigned debugLevelIn )
{
	this->m_debugLevel = debugLevelIn;
}

#if 0
inline EventMask caServerI :: valueEventMask() const
{
    return this->m_valueEvent;
}

inline EventMask caServerI :: logEventMask() const
{
    return this->m_logEvent;
}

inline EventMask caServerI :: alarmEventMask() const
{
    return this->m_alarmEvent;
}
#endif

#if ! defined ( __GNUC__ ) || __GNUC__ > 2 || ( __GNUC__ == 2 && __GNUC_MINOR__ >= 95 )
} // end of namespace ca
#endif

#endif // caServerIh
