
/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *
 *
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Copyright, 1986, The Regents of the University of California.
 *
 *
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#define epicsExportSharedSymbols
#include "caStringReaderIterIn.h"

namespace ca {

size_t StringReaderIterIn :: size () const
{
    return m_iterInReader.size ();
}

// Comparison functions
bool StringReaderIterIn :: 
    equals ( const StringReader & rhs ) const
{
    return this->contains ( 0u, rhs );
}

bool StringReaderIterIn :: 
    contains ( size_t posLHS, const StringReader & rhs ) const
{
    size_t tlLHS = m_iterInReader.size ();
    size_t tlRHS = rhs.size ();
    if ( posLHS >= tlLHS ) {
        return false;
    }
    tlLHS -= posLHS;
    if ( tlRHS > tlLHS ) {
        return false;
    }
    m_iterInReader.rewind ();
    IterIn iterIn ( m_iterInReader );
    iterIn.moveForward ( posLHS );
    size_t posRHS = 0u;
    do {
        char buf[128];
        size_t nBytes = tlRHS;
        if ( nBytes > sizeof ( buf ) ) {
            nBytes = sizeof ( buf );
        }
        nBytes = rhs.extract ( posRHS, buf, nBytes );
        assert ( nBytes <= tlRHS );
        for ( size_t i = 0u; i < nBytes; i++ ){
            if ( buf[i] != iterIn.pop < char > () ) {
                return false;
            }
        }
        tlRHS -= nBytes;
    }
    while ( tlRHS );
    return true;
}

bool StringReaderIterIn :: 
    contains ( size_t pos, const char *buf, size_t len ) const
{
    size_t tl = m_iterInReader.size ();
    if ( pos >= tl ) {
        return false;
    }
    tl -= pos;
    if ( tl > len ) {
        return false;
    }
    m_iterInReader.rewind ();
    IterIn iterIn ( m_iterInReader );
    iterIn.moveForward ( pos );
    for ( size_t i = 0u; i < tl; i++ ){
        if ( buf[i] != iterIn.pop < char > () ) {
            return false;
        }
    }
    return true;
}

// Indexed byte access - very inefficent
// with iterator based string
char StringReaderIterIn :: 
    at ( size_t pos ) const
{
    m_iterInReader.rewind ();
    IterIn iterIn ( m_iterInReader );
    iterIn.moveForward ( pos );
    return iterIn.pop < char > ();
}

// Copy out to buffer, no terminator
size_t StringReaderIterIn :: 
    extract ( size_t pos, char *buf, size_t cap ) const
{
    size_t tl = m_iterInReader.size ();
    if ( pos >= tl ) {
        return 0u;
    }
    tl -= pos;
    if ( tl > cap ) {
        tl = cap;
    }
    m_iterInReader.rewind ();
    IterIn iterIn ( m_iterInReader );
    iterIn.moveForward ( pos );
    epicsUInt8 * const pUint8 = 
        reinterpret_cast < epicsUInt8 *> ( buf );
    STATIC_ASSERT ( sizeof ( buf[0] ) == sizeof ( pUint8[0] ) );
    iterIn.popBytes ( pUint8, tl );
    return tl;
}

} // end of namespace ca


