
#ifndef caSvrCirMgrChanh
#define caSvrCirMgrChanh

#include <exception>

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_caSvrCirMgrChanh
#   undef epicsExportSharedSymbols
#endif

#include "tsDLList.h"
#include "resourceLib.h"

#ifdef epicsExportSharedSymbols_caSvrCirMgrChanh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "caSvrCirMgr.h"
#include "caSvrCirRspThrCirMon.h"

namespace ca {

class IterIn;
struct HdrLargeArray;

namespace Svr {
namespace Cir {

using std :: exception;

class Mgr :: Chan {
public:
    class Read;
    class Write;
    class CircMon;
    class Subscribe;
    class HdrOnly;
    class ChanCreate;
    typedef Ptr < ReqNotify > PReqNotify;
    static Ptr < Chan >
        factory ( Mgr &, caResId sid, caResId cid );
    Chan ( Mgr &, caResId sid, caResId cid );
    ~Chan (); 
    void casChannelDestroyFromInterfaceNotify ();
    void postAccessRightsEvent ();
    void postFailure ( const PResp &, const char * pCtx );
    PResp installCircuitMonitor ( const PReq & pCirMonReq );
    PCat channelDescription () const;
#if 0
    void read ( const HdrLargeArray & );
    void subscribe ( const HdrLargeArray &, PCat & payloadCatalog );
    void restartSubscriptions ();
    void write ( const HdrLargeArray &, PCat & payloadCatalog );
#endif
    void show ( unsigned level ) const;
    void enterFlowControlMode ();
    void exitFlowControlMode ();
    void readLackingCallbackCancel ();
    PResp createReadResp ( const PReq &, 
        const HdrLargeArray &, caResId cid );
    PResp createWriteResp ( const PReq &, 
        const HdrLargeArray &, caResId cid );
    PResp createSubscResp ( const PReq &, 
        const HdrLargeArray &, caResId cid, const PCat & );
    bool unsubscribe ( ca_uint32_t clientIdIn );
private:
    class M_MgrChanHandle : 
        public HandleIntf < Chan > {
    private:
        TargetMutexPair m_targetMutexPair ();
        HandleIntf < Chan > & m_clone ();
        void m_release ();
        typedef const epics :: EnclLnk < Chan, 
                                    M_MgrChanHandle > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    class M_CirMonServRespHandle : 
        public HandleIntf < Responder > {
    private:
        TargetMutexPair m_targetMutexPair ();
        HandleIntf < Responder > & m_clone ();
        void m_release ();
        typedef const epics :: EnclLnk < Chan, 
                                    M_CirMonServRespHandle > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    class M_CirMonRespThrHandle : 
        public HandleIntf < Responder > {
    private:
        TargetMutexPair m_targetMutexPair ();
        HandleIntf < Responder > & m_clone ();
        void m_release ();
        typedef const epics :: EnclLnk < Chan, 
                                    M_CirMonRespThrHandle > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    class M_CirMonServiceResp :         
        public Responder {
    private:
        void m_responseNotify ( const PCat & );
        void m_show ( unsigned ) const;
        typedef const epics :: EnclLnk < Chan, 
                                    M_CirMonServiceResp > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    // types
    RspThr :: CircMon m_respThrCirMon;
    tsDLList < ReqNotify > m_reqList;
    M_CirMonServiceResp m_cirMonServiceResp;
    M_MgrChanHandle m_mgrChanHandle;
    M_CirMonServRespHandle m_cirMonServRespHandle;
    M_CirMonRespThrHandle m_cirMonRespThrHandle;
    Mgr & m_mgr;
    PReq m_pCirMonReq;
    Channel :: CircuitState m_state;
    RefCtr m_refCtr;
    //bool m_ioBlocked;
    //bool m_serverDeletePending;
    //bool m_accessRightsEvPending;
    //epicsShareFunc virtual void destroy ();
    Chan ( const Chan & ); //not implemented
    Chan & operator = ( const Chan & ); //not implemented
    void m_cirStateChangeExceptDiagnostic ( const exception & );
    typedef void ( Chan :: * PStateTrans )( const PCat & );
    void m_becomeDisconnected ( const PCat & );
    void m_becomeUnresponsive ( const PCat & );
    void m_becomeResponsive ( const PCat & );
    void m_commonUnreference ();
    static const PStateTrans m_pStateChangeFunc [];
};

inline PResp Mgr :: Chan :: 
    installCircuitMonitor ( const PReq & pCirMonReq )
{
    assert ( pCirMonReq.isntNill () );
    m_pCirMonReq = pCirMonReq;
    return PResp ( m_cirMonServRespHandle );
}

} // end of namespace Cir
} // end of namespace Svr
} // end of namespace ca

#if 0
class Mgr :: Chan :: Req :
    public tsDLNode < Req >
{
public:
    struct Notify;
    Req ( Chan &, Notify & );
    ~Req ();
    void forceFailResponse ( PResp &, const std :: string & failCtx );
    void show ( unsigned level ) const;
    bool subscrDestroy ( caResId );
    void readLackingCallbackDestroy ();
    void enterFlowControlMode ();
    void exitFlowControlMode ();
    PResp pResponse ();
private:
    class HandleResponderService : 
        public HandleIntf < Responder > {
    public:
        Responder & lock ();  
        Responder & lockRecursive ();  
        void unlock ();
        HandleIntf < Responder > & clone ();
        void release ();
    };
    // The m_responseNotify in this class has the 
    // proper lock for accessing the event queue
    class ResponderService :         
        public Responder {
    private:
        void m_responseNotify ( const PCat & );
    };
    HandleResponderService m_handleResponderService;
    ResponderService m_responderService;
    PReq m_pRequest;
    PResp m_pEventQueueResp;
    Notify & m_notify;
    Chan & m_chan;
    RefCtr m_refCtr;
    Req ( const Req &); // disabled
    Req & operator = ( const Req & ); // disabled
};
#endif

#if 0

inline const caResId Mgr :: Chan :: 
    getCID () const
{
	return m_cid;
}

inline void Mgr :: Chan :: 
    postAccessRightsEvent ( Guard & guard )
{
    guard.assertIdenticalMutex ( m_mutexId );
	m_privateForPV.client().
        addToEventQueue ( *this, m_accessRightsEvPending );
}

inline void Mgr :: Chan :: 
    installIO ( Guard & guard, casAsyncIOI & io )
{
    guard.assertIdenticalMutex ( m_mutexId );
    m_ioList.add ( io );
}

inline void Mgr :: Chan :: 
    uninstallIO ( Guard & guard, casAsyncIOI & io )
{
    guard.assertIdenticalMutex ( m_mutexId );
    m_ioList.remove ( io );
    m_iobl.signal ();
}

inline void Mgr :: Chan :: 
    casChannelDestroyFromInterfaceNotify ( Guard & guard )
{
    guard.assertIdenticalMutex ( m_mutexId );
    if ( ! m_serverDeletePending ) {
        m_privateForPV.client().
            casChannelDestroyFromInterfaceNotify ( *this, true );
    }
}
#endif

#endif // ifndef caSvrCirMgrChanh
