

#ifndef caSvrCirMgrChanCircMonh
#define caSvrCirMgrChanCircMonh

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_caSvrCirMgrChanCircMonh
#   undef epicsExportSharedSymbols
#endif

#ifdef epicsExportSharedSymbols_caSvrCirMgrChanCircMonh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "caSvrCirMgrChan.h"
#include "caSvrEvQueSubscr.h"

namespace ca {
namespace Svr {
namespace Cir {

class Mgr :: Chan :: CircMon 
{
public:
    CircMon ( PReq & );
    void show ( unsigned level ) const;
private:
    typedef Mgr :: Chan Chan;
    class M_ReqNotify :
        public ReqNotify {
    public:
        void show ( unsigned level ) const;
        bool subscrDestroy ( caResId );
        void readLackingCallbackDestroy ();
        void enterFlowControlMode ();
        void exitFlowControlMode ();
        void channelDestroyNotify ();
    };
    class M_Handle : 
        public HandleIntf < Responder > {
    private:
		TargetMutexPair m_targetMutexPair ();
		HandleIntf < Responder > & m_clone ();
		void m_release ();
    };
    class M_Responder :         
        public Responder {
    private:
        void m_responseNotify ( const PCat & );
        void m_show ( unsigned ) const;
    };
    M_Handle m_handle;
    M_Responder m_responder;
    M_ReqNotify m_reqNotify;
    EvQue :: Subscr m_subscr;
    caResId m_cid;
    caResId m_sid;
    Channel :: CircuitState m_circuitState;
    bool m_createChannelRespWasSent;
    void m_unresponsiveNotify ( Guard &, const Catalog & );
    void m_responsiveNotify ( Guard &, const Catalog & );
    void m_disconnectNotify ( Guard &, const Catalog & );
    typedef void ( CircMon :: * PStateNotifyFunc ) 
                                    ( Guard &, const Catalog & );
    static const PStateNotifyFunc m_stateNotifyDispatch [3];
    CircMon ( const CircMon & ); // not implemented
    CircMon & operator = ( const CircMon & ); // not implemented
    //
    // Circuit State Notify Strategy
    // -----------------------------
    // o For clients at V4.12 or higher send the channel create
    // response immediately, and dont wait for the service to 
    // inform us that the channel is responsive. For before 
    // V4.12 postpone sending the create channel response until 
    // only after channel becomes responsive for the first time. 
    // o Never send the "server deletes PV" response when when 
    // service notifies the server that the channel is disconnected. 
    // The client library will nevertheless need to continue to 
    // continue to support this feature for some time due to the 
    // legacy portable server code (in the gateway).
    // o If we are unable to allocate memory for the channel then
    // send the CA_PROTO_CREATE_CH_FAIL response. Otherwise,
    // if the channel disconnects after we create send the
    // CA_PROTO_CH_CONN_STATE response to V4.12 clients, and if its
    // an old an old client rudely disconnect them at the TCP level.
    // o For V4.12 clients communicate channel connectivity in the 
    // service using the new CA_PROTO_CH_CONN_STATE response code.
    // Older clients will be rudely disconnected at the tcp level if
    // the service tells us that the channel has disconnected. That 
    // will be dissruptive, but it will primarily impact non-IOC 
    // clients (on the console side of the gateway), and an easy fix
    // there will be to recompile console clients to use the newer
    // client library.
    //
    // With this strategy we guarantee that the MEDM screen will
    // always go to disconnected white in response to a gateway 
    // loss of service as the users will demand. Alarm state changes
    // are not a sufficent mechanism to communicate loss of 
    // connectivity given the coding practices in  many client 
    // side applications.
    //
    static PType m_pSubscrReqType;
    static PType m_pSubscrResType;
};

} // end of namespace Cir
} // end of namespace Svr
} // end of namespace ca

#endif // caSvrCirMgrChanCircMonh
