
/*
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#define epicsExportSharedSymbols
#include "caSvrCirMgrChanRead.h"
#include "caSvrCirMgrFreeLists.h"

namespace ca {
namespace Svr {
namespace Cir {

PResp Mgr :: Chan :: Read ::
        factory ( Chan & chan, const PReq & pReq, 
        const HdrLargeArray & hdr, caResId cid )
{
    Read * pRead = new ( chan.m_mgr.m_pFreeLists->m_chanRead ) 
                Read ( chan, pReq, hdr, cid );
    return PResp ( pRead->m_servRespHandle );
}

Mgr :: Chan :: Read ::
    Read ( Chan & chan, const PReq & pReq, 
            const HdrLargeArray & hdr, caResId cid ) :
    m_rspThrRead ( hdr, cid ),
    m_pReq ( pReq ),
    m_pChan ( & chan ), 
    m_mgr ( chan.m_mgr )
{
    m_mgr.m_eventQueue.reserve ( 1u );
    chan.m_reqList.add ( m_reqNotify );
}

Mgr :: Chan :: Read ::
    ~Read ()
{
    if ( m_pChan ) {
        m_pChan->m_reqList.remove( m_reqNotify );
    }
    m_mgr.m_eventQueue.release ( 1u );
}

//
// M_Responder
//
Mgr :: Chan :: Read :: M_Responder :: M_EnclLnk
    Mgr :: Chan :: Read :: M_Responder ::  
        m_enclLnk ( & Read :: m_responder );
void Mgr :: Chan :: Read :: M_Responder ::
    m_responseNotify ( const PCat & pCat )
{
    Read & read ( m_enclLnk.enclosure ( *this ) );
    if ( read.m_pChan ) {
        read.m_pChan->m_mgr.m_eventQueue.
            post ( PResp ( read.m_eqRespHandle ), pCat );
        read.m_pChan->m_reqList.remove ( read.m_reqNotify );
        read.m_mgr.m_danglingReqList.add ( read.m_reqNotify );
        read.m_pChan = 0;
    }
}
void Mgr :: Chan :: Read :: M_Responder ::
    m_show ( unsigned level ) const 
{
    const Read & read ( m_enclLnk.enclosure ( *this ) );
    read.show ( level );
}

void Mgr :: Chan :: Read :: 
    show ( unsigned level ) const
{
    printf ( "Mgr :: Chan :: Read:\n" );
    if ( level > 0u ) {
        m_rspThrRead.show ( level - 1 );
        m_refCtr.show ( level - 1 );
        printf ( "m_pChan = %p\n", m_pChan );
        if ( m_pReq.isntNill () ) {
            m_pReq->show ( level - 1 );
        }
    }
}

//
// M_ReqNotify
//
Mgr :: Chan :: Read :: M_ReqNotify :: M_EnclLnk
    Mgr :: Chan :: Read :: M_ReqNotify ::  
        m_enclLnk ( & Read :: m_reqNotify );
void Mgr :: Chan :: Read :: M_ReqNotify ::
    show ( unsigned level ) const
{
    const Read & read ( m_enclLnk.enclosure ( *this ) );
    read.show ( level );
}
bool Mgr :: Chan :: Read :: M_ReqNotify ::
    subscrDestroy ( caResId )
{
    // noop, this isnt a subscription
    return false;
}
void Mgr :: Chan :: Read :: M_ReqNotify ::
    enterFlowControlMode ()
{
    // Noop, this isnt a subscription
}
void Mgr :: Chan :: Read :: M_ReqNotify ::
    exitFlowControlMode ()
{
    // Noop, this isnt a subscription
}
void Mgr :: Chan :: Read :: M_ReqNotify ::
    readLackingCallbackDestroy ()
{
    Read & read ( m_enclLnk.enclosure ( *this ) );
    if ( read.m_rspThrRead.isGetCopy () ) {
        read.m_ioCancel ( 
            "get copy operation "
            "canceled by old client" );
    }
}
void Mgr :: Chan :: Read :: M_ReqNotify :: 
    channelDestroyNotify ()
{
    Read & read ( m_enclLnk.enclosure ( *this ) );
    read.m_ioCancel ( 
        "read operation canceled due "
        "to client initiated channel destroy" );
}

void Mgr :: Chan :: Read :: 
    m_ioCancel ( const char *pCtx )
{
    assert ( m_pChan );
    m_pChan->m_reqList.remove ( m_reqNotify );
    m_mgr.m_danglingReqList.add ( m_reqNotify );
    m_pChan->m_mgr.m_eventQueue.
        postFailure ( PResp ( m_eqRespHandle ), pCtx );
    m_pChan = 0;
    if ( m_pReq.isntNill () ) {
        m_pReq->cancel ();
    }
}

//////////////////////////////////////////////
// M_ServRespHandle
//////////////////////////////////////////////
Mgr :: Chan :: Read :: M_ServRespHandle :: M_EnclLnk
    Mgr :: Chan :: Read :: M_ServRespHandle ::  
        m_enclLnk ( & Read :: m_servRespHandle );
Mgr :: Chan :: Read :: M_ServRespHandle :: TargetMutexPair 
    Mgr :: Chan :: Read :: M_ServRespHandle :: 
        m_targetMutexPair ()
{
    Read & read ( m_enclLnk.enclosure ( *this ) );
    return TargetMutexPair ( 
        read.m_responder, read.m_mgr.m_mutex );
}
HandleIntf < Responder > & 
    Mgr :: Chan :: Read :: M_ServRespHandle ::
        m_clone ()
{
    Read & read ( m_enclLnk.enclosure ( *this ) );
    read.m_refCtr.reference ();
    return *this;
}
void Mgr :: Chan :: Read :: M_ServRespHandle :: 
    m_release ()
{
    Read & read ( m_enclLnk.enclosure ( *this ) );
    read.m_sharedRelease ();
    // no code below this point considering above delete
}

//////////////////////////////////////////////
// M_EQRespHandle
//////////////////////////////////////////////
Mgr :: Chan :: Read :: M_EQRespHandle :: M_EnclLnk
    Mgr :: Chan :: Read :: M_EQRespHandle ::  
        m_enclLnk ( & Read :: m_eqRespHandle );
Mgr :: Chan :: Read :: M_EQRespHandle :: TargetMutexPair 
    Mgr :: Chan :: Read :: M_EQRespHandle :: 
        m_targetMutexPair ()
{
    Read & read ( m_enclLnk.enclosure ( *this ) );
    return TargetMutexPair ( read.m_rspThrRead, 
                        read.m_mgr.m_noopMutex );
}
HandleIntf < Responder > & 
    Mgr :: Chan :: Read :: M_EQRespHandle ::
        m_clone ()
{
    Read & read ( m_enclLnk.enclosure ( *this ) );
    read.m_refCtr.reference ();
    return *this;
}
void Mgr :: Chan :: Read :: M_EQRespHandle :: 
    m_release ()
{
    Read & read ( m_enclLnk.enclosure ( *this ) );
    read.m_sharedRelease ();
    // no code below this point considering above delete
}

void Mgr :: Chan :: Read :: m_sharedRelease ()
{
    if ( m_refCtr.unreference () ) {
        Mgr & mgr ( m_mgr );
        epics :: Mutex :: Guard guard ( mgr.m_mutex );
        if ( m_pChan ) {
            // its a bit weird to cancel the Request from 
            // the Response unreference callback, but 
            // unreferencing the Response before calling
            // the IO completion callback is also a bit 
            // of an odd thing to do
            m_ioCancel ( 
                "service prematurely released the Response?" );
        }
        // we print a warning message if the reference to
        // the response still exists during the Mgr dtor
        // and we know we have already done all of the following
        // o canceled the Request
        // o released the reference to the Request
        // o released the reference to the Channel
        mgr.m_danglingReqList.remove ( m_reqNotify );
        mgr.m_pFreeLists->m_chanRead.deleteProduct ( *this );
        // no code below this point considering above delete
    }
    // no code below this point considering above delete
}

} // end of namespace Cir
} // end of namespace Svr
} // end of namespace ca
