
/*
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#define epicsExportSharedSymbols
#include "caSvrCirMgrChanSubscribe.h"
#include "caSvrCirMgrFreeLists.h"

namespace ca {
namespace Svr {
namespace Cir {

PResp Mgr :: Chan :: Subscribe ::
        factory ( Chan & chan, const PReq & pReq, 
            const HdrLargeArray & hdr, caResId cid, 
            const PCat & pReqParms )
{
    Subscribe * pSubsr = 
        new ( chan.m_mgr.m_pFreeLists->m_chanSubscribe ) 
                Subscribe ( chan, pReq, hdr, cid, pReqParms );
    return PResp ( pSubsr->m_servRespHandle );
}

Mgr :: Chan :: Subscribe :: 
    Subscribe ( Chan & chan, const PReq & pReq, 
        const HdrLargeArray & hdr, caResId cid, 
        const PCat & pReqParms ) :
    m_rspThrSubscr ( hdr, cid ),
    m_eqSubsr ( chan.m_mgr.m_eventQueue, 
            PResp ( m_eqRespHandle ), pReqParms ),
    m_pReq ( pReq ), 
    m_pChan ( & chan ), 
    m_mgr ( chan.m_mgr )
{
}

Mgr :: Chan :: Subscribe ::
    ~Subscribe ()
{
    if ( m_pChan ) {
        m_pChan->m_reqList.remove( m_reqNotify );
    }
}

//
// M_Responder
// 
Mgr :: Chan :: Subscribe :: M_Responder :: M_EnclLnk
    Mgr :: Chan :: Subscribe :: M_Responder ::  
        m_enclLnk ( & Subscribe :: m_responder );
void Mgr :: Chan :: Subscribe :: M_Responder ::
    m_responseNotify ( const PCat & pCat )
{
    Subscribe & subscr ( m_enclLnk.enclosure ( *this ) );
    if ( subscr.m_pChan ) {
        subscr.m_eqSubsr.responseNotify ( pCat );
    }
}
void Mgr :: Chan :: Subscribe :: M_Responder ::
    m_show ( unsigned level ) const
{
    const Subscribe & subscr ( m_enclLnk.enclosure ( *this ) );
    subscr.show ( level );
}

void Mgr :: Chan :: Subscribe :: 
    show ( unsigned level ) const
{
    printf ( "Mgr :: Chan :: Subscribe\n" );
    if ( level > 0u ) {
	    if ( m_pReq.isntNill () ) {
            m_pReq->show ( level - 1 );
	    }
        m_eqSubsr.show ( level - 1 );
	    m_refCtr.show ( level - 1 );
	    printf ( "m_pChan = %p\n", m_pChan );
    }
}

//
// M_ReqNotify
//
Mgr :: Chan :: Subscribe :: M_ReqNotify :: M_EnclLnk
    Mgr :: Chan :: Subscribe :: M_ReqNotify ::  
        m_enclLnk ( & Subscribe :: m_reqNotify );
bool Mgr :: Chan :: Subscribe :: M_ReqNotify ::
    subscrDestroy ( caResId subscrId )
{
    Subscribe & subscr ( m_enclLnk.enclosure ( *this ) );
    if ( subscrId != subscr.m_rspThrSubscr.clientId () ) {
        return false;
    }
    subscr.m_ioCancel ( 
        "subscription operation canceled due "
        "to client initiated subscription destroy" );
    delete & subscr;
    // no access to this object after this point
    // due to above delete
    return true;
}
Mgr :: Chan :: Subscribe :: M_ReqNotify ::
    ~M_ReqNotify ()
{
}
void Mgr :: Chan :: Subscribe :: M_ReqNotify ::
    enterFlowControlMode ()
{
    Subscribe & subscr ( m_enclLnk.enclosure ( *this ) );
    subscr.m_eqSubsr.enterFlowControlMode ();
}
void Mgr :: Chan :: Subscribe :: M_ReqNotify ::
    exitFlowControlMode ()
{
    Subscribe & subscr ( m_enclLnk.enclosure ( *this ) );
    subscr.m_eqSubsr.exitFlowControlMode ();
}

void Mgr :: Chan :: Subscribe :: M_ReqNotify ::
    readLackingCallbackDestroy ()
{
    // noop, this isnt a read request
}
void Mgr :: Chan :: Subscribe :: M_ReqNotify :: 
    channelDestroyNotify ()
{
    Subscribe & subscr ( m_enclLnk.enclosure ( *this ) );
    subscr.m_ioCancel ( 
        "subscription operation canceled due "
        "to client initiated channel destroy" );
}
void Mgr :: Chan :: Subscribe :: M_ReqNotify :: 
    show ( unsigned level ) const
{
    const Subscribe & subscr ( m_enclLnk.enclosure ( *this ) );
    subscr.show ( level );
}

void Mgr :: Chan :: Subscribe :: 
    m_ioCancel ( const char *pCtx )
{
    assert ( m_pChan );
    m_pChan->m_reqList.remove ( m_reqNotify );
    m_mgr.m_danglingReqList.add ( m_reqNotify );
    m_pChan = 0;
    m_eqSubsr.cancel ();
    if ( m_pReq.isntNill () ) {
        m_pReq->cancel ();
    }
}

//////////////////////////////////////////////
// M_EQRespHandle
//////////////////////////////////////////////
Mgr :: Chan :: Subscribe :: M_EQRespHandle :: M_EnclLnk
    Mgr :: Chan :: Subscribe :: M_EQRespHandle ::  
        m_enclLnk ( & Subscribe :: m_eqRespHandle );
Mgr :: Chan :: Subscribe :: M_EQRespHandle :: TargetMutexPair 
    Mgr :: Chan :: Subscribe :: M_EQRespHandle :: m_targetMutexPair ()
{
    Subscribe & subscr ( m_enclLnk.enclosure ( *this ) );
    return TargetMutexPair ( 
        subscr.m_rspThrSubscr, subscr.m_mgr.m_noopMutex );
}

HandleIntf < Responder > & 
Mgr :: Chan :: Subscribe :: M_EQRespHandle :: m_clone ()
{
    Subscribe & subscr ( m_enclLnk.enclosure ( *this ) );
    subscr.m_refCtr.reference ();
    return *this;
}

void Mgr :: Chan :: Subscribe :: M_EQRespHandle :: m_release ()
{
    Subscribe & subscr ( m_enclLnk.enclosure ( *this ) );
    subscr.m_commonRelease ();
    // no code below this point considering above delete
}

//////////////////////////////////////////////
// M_ServRespHandle
//////////////////////////////////////////////
Mgr :: Chan :: Subscribe :: M_ServRespHandle :: M_EnclLnk
    Mgr :: Chan :: Subscribe :: M_ServRespHandle ::  
        m_enclLnk ( & Subscribe :: m_servRespHandle );
Mgr :: Chan :: Subscribe :: M_ServRespHandle :: TargetMutexPair 
    Mgr :: Chan :: Subscribe :: M_ServRespHandle :: 
        m_targetMutexPair ()
{
    Subscribe & subscr ( m_enclLnk.enclosure ( *this ) );
    return TargetMutexPair ( subscr.m_responder, subscr.m_mgr.m_mutex );
}
HandleIntf < Responder > & 
    Mgr :: Chan :: Subscribe :: M_ServRespHandle ::
        m_clone ()
{
    Subscribe & subscr ( m_enclLnk.enclosure ( *this ) );
    subscr.m_refCtr.reference ();
    return *this;
}
void Mgr :: Chan :: Subscribe :: M_ServRespHandle :: 
    m_release ()
{
    Subscribe & subscr ( m_enclLnk.enclosure ( *this ) );
    subscr.m_commonRelease ();
    // no code below this point considering above delete
}

void Mgr :: Chan :: Subscribe :: m_commonRelease ()
{
    if ( m_refCtr.unreference () ) {
        Mgr & mgr ( m_mgr );
        Guard guard ( mgr.m_mutex );
        m_eqSubsr.cancel ();
        if ( m_pChan ) {
            // its a bit weird to cancel the Request from 
            // the Response unreference callback, but 
            // unreferencing the Response before calling
            // the IO completion callback is also a bit 
            // of an odd thing to do
            m_ioCancel ( 
                "service prematurely released the Response?" );
        }
        // we print a warning message if the reference to
        // the response still exists during the Mgr dtor
        // and we know we have already done all of the following
        // o canceled the Request
        // o released the reference to the Request
        // o released the reference to the Channel
        mgr.m_danglingReqList.remove ( m_reqNotify );
        mgr.m_pFreeLists->m_chanSubscribe.deleteProduct ( *this );
        // no code below this point considering above delete
    }
    // no code below this point considering above delete
}

} // end of namespace Cir
} // end of namespace Svr
} // end of namespace ca
