
/*
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#define epicsExportSharedSymbols
#include "caSvrCirMgrChanWrite.h"
#include "caSvrCirMgrFreeLists.h"

namespace ca {
namespace Svr {
namespace Cir {

PResp Mgr :: Chan :: Write ::
        factory ( Chan & chan, const PReq & pReq,
            const HdrLargeArray & hdr, caResId cid )
{
    Write * pWrite = new ( chan.m_mgr.m_pFreeLists->m_chanWrite ) 
                Write ( chan, pReq, hdr, cid );
    return PResp ( pWrite->m_servRespHandle );
}

Mgr :: Chan :: Write :: 
    Write ( Chan & chan, const PReq & pReq, 
        const HdrLargeArray & hdr, caResId cid ) :
    m_rspThrWrite ( hdr, cid ),
    m_pReq ( pReq ), 
    m_pChan ( & chan ), 
    m_mgr ( chan.m_mgr )
{
    m_mgr.m_eventQueue.reserve ( 1u );
    m_pChan->m_reqList.add ( m_reqNotify );
}

Mgr :: Chan :: Write ::
    ~Write ()
{
    if ( m_pChan ) {
        m_pChan->m_reqList.remove( m_reqNotify );
    }
    m_mgr.m_eventQueue.release ( 1u );
}

//
// M_Responder
// 
Mgr :: Chan :: Write :: M_Responder :: M_EnclLnk
    Mgr :: Chan :: Write :: M_Responder ::  
        m_enclLnk ( & Write :: m_responder );
void Mgr :: Chan :: Write :: M_Responder ::
    m_responseNotify ( const PCat & pCat )
{
    Write & write ( m_enclLnk.enclosure ( *this ) );
    if ( write.m_pChan ) {
        write.m_pChan->m_mgr.m_eventQueue.
            post ( PResp ( write.m_eqRespHandle ), pCat );
        write.m_pChan->m_reqList.remove ( write.m_reqNotify );
        write.m_mgr.m_danglingReqList.add ( write.m_reqNotify );
        write.m_pChan = 0;
    }
}
void Mgr :: Chan :: Write :: M_Responder ::
    m_show ( unsigned level ) const
{
    const Write & write ( m_enclLnk.enclosure ( *this ) );
    write.show ( level );
}

void Mgr :: Chan :: Write :: 
    show ( unsigned level ) const
{
    printf ( "Mgr :: Chan :: Write:\n" );
    if ( level > 0u ) {
        m_rspThrWrite.show ( level - 1 );
        m_refCtr.show ( level - 1 );
	if ( m_pChan ) {
            m_pChan->show ( level - 1 );
	}
	printf ( "m_pChan = %p\n", m_pChan );
    }
}

//
// M_ReqNotify
//
Mgr :: Chan :: Write :: M_ReqNotify :: M_EnclLnk
    Mgr :: Chan :: Write :: M_ReqNotify ::  
        m_enclLnk ( & Write :: m_reqNotify );
void Mgr :: Chan :: Write :: M_ReqNotify ::
    show ( unsigned level ) const
{
    const Write & write ( m_enclLnk.enclosure ( *this ) );
    write.show ( level );
}
bool Mgr :: Chan :: Write :: M_ReqNotify ::
    subscrDestroy ( caResId )
{
    // noop, this isnt a subscription
    return false;
}
void Mgr :: Chan :: Write :: M_ReqNotify ::
    enterFlowControlMode ()
{
    // Noop, this isnt a subscription
}
void Mgr :: Chan :: Write :: M_ReqNotify ::
    exitFlowControlMode ()
{
    // Noop, this isnt a subscription
}
void Mgr :: Chan :: Write :: M_ReqNotify ::
    readLackingCallbackDestroy ()
{
    // Noop, this isnt a read
}
void Mgr :: Chan :: Write :: M_ReqNotify :: 
    channelDestroyNotify ()
{
    Write & write ( m_enclLnk.enclosure ( *this ) );
    write.m_ioCancel ( 
        "write operation canceled due "
        "to client initiated channel destroy" );
}

void Mgr :: Chan :: Write :: 
    m_ioCancel ( const char * pCtx )
{
    assert ( m_pChan );
    m_pChan->m_reqList.remove ( m_reqNotify );
    m_mgr.m_danglingReqList.add ( m_reqNotify );
    m_pChan->m_mgr.m_eventQueue.
        postFailure ( PResp ( m_eqRespHandle ), pCtx );
    m_pChan = 0;
    if ( m_pReq.isntNill () ) {
        m_pReq->cancel ();
    }
}

///////////////////////////////////////////////
// M_ServRespHandle
///////////////////////////////////////////////
Mgr :: Chan :: Write :: M_ServRespHandle :: M_EnclLnk
    Mgr :: Chan :: Write :: M_ServRespHandle ::  
        m_enclLnk ( & Write :: m_servRespHandle );
Mgr :: Chan :: Write :: M_ServRespHandle :: TargetMutexPair 
    Mgr :: Chan :: Write :: M_ServRespHandle :: 
        m_targetMutexPair ()
{
    Write & write ( m_enclLnk.enclosure ( *this ) );
    return TargetMutexPair ( write.m_responder, write.m_mgr.m_mutex );
}
HandleIntf < Responder > & 
    Mgr :: Chan :: Write :: M_ServRespHandle ::
        m_clone ()
{
    Write & write ( m_enclLnk.enclosure ( *this ) );
    write.m_refCtr.reference ();
    return *this;
}
void Mgr :: Chan :: Write :: M_ServRespHandle :: 
    m_release ()
{
    Write & write ( m_enclLnk.enclosure ( *this ) );
    write.m_commonRelease ();
}

///////////////////////////////////////////////
// M_EQRespHandle
///////////////////////////////////////////////
Mgr :: Chan :: Write :: M_EQRespHandle :: M_EnclLnk
    Mgr :: Chan :: Write :: M_EQRespHandle ::  
        m_enclLnk ( & Write :: m_eqRespHandle );
Mgr :: Chan :: Write :: M_EQRespHandle :: TargetMutexPair 
    Mgr :: Chan :: Write :: M_EQRespHandle :: 
        m_targetMutexPair ()
{
    Write & write ( m_enclLnk.enclosure ( *this ) );
    return TargetMutexPair ( write.m_rspThrWrite, write.m_mgr.m_noopMutex );
}
HandleIntf < Responder > & 
    Mgr :: Chan :: Write :: M_EQRespHandle ::
        m_clone ()
{
    Write & write ( m_enclLnk.enclosure ( *this ) );
    write.m_refCtr.reference ();
    return *this;
}
void Mgr :: Chan :: Write :: M_EQRespHandle :: 
    m_release ()
{
    Write & write ( m_enclLnk.enclosure ( *this ) );
    write.m_commonRelease ();
}

void Mgr :: Chan :: Write ::  
    m_commonRelease ()
{
    if ( m_refCtr.unreference () ) {
        Mgr & cirSvr ( m_mgr );
        epics :: Mutex :: Guard guard ( cirSvr.m_mutex );
        if ( m_pChan ) {
            // its a bit weird to cancel the Request from 
            // the Response unreference callback, but 
            // unreferencing the Response before calling
            // the IO completion callback is also a bit 
            // of an odd thing to do
            m_ioCancel ( "service prematurely released the Response?" );
        }
        // we print a warning message if the reference to
        // the response still exists during the Mgr dtor
        // and we know we have already done all of the following
        // o canceled the Request
        // o released the reference to the Request
        // o released the reference to the Channel
        cirSvr.m_danglingReqList.remove ( m_reqNotify );
        cirSvr.m_pFreeLists->m_chanWrite.deleteProduct ( *this );
    }
    // no code below this point considering above delete
}

} // end of namespace Cir
} // end of namespace Svr
} // end of namespace ca
