
#ifndef caSvrCirRcvThrh
#define caSvrCirRcvThrh

#include <vector>

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_caSvrCirRcvThrh
#   undef epicsExportSharedSymbols
#endif

#include "tsDLList.h"
#include "resourceLib.h"
#include "epicsThread.h"
#include "epicsTime.h"
#include "FixedString.h"
#include "EnclLnk.h"

#ifdef epicsExportSharedSymbols_caSvrCirRcvThrh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "casdef2.h"
#include "caHdrLargeArray.h"
#include "caChannelDescription.h"
#include "IterIn.h"

namespace ca {

struct CircuitIn;
struct Circuit;

namespace Svr {
namespace Cir {

class Mgr;
class RspThr;

class RcvThr {
public:
    static Ptr < RcvThr > 
        factory ( const PService &, const Ptr < Mgr > &, 
            const Ptr < RspThr > &, std :: auto_ptr < CircuitIn > );
    RcvThr ( const PService &, const Ptr < Mgr > &, 
            const Ptr < RspThr > &, std :: auto_ptr < CircuitIn > );
    ~RcvThr ();
    void show ( unsigned level ) const;
    void setPriority ( unsigned epicsPriority );
    void start ();
private:
    class M_Handle : 
        public HandleIntf < RcvThr > {
    private:
        TargetMutexPair m_targetMutexPair ();
        HandleIntf < RcvThr > & m_clone ();
        void m_release ();
        typedef const epics :: EnclLnk < RcvThr, 
                                M_Handle > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    class M_CircuitInReader : 
        public Reader {
    public:
        M_CircuitInReader ( std :: auto_ptr < CircuitIn > );
        Vec < const epicsUInt8 > read ( size_t nBytesMax );
        void rewind (); // circuits dont rewind
        void shutdown ();
    private:
        std :: auto_ptr < CircuitIn > m_pCircuitIn;
        typedef const epics :: EnclLnk < RcvThr, 
                                M_CircuitInReader > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    class M_EntryPt : 
        public epicsThreadRunable {
    public:
        void run ();
		void show ( unsigned level ) const;
        typedef const epics :: EnclLnk < RcvThr, 
                                M_EntryPt > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    class Chan;
    typedef void ( RcvThr :: * PMsgHandler ) ( IterIn & );
    M_Handle m_handle;
    M_EntryPt m_entryPt;
    M_CircuitInReader m_circuitInReader;
    epicsThread m_thread;
    epics :: Mutex m_mutex;
    HdrLargeArray m_curMsg;
    //VecAllocator m_vecAllocator;
    //tsFreeList < ChannelDescription > m_channelDescrFreeList;
    tsFreeList < Chan > m_chanFreeList;
    chronIntIdResTable < Chan > m_chanTable;
    tsDLList < Chan > m_chanList;
    epicsTime m_lastRecvTS;
    PService m_pService;
    Ptr < Mgr > m_pMgr;
    Ptr < RspThr > m_pRspThr;
    std :: vector < PType > m_dbrTypePtrs;
    FixedString < 128 > m_hostNameStr;
    FixedString < 128 > m_userNameStr;
    RefCtr m_refCtr;
    ca_uint16_t m_minorVersionNumber;
    bool m_threadShutdownCmd;
    PCat m_dbrCatalogFactory ( IterIn & );
    Chan & m_verifyRequest ();
    void m_setVersion ( ca_uint16_t clientVersion );
    void m_readLackingCallbackCancel ();
    void m_processMsg ( IterIn & );
    void m_msgStubExceptDiag ( const std::exception & ) const;
    void m_msgStubProtocolErrorDiag ( const std::exception & ) const;
    static unsigned m_calcOffsetPriority ( unsigned reqPriority );

    // one function for each CA request type
    void m_uknownMessageAction ( IterIn & );
    void m_ignoreMsgAction ( IterIn & );
    void m_versionAction ( IterIn & );
    void m_echoAction ( IterIn & );
    void m_subscribeAction ( IterIn & );
    void m_eventCancelAction ( IterIn & );
    void m_readAction ( IterIn & );
    void m_writeAction ( IterIn & );
    void m_eventsOffAction ( IterIn & );
    void m_eventsOnAction ( IterIn & );
    void m_readSyncAction ( IterIn & );
    void m_clearChannelAction ( IterIn & );
    void m_createChannelAction ( IterIn & );
    void m_clientNameAction ( IterIn & );
    void m_hostNameAction ( IterIn & );
    static PMsgHandler const m_msgHandlers[CA_PROTO_LAST_CMMD+1u];
};

} // end of namespace Cir
} // end of namespace Svr
} // end of namespace ca

#endif // caSvrCirRcvThrh

