
/*
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#ifndef caSvrCirRcvThrChanh
#define caSvrCirRcvThrChanh

#include "caSvrCirRcvThr.h"
#include "caSvrCirMgrChan.h" // no fwd decl of nested class

namespace ca {

class IterIn;

namespace Svr {
namespace Cir {

class RcvThr :: Chan : 
    public tsDLNode < Chan >, 
    public chronIntIdRes < Chan > {
public:
    Chan ( RcvThr &, PChan &, caResId cid );
    // a two step init is needed due to chicken
    // and egg issues
    void setServerChan ( const Mgr :: PChan & ); 
    void casChannelDestroyFromInterfaceNotify ();
    //const caResId getCID () const;
    const caResId getSID () const;
    void postAccessRightsEvent ();
    void read ( const HdrLargeArray & );
    void write ( const HdrLargeArray &, const PCat & val );
    void subscribe ( const HdrLargeArray &, const PCat & reqParms );
    bool unsubscribe ( ca_uint32_t clientIdIn );
    void restartSubscriptions ();
    void show ( unsigned level ) const;
    void enterFlowControlMode ();
    void exitFlowControlMode ();
    void readLackingCallbackCancel ();
private:

    // types
    static Type & typeCircuitState;
    // we dont invoke PChan from CirSvr lock domain
    PChan m_pServiceChan; 
    Mgr :: PChan m_pServerChan;
    RcvThr & m_recvThr;
    caResId m_cid; // client id 
    void m_exceptionResponse ( const PResp &, 
            const std :: exception &, const char * pCtx );
    //Channel :: CircuitState m_state;
    //RspThr :: CircuitMonitor m_circuitMonitor;
    
    //bool m_ioBlocked;
    //bool m_serverDeletePending;
    //bool m_accessRightsEvPending;
    //epicsShareFunc virtual void destroy ();
    Chan ( const Chan & ); // disabled
    Chan & operator = ( const Chan & ); // disabled
    void cirStateNotify ( const Catalog & );
    //void cirStateNotifyFailure ( const Abberation & );
};
  
inline void RcvThr :: Chan ::
    setServerChan ( const Mgr :: PChan & pChan ) 
{
    assert ( pChan.isntNill () );
    m_pServerChan = pChan;
}

inline const caResId RcvThr :: Chan :: 
    getSID () const
{
    return this->chronIntIdRes < Chan > :: getId ();
}

inline void RcvThr :: Chan :: enterFlowControlMode ()
{
    m_pServerChan->enterFlowControlMode ();
}

inline void RcvThr :: Chan :: exitFlowControlMode ()
{
    m_pServerChan->exitFlowControlMode ();
}

inline void RcvThr :: Chan :: readLackingCallbackCancel ()
{
    m_pServerChan->readLackingCallbackCancel ();
}

} // end of namespace Cir
} // end of namespace Svr
} // end of namespace ca

#endif // caSvrCirRcvThrChanh
