
#ifndef caSvrCirRspThrh
#define caSvrCirRspThrh

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_caSvrCirRspThrh
#   undef epicsExportSharedSymbols
#endif

#include "epicsThread.h"
#include "epicsTime.h"
#include "caProto.h"
#include "EnclLnk.h"

#ifdef epicsExportSharedSymbols_caSvrCirRspThrh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "casdef2.h"
#include "ByteSeqOut.h"
#include "caRefCtr.h"

namespace ca {

struct CircuitOut;
struct HdrLargeArray;
struct Circuit;

namespace Svr {
namespace Cir {

class Mgr;

// All of the public methods of this class can be 
// safely called from multiple threads w/o locking.
// The private internals of this class are only
// accessed by a single private thread which, among
// other tasks, processes events on the event queue.
class RspThr {
public:
    static Ptr < RspThr > factory ( const Ptr < Mgr > &, 
        std :: auto_ptr < CircuitOut >, const Ptr < VecAllocator > & );
    RspThr ( const Ptr < Mgr > &, 
        std :: auto_ptr < CircuitOut >, const Ptr < VecAllocator > & );
    ~RspThr ();
    // Show is currently not safe to be called 
    // from the public interface which isnt locked.
    // If we need a show we could post an event
    // to the event queue which cause the response
    // thread's internals to be dumped.
    void show ( unsigned level ) const;
    void eventSignal ();
    void setPriority ( unsigned epicsThreadPriority );
    void start ();
    void chanCreateFailResp ( caResId id, const std :: exception & );
    void readFailResp ( const HdrLargeArray & );
    void writeFailResp ( const HdrLargeArray & );
    void subscribeFailResp ( const HdrLargeArray & );
    class Read;
    class Write;
    class Subscribe;
    class ChanCreate;
    class ChanDelete;
    class CircMon;
    class ClientVerSet;
    class HdrOnly;
    //class BadMsgLen
    static RspThr * pAttachedRspThr ();
private:
    class M_Handle : 
        public HandleIntf < RspThr > {
    private:
        TargetMutexPair m_targetMutexPair ();
        HandleIntf < RspThr > & m_clone ();
        void m_release ();
        typedef const epics :: EnclLnk < RspThr, 
                                M_Handle > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    class M_EntryPt : 
        public epicsThreadRunable {
    public:
        void run ();
        void show ( unsigned level ) const;
        typedef const epics :: EnclLnk < RspThr, 
                                M_EntryPt > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    class CircuitOutWriter;
    M_Handle m_handle;
    M_EntryPt m_entryPt;
    mutable epics :: Mutex m_mutex;
    epicsThread m_thread;
    ByteSeqOut m_byteSeqOut;
    std :: auto_ptr < CircuitOut > m_pOut;
    Ptr < Mgr > m_pMgr;
    epicsEvent m_event;
    RefCtr m_refCtr;
    epicsTime m_lastSendTS;
    ca_uint16_t m_minorVersionNumber;
    bool m_threadShutdownCmd;

    void m_flush ();
    void m_sendVersion ();
    size_t m_modalMaxEventQueueEntires ();
    void m_verifyCorrectThread () const;
    void m_accessRightsResponse ( caResId cid, const Catalog & );
#if 0
    void m_cirMonResponse ( Catalog &, Chan & );
    void m_readExceptionResponse ( 
        Chan & chan, 
        const HdrLargeArray & msg, 
        const char * pExceptName );
    void m_readNotifyResponseSuccess ( 
        ca_uint32_t cid, 
        const HdrLargeArray & reqMsg, 
        const Catalog & );
    void m_readNotifyResponseFailure ( 
        ca_uint32_t cid, 
        const HdrLargeArray & reqMsg, 
        const Catalog & );
    void m_writeResponse ( 
        Catalog &, Chan &, 
        size_t nElem, ca_uint16_t dbrType );
    void m_subscrUpdateResponse ( 
        Catalog &, Chan &, 
        size_t nElem, ca_uint16_t dbrType );

#endif
};

class RspThr :: ClientVerSet 
{
public:
    ClientVerSet ( ca_uint16_t clientVersion );
private:
    class EventCallback : 
        public Responder {
    private:
        void m_responseNotify ( const PCat & );
        void m_show ( unsigned level ) const;
    };
    EventCallback m_eventCallback;
    ca_uint16_t m_minorVersionNumber;
    ClientVerSet ( const HdrOnly & ); 
    ClientVerSet & operator = ( const HdrOnly & ); 
};

#if 0
class RspThr :: BadMsgLen
{
public:
    BadMsgLen ( Guard &, CirSvr &, size_t msgSize );
private:
    class EventNotifyBinder {
    public:
        EventNotifyBinder ( BadMsgLen & );
        void success ( Guard &, PCat & );
        void fail ( Guard &, PCat & );
    private:
        BadMsgLen & m_req;
    };
    EventNotifyBinder m_eventNotify;
    size_t m_msgSize;
    class Mgr & m_mgr;
    void m_sendResponse ( Guard & );
    void m_release ( Guard & );
    ~BadMsgLen (); // disabled - see m_destructor
    BadMsgLen ( const BadMsgLen & ); // disabled
    BadMsgLen & operator = ( const BadMsgLen & ); // disabled
};
#endif

inline void RspThr :: 
    setPriority ( unsigned epicsThreadPriority )
{
    m_thread.setPriority ( epicsThreadPriority );
}

inline void RspThr :: start ()
{
    m_thread.start ();
}

inline void RspThr :: eventSignal ()
{
    m_event.signal ();
}

} // end of namespace Cir
} // end of namespace Svr
} // end of namespace ca

#endif // ifndef caSvrCirRspThrh
