
/*
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#define epicsExportSharedSymbols
#include "caSvrCirMgr.h"
#include "caSvrCirRspThrChanCreate.h"
#include "v3Proto.h"

namespace ca {
namespace Svr {
namespace Cir {

RspThr :: ChanCreate :: 
    ChanCreate ( const Guard & guard, 
        RspThr & respThread, caResId cid ) :
    m_event ( guard, m_respThread.m_eventQueue ), 
    m_respThread ( respThread ), 
    m_cid ( cid ), 
    m_sid ( 0 )
{
#error this needs to reserve space on the event queue?
}

RspThr :: ChanCreate :: 
    ~ChanCreate ()
{
#error this needs to release space on the event queue?
}

void RspThr :: ChanCreate :: 
    show ( unsigned level ) const
{
    printf ( "RspThr :: ChanCreate\n" );
    if ( level > 0 ) {
        m_event.show ( guard, level - 1 );
    }
}

void RspThr :: ChanCreate :: 
    enqueuSuccessResp ( caResId sid ) 
{
    m_sid = sid;
    PCat emptyCatalogPtr;
    m_event.postSuccess ( 
        guard, m_eventCallback, emptyCatalogPtr );
}

void RspThr :: ChanCreate :: 
    enqueuFailResp ( const Guard & guard )
{
    guard.assertIdenticalMutex ( m_respThread.m_mgr.m_mutex );
    PCat emptyCatalogPtr;
    m_event.postFailure ( guard, m_eventCallback, emptyCatalogPtr );
}

void RspThr :: ChanCreate :: EventCallback ::
    success ( Guard & guard, PCat & )
{
    EnclBinding :: Ptr < ChanCreate > pChanCreate ( *this );
    guard.assertIdenticalMutex (
        pChanCreate->m_respThread.m_byteSeqMutex );
    Mgr & mgr ( pChanCreate->m_respThread.m_mgr );
    // In V4.13 native type is sent later once we 
    // receive notification that the channel is in 
    // a responsive state. In older versions we have 
    // no choice but to wait to send channel create 
    // confirmation until we receive responsive state 
    // notification from the service.
    if ( CA_V413 ( mgr.m_minorVersionNumber ) ) {
        HdrLargeArray respHdr;
        respHdr.m_cmmd = CA_PROTO_CREATE_CHAN;
        respHdr.m_postsize = 0;
        respHdr.m_dataType = 0;
        respHdr.m_count = 0;
        respHdr.m_cid = pChanCreate->m_cid;
        respHdr.m_available = pChanCreate->m_sid;
        MsgMinderV3 msgMinderV3 ( 
                pChanCreate->m_respThread.m_byteSeqOut,
                mgr.m_minorVersionNumber );
        msgMinderV3.pushResp ( respHdr );
        msgMinderV3.commit ();
    }
}

void RspThr :: ChanCreate :: EventCallback ::
    fail ( Guard & guard, PCat & )
{
    EnclBinding :: Ptr < ChanCreate > pChanCreate ( *this );
    guard.assertIdenticalMutex ( 
        pChanCreate->m_respThread.m_byteSeqMutex );
    Mgr & mgr ( pChanCreate->m_respThread.m_mgr );
    HdrLargeArray respHdr;
    respHdr.m_cmmd = CA_PROTO_CREATE_CH_FAIL;
    respHdr.m_postsize = 0;
    respHdr.m_dataType = 0;
    respHdr.m_count = 0;
    respHdr.m_cid = pChanCreate->m_cid;
    respHdr.m_available = 0;
    MsgMinderV3 msgMinderV3 ( 
            pChanCreate->m_respThread.m_byteSeqOut,
            mgr.m_minorVersionNumber );
    msgMinderV3.pushResp ( respHdr );
    msgMinderV3.commit ();
}

void RspThr :: ChanCreate :: EventCallback ::
    release ( const Guard & guard )
{
    EnclBinding :: Ptr < ChanCreate > pChanCreate ( *this );
    guard.assertIdenticalMutex ( 
        pChanCreate->m_respThread.m_mgr.m_mutex );
    pChanCreate->m_event.cleanup ( guard );
    pChanCreate->m_respThread.m_mgr.m_chanCreateFreeList.
        deleteProduct ( *pChanCreate.get () );
}

} // end of namespace Cir
} // end of namespace Svr
} // end of namespace ca
