
/*
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#include "db_access.h"

#define epicsExportSharedSymbols
#include "caSvrCirRspThrRead.h"
#include "FailureContext.h"
#include "protoDiagnostics.h"
#include "v3Proto.h"

namespace ca {
namespace Svr {
namespace Cir {

RspThr :: Read ::
    Read ( const HdrLargeArray & reqMsg, caResId cid ) :
    m_reqMsg ( reqMsg ), m_cid ( cid )
{
}

RspThr :: Read ::
    ~Read ()
{
}

#if 0
void RspThr :: Read ::
    initiate ( PChan & pChan )
{
    try {
        bool invalidType = ( m_reqMsg.m_dataType >= 
                m_respThread.m_mgr.m_dbrTypePtrs.size () );
        if ( invalidType || m_respThread.m_mgr.
            m_dbrTypePtrs[ m_reqMsg.m_dataType ].isNill () ) {
            throw std :: range_error ( 
                "specified dbr type isnt supported by this server" );
        }
        Ptr < const Type > voidReqType;
        PReq reqPtr = pChan->createRequest ( 
            Channel :: read, voidReqType, 
            m_respThread.m_mgr.m_dbrTypePtrs[ m_reqMsg.m_dataType ] );
        m_req.initiate ( reqPtr, PResp ( m_eventCallbackHandle ) );
    }
    catch ( std :: exception & except ) {
        m_onceReq.forceFailResponse ( 
            PResp ( m_eventCallbackHandle ),
            std :: string (            
                "ca::Svr::Cir::RspThr::Read initiate: "
                "exception when creating "
                "\"read\" request in service was \"" ) +
            except.what () + 
            std :: string ( "\"" ) );
    }
    catch ( ... ) {
        m_onceReq.forceFailResponse ( 
            PResp ( m_eventCallbackHandle ),
            "ca::Svr::Cir::RspThr::RspThr::Read initiate: "
            "non-standard exception when creating "
            "\"read\" request in service" );
    }
}
#endif

inline RspThr :: Read :: ResponseAgentSuccess ::
    ResponseAgentSuccess ( Read & read, const Catalog & respParms ) :
    m_read ( read ), m_respParm ( m_respParm )
{
}

void RspThr :: Read :: ResponseAgentSuccess ::
    assignDefault ()
{
    // ok, success indication wasnt found so we 
    // probe now for failure indication
    ResponseAgentFailure responseAgentFailure ( m_read );
    get ( m_respParm, Responder :: failure, responseAgentFailure );
}

void RspThr :: Read :: ResponseAgentSuccess ::
    assignScalarCatalog ( const Catalog & successParms )
{
    HdrLargeArray respHdr;
    respHdr.m_cmmd = m_read.m_reqMsg.m_cmmd;
    respHdr.m_postsize = 0; // tbd
    respHdr.m_dataType = m_read.m_reqMsg.m_dataType;
    respHdr.m_count = m_read.m_reqMsg.m_count;
    respHdr.m_cid = m_read.m_cid;
    if ( m_read.m_reqMsg.m_cmmd == CA_PROTO_READ_NOTIFY ) {
        respHdr.m_available = ECA_NORMAL;
    }
    else {
        respHdr.m_available = m_read.m_reqMsg.m_available;
    }
    try {
        RspThr * pRspThr = RspThr :: pAttachedRspThr ();
        if ( pRspThr ) {
            MsgMinderV3 msgMinderV3 ( pRspThr->m_byteSeqOut,
                pRspThr->m_minorVersionNumber );
            msgMinderV3.pushResp ( respHdr, successParms );
            msgMinderV3.commit ();
        }
    }
    catch ( exception & except ) {
        FailureContext failureContext ( except.what () );
        m_read.m_failResponse ( failureContext );
    }
}

inline RspThr :: Read :: ResponseAgentFailure ::
    ResponseAgentFailure ( Read & read ) :
    m_read ( read )
{
}

void RspThr :: Read :: ResponseAgentFailure ::
    assignDefault ()
{
    FailureContext failureContext ( 
        "service read response payload was "
        "success/failure ambiguous" );
    m_read.m_failResponse ( failureContext );
}

void RspThr :: Read :: ResponseAgentFailure ::
    assignScalarCatalog ( const Catalog & failureParms )
{
    m_read.m_failResponse ( failureParms );
}

void RspThr :: Read :: 
    m_responseNotify ( const PCat & pRespParms )
{
    // This tells the dtor that the m_responseNotify
    // from the Request has arrived so the request 
    // doesnt need to be canceled
    if ( pRespParms.isntNill () ) {
        PCat :: Locker pRespParmsLocked = pRespParms;
        ResponseAgentSuccess 
            responseAgentSuccess ( *this, *pRespParmsLocked ); 
        get ( *pRespParmsLocked, success, responseAgentSuccess );
    }
    else {
        FailureContext failureContext ( 
            "received service read response notification, "
            "but success/failure indication wasnt in "
            "response (empty) payload" );
        m_failResponse ( failureContext );
    }
}

void RspThr :: Read ::  
    m_show ( unsigned level ) const
{
    printf ( "ca::Svr::Cir::RspThr::Read\n" );
    if ( level > 0 ) {
        dumpMsg ( m_reqMsg );
        printf ( "ca::Svr::Cir::RspThr::Read: m_cid = %u\n", m_cid );
    }
}

void RspThr :: Read ::
    m_failResponse ( const Catalog & failureContext )
{
    RspThr * pRspThr = RspThr :: pAttachedRspThr ();
    if ( pRspThr ) {
        MsgMinderV3 msgMinderV3 ( pRspThr->m_byteSeqOut,
                    pRspThr->m_minorVersionNumber );
        if ( m_reqMsg.m_cmmd == CA_PROTO_READ_NOTIFY ) {
            ca_uint32_t payloadSize = 
                dbr_size_n ( m_reqMsg.m_dataType, m_reqMsg.m_count );
            HdrLargeArray respHdr;
            respHdr.m_cmmd = m_reqMsg.m_cmmd;
            respHdr.m_postsize = payloadSize;
            respHdr.m_dataType = m_reqMsg.m_dataType;
            respHdr.m_count = m_reqMsg.m_count;
            respHdr.m_cid = m_cid;
            respHdr.m_available = ECA_GETFAIL;
            // in the future it might be useful to, instead of
            // copying in a nill payload, copy the failure 
            // detail string into the payload here, but that 
            // might result in some protocol issues.
            // 1) We would need to reply specifying DBR_STRING, 
            // and that would of course, typically, not match 
            // the request
            // 2) The client library would detect that the 
            // failure response was of the new type by noticing 
            // that the response type was DBR_STRING and that 
            // the payload was not set to zeros.
            msgMinderV3.pushResp ( respHdr );
        }
        //
        // for now, we send them the error detail string in an
        // independent exception message for both the get and the
        // get callback situtions so that they will always
        // receive the details
        //
        msgMinderV3.errorResponse ( 
            m_reqMsg, m_cid, ECA_GETFAIL, failureContext );
        msgMinderV3.commit ();
    }
}

} // end of namespace Cir
} // end of namespace Svr
} // end of namespace ca
