
/*
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#ifndef caSvrCirRspThrReadh
#define caSvrCirRspThrReadh

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_caSvrCirRspThrReadh
#   undef epicsExportSharedSymbols
#endif

#ifdef epicsExportSharedSymbols_caSvrCirRspThrReadh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "caSvrCirRspThr.h"
#include "caHdrLargeArray.h" 

namespace ca {
namespace Svr {
namespace Cir {

class RspThr :: Read : 
    public Responder
{
public:
    Read ( const HdrLargeArray &, caResId cid );
    ~Read ();
    bool isGetCopy () const;
private:
    class ResponseAgentSuccess :
        public MutatorCatalog {
    public:
        ResponseAgentSuccess ( Read &, const Catalog & respParms );
        void assignDefault ();
        void assignScalarCatalog ( const Catalog & src );
    private:
        Read & m_read;
        const Catalog & m_respParm;
    };
    class ResponseAgentFailure :
        public MutatorCatalog {
    public:
        ResponseAgentFailure ( Read & );
        void assignDefault ();
        void assignScalarCatalog ( const Catalog & src );
    private:
        Read & m_read;
    };
    const HdrLargeArray m_reqMsg;
    const caResId m_cid;
    void m_responseNotify ( const PCat & );
    void m_show ( unsigned ) const;
    void m_failResponse ( const Catalog & );
    Read ( const Read & ); // disabled
    Read & operator = ( const Read & ); // disabled
};

inline bool RspThr :: Read :: isGetCopy () const
{
    return m_reqMsg.m_cmmd == CA_PROTO_READ;
}

} // end of namespace Cir
} // end of namespace Svr
} // end of namespace ca

#endif // caSvrCirRspThrReadh
