
/*
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */
#include "db_access.h"

#define epicsExportSharedSymbols
#include "caSvrCirMgr.h"
#include "caSvrCirRspThrSubscribe.h"
#include "FailureContext.h"
#include "protoDiagnostics.h"
#include "v3Proto.h"

namespace ca {
namespace Svr {
namespace Cir {

RspThr :: Subscribe :: 
    Subscribe ( const HdrLargeArray & reqMsg, caResId cid ) :
    m_reqMsg ( reqMsg ), m_cid ( m_cid )
{
}

#if  0
void RspThr :: Subscribe :: 
    initiate ( Guard & guard, PChan & pChan, 
                        PCat & reqModifiers )
{
    try {
        bool invalidType = ( m_reqMsg.m_dataType >= 
                m_respThread.m_mgr.m_dbrTypePtrs.size () );
        if ( invalidType || m_respThread.m_mgr.
            m_dbrTypePtrs[ m_reqMsg.m_dataType ].isNill () ) {
            throw std :: range_error ( 
                "specified dbr type isnt supported by this server" );
        }
#pragma message ("eventually the request modifier type needs to be a valid")
        PType pTypeNill;
        PReq reqPtr = pChan->createRequest ( Channel :: subscribe, 
            m_respThread.m_mgr.m_dbrTypePtrs[ m_reqMsg.m_dataType ], pTypeNill );
#pragma message ("eventually the request modifier type need to be passed here")
        //m_subscrReq.initiate ( guard, m_eventCallback, reqPtr, reqModifiers );
        m_subscrReq.initiate ( guard, m_eventCallback, reqPtr );
    }
    catch ( std :: exception & except ) {
        m_subscrReq.forceFailResponse ( 
            std :: string (            
                "RspThr :: Subscribe initiate: "
                "exception when creating "
                "\"subscribe\" request in service was \"" ) +
            except.what () + 
            std :: string ( "\"" ) );
    }
    catch ( ... ) {
        m_subscrReq.forceFailResponse ( 
            "RspThr :: Subscribe initiate: "
            "non-standard exception when creating "
            "\"subscribe\" request in service" );
    }
}
#endif

inline RspThr :: Subscribe :: ResponseAgentSuccess ::
    ResponseAgentSuccess ( Subscribe & subscribe, const Catalog & respParms ) :
    m_subscribe ( subscribe ), m_respParms ( respParms )
{
}

void RspThr :: Subscribe :: ResponseAgentSuccess ::
    assignDefault ()
{
    // ok, success indication wasnt found so we 
    // probe now for failure indication
    ResponseAgentFailure responseAgentFailure ( m_subscribe );
    get ( m_respParms, Responder :: failure, responseAgentFailure );
}

void RspThr :: Subscribe :: ResponseAgentSuccess ::
    assignScalarCatalog ( const Catalog & successParms )
{
    HdrLargeArray respHdr;
    respHdr.m_cmmd = m_subscribe.m_reqMsg.m_cmmd;
    respHdr.m_postsize = 0; // dbd
    respHdr.m_dataType = m_subscribe.m_reqMsg.m_dataType;
    respHdr.m_count = m_subscribe.m_reqMsg.m_count;
    respHdr.m_cid = ECA_NORMAL;
    respHdr.m_available = m_subscribe.m_reqMsg.m_available;
    try {
        RspThr * pRspThr = RspThr :: pAttachedRspThr ();
        MsgMinderV3 msgMinderV3 ( pRspThr->m_byteSeqOut,
                pRspThr->m_minorVersionNumber );
        msgMinderV3.pushResp ( respHdr, successParms );
        msgMinderV3.commit ();
    }
    catch ( exception & except ) {
        FailureContext failureContext ( except.what () );
        m_subscribe.m_failResponse ( failureContext );
    }
}

inline RspThr :: Subscribe :: ResponseAgentFailure ::
    ResponseAgentFailure ( Subscribe & subscribe ) :
    m_subscribe ( subscribe )
{
}

void RspThr :: Subscribe :: ResponseAgentFailure ::
    assignDefault ()
{
    FailureContext failureContext ( 
        "service subscription update payload was "
        "success/failure ambiguous" );
    m_subscribe.m_failResponse ( failureContext );
}

void RspThr :: Subscribe :: ResponseAgentFailure ::
    assignScalarCatalog ( const Catalog & failureParms )
{
    m_subscribe.m_failResponse ( failureParms );
}

void RspThr :: Subscribe :: 
    m_responseNotify ( const PCat & pRespParms )
{
    if ( pRespParms.isntNill () ) {
        PCat :: Locker pRespParmsLocked = pRespParms;
        ResponseAgentSuccess 
            responseAgentSuccess ( *this, *pRespParmsLocked ); 
        get ( *pRespParmsLocked, success, responseAgentSuccess );
    }
    else {
        FailureContext failureContext ( 
            "received service subscription update notification, "
            "but success/failure indication wasnt in "
            "response (empty) payload" );
        m_failResponse ( failureContext );
    }
}

void RspThr :: Subscribe :: 
    m_show ( unsigned level ) const
{
    printf ( "RspThr::Subscribe\n" );
    if ( level > 0 ) {
        dumpMsg ( m_reqMsg );
        printf ( "RspThr::Subscribe: m_cid = %u\n", m_cid );
    }
}

void RspThr :: Subscribe :: 
    m_failResponse ( const Catalog & resp )
{
	ca_uint32_t payloadSize = dbr_size_n ( 
        m_reqMsg.m_dataType, m_reqMsg.m_count );
    HdrLargeArray respHdr;
    respHdr.m_cmmd = m_reqMsg.m_cmmd;
    respHdr.m_postsize = payloadSize;
    respHdr.m_dataType = m_reqMsg.m_dataType;
    respHdr.m_count = m_reqMsg.m_count;
    respHdr.m_cid = ECA_GETFAIL;
    respHdr.m_available = m_reqMsg.m_available;
#pragma message ( "Maybe if protocol level exceeds NNN" )
#pragma message ( "then place diag str in payload here" )
    RspThr * pRspThr = RspThr :: pAttachedRspThr ();
    MsgMinderV3 msgMinderV3 ( pRspThr->m_byteSeqOut, 
                        pRspThr->m_minorVersionNumber );
    msgMinderV3.pushResp ( respHdr );
    msgMinderV3.commit ();
}

} // end of namespace Cir
} // end of namespace Svr
} // end of namespace ca
