
/*
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#define epicsExportSharedSymbols
#include "caSvrCirRspThrWrite.h"
#include "FailureContext.h"
#include "protoDiagnostics.h"
#include "v3Proto.h"

namespace ca {
namespace Svr {
namespace Cir {

RspThr :: Write ::
    Write ( const HdrLargeArray & reqMsg, caResId cid ) :
    m_reqMsg ( reqMsg ), 
    m_cid ( cid )
{
}

RspThr :: Write ::
    ~Write ()
{
}

#if 0
void RspThr :: Write :: 
    initiate ( Guard & guard, PChan & pChan, PCat & pVal )
{
    try {
        bool invalidType = ( m_reqMsg.m_dataType >= 
                m_respThread.m_mgr.m_dbrTypePtrs.size () );
        if ( invalidType || m_respThread.m_mgr.
            m_dbrTypePtrs[ m_reqMsg.m_dataType ].isNill () ) {
            throw std :: range_error ( 
                "specified dbr type isnt supported by this server" );
        }
        PType pTypeNill;
        PReq pReq = pChan->createRequest ( Channel :: write, 
                        m_respThread.m_mgr.m_dbrTypePtrs[ m_reqMsg.m_dataType ], 
                        pTypeNill );
        m_onceReq.initiate ( guard, m_eventCallback, pReq, pVal );
    }
    catch ( std :: exception & except ) {
        m_onceReq.forceFailResponse ( 
            std :: string (            
                "RspThr :: Write initiate: "
                "exception when creating "
                "\"write\" request in service was \"" ) +
            except.what () + 
            std :: string ( "\"" ) );
    }
    catch ( ... ) {
        m_onceReq.forceFailResponse ( 
            "RspThr :: Write initiate: "
            "non-standard exception when creating "
            "\"write\" request in service" );
    }
}
#endif

inline RspThr :: Write :: ResponseAgentSuccess ::
    ResponseAgentSuccess ( Write & write, const Catalog & respParms ) :
    m_write ( write ), m_respParms ( respParms )
{
}

void RspThr :: Write :: ResponseAgentSuccess ::
    assignDefault ()
{
    // ok, success indication wasnt found so we 
    // probe now for failure indication
    ResponseAgentFailure responseAgentFailure ( m_write );
    get ( m_respParms, Responder :: failure, responseAgentFailure );
}

void RspThr :: Write :: ResponseAgentSuccess ::
    assignScalarCatalog ( const Catalog & successParms )
{
    // a CA put callback has a protocol response, but an
    // ordinary CA put has no response
    if ( m_write.m_reqMsg.m_cmmd == CA_PROTO_WRITE_NOTIFY ) {
        HdrLargeArray respHdr;
        respHdr.m_cmmd = m_write.m_reqMsg.m_cmmd;
        respHdr.m_postsize = 0;
        respHdr.m_dataType = m_write.m_reqMsg.m_dataType;
        respHdr.m_count = m_write.m_reqMsg.m_count;
        respHdr.m_cid = ECA_NORMAL;
        respHdr.m_available = m_write.m_reqMsg.m_available;
        RspThr * pRspThr = RspThr :: pAttachedRspThr ();
        MsgMinderV3 msgMinderV3 ( pRspThr->m_byteSeqOut, 
                pRspThr->m_minorVersionNumber );
        msgMinderV3.pushResp ( respHdr );
        msgMinderV3.commit ();
    }
}

inline RspThr :: Write :: ResponseAgentFailure ::
    ResponseAgentFailure ( Write & write ) :
    m_write ( write )
{
}

void RspThr :: Write :: ResponseAgentFailure ::
    assignDefault ()
{
    FailureContext failureContext ( 
        "service write response payload was "
        "success/failure ambiguous" );
    m_write.m_failResponse ( failureContext );
}

void RspThr :: Write :: ResponseAgentFailure ::
    assignScalarCatalog ( const Catalog & failureParms )
{
    m_write.m_failResponse ( failureParms );
}

void RspThr :: Write :: 
    m_responseNotify ( const PCat & pRespParms )
{
    if ( pRespParms.isntNill () ) {
        PCat :: Locker pRespParmsLocked = pRespParms;
        ResponseAgentSuccess 
            responseAgentSuccess ( *this, *pRespParmsLocked ); 
        get ( *pRespParmsLocked, success, responseAgentSuccess );
    }
    else {
        FailureContext failureContext ( 
            "received service write response notification, "
            "but success/failure indication wasnt in "
            "response (empty) payload" );
        m_failResponse ( failureContext );
    }
}

void RspThr :: Write :: 
    m_show ( unsigned level ) const
{
    printf ( "RspThr::Write\n" );
    if ( level > 0 ) {
        dumpMsg ( m_reqMsg );
        printf ( "RspThr::Write: m_cid = %u\n", m_cid );
    }
}

void RspThr :: Write :: 
    m_failResponse ( const Catalog & resp )
{
    RspThr * pRspThr = RspThr :: pAttachedRspThr ();
    MsgMinderV3 msgMinderV3 ( pRspThr->m_byteSeqOut,
                    pRspThr->m_minorVersionNumber );
    if ( m_reqMsg.m_cmmd == CA_PROTO_WRITE_NOTIFY ) {
        HdrLargeArray respHdr;
        respHdr.m_cmmd = m_reqMsg.m_cmmd;
        respHdr.m_postsize = 0;
        respHdr.m_dataType = m_reqMsg.m_dataType;
        respHdr.m_count = m_reqMsg.m_count;
        respHdr.m_cid = ECA_PUTFAIL;
        respHdr.m_available = m_reqMsg.m_available;
        msgMinderV3.pushResp ( respHdr );
    }
    //
    // for now, we send them the error detail string in an
    // independent exception message for both the put and the
    // put callback situtions so that they will always
    // receive the details
    //
    msgMinderV3.errorResponse ( m_reqMsg, m_cid, ECA_PUTFAIL, resp );
    msgMinderV3.commit ();
}

} // end of namespace Cir
} // end of namespace Svr
} // end of namespace ca
