
/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

#ifndef caSvrDGMgrh
#define caSvrDGMgrh

#include <list>
#include <memory>

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_caSvrDGMgrh
#   undef epicsExportSharedSymbols
#endif

#include "epicsTimer.h"
#include "epicsMutex.h"
#include "caProto.h"
#include "epicsTime.h"
#include "tsFreeList.h"
#include "EnclLnk.h"

#ifdef epicsExportSharedSymbols_caSvrDGMgrh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "DGParm.h"
#include "caSvrEvQue.h"
#include "caIntfIO.h"

namespace ca {

class IterIn;
class Server;
struct Request;
typedef Ptr < Request > PReq;
struct Responder;
typedef Ptr < Responder > PResp;

namespace Svr {
namespace DG {

class Mgr;
typedef Ptr < Mgr > PMgr;
class RcvThr;
typedef Ptr < RcvThr > PRcvThr;
class RspThr;
typedef Ptr < RspThr > PRspThr;
typedef std :: auto_ptr < DatagramIn > InPtr;

struct ReqNotify : public tsDLNode < ReqNotify > {
    virtual void show ( unsigned level ) const = 0;
    virtual void serverDestroyNotify () = 0;
};

class Mgr {
public:
    static PMgr factory ( const PService & pService, 
                        const PCat & pNameResChanDescr,
                        epicsTimerQueue & timerQueue, 
                        std :: auto_ptr < DatagramAgent >, 
                        const Ptr < VecAllocator > & );
    Mgr ( const PService & pService, const PCat & pNameResChanDescr,
            epicsTimerQueue & timerQueue, std :: auto_ptr < DatagramAgent >,
            const Ptr < VecAllocator > & );
    ~Mgr ();
    void generateBeaconAnomaly ();
    struct InitiateParms {
        InitiateParms ( const PResp &, const PCat & );
        InitiateParms ( PResp :: Hdl &, PCat :: Hdl & );
        PResp pResp;
        PCat pReqParms;
    };
    void processEvents ();
    InitiateParms createNameResResp ( const PReq &, const DGParm &, caResId cid, 
            bool failRespRequired, const StringReader & name );
	void show ( unsigned level ) const;
#if 0
    struct IO {
        virtual void sendBeacon ( char & msg, size_t length, 
            ca_uint16_t & portField, ca_uint32_t & addrField ) = 0;
        virtual NetAddr serverAddress () const = 0;
    };
#endif
private:
    class M_NewEventSignal :
        public EvQue :: NewEventSignal {
    public:
        void signal ();
        typedef const epics :: EnclLnk < Mgr, 
                                    M_NewEventSignal > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    class M_Handle : 
        public HandleIntf < Mgr > {
    private:
		TargetMutexPair m_targetMutexPair ();
		HandleIntf < Mgr > & m_clone ();
		void m_release ();
        typedef const epics :: EnclLnk < Mgr, 
                                    M_Handle > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    class Echo;
    class NameRes;
    class BeaconTimer;
    class BeaconAnomGov;
    // order is significant here
    M_NewEventSignal m_newEventSignal;
    M_Handle m_handle;
    mutable Mutex m_mutex;
    mutable Mutex m_noopMutex; 
    EvQue m_eventQueue;
    PRspThr m_pRspThr;
    std :: vector < PRcvThr > m_recvThreadPtrVector;
    tsFreeList < Echo > m_echoFreeList;
    tsFreeList < NameRes > m_nameResFreeList;
    tsDLList < ReqNotify > m_reqList;
    std :: auto_ptr < BeaconTimer > m_pBeaconTimer;
    std :: auto_ptr < BeaconAnomGov > m_pBeaconAnomGov;
    RefCtr m_refCtr;
#if 0
	casStreamWriteIO::flushCondition xSend ( 
        char *pBufIn, size_t nBytesAvailableToSend, 
		size_t nBytesNeedToBeSent, size_t &nBytesSent );
	casStreamReadIO::fillCondition xRecv ( 
        char * pBufIn, size_t nBytesToRecv, 
        fillParameter parm, size_t & nByesRecv );
	virtual casStreamWriteIO::flushCondition osdSend ( 
            const char * pBuf, size_t nBytesReq, 
            const NetAddr & addr ) = 0;
	virtual casStreamReadIO::fillCondition osdRecv ( 
        char *pBuf, size_t nBytesReq,
	    fillParameter parm, size_t &nBytesActual, 
        NetAddr & addr ) = 0;
    ca_uint32_t m_datagramSequenceNumber () const;
    struct cadg {
        NetAddr cadg_addr; // invalid address indicates pad
        size_t cadg_nBytes;
    };
#endif
	Mgr ( const Mgr & ); // disabled 
	Mgr & operator = ( const Mgr & ); // disabled
};

class Mgr :: Echo {
public:
    static PResp factory ( Mgr &, const PResp & );
    Echo ( Mgr &, const PResp & );
    ~Echo ();
    void show ( unsigned level ) const;
private:
    class M_Responder :         
        public Responder {
    private:
        void m_responseNotify ( const PCat & );
        void m_show ( unsigned ) const;
    };
    class M_RespHandle : 
        public HandleIntf < Responder > {
    private:
		TargetMutexPair m_targetMutexPair ();
		HandleIntf < Responder > & m_clone ();
		void m_release ();
    };
    class M_ReqNotify :
        public ReqNotify {
    public:
        void show ( unsigned level ) const;
        void serverDestroyNotify ();
    };
    M_RespHandle m_respHandle;
    M_Responder m_responder;
    M_ReqNotify m_reqNotify;
    // This is a reference to the RspThr :: Echo
    // which is posted to the event queue. We 
    // preallocate this object so that we dont
    // end up being unable to complete the IO 
    // request due to lack of memory.
    PResp m_pResp;
    Mgr & m_mgr;
    RefCtr m_refCtr;
    void m_ioCancel ( const char *pCtx );
    Echo ( const Echo & ); // disabled
    Echo & operator = ( const Echo & ); // disabled
};

inline Mgr :: InitiateParms ::
    InitiateParms ( const PResp & pRespIn, 
                    const PCat & pReqParmsIn ) :
    pResp ( pRespIn ), pReqParms ( pReqParmsIn )
{
}

inline Mgr :: InitiateParms ::
    InitiateParms ( PResp :: Hdl & pRespHdl, 
                    PCat :: Hdl & pCatHdl ) :
    pResp ( pRespHdl ), pReqParms ( pCatHdl )
{
}

inline void Mgr :: processEvents ()
{
    m_eventQueue.processEvents ();
}

} // end of namespace DG
} // end of namespace Svr
} // end of namespace ca

#endif // ifndef caSvrDGMgrh

