
/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

#ifndef caSvrDGMgrBeaconTimerh
#define caSvrDGMgrBeaconTimerh

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_caSvrDGMgrBeaconTimerh
#   undef epicsExportSharedSymbols
#endif

#include "EnclLnk.h"

#ifdef epicsExportSharedSymbols_caSvrDGMgrBeaconTimerh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "caSvrDGMgr.h"
#include "caSvrDGRspThrBeacon.h"

namespace ca {
namespace Svr {
namespace DG {

class Mgr :: BeaconTimer {
public:
    BeaconTimer ( Mgr &, epicsTimerQueue & );
    ~BeaconTimer ();
    void start ();
    void generateBeaconAnomaly ();
    void show ( unsigned level ) const;
private:
    class M_TimerNotify :
        public epicsTimerNotify {
    public:
        expireStatus expire ( const epicsTime & currentTime );
        void show ( unsigned ) const;
        typedef const epics :: EnclLnk < BeaconTimer, 
                                    M_TimerNotify > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    class M_RespHandle : 
        public HandleIntf < Responder > {
    private:
		TargetMutexPair m_targetMutexPair ();
		HandleIntf < Responder > & m_clone ();
		void m_release ();
        typedef const epics :: EnclLnk < BeaconTimer, 
                                    M_RespHandle > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    RspThr :: Beacon m_rspThrBeacon;
    M_RespHandle m_respHandle;
    M_TimerNotify m_timerNotify;
    double m_beaconPeriod;
    double m_maxBeaconInterval;
    epicsTimer & m_timer;
    Mgr & m_mgr;
    BeaconTimer ( const BeaconTimer & ); // disabled
    BeaconTimer & operator = ( const BeaconTimer & ); // disabled
};

} // end of namespace DG
} // end of namespace Svr
} // end of namespace ca

#endif // ifndef caSvrDGMgrBeaconTimerh
