/*
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */

#include "osiSock.h"
#include "errlog.h"

#define epicsExportSharedSymbols
#include "caSvrDGMgrNameRes.h"

namespace ca {
namespace Svr {
namespace DG {

Mgr :: InitiateParms
    Mgr :: NameRes :: factory ( 
        Mgr & mgr, const PReq & pReq, const DGParm & dgParm, 
        caResId cid, bool failRespRequired, 
        const StringReader & name )
{
    NameRes * pNameRes = new ( mgr.m_nameResFreeList ) 
                    NameRes ( mgr, pReq, dgParm, 
                        cid, failRespRequired, name );
    return InitiateParms ( pNameRes->m_servRespHandle, 
                            pNameRes->m_reqParmsHandle );
}

Mgr :: NameRes :: 
    NameRes ( Mgr & mgr, const PReq & pReq, 
        const DGParm & dgParm, caResId cid, 
        bool failRespRequired, const StringReader & name ) :
    m_rspThrNameResp ( dgParm, cid, failRespRequired ),
    m_channelDescription ( name ), m_pReq ( pReq ),
    m_mgr ( mgr ), m_servRespNotifyOccurred ( false )
{
    m_mgr.m_reqList.add ( this->m_reqNotify );
}

#if 0
    try {
        PReq reqPtr =
            m_respCtx.m_pNameResolver->createRequest ( 
                    Channel :: nameResolve, 
                    m_respCtx.m_pChannelDescriptionType,
                    m_respCtx.m_pChannelAddressType );
        m_onceReq.initiate ( guard, m_eventCallback, reqPtr );
    }
    catch ( std :: exception & except ) {
        m_onceReq.forceFailResponse ( 
            std :: string (            
                "RespCtx :: NameRes ctor: "
                "exception when creating "
                "\"nameResolve\" request in Service was \"" ) +
            except.what () + 
            std :: string ( "\"" ) );
    }
    catch ( ... ) {
        m_onceReq.forceFailResponse ( 
            "RespCtx :: NameRes ctor: "
            "non-standard exception when creating "
            "\"nameResolve\" request in Service" );
    }
#endif

void Mgr :: NameRes ::
    show ( unsigned level ) const
{
    printf ( "Mgr :: NameRes\n" );
    if ( level > 0 ) {
        m_refCtr.show ( level - 1 );
        m_pReq->show ( level - 1 );
        printf ( "m_servRespNotifyOccurred = %c\n", 
            m_servRespNotifyOccurred ? 'T' : 'F' );
    }
}

///////////////////////////////////////////////////////
// M_EQRespHandle event queue m_rspThrNameResp handle
///////////////////////////////////////////////////////
Mgr :: NameRes :: M_EQRespHandle :: M_EnclLnk
    Mgr :: NameRes :: M_EQRespHandle ::   
        m_enclLnk ( & Mgr :: NameRes :: m_eqRespHandle );
Mgr :: NameRes :: M_EQRespHandle :: TargetMutexPair 
    Mgr :: NameRes :: M_EQRespHandle :: m_targetMutexPair ()
{
    NameRes & nameRes ( m_enclLnk.enclosure ( *this ) );
    return TargetMutexPair ( nameRes.m_rspThrNameResp, 
                        nameRes.m_mgr.m_noopMutex );
}
HandleIntf < Responder > & 
    Mgr :: NameRes :: M_EQRespHandle :: m_clone ()
{
    NameRes & nameRes ( m_enclLnk.enclosure ( *this ) );
    nameRes.m_refCtr.reference ();
    return *this;
}
void Mgr :: NameRes :: M_EQRespHandle :: m_release ()
{
    NameRes & nameRes ( m_enclLnk.enclosure ( *this ) );
    nameRes.m_sharedRelease ();
    // no code below here due to above delete
}

///////////////////////////////////////////////////////
// M_ServRespHandle 
///////////////////////////////////////////////////////
Mgr :: NameRes :: M_ServRespHandle :: M_EnclLnk
    Mgr :: NameRes :: M_ServRespHandle ::   
        m_enclLnk ( & Mgr :: NameRes :: m_servRespHandle );
Mgr :: NameRes :: M_ServRespHandle :: TargetMutexPair 
    Mgr :: NameRes :: M_ServRespHandle :: m_targetMutexPair ()
{
    NameRes & nameRes ( m_enclLnk.enclosure ( *this ) );
    return TargetMutexPair ( nameRes.m_responder, 
                        nameRes.m_mgr.m_mutex );
}
HandleIntf < Responder > & 
    Mgr :: NameRes :: M_ServRespHandle :: m_clone ()
{
    NameRes & nameRes ( m_enclLnk.enclosure ( *this ) );
    nameRes.m_refCtr.reference ();
    return *this;
}
void Mgr :: NameRes :: M_ServRespHandle :: m_release ()
{
    NameRes & nameRes ( m_enclLnk.enclosure ( *this ) );
    nameRes.m_sharedRelease ();
    // no code below here due to above delete
}

//////////////////////////////////////////////////////////
// request parameters m_channelDescription handle
/////////////////////////////////////////////////////////
Mgr :: NameRes :: M_ReqParmsHandle :: M_EnclLnk
    Mgr :: NameRes :: M_ReqParmsHandle ::   
        m_enclLnk ( & Mgr :: NameRes :: m_reqParmsHandle );
Mgr :: NameRes :: M_ReqParmsHandle :: TargetMutexPair 
    Mgr :: NameRes :: M_ReqParmsHandle :: m_targetMutexPair ()
{
    NameRes & nameRes ( m_enclLnk.enclosure ( *this ) );
    return TargetMutexPair ( nameRes.m_channelDescription, 
                            nameRes.m_mgr.m_noopMutex );
}
HandleIntf < const Catalog > & 
    Mgr :: NameRes :: M_ReqParmsHandle :: m_clone ()
{
    NameRes & nameRes ( m_enclLnk.enclosure ( *this ) );
    nameRes.m_refCtr.reference ();
    return *this;
}
void Mgr :: NameRes :: M_ReqParmsHandle :: m_release ()
{
    NameRes & nameRes ( m_enclLnk.enclosure ( *this ) );
    nameRes.m_sharedRelease ();
    // no code below here due to above delete
}

//////////////////////////////////////////////////////////////
// m_sharedRelease (common to all handles)
//////////////////////////////////////////////////////////////
void Mgr :: NameRes :: m_sharedRelease ()
{
    if ( m_refCtr.unreference () ) {
        Guard guard ( m_mgr.m_mutex );
        // look for service destroys before receiving
        // completion notify
        if ( ! m_servRespNotifyOccurred ) {
            m_mgr.m_reqList.remove ( m_reqNotify );
        }
        m_mgr.m_nameResFreeList.deleteProduct ( *this );
        // no code below here due to above delete
    }
    // no code below here due to above delete
}

//////////////////////////////////////////////////////////////
// responder callback from service
//////////////////////////////////////////////////////////////
Mgr :: NameRes :: M_Responder :: M_EnclLnk
    Mgr :: NameRes :: M_Responder ::   
        m_enclLnk ( & Mgr :: NameRes :: m_responder );
void Mgr :: NameRes :: M_Responder ::
    m_responseNotify ( const PCat & pCat )
{
    NameRes & nameRes ( m_enclLnk.enclosure ( *this ) );
    if ( ! nameRes.m_servRespNotifyOccurred ) {
        nameRes.m_mgr.m_reqList.remove ( nameRes.m_reqNotify );
        nameRes.m_mgr.m_eventQueue.
            post ( PResp ( nameRes.m_eqRespHandle ), pCat );
        nameRes.m_servRespNotifyOccurred = true;
    }
}
void Mgr :: NameRes :: M_Responder ::
    m_show ( unsigned level ) const
{
    const NameRes & nameRes ( m_enclLnk.enclosure ( *this ) );
    nameRes.show ( level );
}

////////////////////////////////////////////////////////
// server M_ReqNotify
////////////////////////////////////////////////////////
Mgr :: NameRes :: M_ReqNotify :: M_EnclLnk
    Mgr :: NameRes :: M_ReqNotify ::   
        m_enclLnk ( & Mgr :: NameRes :: m_reqNotify );
void Mgr :: NameRes :: M_ReqNotify ::
    serverDestroyNotify ()
{
    NameRes & nameRes ( m_enclLnk.enclosure ( *this ) );
    if ( ! nameRes.m_servRespNotifyOccurred ) {
        nameRes.m_mgr.m_reqList.remove ( nameRes.m_reqNotify );
        nameRes.m_servRespNotifyOccurred = true;
        {
            // dont reverse the lock hierarchy
            epics :: Mutex :: Unlocker 
                unlocker ( nameRes.m_mgr.m_mutex );
            nameRes.m_pReq->cancel ();
        }
    }
}
void Mgr :: NameRes :: M_ReqNotify ::
    show ( unsigned level ) const
{
    const NameRes & nameRes ( m_enclLnk.enclosure ( *this ) );
    nameRes.show ( level );
}

} // end of namespace DG
} // end of namespace Svr
} // end of namespace ca

