
/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

#ifndef caSvrDGMgrNameResh
#define caSvrDGMgrNameResh

#include <memory>
#include <vector>

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_caSvrDGMgrNameResh
#   undef epicsExportSharedSymbols
#endif

#include "EnclLnk.h"

#ifdef epicsExportSharedSymbols_caSvrDGMgrNameResh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "caSvrDGMgr.h"
#include "caChannelDescription.h"
#include "caSvrDGRspThrNameRes.h"

namespace ca {
namespace Svr {
namespace DG {

class Mgr :: NameRes {
public:
    static InitiateParms 
        factory ( 
            Mgr &, const PReq &, const DGParm &, caResId cid, 
            bool failRespRequired, const StringReader & name );
    NameRes ( Mgr &, const PReq &, const DGParm &, caResId cid, 
        bool failRespRequired, const StringReader & name );
    void show ( unsigned level ) const;
private:
    // event queue response handle
    class M_EQRespHandle : 
        public HandleIntf < Responder > {
    private:
		TargetMutexPair m_targetMutexPair ();
		HandleIntf < Responder > & m_clone ();
		void m_release ();
        typedef const epics :: EnclLnk < NameRes, 
                                    M_EQRespHandle > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    // service response handle
    class M_ServRespHandle : 
        public HandleIntf < Responder > {
    private:
		TargetMutexPair m_targetMutexPair ();
		HandleIntf < Responder > & m_clone ();
		void m_release ();
        typedef const epics :: EnclLnk < NameRes, 
                                    M_ServRespHandle > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    // request parameters handle
    class M_ReqParmsHandle : 
        public HandleIntf < const Catalog > {
    private:
		TargetMutexPair m_targetMutexPair ();
		HandleIntf < const Catalog > & m_clone ();
		void m_release ();
        typedef const epics :: EnclLnk < NameRes, 
                                    M_ReqParmsHandle > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
#if 0
    class M_ReqNotifyHandle : 
        public HandleIntf < const Catalog > {
    public:
        typedef M_ReqNotifyHandle NameRes :: * PMemSelf;
        static PMemSelf pMemSelf ();
    private:
		TargetMutexPair m_targetMutexPair ();
		HandleIntf < const Catalog > & m_clone ();
		void m_release ();
    };
#endif
    // responder callback from service
    class M_Responder :         
        public Responder {
    private:
        void m_responseNotify ( const PCat & );
        void m_show ( unsigned ) const;
        typedef const epics :: EnclLnk < NameRes, 
                                    M_Responder > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    // IO request manipulation from Mgr
    class M_ReqNotify :
        public ReqNotify {
    public:
        void show ( unsigned level ) const;
        void serverDestroyNotify ();
    private:
        typedef const epics :: EnclLnk < NameRes, 
                                    M_ReqNotify > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    RspThr :: NameRes m_rspThrNameResp;
    M_ReqParmsHandle m_reqParmsHandle;
    M_ServRespHandle m_servRespHandle;
    M_EQRespHandle m_eqRespHandle;
    M_Responder m_responder;
    M_ReqNotify m_reqNotify;
    const ChannelDescription m_channelDescription;
    // We keep a reference to the request until 
    // this object is destroyed. Its anti-social 
    // to destroy the request when we are in the 
    // service's call to the response callback.
    PReq m_pReq;
    Mgr & m_mgr;
    RefCtr m_refCtr;
    bool m_servRespNotifyOccurred;
    void m_ioCancel ( const char *pCtx );
    void m_sharedRelease ();
    NameRes ( const NameRes & ); // disabled
    NameRes & operator = ( const NameRes & ); // disabled
};

} // end of namespace DG
} // end of namespace Svr
} // end of namespace ca

#endif // ifndef caSvrDGMgrNameResh


