
/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

#ifndef caSvrDGRcvThrh
#define caSvrDGRcvThrh

#include <memory>
#include <vector>

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_caSvrDGRcvThrh
#   undef epicsExportSharedSymbols
#endif

#include "EnclLnk.h"

#ifdef epicsExportSharedSymbols_caSvrDGRcvThrh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "caHdrLargeArray.h"
#include "caSvrDGMgr.h"

namespace ca {
namespace Svr {
namespace DG {

typedef std :: auto_ptr < DatagramIn > InPtr;

class RcvThr {
public:
    static Ptr < RcvThr > factory ( const PService & pService, 
        const PCat & pNameResChanDescr, const PMgr &, InPtr & );
    RcvThr ( const PService & pService, const PCat & pNameResChanDescr, 
            const PMgr &, InPtr & );
    ~RcvThr ();
    void start ();
    void show ( unsigned level ) const;
private:
    class DatagramInReader;
    class RootChannelDescr;
    class M_ThreadIntf : 
        public epicsThreadRunable {
    public:
        void run ();
    private:
        typedef const epics :: EnclLnk < RcvThr, 
                                    M_ThreadIntf > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    class M_Handle : 
        public HandleIntf < RcvThr > {
    private:
        TargetMutexPair m_targetMutexPair ();
        HandleIntf < RcvThr > & m_clone ();
        void m_release ();
        typedef const epics :: EnclLnk < RcvThr, 
                                    M_Handle > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    typedef void ( RcvThr :: * MsgHandler ) ( IterIn & );
    DGParm m_curDGParm;
    HdrLargeArray m_curMsg;
	epicsThread m_thread;
    mutable epics :: Mutex m_mutex;
    epicsTime m_lastRecvTS;
    M_ThreadIntf m_threadIntf;
    M_Handle m_handle;
    InPtr m_pIn;
    PChan m_pChanNameRes;
    PService m_pService;
    PMgr m_pMgr;
    PType m_pChannelAddressType;
    PType m_pChannelDescriptionType;
    RefCtr m_refCtr;
    bool m_threadShutdownCmd;
    static MsgHandler const m_msgHandlers[CA_PROTO_LAST_CMMD+1u];
    void m_processDG ( IterIn & );
    // one function for each datagram request type
    void m_searchAction ( IterIn & );
    void m_echoAction ( IterIn & );
    void m_versionAction ( IterIn & );
    void m_uknownMessageAction ( IterIn & );
    void m_badMessageAction ( const char * pFormat, ... );
    void m_badMessageAction ( const char * pFormat, va_list & );
	RcvThr ( const RcvThr & ); // disabled 
	RcvThr & operator = ( const RcvThr & ); // disabled
};

class RcvThr :: DatagramInReader : 
    public Reader {
public:
    DatagramInReader ( DatagramIn & );
    Vec < const epicsUInt8 > read ( size_t nBytesMax );
    void rewind (); // datagram streams dont rewind
private:
    DatagramIn & m_datagramIn;
};

inline RcvThr :: DatagramInReader :: 
    DatagramInReader ( DatagramIn & datagramIn ) :
    m_datagramIn ( datagramIn )
{
}

} // end of namespace DG
} // end of namespace Svr
} // end of namespace ca

#endif // ifndef caSvrDGRcvThrh
