/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

#ifndef caSvrDGRspThrh
#define caSvrDGRspThrh

#include <memory>

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_caSvrDGRspThrh
#   undef epicsExportSharedSymbols
#endif

#include "epicsTimer.h"
#include "epicsMutex.h"
#include "tsFreeList.h"
#include "EnclLnk.h"

#ifdef epicsExportSharedSymbols_caSvrDGRspThrh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "casdef2.h"
#include "caRefCtr.h"

namespace ca {

struct DatagramOut;
struct VecAllocator;
struct DatagramAgent;

namespace Svr {
namespace DG {

class Mgr;
typedef Ptr < Mgr > PMgr;

class RspThr {
public:
    class Echo;
    class Beacon;
    class NameRes;
    static Ptr < class RspThr > 
        factory ( const PMgr &, epicsTimerQueue &, 
            std :: auto_ptr < DatagramOut >,
            const Ptr < VecAllocator > & );
    RspThr ( const PMgr &, epicsTimerQueue &, 
            std :: auto_ptr < DatagramOut >,
            const Ptr < VecAllocator > & );
    ~RspThr ();
    void eventSignal ();
    void show ( unsigned level ) const;
    static RspThr * pAttachedRspThr ();
private:
    class DGCoalescer;
    class DatagramOutWriter;
    class M_Handle : 
        public HandleIntf < RspThr > {
    private:
		TargetMutexPair m_targetMutexPair ();
		HandleIntf < RspThr > & m_clone ();
		void m_release ();
        typedef const epics :: EnclLnk < RspThr, 
                                    M_Handle > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
    class M_EntryPt : 
        public epicsThreadRunable {
    public:
        void run ();
		void show ( unsigned level ) const;
    private:
        typedef const epics :: EnclLnk < RspThr, 
                                    M_EntryPt > M_EnclLnk;
        static M_EnclLnk m_enclLnk;
    };
#if 0
    class M_NewEventSignal : 
        public EvQue :: NewEventSignal {
    public:
        void signal ();
    };
#endif
    epicsThread m_thread;
    epicsTime m_lastSendTS;
    M_Handle m_handle;
    M_EntryPt m_entryPt;
    std :: auto_ptr < DGCoalescer > m_pDGCoalescer;
    std :: auto_ptr < DatagramOut > m_pDatagramOut;
    tsFreeList < NameRes > m_nameResReqFreeList;
    tsFreeList < Echo > m_echoResReqFreeList;
    PMgr m_pMgr;
    mutable epics :: Mutex m_mutex; 
    epicsEvent m_event;
    RefCtr m_refCtr;
    bool m_threadShutdownCmd;
#if 0
    void m_searchResponse ( 
        Guard &, const HdrLargeArray &, const caNetAddr & );
    void m_searchResponse (
        Guard &, const caNetAddr & outAddr, 
        const HdrLargeArray &, const caNetAddr & pvLocation,
        ca_uint16_t protocolRevision, ca_uint32_t sequenceNumber );
    void m_sendVersion ( Guard & );
#endif
	RspThr ( const RspThr & ); // disabled 
	RspThr & operator = ( const RspThr & ); // disabled
};

inline void RspThr :: eventSignal ()
{
    m_event.signal ();
}

} // end of namespace DG
} // end of namespace Svr
} // end of namespace ca

#endif // ifndef caSvrDGRspThrh
