/*
 *      caSvrDGRspThrDGCoalescer.cpp,v 1.6 2009/11/24 01:34:25 jhill Exp
 *
 *      Author  Jeffrey O. Hill
 *              johill@lanl.gov
 *              505 665 1831
 */

#define epicsExportSharedSymbols
#include "caSvrDGRspThrDGCoalescer.h"
#include "caSvrDGRspThrDatagramOutWriter.h"
#include "caHdrLargeArray.h"

namespace ca {
namespace Svr {
namespace DG {

RspThr :: DGCoalescer :: 
    DGCoalescer ( RspThr & rspThr, const Ptr < VecAllocator > & pVecAllocator ) :
    m_byteSeqOut ( pVecAllocator ), m_rspThr ( rspThr )
{
}

RspThr :: DGCoalescer :: 
    ~DGCoalescer ()
{
}

void RspThr :: DGCoalescer :: 
    m_flushOnDemand ( DGParm & dgParm, size_t msgSize )
{
    static const size_t maxSend = 
            MAX_UDP_SEND - sizeof ( ca_hdr );
    if ( msgSize > maxSend ) {
        throw std :: runtime_error ( 
            "CA message size exceeds "
            "maximum datagram size" );
    }
    if ( m_currentDGParm != dgParm ) {
        m_currentDGParm = dgParm;
        flush ();
    }
    else {
        const size_t bytesBuffered = m_byteSeqOut.nBytesBuffered ();
        if ( bytesBuffered > maxSend - msgSize ) {
            flush ();
        }
    }
}

void RspThr :: DGCoalescer :: 
    flush ()
{
    const size_t bytesBuffered = m_byteSeqOut.nBytesBuffered ();
    if ( bytesBuffered > sizeof ( ca_hdr ) ) {
        m_rspThr.m_pDatagramOut->beginFrame ( m_currentDGParm.srcAddr () );
        DatagramOutWriter datagramOutWriter ( *m_rspThr.m_pDatagramOut );
        m_byteSeqOut.flush ( datagramOutWriter );
        m_rspThr.m_pDatagramOut->endFrame ();
        m_pushDGHeader ();
    }
}

void RspThr :: DatagramOutWriter ::
    write ( const Vec < const epicsUInt8 > & pBuf )
{
    m_datagramOut.writeFrameSeg ( pBuf );
}

void RspThr :: DGCoalescer :: 
    pushHeader ( DGParm & dgParm, const caHdr & hdr )
{
    assert ( hdr.m_postsize == 0 );
    const size_t msgSize = sizeof ( hdr );
    m_flushOnDemand ( dgParm, msgSize );
    Iter iter ( m_byteSeqOut );
    m_pushHeader ( iter, hdr );
    iter.commit ();
}

void RspThr :: DGCoalescer :: 
    pushHeaderAndRev ( DGParm & dgParm, const caHdr & hdr )
{
    const size_t payloadSize = sizeof ( ca_uint16_t );
    assert ( hdr.m_postsize == payloadSize );
    const size_t msgSize = sizeof ( hdr ) + payloadSize;
    m_flushOnDemand ( dgParm, msgSize );
    Iter iter ( m_byteSeqOut );
    m_pushHeader ( iter, hdr );
    iter.pushUInt16 ( CA_MINOR_PROTOCOL_REVISION );
    iter.commit ();
}

// send minor protocol revision to the client
void RspThr :: DGCoalescer :: 
    m_pushDGHeader ()
{
    // in current versions of CA version 3, all datagrams 
    // begin with a version header
    Iter iter ( m_byteSeqOut );
    iter.pushUInt16 ( CA_PROTO_VERSION ); 
    iter.pushUInt16 ( 0 ); 
    iter.pushUInt16 ( 0 ); 
    iter.pushUInt16 ( CA_MINOR_PROTOCOL_REVISION );
    iter.pushUInt32 ( 0 );
    iter.pushUInt32 ( 0 );
    iter.commit ();
}

void RspThr :: DGCoalescer :: 
    m_pushHeader ( Iter & iter, const caHdr & hdr )
{
    iter.pushUInt16 ( hdr.m_cmmd ); 
    iter.pushUInt16 ( hdr.m_postsize ); 
    iter.pushUInt16 ( hdr.m_dataType ); 
    iter.pushUInt16 ( hdr.m_count );
    iter.pushUInt32 ( hdr.m_cid );
    iter.pushUInt32 ( hdr.m_available );
}

void RspThr :: DGCoalescer :: 
    show ( unsigned level ) const
{
    printf ( "RspThr :: DGCoalescer ::\n" );
    if ( level > 0 ) {
        m_currentDGParm.show ( level - 1 );
        m_byteSeqOut.show ( level - 1 );
    }
}

#if 0
void RspThr :: DGCoalescer :: 
    pushHeader ( Guard & guard, DGParm & dgParm, 
        const caHdr & hdr, ByteSeqOut :: Ptr & ptr )
{
    guard.assertIdenticalMutex ( m_mutex );
#error
}

void RspThr :: DGCoalescer :: 
    commit ( Guard & )
{
    guard.assertIdenticalMutex ( m_mutex );
#error
}
#endif

} // end of namespace DG
} // end of namespace Svr
} // end of namespace ca
