

// Author: Jeff HIll

#define epicsExportSharedSymbols
#include "caSvrEvQue.h"

namespace ca {
namespace Svr {

EvQue :: EntryBlock :: 
    ~EntryBlock ()
{
    for ( size_t i = 0; i < m_nextEntry; i++ ) {
        const size_t rawIndex = i * sizeof ( Entry );
        Entry & entry = * reinterpret_cast < Entry * > 
                                ( &m_entries[rawIndex] );
        entry.~Entry ();
    }
}

bool EvQue :: EntryBlock :: 
    post ( const PResp & pResponder, const PCat & pPayload )
{
    bool success = m_nextEntry < m_nEntries;
    if ( success ) {
        const size_t rawIndex = m_nextEntry * sizeof ( Entry );
		Entry & entry = * new ( &m_entries[rawIndex] ) 
                        Entry ( pResponder, pPayload );
        m_nextEntry++;
    }
    return success;
}

void EvQue :: EntryBlock :: 
    show ( unsigned level ) const
{
    printf ( "EvQue :: EntryBlock:\n" );
    if ( level > 0 ) {
        printf ( "EvQue :: EntryBlock m_nextEntry = %u:\n", 
            m_nextEntry );
        printf ( "EvQue :: EntryBlock entries:\n" );
        for ( size_t i = 0u; i < m_nextEntry; i++ ) {
            const size_t rawIndex = i * sizeof ( Entry );
            const Entry & entry = * reinterpret_cast < const Entry * > 
                                    ( &m_entries[rawIndex] );
            entry.show ( level - 1u );
        }
    }
}

void EvQue :: EntryBlock :: 
    action ()
{
	if ( m_nextEntry ) {
		{
            Entry * pEntry = reinterpret_cast < Entry * > ( m_entries );
			PResp :: Locker pRespLocked = 
				pEntry->m_pResponder.operator -> ();
			pRespLocked->responseNotify ( pEntry->m_pPayload );
			for ( size_t i = 1u; i < m_nextEntry; i++ ) {
                pEntry++;
				// hold onto the lock to the previous entry 
				// if they are both using the same mutex
				pRespLocked = pEntry->m_pResponder;
				pRespLocked->responseNotify ( pEntry->m_pPayload );
			}
		}
		// Now that the lock is released then release 
		// all of the entries. Somewhat costly but
		// perhaps unavoidable. Its necessary to do 
		// do this now when the lock isnt applied
        {
            Entry * pEntry = reinterpret_cast < Entry * > ( m_entries );
		    for ( size_t i = 0u; i < m_nextEntry; i++ ) {
                pEntry->~Entry ();
                pEntry++;
		    }
        }
        m_nextEntry = 0u;
	}
}

void EvQue :: EntryBlock :: 
    replRespIfMatch ( const PResp & pMatch, PResp & pReplace )
{
    Entry * pEntry = reinterpret_cast < Entry * > ( m_entries );
    for ( size_t i = 0; i < m_nextEntry; i++ ) {
		pEntry->m_pResponder.replaceIfMatch ( pMatch, pReplace );
        pEntry++;
    }
}

void EvQue :: EntryBlock :: Entry :: 
    show ( unsigned level ) const
{
    printf ( "EvQue :: EntryBlock :: Entry\n" );
    if ( level > 0 ) {
        {
            PResp :: Locker pResponderLocked = m_pResponder;
            printf ( "EvQue :: m_pNotify \"%s\"\n",
                typeid ( *pResponderLocked ).name () );
        }

        {
            PCat :: Locker pPayloadLocked = m_pPayload;
            printf ( "EvQue :: m_pPayload \"%s\"\n",
                typeid ( *pPayloadLocked ).name () );
#pragma message ( "call DA dump utility here" )
        }
    }
}

} // end of namespace Svr
} // end of namespace ca
