
/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 */


#define epicsExportSharedSymbols
#include "ServerThr.h"

namespace ca {
namespace Svr {

ServerThr :: M_VecAllocator :: M_EnclLnk
    ServerThr :: M_VecAllocator :: m_enclLnk = 
            & ServerThr :: m_vecAllocator;

Vec < epicsUInt8 > ServerThr :: M_VecAllocator :: alloc ()
{
    epics :: Mutex :: Guard guard ( m_mutex );
    ComBuf * pComBuf = new ( m_comBufFreeList ) ComBuf ( m_destructor );
    return pComBuf->bufPtr ();
}

void ServerThr :: M_VecAllocator :: show ( unsigned level ) const
{
    epics :: Mutex :: Guard guard ( m_mutex );
    printf ( "ServerThr :: M_VecAllocator\n" );
    if ( level > 0 ) {
        level--;
        m_mutex.show ( level );
        m_comBufFreeList.show ( level );
    }
}

size_t ServerThr :: M_VecAllocator :: blockSize () const
{
    return ComBuf :: size ();
}

ServerThr :: M_VecAllocator :: M_Destructor :: M_EnclLnk
    ServerThr :: M_VecAllocator :: M_Destructor :: m_enclLnk = 
            & ServerThr :: M_VecAllocator :: m_destructor;

void ServerThr :: M_VecAllocator :: M_Destructor :: 
    destroy ( ComBuf & comBuf )
{
    M_VecAllocator & vecAllocator ( m_enclLnk.enclosure ( *this ) );
    epics :: Mutex :: Guard guard ( vecAllocator.m_mutex );
    vecAllocator.m_comBufFreeList.deleteProduct ( comBuf );
}

} // end of namespace Svr
} // end of namespace ca
