
/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *
 *      Author  Jeffrey O. Hill
 *              johill@lanl.gov
 *              505 665 1831
 */

#error defunct 

#ifndef casCoreClienth
#define casCoreClienth

#include "caServerI.h"
#include "ioBlocked.h"
#include "casMonitor.h"
#include "casEventSys.h"
#include "casCtx.h"
#include "casdef2.h"

//
// casCoreClient
// (this will eventually support direct communication
// between the client lib and the server lib)
//
class casCoreClient : 
    private casMonitorCallbackInterface {
public:
	casCoreClient ( caServerI & serverInternal ); 
	virtual ~casCoreClient ();
	virtual void show ( unsigned level ) const;

    void installAsynchIO ( class casAsyncPVAttachIOI & io );
    void uninstallAsynchIO ( class casAsyncPVAttachIOI & io );
    void installAsynchIO ( class casAsyncPVExistIOI & io );
    void uninstallAsynchIO ( class casAsyncPVExistIOI & io );

	caServerI & getCAS () const;

	//
	// one virtual function for each CA request type that has
	// asynchronous completion
	//
	virtual void m_searchResponse (
        Guard &, const caNetAddr & outAddr, 
		const caHdrLargeArray &, const pvExistReturn &,
        ca_uint16_t protocolRevision, ca_uint32_t sequenceNumber ) = 0;
	//virtual void createChanResponse (
    //    Guard &,
	//    casCtx &, const pvAttachReturn &) = 0;
	virtual void createChanFailResponse ( 
        Guard &,
        const caHdrLargeArray &, 
        const ca :: Abberation & ) = 0;
	virtual void readResponse (
        Guard &,
		casChannelI &, const caHdrLargeArray &, 
        const ca :: Catalog & ) = 0; 
	virtual void readFailResponse ( 
        Guard &,
        casChannelI &, const caHdrLargeArray &,
        const ca :: Abberation & ) = 0;
	virtual void writeFailResponse ( 
        Guard &, 
        casChannelI &, const caHdrLargeArray &,
        const ca :: Abberation & ) = 0;
	virtual void writeNotifyResponse ( 
        Guard &, casChannelI &, 
        const caHdrLargeArray & ) = 0;
	virtual void writeNotifyFailResponse ( 
        Guard &, casChannelI &, 
        const caHdrLargeArray &, 
        const ca :: Abberation & ) = 0;
	virtual void subscriptionUpdateResponse (
        Guard &,
		casChannelI &, const caHdrLargeArray &, 
        const ca :: Catalog & ) = 0; 
	virtual void subscriptionUpdateFailResponse ( 
        Guard &,
        casChannelI &, const caHdrLargeArray &,
        const ca :: Abberation & ) = 0;

	virtual void accessRightsResponse ( 
        Guard &, casChannelI * ) = 0;
    virtual void enumPostponedCreateChanResponse ( 
        Guard &,
        casChannelI &, const caHdrLargeArray & ) = 0;
    virtual void channelDestroyEventNotify ( 
        Guard & guard, 
        casChannelI * const pChan, ca_uint32_t sid ) = 0;
    virtual void casChannelDestroyFromInterfaceNotify ( 
       casChannelI & chan, bool immediateDestroyNeeded ) = 0;

	virtual ca_uint16_t protocolRevision () const = 0;

	// used only with DG clients 
	virtual caNetAddr fetchLastRecvAddr () const;
    virtual ca_uint32_t datagramSequenceNumber () const;

	void setDestroyPending ();

    casEventSys::processStatus eventSysProcess();

	caStatus addToEventQueue ( casAsyncIOI &, 
        bool & onTheQueue, bool & posted );
    void removeFromEventQueue ( casAsyncIOI &, 
        bool & onTheEventQueue );
	void addToEventQueue ( 
        casChannelI &, bool & inTheEventQueue );
    void removeFromEventQueue ( class casChannelI &, 
        bool & inTheEventQueue );
    void addToEventQueue ( class channelDestroyEvent & ev );
    void enableEvents ();
    void disableEvents ();
    caStatus casMonitorCallBack ( 
        Guard &, 
        casMonitor &, const gdd & );
    void postEvent ( tsDLList <casMonitor > &, 
        const EventMask &select, const gdd &event );

    casMonitor & monitorFactory ( 
        casChannelI & ,
        caResId clientId, 
        const unsigned long count, 
        const unsigned type, 
        const EventMask & );
    void destroyMonitor ( casMonitor & mon );

    void casMonEventDestroy ( 
        casMonEvent &, epicsGuard < evSysMutex > & );

protected:
    casEventSys m_eventSys;
    mutable Mutex m_mutex;
	casCtx m_ctx;

private:
    // for io that does not have a channel
	tsDLList < casAsyncIOI > m_ioList;

    // not pure because the base class noop must be called
    // when in the destructor
    virtual void eventSignal ();

	casCoreClient ( const casCoreClient & );
	casCoreClient & operator = ( const casCoreClient & );
};

inline caServerI & casCoreClient::getCAS() const
{
	return *m_ctx.getServer();
}

inline void casCoreClient::postEvent ( 
    tsDLList < casMonitor > & monitorList, 
    const EventMask & select, const gdd & event )
{
    bool signalNeeded = 
        m_eventSys.postEvent ( monitorList, select, event );
    if ( signalNeeded ) {
        this->eventSignal ();
    }
}

inline casEventSys::processStatus casCoreClient::eventSysProcess ()
{
    Guard guard ( m_mutex );
	return m_eventSys.process ( guard );
}

inline caStatus casCoreClient::addToEventQueue ( casAsyncIOI & io, 
                      bool & onTheQueue, bool & posted )
{
    bool wakeupNeeded;
	caStatus status =  m_eventSys.addToEventQueue ( io, 
        onTheQueue, posted, wakeupNeeded );
	if ( wakeupNeeded ) {
		this->eventSignal ();
	}
    return status;
}

inline void casCoreClient::removeFromEventQueue ( 
    casAsyncIOI & io, bool & onTheEventQueue )
{
    m_eventSys.removeFromEventQueue ( io, onTheEventQueue );
}

inline void casCoreClient::addToEventQueue ( 
    casChannelI & ev, bool & inTheEventQueue )
{
	bool signalNeeded = 
        m_eventSys.addToEventQueue ( ev, inTheEventQueue );
	if ( signalNeeded ) {
		this->eventSignal ();
	}
}

inline void casCoreClient::removeFromEventQueue ( class casChannelI & io, 
    bool & inTheEventQueue )
{
    m_eventSys.removeFromEventQueue ( io, inTheEventQueue );
}

inline void casCoreClient::addToEventQueue ( class channelDestroyEvent & ev )
{
    bool wakeUpNeeded = m_eventSys.addToEventQueue ( ev );
    if ( wakeUpNeeded ) {
		this->eventSignal ();
    }
}

inline void casCoreClient::enableEvents ()
{
    m_eventSys.eventsOn ();
    eventSignal (); // wake up the event queue consumer
}

inline void casCoreClient::disableEvents ()
{
    bool signalNeeded = m_eventSys.eventsOff ();
    if ( signalNeeded ) {
		eventSignal ();
    }
}

inline void casCoreClient::setDestroyPending ()
{
    m_eventSys.setDestroyPending ();
    eventSignal ();
}

inline void casCoreClient::casMonEventDestroy ( 
    casMonEvent & ev, epicsGuard < evSysMutex > & guard )
{
    m_eventSys.casMonEventDestroy ( ev, guard );
}

#endif // casCoreClienth

