

/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *      casCtx.h,v 1.2 2009/09/04 23:23:08 jhill Exp
 *
 *      Author  Jeffrey O. Hill
 *              johill@lanl.gov
 *              505 665 1831
 */

#error defunct

#ifndef casCtxh
#define casCtxh

#include "caHdrLargeArray.h"

#if ! defined ( __GNUC__ ) || __GNUC__ > 2 || ( __GNUC__ == 2 && __GNUC_MINOR__ >= 95 )
namespace ca {
#endif

class casCtx {
public:
	casCtx();
	const caHdrLargeArray & getMsg () const;
	const void * getData () const;
	class caServerI * getServer () const;
	class casCoreClient * getClient () const;
	class casPVI * getPV () const;
	class casChannelI * getChannel () const;
	void setMsg ( const caHdrLargeArray &, const void * pBody );
	void setServer ( class caServerI * p );
	void setClient ( class casCoreClient * p );
	void setPV ( class casPVI * p );
	void setChannel ( class casChannelI * p );
	void show ( unsigned level ) const;
private:
	caHdrLargeArray msg;	// ca message header
	const void * pData; // pointer to data following header
	caServerI * pCAS;
	casCoreClient * pClient;
	casChannelI * pChannel;
	casPVI * pPV;
	unsigned nAsyncIO; // checks for improper use of async io
};

inline const caHdrLargeArray & casCtx :: getMsg() const 
{	
	return this->msg;
}

inline const void * casCtx::getData() const 
{
	return this->pData;
}

inline caServerI * casCtx::getServer() const 
{
	return this->pCAS;
}

inline casCoreClient * casCtx::getClient() const 
{
	return this->pClient;
}

inline casPVI * casCtx::getPV() const 
{
	return this->pPV;
}

inline casChannelI * casCtx::getChannel() const 
{	
	return this->pChannel;
}

inline void casCtx::setMsg ( 
    const caHdrLargeArray & msgIn, const void * pBody )
{
	this->msg = msgIn;
    this->pData = pBody;
}

inline void casCtx::setServer(caServerI *p)
{
	this->pCAS = p;
}

inline void casCtx::setClient(casCoreClient *p) 
{
	this->pClient = p;
}

inline void casCtx::setPV(casPVI *p) 
{
	this->pPV = p;
}

inline void casCtx::setChannel(casChannelI *p) 
{
	this->pChannel = p;
}

#if ! defined ( __GNUC__ ) || __GNUC__ > 2 || ( __GNUC__ == 2 && __GNUC_MINOR__ >= 95 )
} // end of namespace ca
#endif

#endif // casCtxh
