/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE is distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *
 *      Author  Jeffrey O. Hill
 *              johill@lanl.gov
 *              505 665 1831
 */

#include <stdio.h>
#include <limits.h>

#include "errlog.h"

#define epicsExportSharedSymbols
#include "casdef.h"
#include "EventRegistry.h"

#ifdef TEST
main ()
{
	EventRegistry 	reg;
	EventMask		bill1 (reg, "bill");
	EventMask		bill2 (reg, "bill");
	EventMask		bill3 (reg, "bill");
	EventMask		art1 (reg, "art");
	EventMask		art2 (reg, "art");
	EventMask		jane (reg, "jane");
	EventMask		artBill;
	EventMask		tmp;

	bill1.show(10u);
	reg.show(10u);
	bill2.show(10u);
	reg.show(10u);
	bill3.show(10u);
	reg.show(10u);
	jane.show(10u);
	reg.show(10u);
	art1.show(10u);
	reg.show(10u);
	art2.show(10u);
	reg.show(10u);

	assert (bill1 == bill2);
	assert (bill1 == bill3);
	assert (jane != bill1);
	assert (jane != art1);
	assert (bill1 != art1);
	assert (art1 == art2);

	artBill = art1 | bill1;
	tmp = artBill & art1;
	assert (tmp.eventsSelected());
	tmp = artBill & bill1;
	assert (tmp.eventsSelected());
	tmp = artBill&jane;
	assert (tmp.noEventsSelected());
}
#endif

EventMask EventRegistry::maskAllocator ()
{
	EventMask    evMask;
 
	if ( this->maskBitAllocator < CHAR_BIT * sizeof ( evMask.mask ) ) {
		evMask.mask = 1u << ( this->maskBitAllocator++ );
	}
	return evMask;
}

EventMask EventRegistry::registerEvent ( const char *pName )
{
	//
	// NOTE: pName outlives id here
	// (so the refString option is ok)
	//
	stringId id ( pName, stringId::refString );
	EventMaskEntry * pEntry;
	EventMask mask;

	pEntry = this->lookup ( id );
	if (pEntry) {
		mask = *pEntry;
	}
	else {
		mask = this->maskAllocator ();
		if ( mask.mask == 0u ) {
			errMessage ( S_cas_tooManyEvents, "EventRegistry::registerEvent" );
		}
		else {
			pEntry = new EventMaskEntry ( *this, mask, pName );
			mask = *pEntry;
		}
	}
	return mask;
}

void EventMask::show ( unsigned level ) const
{
	if ( level > 0u ) {
		printf ( "EventMask = %x\n", this->mask );
	}
}

EventMask::EventMask ( EventRegistry & reg, const char * pName )
{
        *this = reg.registerEvent ( pName );
}

void EventRegistry::show ( unsigned level ) const
{
	if ( level > 1u ) {
		printf ("EventRegistry: bit allocator = %d\n", 
				this->maskBitAllocator);
	}
	this->resTable < EventMaskEntry, stringId >::show ( level );
}

EventMaskEntry::EventMaskEntry (
	EventRegistry & regIn, EventMask maskIn, const char * pName ) :
	EventMask ( maskIn ), stringId ( pName ), reg ( regIn )
{
	int 	stat;

	assert ( this->resourceName() != NULL );
	stat = this->reg.add ( *this );
	assert ( stat == 0 );
}

EventMaskEntry::~EventMaskEntry()
{
        this->reg.remove ( *this );
}

void EventMaskEntry::destroy ()
{
	delete this;
}

void EventMaskEntry::show ( unsigned level ) const 
{
	this->EventMask::show ( level );
	this->stringId::show ( level );
}

