/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *      casEventMask.h,v 1.13 2009/09/04 23:23:29 jhill Exp
 *
 *      Author  Jeffrey O. Hill
 *              johill@lanl.gov
 *              505 665 1831
 */


#ifndef casEventMaskH
#define casEventMaskH

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_casEventMaskH
#   undef epicsExportSharedSymbols
#endif
#include "resourceLib.h"
#ifdef epicsExportSharedSymbols_casEventMaskH
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif
 
class EventRegistry;

class epicsShareClass EventMask {
public:
    void clear ()
    {
        this->mask = 0u;
    }
    
    EventMask ( EventRegistry &, const char *pName );
    
    EventMask () 
    {
        this->clear();
    }
    
    void show ( unsigned level ) const;
    
    bool eventsSelected () const
    {
        return this->mask != 0u;
    }
    bool noEventsSelected () const
    {
        return this->mask == 0u;
    }
    
    inline void operator |= ( const EventMask & rhs );
    inline void operator &= ( const EventMask & rhs );
    
private:
    unsigned mask;
    friend inline EventMask operator | (
        const EventMask & lhs, const EventMask & rhs );
    friend inline EventMask operator & (
        const EventMask & lhs, const EventMask & rhs );
    friend inline int operator == (
        const EventMask & lhs, const EventMask & rhs );
    friend inline int operator != (
        const EventMask & lhs, const EventMask & rhs );
    friend class EventRegistry;
};

inline EventMask operator | (
    const EventMask & lhs, const EventMask & rhs )
{
        EventMask result;
        result.mask = lhs.mask | rhs.mask;
        return result;
}
inline EventMask operator & (
    const EventMask & lhs, const EventMask & rhs )
{
        EventMask result;
        result.mask = lhs.mask & rhs.mask;
        return result;
}

inline int operator == (
    const EventMask & lhs, const EventMask & rhs )
{
        return lhs.mask == rhs.mask;
}

inline int operator != (
    const EventMask & lhs, const EventMask & rhs )
{
        return lhs.mask != rhs.mask;
}

inline void EventMask :: operator |= (
    const EventMask & rhs ) 
{
	*this = *this | rhs;		
}

inline void EventMask :: operator
    &= ( const EventMask & rhs ) {
	*this = *this & rhs;		
}

#endif // casEventMaskH

