


/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *      casEventRegistry.h,v 1.2 2009/09/04 23:23:29 jhill Exp
 *
 *      Author  Jeffrey O. Hill
 *              johill@lanl.gov
 *              505 665 1831
 */

#ifndef casEventRegistryh
#define casEventRegistryh

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_casEventRegistryh
#   undef epicsExportSharedSymbols
#endif

#include "tsSLList.h"

#ifdef epicsExportSharedSymbols_casEventRegistryh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "casEventMask.h"

namespace ca {

class EventMaskEntry : 
    public tsSLNode < EventMaskEntry >,
	public EventMask, 
    public stringId {
public:
	EventMaskEntry (EventRegistry &regIn,
	    EventMask maskIn, const char *pName);
	virtual ~EventMaskEntry();
	void show (unsigned level) const;

	virtual void destroy();
private:
	EventRegistry &reg;
	EventMaskEntry ( const EventMaskEntry & );
	EventMaskEntry & operator = ( const EventMaskEntry & );
};

class EventRegistry : 
    private resTable < EventMaskEntry, stringId > {
public:
    EventRegistry () : maskBitAllocator ( 0 ) {}
    virtual ~EventRegistry();
    EventMask registerEvent ( const char * pName );
    void show ( unsigned level ) const;
private:
    unsigned maskBitAllocator;

    EventMask maskAllocator();
	EventRegistry ( const EventRegistry & );
	EventRegistry & operator = ( const EventRegistry & );
    friend class EventMaskEntry;
};

} // end of namespace ca

#endif // casEventRegistryh
