

/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *
 *      Author  Jeffrey O. Hill
 *              johill@lanl.gov
 *              505 665 1831
 */

#error defunct

#ifndef casEventSysh
#define casEventSysh

#if defined ( epicsExportSharedSymbols  )
#   undef epicsExportSharedSymbols
#   define casEventSysh_restore_epicsExportSharedSymbols
#endif

#include "tsDLList.h"
#include "tsFreeList.h"
#include "epicsMutex.h"

#if defined ( casEventSysh_restore_epicsExportSharedSymbols )
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

template < class MUTEX > class epicsGuard;

#include "casdef.h"
#include "casEvent.h"

/*
 * maximum peak log entries for each event block (registartion)
 * (events cached into the last queue entry if over flow occurs)
 * (if this exceeds 256 then the casMonitor::nPend must
 * be assigned a new data type)
 */
#define individualEventEntries 16u

/*
 * maximum average log entries for each event block (registartion)
 * (events cached into the last queue entry if over flow occurs)
 * (if this exceeds 256 then the casMonitor::nPend must
 * be assigned a new data type)
 */
#define averageEventEntries 4u

enum casProcCond { casProcOk, casProcDisconnect };

class casMonitor;
class casMonEvent;

class casEventSys {
public:
	casEventSys ( 
		Guard :: Id );
	~casEventSys ();
	void show ( 
		Guard &, unsigned level ) const;
    struct processStatus {
        casProcCond cond;
        unsigned nAccepted;
    };
	processStatus process ( 
        Guard & );
	void installMonitor (
		Guard & );
	void removeMonitor (
		Guard & );
    void prepareMonitorForDestroy ( 
		Guard &, casMonitor & mon );
    bool postEvent ( 
		Guard &,
		tsDLList < casMonitor > & monitorList, 
        const EventMask & select, const gdd & event );
	caStatus addToEventQueue ( 
		Guard &, class casAsyncIOI &, 
        bool & onTheQueue, bool & posted, bool & signalNeeded );
    void removeFromEventQueue ( 
		Guard &, class casAsyncIOI &, 
        bool & onTheEventQueue );
	bool addToEventQueue ( 
		Guard &,
        casChannelI &, bool & inTheEventQueue );
    void removeFromEventQueue ( 
		Guard &, class casChannelI &, 
        bool & inTheEventQueue );
	bool addToEventQueue ( 
		Guard &, class channelDestroyEvent & );
	bool getNDuplicateEvents (
		Guard & ) const;
	void setDestroyPending (
		Guard & );
	void eventsOn (
		Guard & );
	bool eventsOff (
		Guard & );
    void casMonEventDestroy ( 
		Guard &,
        casMonEvent & );
private:
	Guard :: Id m_mutexId;
	tsDLList < casEvent > m_eventLogQue;
    tsFreeList < casMonEvent, 1024, epicsMutexNOOP > m_casMonEventFreeList;
	class casEventPurgeEv * m_pPurgeEvent; // flow control purge complete event
	unsigned m_numSubscriptions; // N subscriptions installed
	unsigned m_maxLogEntries; // max log entries
	bool m_destroyPending;
	bool m_replaceEvents; // replace last existing event on queue
	bool m_dontProcess; // flow ctl is on - dont process event queue

	bool full () const;
	casEventSys ( const casEventSys & );
	casEventSys & operator = ( const casEventSys & );
    friend class casEventPurgeEv;
};

/*
 * when this event reaches the top of the queue we
 * know that all duplicate events have been purged
 * and that now no events should not be sent to the
 * client until it exits flow control mode
 */
class casEventPurgeEv : public casEvent {
public:
    casEventPurgeEv ( class casEventSys & );
private:
    casEventSys & evSys;
	caStatus cbFunc ( 
        casCoreClient &, 
        Guard & );
    casEventPurgeEv & operator = ( const casEventPurgeEv & ); 
    casEventPurgeEv ( const casEventPurgeEv & ); 
};

//
// casEventSys::casEventSys ()
//
inline casEventSys :: casEventSys ( 
    Guard :: Id mutexId ) :
	m_mutexId ( mutexId ),
	m_pPurgeEvent ( NULL ),
	m_numSubscriptions ( 0u ),
	m_maxLogEntries ( individualEventEntries ),
	m_destroyPending ( false ),
	m_replaceEvents ( false ), 
	m_dontProcess ( false ) 
{
}

#endif // casEventSysh

