

#ifndef casdef2h
#define casdef2h

// Issues
// o I currently have properties and request methods in the same name space.
// Thats probably a lazy implementation, and a bad design.

#include <exception>

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_casdef2h
#   undef epicsExportSharedSymbols
#endif

#include "epicsAssert.h"
#include "da.h"
#include "caerr.h" // legacy CA API error codes

#ifdef epicsExportSharedSymbols_casdef2h
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "caPtr.h"

//template < class M > class epicsGuard;
class StringEditor;

namespace ca {

using namespace da;
using std :: exception; // Tornado 2 workaround

void ignoredExceptMsg ( const type_info & intfType, 
    const char * pFuncName );
void ignoredExceptMsg ( const type_info & intfType, 
    const char * pFuncName, exception & );

// Comments on error context reporting (strings verses codes)
// ----------------------------------------------------------
// 1) Failure status codes are more effiently moved through the 
// system compared to strings. They are a type of data compression
// technique. Failure status codes also allow the application
// to take evasive acction for certain types of errors although
// perhaps this is perhaps rare in practice.
// 2) Strings are more flexible than status codes especially when 
// crossing the network, when multiple subsystems need to chain 
// together error context, or when some of the status is purely
// runtime dynamic in nature.
// 2a) Issues with sending failure status codes over the network can be 
// solved by providing functions that return a string corresponding 
// to a status code, and by defining protocol that defines a string 
// that is to be associated with an error code in the future.
// 2b) Issues with chaining together error context might be solved
// by sendinng a message containg an array of error codes.
// 2c) Runtime dynamic information is frequently providing addressing
// information for the device, channel, record, etc. It is far more 
// difficult to come up with a mechanism to compress this type of 
// information.
//
// Since strings are easier and more flexible, then why not use 
// them all of the time? Presumably, its ok for the system to run 
// slower if there are unfixed connfiguration problems? We are 
// already passing strings for certain types of failures.
//
// If so, then perhaps pass a reference counted smart pointer
// for a string type?

// maybe there isnt a compelling reason to pass this, 
// or any std::exception derived class,  to the IO completion 
// failures callbacks, but this still might be useful
// for providing backwards compatible status during the 
// transition.
struct Abberation : public exception {
    virtual int legacyStatus () const throw () = 0;
};

typedef Ptr < const Catalog > PCat;

struct Type;
typedef Ptr < const Type > PType;

struct Channel;
typedef Ptr < Channel > PChan;

// When creating a service we might specify to be used to synchronize
// access to the service. Therefore, sometimes the Service interface 
// is in a lock domain specified by its client. In this way we can 
// improve performance by not requiring lock / unlock overhead 
// each time that we call the service. 
struct Service { 
    virtual ~Service () = 0; 

    // This creates a tag that can be used to efficently
    // identify a particular configuration of properties 
    // within a Catalog. If the catalog's property order, 
    // topology, primitive types, or bounds change then 
    // a new Type must be obtained from the service.
    // !!!! But, when traversing the Catalog its
    // !!!! possible for primitive type and or bounds to 
    // !!!! be left unspecified for certain properties  
    // !!!! whoose type information needs to
    // !!!! be dynamic. The revealNexus method in
    // !!!! da :: Introspector and / or da :: Viewer
    // !!!! could be called when the primitive type
    // !!!! and or bounds are not known apriori.
    // !!!! This might have several benefits
    // !!!! 1) Good management of type permutations. Perhaps
    // !!!! we prefer not to create a new type when it is
    // !!!! used infrequently, and it would be better to
    // !!!! create a general purpose type that can be used 
    // !!!! in more situations - we prefer in general that 
    // !!!! types need not be channel spoecific.
    // !!!! 2) We certainly dont care if the array bounds
    // !!!! information accompanies every array on the wire
    // !!!! if an array is significantly greater in size 
    // !!!! than the bounds information.
    // !!!! 3) For certain types of interfaces flexibility
    // !!!! might be more important than efficency.
    //
    PType createType ( const Catalog & description );    

    // description supplies ( see Ident defined by Channel)
    // o name
    // o priority
    // o property path
    // o bounds
    // ...
    //
    PChan createChannel ( const PCat & pDescription ); 

    void flush () throw (); 
    void show ( unsigned level ) const throw (); 

    // exceptions
    class InvalidChannelPriority;
    class ServiceDisconnect; 
private:
    virtual PType m_createType ( const Catalog & description ) = 0;    
    virtual PChan m_createChannel ( const PCat & pDescription ) = 0; 
    virtual void m_flush () = 0; 
    virtual void m_show ( unsigned level ) const = 0; 
};

typedef Ptr < Service > PService;
    
struct Request;
typedef Ptr < Request > PReq;

struct Channel
{
    virtual ~Channel () = 0;

    typedef unsigned Priority;
    static const Priority priorityMax;
    static const Priority priorityMin;

    static Ident name;
    static Ident priority;

    // circuit state property
    // (initially, after creating the channel, its assumed to 
    // be unresponsive)
    // hm ... perhaps they should know only about connected and 
    // disconnected?
    static Ident circuitState;
    enum CircuitState {
	    csDisconn,
	    csUnresp,
	    csResp
    };
    static Ident nativeType;
    enum NativeType {
	    csFloat,
	    csInt,
        csEnum,
        csString
    };
    static Ident nativeTypeSize;
    static Ident nativeBounds;

    //Ident classId ( const Guard & ) const;
    void cannonicalName ( StringEditor & ) const;
    void serviceName ( StringEditor & ) const;
    //bool isConnected ( const Guard & ) const;
    //bool isResponsive ( const Guard & ) const;
    void show ( unsigned level ) const throw ();
        
    //
    // IO request factory
    //
    // To implement this a server tool might:
    // o perform hash lookup on method id
    // o use da based assign to copy incoming parameters 
    // to request parms
    // o invoke the method
    // o use da based assign to copy response parms to 
    // outgoing parms
    //
    // The specification of request and response parameter 
    // Type here is useful for
    // o With a network remoting service we can send the 
    // type description over the wire only the first time 
    // it is used, and for all subsequent uses pass only a 
    // type definition identifier.
    // o With an in-memory service the service can overload 
    // the method invocation, and for example implement 
    // specific code for read/write/subscribe methods specific 
    // to a particular Types that is being passed.
    //
    // Presumably, the optimizations possible when binding a Request
    // to a particular channel outweigh any memory savings benefits
    // obtained from allowing a Request creater to reuse the same 
    // Request with multiple channels. It also seems likely that 
    // most applications probably wouldnt bother to try to use the 
    // same request with multiple channels.
    //
    //
// 1) method allows us to pick read / write / subscribe
// 2) circuit state is observed here also?
// 3) can we remove survey as a special type in da by creating
//    standard properties that describe properties. So a property
//    might be a list of properties that describe properties. This
//    would require that they switch or test a property type code
//    unless a property's primitive type could be a callable Survey 
//    interface
//
// Observations:
// 1) The callback's response parameters Catalog is permitted
// to contain parameters that were not requested 
// 2) The callback's response parameters Catalog is permitted
// to contain parameters with different primitive types than 
// were requested.
//
// 3) An implementation challenge for the service here will be, 
// when methods are overloaded efficently mapping a type into 
// the argument signature that best matches that type.
//
    // Issues for writes
    // -----------------
    // o in high throughput situations can intermediate writes 
    // be discarded
    // o does completion notification get sent when the 
    // request is queued, or when the operation completes
    // Issues for Reads
    // ----------------
    // o are we allowed to fetch out of a cache or not 
    // 

    // o The service can respond efficently with any parameter 
    // set it wants for a particular method. In contrast, this 
    // function is perhaps harder for the service to implement at 
    // runtime.
    // It seems that a library function that matches the 
    // argument signature advertised by the service with the
    // argument signature being supplied by the client will
    // be necessary.
    //
    // for predefined methods - see Ident defined in Request
    //
    static const PType & pEmptyType;
    PReq createRequest (
        const Ident & method, 
        const PType & pReqParmType = pEmptyType, 
        const PType & pRspParmType = pEmptyType );

    // If implemented, this would allow the same lock to be used by the
    // service to create the request and also initiate it. It seems 
    // really likely that the channel and the channel's Request would
    // choose to use the same lock.
#if 0
    void invoke ( const Ident & method );
    void invoke ( const Ident & method, 
        const PType & pReqParmType, const PCat &, 
        const PType & pRspParmType, const PResp & );
    void invokePut ( const Ident & method, const PType & pReqParmType, const PCat & );
    void invokeGet ( const Ident & method, const PType & pRspParmType, const PResp & );
#endif
private:
    virtual void m_cannonicalName ( StringEditor & ) const = 0;
    virtual void m_serviceName ( StringEditor & ) const = 0;
    virtual void m_show ( unsigned level ) const = 0;
    virtual PReq m_createRequest ( const Ident & method, 
        const PType & pRequestParamType, 
        const PType & pResponseParamType ) = 0;
#if 0
    virtual PReq m_invoke ( const Ident & method, 
        const PType & pReqParmType, const PCat &,
        const PType & pRspParmType, const PResp & ) = 0;
#endif
};

// A Type might be created in many different ways, but the 
// user should expect an exception to be thrown, if a Type 
// not created by the Channel's Service is specified when 
// creating a Request. The only exceptions might be built-in
// types which can be used with any Service.
struct Type {
    virtual ~Type () = 0;
};

struct Responder;
typedef Ptr < Responder > PResp;

//
// flow control of the event queue is handled by canceling
// and resuming the subscruiption request
//
struct Request {
    // predefined Request method identifiers

    static Ident read; 
    static Ident write;
    static Ident subscribe;
    static Ident resolveName;
    //
    // response params for monitorCircuitState
    // o circuitState - one of CircuitState (required)
    // o nativeType - one of Xxxxxxx (default to string)
    // o nativeTypeSize - number of bytes (default to widest)
    // o nativeBounds - array of element count (defaults to scalar)
    static Ident monitorCircuitState;

    virtual ~Request () = 0;
    //
    // o Good design dictates that virtual interfaces 
    // dont have default arguments. Therefore, we dont 
    // default here to an empty catalog pointer.
    // o The PCat refernece passed to initiate 
    // allows the service to create a private 
    // reference to the server's data structure.
    // This works fine when the types are known at
    // compile time - for example dbr_XXX. Its harder 
    // when the types are defined at runtime - for
    // example next generation protocol. The probable
    // solution is a maintaining one runtime flexible 
    // free list (the size of the free list elemennts 
    // is defined at runtime) for each Type that is
    // defined by the client via the next generation
    // protocol.
    // o of course the above comment doesnt address 
    // the issues associated with DBR atomic arrays. 
    // Incoming arrays will need to be referenced
    // by the PCat directly in the unaligned
    // protocol stream.
    // o so this leaves the question open, should
    // they be allowed to reference scalars in the 
    // protocol stream. In principal, a scalar isnt
    // different from a small array in terms of the 
    // potential to trap a large block in order to
    // hold a small datum.
    //
    // o Frequently, Request and Channel might be in the
    // same lock domain, so perhaps the user is advised 
    // test and see if the objects have the same Guard
    // and use the Guard in PtrGuard < Channel > to speed
    // up creation of PtrGuard < Request >?
    //
    void initiate ();
    void initiate ( const PCat &, const PResp & );
    void initiatePut ( const PCat & );
    void initiateGet ( const PResp & );

    // A Request, which has a response to a Responder
    // pending, may be canceled. A Request, which 
    // doesnt have a response to a Responder pending, 
    // may not be canceled. The act of canceling a 
    // pending Request will always properly clean up
    // dangling resources, but of course it cant be
    // guaranteed that cancel will always abort a 
    // pending request before it is executed.
    // Destroying a Request while a response to a
    // Responder is pending will include cancelation
    // as a subset effect.
    void cancel () throw ();
    void show ( unsigned level ) const throw ();
private:
    virtual void m_initiate ( const PCat &, const PResp & ) = 0;
    virtual void m_cancel () = 0;
    virtual void m_show ( unsigned level ) const = 0;
    static const PCat & pEmptyCat;
    static const PResp & pNoResp;
    //static Responder & m_noResponder;
};

struct Responder {
    virtual ~Responder () = 0;
    // The properties in the catalog supplied here will 
    // convey the response parameters.
    // o For successful responses a single top-level container
    // property named "success" shall be present. The structure 
    // of the successful response parameters supplied in this 
    // container are specific to the request, and their 
    // structure is not documented here.
    // o For failure responses a single top-level container
    // property named "failure" shall be present. At a minimum
    // this container will provide a string property named
    // "context" providing detailed context information 
    // about the nature of the failure.
    static Ident success; 
    static Ident failure; 
    static Ident context; 
    void responseNotify ( const PCat & ) throw ();
    // !!!! real possibility of show loops here
    // !!!! perhaps there should not be a show in 
    // !!!! the response callback
    void show ( unsigned level ) const throw ();
    class NotifyAfterCancelExcept;
private:
    // indirect access to the virtual 
    // functions is used for two purposes
    // 1) virtual functions should not have default arguments
    // 2) exceptions occuring in the callback virtual 
    // functions are not propagated to the caller, and
    // instead they are reported as warnings messages
    virtual void m_responseNotify ( const PCat & ) = 0;
    virtual void m_show ( unsigned level ) const = 0;
};

inline PType Service :: 
    createType ( const Catalog & description )
{
    return m_createType ( description );
}

inline PChan Service :: 
    createChannel ( const PCat & pDescription )
{
    return m_createChannel ( pDescription );
}

inline void Service :: 
    flush () throw ()
{
    try {
        m_flush ();
    }
    catch ( std :: exception & except ) {
        ignoredExceptMsg ( typeid ( *this ),
            "m_flush", except );
    }
    catch ( ... ) {
        ignoredExceptMsg ( typeid ( *this ),
            "m_flush" );
    }
}

inline void Service :: 
    show ( unsigned level ) const throw ()
{
    try {
        m_show ( level );
    }
    catch ( std :: exception & except ) {
        ignoredExceptMsg ( typeid ( *this ),
            "m_show", except );
    }
    catch ( ... ) {
        ignoredExceptMsg ( typeid ( *this ),
            "m_show" );
    }
}

inline void Channel :: 
    cannonicalName ( StringEditor & dest ) const
{
    m_cannonicalName ( dest );
}

inline void Channel :: 
    serviceName ( StringEditor & dest ) const
{
    m_serviceName ( dest );
}

inline void Channel :: 
    show ( unsigned level ) const throw ()
{
    try {
        m_show ( level );
    }
    catch ( std :: exception & except ) {
        ignoredExceptMsg ( typeid ( *this ),
            "m_show", except );
    }
    catch ( ... ) {
        ignoredExceptMsg ( typeid ( *this ),
            "m_show" );
    }
}

inline PReq Channel :: 
    createRequest ( const Ident & method, 
            const PType & pRequestParametersType, 
            const PType & pResponseParametersType )
{
    return m_createRequest ( method, pRequestParametersType, 
                                pResponseParametersType );
}

inline void Request :: initiate ()
{
    m_initiate ( pEmptyCat, pNoResp );
}

inline void Request :: initiateGet ( const PResp & pResp )
{
    m_initiate ( pEmptyCat, pResp );
}

inline void Request :: initiatePut ( const PCat & pStim )
{
    m_initiate ( pStim, pNoResp );
}

inline void Request :: 
    initiate ( const PCat & pStim, const PResp & pResp )
{
    m_initiate ( pStim, pResp );
}

inline void Request :: cancel () throw ()
{
    try {
        m_cancel ();
    }
    catch ( std :: exception & except ) {
        ignoredExceptMsg ( typeid ( *this ),
            "m_cancel", except );
    }
    catch ( ... ) {
        ignoredExceptMsg ( typeid ( *this ),
            "m_cancel" );
    }
}

inline void Request :: show ( unsigned level ) const throw () 
{
    try {
        m_show ( level );
    }
    catch ( std :: exception & except ) {
        ignoredExceptMsg ( typeid ( *this ),
            "m_show", except );
    }
    catch ( ... ) {
        ignoredExceptMsg ( typeid ( *this ),
            "m_show" );
    }
}

// Notes:
// A) this prevents ungraceful unexpected exception 
// termination from occurring by explicit use of a try 
// catch block
// B) presumably in such situations the compiler will 
// not insert an additional implicit try catch block 
// enforcing the release function prototype's empty 
// throw specification
// C) the design assumption here is that the overhead 
// for the try catch block is compelling considering 
// that we are crossing a membrane between user code 
// and library code, and that the call is from the 
// library's auxiliary thread which should terminate 
// only at the library's, and not the user's, 
// discretion.
//
inline void Responder :: 
    responseNotify ( const PCat & pCat ) throw ()
{
    try {
        m_responseNotify ( pCat );
    }
    catch ( std :: exception & except ) {
        ignoredExceptMsg ( typeid ( *this ),
            "m_responseNotify", except );
    }
    catch ( ... ) {
        ignoredExceptMsg ( typeid ( *this ),
            "m_responseNotify" );
    }
}

inline void Responder :: 
    show ( unsigned level ) const throw ()
{
    try {
        m_show ( level );
    }
    catch ( std :: exception & except ) {
        ignoredExceptMsg ( typeid ( *this ),
            "m_show", except );
    }
    catch ( ... ) {
        ignoredExceptMsg ( typeid ( *this ),
            "m_show" );
    }
}

//
// Exceptions
//
// Exception Strategy
//
// No matter what else we do we need this
// --------------------------------------
// Report back the following over the network
// 1) A copy of the request message
// 2) Details, as supplied by the exception::what() call
//
// What gets sent in the response message must be request dependent.
// This implies a try / catch block around the virtual function call.
//
// Options for mapping to legacy status codes
// -------------------------------------------
// o Provide non-virtual legacy status returning function in some 
// standard set of exceptions, but allow them to specialize the
// exception :: what () virtual function.
//

template < int LEGACY_STATUS >
struct LegacyStatus : public Abberation {
    const char * what () const throw () 
    { 
        return ca_message ( LEGACY_STATUS ); 
    }
    int legacyStatus () const throw () 
    { 
        return LEGACY_STATUS; 
    }
};

struct ArrayElementSubsetOutOfBounds : 
    public LegacyStatus < ECA_BADCOUNT > 
{
    const char * what () const throw () { 
        return "Array element subset out Of Bounds";
    }
};

struct InvalidEventSelection : 
    public LegacyStatus < ECA_BADMASK > {
    int legacyStatus () const throw ();
};

struct NoWriteAccess : 
    public LegacyStatus < ECA_NOWTACCESS > {
}; 

struct NoReadAccess : 
    public LegacyStatus < ECA_NORDACCESS > {
};

struct ConversionFailure : 
    public LegacyStatus < ECA_NOCONVERT > {
};

struct PropertyValueOutOfBounds :
    public LegacyStatus < ECA_NOCONVERT > {
    const char * what () const throw () { 
        return "Property value out of bounds";
    }
};

struct MessageDoesNotFitInProtocolBuffer : 
    public LegacyStatus < ECA_TOLARGE > {
};

// exceptions
class UnsupportedByService : public Abberation {};

class RequestTimedOut : public Abberation {};

class Unresponsive : public Abberation {};

class Disconnect : public Abberation {};

class UnexpectedServiceCompletionCallback :
    public std :: exception {
    const char * what () const throw ();
};

class UknStdExcept : public Abberation {
public:
    UknStdExcept ( const std :: exception & except );
    const char * what () const throw ();
    int legacyStatus () const throw ();
private:
    const std :: exception & m_stdExcept;
};

class UknExcept : public Abberation {
public:
    UknExcept ();
    const char * what () const throw ();
    int legacyStatus () const throw ();
};

inline UknStdExcept :: 
    UknStdExcept ( const std :: exception & except ) : 
    m_stdExcept ( except ) 
{
}

//
// use cases
//
// A service will be one of the following
// --------------------------------------
// o client library
// o IOC
// o protocol converter
// o a concentrator, proxy, gateway etc
//
// A client will be one of the following
// -------------------------------------
// o the server library
// o a client side app
// o protocol converter
// o a concentrator, proxy, gateway etc
// o an IOC
//
// Design Considerations
// ---------------------
//
// 1) Client application must register each of its
// types with the service, and receive an abstract 
// Type reference for future use when creating an 
// abstract Request reference.
//
// 1a) A future client library will need a capability
// for defining a service specific Type and then 
// referencing it later when the client creates 
// a request.
// This is so that the data description can be
// passed over the wire only once, and subsequently
// the data instance is passed over the wire in a 
// tightly packed binary form.
//
// 1b) Its very likely that other services will
// make good use of this feature so that data 
// type binding, which might be slow, can be 
// performed a minimum number of times.
//
// 2) The current design has it so that  
// a reference counting Catalog pointer is
// passed in the arguments of the IO completion 
// callback, and a reference counting Catalog 
// pointer is passed to the IO request initiate
// virtual method.
//
// Subscription updates are passed as a special
// type of IO completion callback where the IO
// request object is persisting over many
// completion callbacks - one for each subscription
// update.
//
// Subscription updates are passed as a special
// type of IO completion callback where the IO
// request object is persisting over many
// completion callbacks - one for each subscription
// update. Some of the intermediate updates might 
// be discarded if queueing capacity is exceeded.
// THE SERVICE CHOOSES AS FOLLOWS.
// 1) SERVICE NEVER BLOCKS IN THE COMPLETION
// CALLBACK. INTERMEDIATE UPDATES WILL BE DISCARDED 
// IF BUFFERING CAPACITY IS EXCEEDED.
// 2) SERVICE BLOCKS IN THE COMPLETION CALLBACK IF 
// BUFFERING CAPACITY IS EXCEEDED.
// 
// Slider put requests are possibly passed as a 
// special type of IO initiate where the IO
// request object is persisting over many
// overlapping initiate calls - one for each 
// slider position update. 
// THE CLIENT CHOOSES AS FOLLOWS.
// 1) CLIENT NEVER BLOCKS IN THE INITIATE VIRTUAL
// METHOD. INTERMEDIATE UPDATES WILL BE DISCARDED 
// IF BUFFERING CAPACITY IS EXCEEDED.
// 2) CLIENT BLOCKS IN THE INITIATE VIRTUAL 
// METHOD IF BUFFERING CAPACITY IS EXCEEDED.
//
// For the blocking interface a Catalog 
// reference is passed.
//
// For the non-blocking interface a reference
// counting pointer to the Catalog is passed.
// The semantics of the non-blocking interface
// are as follows.
// 1) The storage for the object must persist
// for the duration of any references that
// are outstanding.
// 1) The datum allocator (request initiator
// or completion poster) must not allow the
// datum to modified while there are references
// outstanding (OR ALTERNATIVELY WHILE THE
// LOCK ISNT APPLIED).
//
// This approach allows the storage management
// for subscription update datums to reside in
// the Service where it can be 
// implemented efficently. For example, an
// ordinary binary packed data structure can
// be used, and we dont need to use a linked 
// of paramters, each of them containing a 
// description of the data, which is a flexible
// but inefficent approach at runtime.
//
// What types of Service objects will be posting
// events to the event queue, and how might they 
// manage storage the update datums?
//
// o In the IOC's database Service record support 
// and device support snap-ins can manage storage 
// using free lists and ordinary binary packed 
// data structures. This is very efficent in terms
// of runtime overhead and also in terms of
// coding efficency.
//
// o In a future client library (which is also a 
// Service) packed binary data will be streaming
// in off the wire. There are two options.
//
// Option 1 => Unload from the wire into a
// naturally alligned binary packed storage 
// allocated off a free list that learns the size
// of its items at runtime. This would not be
// a conventional C/C++ data structure, but
// have the same storage efficency and layout.
// A table of offsets and primitive types 
// would be created only once when the client 
// application registers a Type.
//
// Option 2 => The datum is revealed by the
// Catalog directly out of the protocol buffers.
// Scalars would be byte swapped into naturally
// alligned temporary storage just before they 
// are revealed.
// Arrays would be byte swapped into naturally
// alligned temporary storage, blocks of elements,
// just before they are revealed.
// With the current interface, the client 
// application would be permitted to keep
// a reference counting pointer to the protocol
// buffers and so there is potential for a
// small datum to trap a large protocol buffer
// with a resulting loss in storage efficency.
//
//
// 2a) It seems consistent to have a symetric 
// interface when initiating an IO request,
// and when posing an IO completion response. That
// type of storage management is similar to how 
// the current db put callback interface is 
// implemented. but this adds extra work for the 
// client app - without question.
//
// Allowing the service to have a Ptr reference 
// to the incoming protocol stream leaves open a
// potential for a small datum ref to capture a 
// large block resulting in a dismal memory 
// management performance. 
// Is it a solution to just say that a service 
// shouldnt keep a refernce to an IO object after
// the Request completes? 
//



} // end of namespace ca

#endif // ifndef casdef2h

