
#error defunct

//
// cadbr.h,v 1.1 2009/10/01 00:29:42 jhill Exp
//
// Authors: Jeff Hill, johill@lanl.gov
//          Ralph Lange, Ralph.Lange@aps.anl.gov

//
// Notes:
// 1) It would be nice to assign through temporaries here
// so that we guarantee that no modifications occurr if 
// an exception is thrown during the assign, but its 
// difficult to create temporaries on the stack for the 
// dbr types when the element count is greaater than one. 
// Therefore, a solution for this problem, and the associated 
// memory management for temporaries during the assign, is left
// to code layered above the assign interface for the highest 
// Catalog in the hierarchy. To remauin consistent, this 
// policy is being adopted for both scalar and vector dbr 
// types. This approach is also more efficent because we need 
// not assign through temporaries in each of subordinate
// container types (the only practical unit for temporaries
// seems to be the entire dbr type even if we are assigning
// say only to the alarm limits).
//
// 2) all of these implementation details should be hidden 
// from the user - this should be invoked through a jump table 
// based on the DBR_XXXX type.
// 
#ifndef cadbr_h
#define cadbr_h

#include <cstdlib>

#ifdef epicsExportSharedSymbols
#   undef epicsExportSharedSymbols
#   define epicsExportSharedSymbolsRestore_cadbr
#endif

#include "db_access.h"

#include "daIdent.h"
#include "daViewer.h"
#include "daIntrospector.h"
#include "daCatalog.h"
#include "daClerk.h"
#include "daAtomic.h"
#include "daMutator.h"
#include "daArrayVector.h"
#include "daStringInFixedLenCharArray.h"
#include "daCatalogSingle.h"

#ifdef epicsExportSharedSymbolsRestore_cadbr
#   define epicsExportSharedSymbols
#endif

#include "shareLib.h"

namespace ca {
namespace dbr {

//
// DBR Property Hierarchy
// ----------------------
// PV
//   time
//   precision
//     decimalDigits
//   units
//   limits
//     control
//       lower
//       upper
//     display
//       lower
//       upper
//     alarm
//       lower
//         major
//         minor
//       upper
//         major
//         minor
//   alarm 
//     condition
//       severity
//       status
//     acknowledgement
//       pending (is a transient alarm pending acknowledgement)
//       severity (the highest unacknowledged alarm severity)
//   stateSet
//     labels
//       size (for use when modifying the number of states)
//   class
//     name
//
//

epicsShareExtern Ident pi_value;

epicsShareExtern Ident pi_limits;
epicsShareExtern Ident pi_time;
epicsShareExtern Ident pi_units;

//
epicsShareExtern Ident pi_precision;
epicsShareExtern Ident pi_decimalDigits;

//
epicsShareExtern Ident pi_stateSet;
epicsShareExtern Ident pi_labels;
epicsShareExtern Ident pi_size;

//
epicsShareExtern Ident pi_class;
epicsShareExtern Ident pi_name;

// for alarm condition
epicsShareExtern Ident pi_alarm;
epicsShareExtern Ident pi_condition;
epicsShareExtern Ident pi_severity;
epicsShareExtern Ident pi_status;

// for alarm acknowledgement
epicsShareExtern Ident pi_acknowledge;
epicsShareExtern Ident pi_pending;

// for control, display, warning, alarm limits
epicsShareExtern Ident pi_control;
epicsShareExtern Ident pi_display;
epicsShareExtern Ident pi_warning;
epicsShareExtern Ident pi_alarm;
epicsShareExtern Ident pi_major;
epicsShareExtern Ident pi_minor;
epicsShareExtern Ident pi_upper;
epicsShareExtern Ident pi_lower;

template < unsigned > class TypeCodeTraits;

typedef char UnitsString [MAX_UNITS_SIZE];
typedef const char UnitsStringConst [MAX_UNITS_SIZE];

void extractNativeTypeAndCount ( 
    const Catalog &, unsigned & dbrType, size_t & count );

} // end of namespace dbr
} // end of namespace ca

#include "cadbrCatCmpdScalar.h"
#include "cadbrCatCmpdVector.h"
#include "cadbrMutCmpdScalar.h"
#include "cadbrMutCmpdVector.h"
#include "cadbrIntfAtomic.h"
#include "cadbrIntfTime.h"
#include "cadbrIntfCtrl.h"
#include "cadbrIntfSts.h"
#include "cadbrIntfGr.h"

#endif // ifdef cadbr_h
