
//
// cadbrIntfAlarmAck.h,v 1.2 2009/11/24 01:34:27 jhill Exp
//
// Authors: Jeff Hill, johill@lanl.gov
//          Ralph Lange, Ralph.Lange@aps.anl.gov

#ifndef cadbrIntfAlarmAck_h
#define cadbrIntfAlarmAck_h

#ifdef epicsExportSharedSymbols
#undef epicsExportSharedSymbols
#define epicsExportSharedSymbolsRestore_cadbrIntfAlarmAck
#endif

#include "daCatalog.h"

#ifdef epicsExportSharedSymbolsRestore_cadbrIntfAlarmAck
#define epicsExportSharedSymbols
#endif

#include "shareLib.h"

namespace ca {
namespace dbr {

template < class T >
class CatAlarmAck : 
    public Catalog
{
public:
    CatAlarmAck ( const T & dbr );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
private:
    const T & m_dbr;
};

template < class T >
class MutAlarmAck : 
    public MutatorCatalog
{
public:
    MutAlarmAck ( T & dbr );
    void assignCatalog ( const Catalog & src );
private:
    T & m_dbr;
};

template < class T >
inline CatAlarmAck < T > :: 
    CatAlarmAck ( const T & dbr ) : 
    m_dbr ( dbr )
{
}

template < class T >
void CatAlarmAck < T > :: 
    traverse ( Introspector & i ) const
{
    i.reveal ( pi_pending, m_dbr.ackt ); 
    i.reveal ( pi_severity, m_dbr.acks ); 
}

template < class T >
void CatAlarmAck < T > :: 
    find ( const Ident & id, Viewer & v ) const
{
    if ( id == pi_pending ) {
        v.reveal ( m_dbr.ackt ); 
    }
    else if ( id == pi_severity ) {
        v.reveal ( m_dbr.acks ); 
    }
}

template < class T >
inline MutAlarmAck < T > :: 
    MutAlarmAck ( T & dbr ) : 
    m_dbr ( dbr )
{
}

template < class T >
void MutAlarmAck < T > :: 
    assignCatalog (  const Catalog & src )
{
    Clerk clerk ( src );
    clerk.get ( pi_pending, m_dbr.ackt );
    clerk.get ( pi_severity, m_dbr.acks );
}

} // end of namespace dbr
} // end of namespace ca

#endif // cadbrIntfAlarmAck_h

