
//
// cadbrIntfAlarmCond.h,v 1.2 2009/11/24 01:34:27 jhill Exp
//
// Authors: Jeff Hill, johill@lanl.gov
//          Ralph Lange, Ralph.Lange@aps.anl.gov

#ifndef cadbrIntfAlarmCond_h
#define cadbrIntfAlarmCond_h

#ifdef epicsExportSharedSymbols
#undef epicsExportSharedSymbols
#define epicsExportSharedSymbolsRestore_cadbrIntfAlarmCond
#endif

#include "daCatalogSingle.h"

#ifdef epicsExportSharedSymbolsRestore_cadbrIntfAlarmCond
#define epicsExportSharedSymbols
#endif

#include "shareLib.h"

namespace ca {
namespace dbr {

template < class T >
class CatAlarmCond : 
    public Catalog
{
public:
    typedef T DBR;
    CatAlarmCond ( const T & dbr );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
private:
    const T & m_dbr;
};

template < class T >
class MutAlarmCond : 
    public MutatorCatalog
{
public:
    typedef T DBR;
    MutAlarmCond ( T & dbr );
    void assignCatalog ( const Catalog & src );
private:
    T & m_dbr;
};

template < class T >
inline CatAlarmCond < T > :: 
    CatAlarmCond ( const T & dbr ) : 
    m_dbr ( dbr )
{
}

template < class T >
void CatAlarmCond < T > :: 
    traverse ( Introspector & introspector ) const
{
    introspector.reveal ( pi_status, m_dbr.status ); 
    introspector.reveal ( pi_severity, m_dbr.severity ); 
}

template < class T >
void CatAlarmCond < T > :: 
    find ( const Ident & id, Viewer & viewer ) const
{
    if ( id == pi_status ) {
        viewer.reveal ( m_dbr.status ); 
    }
    else if ( id == pi_severity ) {
        viewer.reveal ( m_dbr.severity ); 
    }
}

template < class T >
inline MutAlarmCond < T > :: 
    MutAlarmCond ( T & dbr ) : m_dbr ( dbr )
{
}

template < class T >
void MutAlarmCond < T > :: 
    assignCatalog ( const Catalog & src )
{
    Clerk clerk ( src );
    clerk.get ( pi_status, m_dbr.status );
    clerk.get ( pi_severity, m_dbr.severity );
}

} // end of namespace dbr
} // end of namespace ca

#endif // cadbrIntfAlarmCond_h

