
//
// cadbrIntfAlarmLimits.h,v 1.2 2009/11/24 01:34:27 jhill Exp
//
// Authors: Jeff Hill, johill@lanl.gov
//          Ralph Lange, Ralph.Lange@aps.anl.gov

#ifndef cadbrIntfAlarmLimits_h
#define cadbrIntfAlarmLimits_h

#ifdef epicsExportSharedSymbols
#undef epicsExportSharedSymbols
#define epicsExportSharedSymbolsRestore_cadbrIntfAlarmLimits
#endif

#include "daCatalog.h"

#ifdef epicsExportSharedSymbolsRestore_cadbrIntfAlarmLimits
#define epicsExportSharedSymbols
#endif

#include "shareLib.h"

namespace ca {
namespace dbr {

template < class T >
class CatAlarmLowerLimits : 
    public Catalog
{
public:
    typedef T DBR;
    CatAlarmLowerLimits ( const T & dbr );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
    void assign ( const Catalog & ctx );
private:
    const T & m_dbr;
};

template < class T >
class MutAlarmLowerLimits : 
    public MutatorCatalog
{
public:
    typedef T DBR;
    MutAlarmLowerLimits ( T & dbr );
    void assignCatalog ( const Catalog & src );
private:
    T & m_dbr;
};

template < class T >
class CatAlarmUpperLimits : 
    public Catalog
{
public:
    typedef T DBR;
    CatAlarmUpperLimits ( const T & dbr );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
private:
    const T & m_dbr;
};

template < class T >
class MutAlarmUpperLimits : 
    public MutatorCatalog
{
public:
    typedef T DBR;
    MutAlarmUpperLimits ( T & dbr );
    void assignCatalog ( const Catalog & src );
private:
    T & m_dbr;
};

template < class T >
class CatAlarmLimits : 
    public Catalog
{
public:
    typedef T DBR;
    CatAlarmLimits ( const T & dbr );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
    void assign ( const Catalog & );
private:
    CatAlarmLowerLimits < T > m_lower;
    CatAlarmUpperLimits < T > m_upper;
};

template < class T >
class MutAlarmLimits : 
    public MutatorCatalog
{
public:
    typedef T DBR;
    MutAlarmLimits ( T & dbr );
    void assignCatalog ( const Catalog & src );
private:
    MutAlarmLowerLimits < T > m_lower;
    MutAlarmUpperLimits < T > m_upper;
};

template < class T >
inline CatAlarmLowerLimits < T > :: 
    CatAlarmLowerLimits ( const T & dbr ) : 
    m_dbr ( dbr )
{
}

template < class T >
void CatAlarmLowerLimits < T > :: traverse ( 
    Introspector & introspector ) const
{
    introspector.reveal ( pi_major, m_dbr.lower_alarm_limit ); 
    introspector.reveal ( pi_minor, m_dbr.lower_warning_limit ); 
}

template < class T >
void CatAlarmLowerLimits < T > :: find ( 
    const Ident & id, Viewer & viewer ) const
{
    if ( id == pi_major ) {
        viewer.reveal ( m_dbr.lower_alarm_limit ); 
    }
    else if ( id == pi_minor ) {
        viewer.reveal ( m_dbr.lower_warning_limit ); 
    }
}

template < class T >
inline MutAlarmLowerLimits < T > :: 
    MutAlarmLowerLimits ( T & dbr ) : 
    m_dbr ( dbr )
{
}

template < class T >
void MutAlarmLowerLimits < T > :: 
    assignCatalog ( const Catalog & src )
{
    Clerk clerk ( src );
    clerk.get ( pi_major, m_dbr.lower_alarm_limit );
    clerk.get ( pi_minor, m_dbr.lower_warning_limit );
}

template < class T >
inline CatAlarmUpperLimits < T > :: 
    CatAlarmUpperLimits ( const T & dbr ) : 
    m_dbr ( dbr )
{
}

template < class T >
void CatAlarmUpperLimits < T > :: 
    traverse ( Introspector & introspector ) const
{
    introspector.reveal ( pi_major, m_dbr.upper_alarm_limit ); 
    introspector.reveal ( pi_minor, m_dbr.upper_warning_limit ); 
}

template < class T >
void CatAlarmUpperLimits < T > :: 
    find ( const Ident & id, Viewer & viewer ) const
{
    if ( id == pi_major ) {
        viewer.reveal ( m_dbr.upper_alarm_limit ); 
    }
    else if ( id == pi_minor ) {
        viewer.reveal ( m_dbr.upper_warning_limit ); 
    }
}

template < class T >
inline MutAlarmUpperLimits < T > :: 
    MutAlarmUpperLimits ( T & dbr ) : 
    m_dbr ( dbr )
{
}

template < class T >
void MutAlarmUpperLimits < T > :: 
    assignCatalog ( const Catalog & src )
{
    Clerk clerk ( src );
    clerk.get ( pi_major, m_dbr.upper_alarm_limit );
    clerk.get ( pi_minor, m_dbr.upper_warning_limit );
}

template < class T >
inline CatAlarmLimits < T > :: 
    CatAlarmLimits ( const T & dbr ) : 
    m_lower ( dbr ), m_upper ( dbr )
{
}

template < class T >
void CatAlarmLimits < T > :: traverse ( 
    Introspector & introspector ) const
{
    introspector.reveal ( pi_lower, m_lower );
    introspector.reveal ( pi_upper, m_upper );
}

template < class T >
void CatAlarmLimits < T > :: find ( 
    const Ident & id, Viewer & viewer ) const
{
    if ( id == pi_upper ) {
        viewer.reveal ( m_upper );
    }
    else if ( id == pi_lower ) {
        viewer.reveal ( m_lower );
    }
}

template < class T >
inline MutAlarmLimits < T > :: 
    MutAlarmLimits ( T & dbr ) :
    m_lower ( dbr ), m_upper ( dbr )
{
}

template < class T >
void MutAlarmLimits < T > :: 
    assignCatalog ( const Catalog & src )
{
    Clerk clerk ( src );
    clerk.get ( pi_lower, m_lower );
    clerk.get ( pi_upper, m_upper );
}

} // end of namespace dbr
} // end of namespace ca

#endif // cadbrIntfAlarmLimits_h

