
//
// Author: Jeff Hill, johill@lanl.gov
//

#ifndef cadbrIntfAtomic_h
#define cadbrIntfAtomic_h

#ifdef epicsExportSharedSymbols
#undef epicsExportSharedSymbols
#define epicsExportSharedSymbolsRestore_cadbrIntfAtomic
#endif

#include "daCatalogSingle.h"
#include "daCatalogSingleNexus.h"
#include "daMutatorNexus.h"
#include "daMutatorAtomic.h"
#include "daStringInFixedLenCharArray.h"

#ifdef epicsExportSharedSymbolsRestore_cadbrIntfAtomic
#define epicsExportSharedSymbols
#endif

#include "shareLib.h"

namespace ca {
namespace dbr {

template < class T >
struct IntfTraitsAtomic  
{
    typedef T DBR;
    typedef CatalogSingle < pi_value, const T & > CatScalar;
    typedef CatalogSingle < pi_value, ArrayConstVector < T > > CatVector;
    typedef MutatorAtomic < T > MutScalar;
    typedef MutatorAtomic < ArrayVector < T > > MutVector;
};

template < unsigned DBR_TYPE_CODE > class TypeCodeTraits;

template <>
struct TypeCodeTraits < DBR_STRING > 
{
    typedef dbr_string_t DBR;
    typedef StringInFixedLenCharArray :: 
        ScalarIntfConst < MAX_STRING_SIZE > StringRefConst;
    typedef CatalogSingle < pi_value, StringRefConst > CatScalar;
    typedef CatalogSingle < pi_value, ArrayConstVector < StringRefConst > > CatVector;
    typedef StringInFixedLenCharArray :: 
        ScalarIntf < MAX_STRING_SIZE > StringRef;
    typedef MutatorAtomic < StringRef > MutScalar;
    typedef MutatorAtomic < ArrayVector < StringRef > > MutVector;
};

template <>
struct TypeCodeTraits < DBR_SHORT > :
    public IntfTraitsAtomic < dbr_short_t >
{
};

template <>
struct TypeCodeTraits < DBR_FLOAT > :
    public IntfTraitsAtomic < dbr_float_t >
{
};

template <>
struct TypeCodeTraits < DBR_ENUM > :
    public IntfTraitsAtomic < dbr_enum_t >
{
};

template <>
struct TypeCodeTraits < DBR_CHAR > :
    public IntfTraitsAtomic < dbr_char_t >
{
};

template <>
struct TypeCodeTraits < DBR_LONG > :
    public IntfTraitsAtomic < dbr_long_t >
{
};

template <>
struct TypeCodeTraits < DBR_DOUBLE > :
    public IntfTraitsAtomic < dbr_double_t >
{
};

template <>
struct TypeCodeTraits < DBR_PUT_ACKT > 
{
    typedef dbr_put_ackt_t DBR;
    // pi_value / pi_alarm / pi_acknowledge / pi_pending
    // A CatalogSingleNexus is used for the pi_value property
    // because the pi_value doesnt have a value (but does have
    // facets in this situation.
    typedef CatalogSingle < pi_pending, dbr_put_ackt_t & > CatPending;
    typedef CatalogSingle < pi_acknowledge, CatPending > CatAcknowledge;
    typedef CatalogSingle < pi_alarm, CatAcknowledge > CatAlarm;
    typedef CatalogSingleNexus < pi_value, CatAlarm > CatScalar;
    // pi_alarm / pi_acknowledge / pi_pending
    typedef MutatorAtomic < dbr_put_ackt_t & > MutVal;
    typedef MutatorNexus < pi_pending, MutVal > MutPending;
    typedef MutatorNexus < pi_acknowledge, MutPending > MutAcknowledge;
    typedef MutatorNexus < pi_alarm, MutAcknowledge > MutScalar;
};

template <>
struct TypeCodeTraits < DBR_PUT_ACKS > 
{
    typedef dbr_put_acks_t DBR;
    // pi_value / pi_alarm / pi_acknowledge / pi_severity
    // A CatalogSingleNexus is used for the pi_value property
    // because the pi_value normally has a value, but doesnt
    // in this situation.
    typedef CatalogSingle < pi_pending, dbr_put_acks_t & > CatSeverity;
    typedef CatalogSingle < pi_acknowledge, CatSeverity > CatAcknowledge;
    typedef CatalogSingle < pi_alarm, CatAcknowledge > CatAlarm;
    typedef CatalogSingleNexus < pi_value, CatAlarm > CatScalar;
    // pi_alarm / pi_acknowledge / pi_severity
    typedef MutatorAtomic < dbr_put_acks_t & > MutVal;
    typedef MutatorNexus < pi_severity, MutVal > MutSeverity;
    typedef MutatorNexus < pi_acknowledge, MutSeverity > MutAcknowledge;
    typedef MutatorNexus < pi_alarm, MutAcknowledge > MutScalar;
};

template <>
struct TypeCodeTraits < DBR_CLASS_NAME > 
{
    typedef dbr_class_name_t DBR;
    // pi_value / pi_class / pi_name
    // A CatalogSingleNexus is used for the pi_value property
    // because the pi_value normally has a value, but doesnt
    // in this situation.
    typedef StringInFixedLenCharArray ::
        ScalarIntf < MAX_STRING_SIZE > StringRef;
    typedef CatalogSingle < pi_name, StringRef > CatName;
    typedef CatalogSingle < pi_class, CatName > CatClass;
    typedef CatalogSingleNexus < pi_value, CatClass > CatScalar;
    // pi_class / pi_name
    typedef MutatorAtomic < StringRef > MutVal;
    typedef MutatorNexus < pi_name, MutVal > MutName;
    typedef MutatorNexus < pi_class, MutName > MutScalar;
};

} // end of namespace dbr
} // end of namespace ca

#endif // ifdef cadbrIntfAtomic_h
