
//
// cadbrIntfControlLimits.h,v 1.2 2009/11/24 01:34:27 jhill Exp
//
// Authors: Jeff Hill, johill@lanl.gov
//          Ralph Lange, Ralph.Lange@aps.anl.gov

#ifndef cadbrIntfControlLimits_h
#define cadbrIntfControlLimits_h

#ifdef epicsExportSharedSymbols
#undef epicsExportSharedSymbols
#define epicsExportSharedSymbolsRestore_cadbrIntfControlLimits
#endif

#include "daCatalog.h"

#ifdef epicsExportSharedSymbolsRestore_cadbrIntfControlLimits
#define epicsExportSharedSymbols
#endif

#include "shareLib.h"

namespace ca {
namespace dbr {

template < class T >
class CatControlLimits : 
    public Catalog
{
public:
    CatControlLimits ( const T & dbr );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
private:
    const T & m_dbr;
};

template < class T >
class MutControlLimits : 
    public MutatorCatalog
{
public:
    MutControlLimits ( T & dbr );
    void assignCatalog ( const Catalog & src );
private:
    T & m_dbr;
};

template < class T >
inline CatControlLimits < T > :: 
    CatControlLimits ( const T & dbr ) : 
    m_dbr ( dbr )
{
}

template < class T >
void CatControlLimits < T > :: 
    traverse ( Introspector & i ) const
{
    i.reveal ( pi_upper, m_dbr.upper_ctrl_limit ); 
    i.reveal ( pi_lower, m_dbr.lower_ctrl_limit ); 
}

template < class T >
void CatControlLimits < T > :: 
    find ( const Ident & id, Viewer & v ) const
{
    if ( id == pi_upper ) {
        v.reveal ( m_dbr.upper_ctrl_limit ); 
    }
    else if ( id == pi_lower ) {
        v.reveal ( m_dbr.lower_ctrl_limit ); 
    }
}

template < class T >
inline MutControlLimits < T > :: 
    MutControlLimits ( T & dbr ) : 
    m_dbr ( dbr )
{
}

template < class T >
void MutControlLimits < T > :: 
    assignCatalog ( const Catalog & src )
{
    Clerk clerk ( src );
    clerk.get ( pi_upper, m_dbr.upper_ctrl_limit ); 
    clerk.get ( pi_lower, m_dbr.lower_ctrl_limit ); 
}

} // end of namespace dbr
} // end of namespace ca

#endif // cadbrIntfControlLimits_h

