
//
// Author: Jeff Hill, johill@lanl.gov
//

#ifndef cadbrIntfCtrl_h
#define cadbrIntfCtrl_h

#ifdef epicsExportSharedSymbols
#undef epicsExportSharedSymbols
#define epicsExportSharedSymbolsRestore_cadbrIntfCtrl
#endif

#include "daMutatorCatalogSingle.h"
#include "daStringInFixedLenCharArray.h"

#ifdef epicsExportSharedSymbolsRestore_cadbrIntfCtrl
#define epicsExportSharedSymbols
#endif

#include "cadbrIntfEnum.h"
#include "cadbrIntfAlarmCond.h"
#include "cadbrIntfCtrlLimits.h"

#include "shareLib.h"

namespace ca {
namespace dbr {

template < class T >
class CatSubordCtrl : 
    public Catalog
{
public:
    typedef T DBR;
    CatSubordCtrl ( const DBR & );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
    void assign ( const Catalog & );
private:
    typedef CatAlarmCond < T > CatAlarmCond;
    CatalogSingle < pi_condition, CatAlarmCond > m_catAlarm;
    CatCtrlLimits < T > m_catCtrlLimits;
    StringInFixedLenCharArray :: 
        ScalarIntfConst < MAX_UNITS_SIZE > m_stringReaderUnits;
};

template < class T >
class MutSubordCtrl : 
    public MutatorCatalog
{
public:
    typedef T DBR;
    MutSubordCtrl ( T & dbr );
    void assignCatalog ( const Catalog & src );
private:
    typedef MutAlarmCond < T > MutAlarmCond;
    MutatorCatalogSingle < pi_condition, MutAlarmCond > m_mutAlarm;
    MutCtrlLimits < T > m_mutCtrlLimits;
    StringInFixedLenCharArray :: 
        ScalarIntf < MAX_UNITS_SIZE > m_stringEditorUnits;
};

template < class T >
class CatSubordCtrlFloat : 
    public CatSubordCtrl < T >
{
public:
    typedef T DBR;
    CatSubordCtrlFloat ( const T & );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
    void assign ( const Catalog & );
private:
    const DBR & m_dbr;
};

template < class T >
class MutSubordCtrlFloat : 
    public MutSubordCtrl < T >
{
public:
    typedef T DBR;
    MutSubordCtrlFloat ( T & dbr );
    void assignCatalog ( const Catalog & src );
private:
    DBR & m_dbr;
};

template < class T >
struct IntfTraitsCtrlInt  
{
    typedef T DBR;
    typedef CatSubordCtrl < DBR > Subord;
    typedef CatCmpdScalar < Subord > CatScalar;
    typedef CatCmpdVector < Subord > CatVector;
    typedef MutSubordCtrl < DBR > MutSubord;
    typedef MutCmpdScalar < MutSubord > MutScalar;
    typedef MutCmpdVector < MutSubord > MutVector;
};

template < class T >
struct IntfTraitsCtrlFloat  
{
    typedef T DBR;
    typedef CatSubordCtrlFloat < DBR > Subord;
    typedef CatCmpdScalar < Subord > CatScalar;
    typedef CatCmpdVector < Subord > CatVector;
    typedef MutSubordCtrlFloat < DBR > MutSubord;
    typedef MutCmpdScalar < MutSubord > MutScalar;
    typedef MutCmpdVector < MutSubord > MutVector;
};

template <>
struct TypeCodeTraits < DBR_CTRL_SHORT > :
    public IntfTraitsCtrlInt < dbr_ctrl_short >
{
};

template <>
struct TypeCodeTraits < DBR_CTRL_CHAR > :
    public IntfTraitsCtrlInt < dbr_ctrl_char >
{
};

template <>
struct TypeCodeTraits < DBR_CTRL_LONG > :
    public IntfTraitsCtrlInt < dbr_ctrl_long >
{
};

template <>
struct TypeCodeTraits < DBR_CTRL_FLOAT > :
    public IntfTraitsCtrlFloat < dbr_ctrl_float >
{
};

template <>
struct TypeCodeTraits < DBR_CTRL_DOUBLE > :
    public IntfTraitsCtrlFloat < dbr_ctrl_double >
{
};

template < class T >
inline CatSubordCtrl < T > :: 
    CatSubordCtrl ( const T & dbr ) :
    m_catAlarm ( dbr ),
    m_catCtrlLimits ( dbr ),
    m_stringReaderUnits ( dbr.units )
{
}

template < class T >
void CatSubordCtrl < T > ::
    traverse ( Introspector & i ) const
{
    i.reveal ( pi_alarm, m_catAlarm );
    i.reveal ( pi_limits, m_catCtrlLimits );
    i.reveal ( pi_units, m_stringReaderUnits );
}

template < class T >
void CatSubordCtrl < T > :: 
    find ( const Ident & id, Viewer & v ) const
{
    if ( id == pi_alarm ) {
        v.reveal ( m_catAlarm );
    }
    else if ( id == pi_limits ) {
        v.reveal ( m_catCtrlLimits );
    }
    else if ( id == pi_units ) {
        v.reveal ( m_stringReaderUnits );
    }
}

template < class T >
inline MutSubordCtrl < T > :: 
    MutSubordCtrl ( T & dbr ) :
    m_mutAlarm ( dbr ),
    m_mutCtrlLimits ( dbr ),
    m_stringEditorUnits ( dbr.units )
{
}

template < class T >
inline void MutSubordCtrl < T > :: 
    assignCatalog ( const Catalog & src ) 
{
    Clerk clerk ( src );
    clerk.get ( pi_alarm, m_mutAlarm );
    clerk.get ( pi_limits, m_mutCtrlLimits );
    clerk.get ( pi_units, m_stringEditorUnits );
}

template < class T >
inline CatSubordCtrlFloat < T > :: 
    CatSubordCtrlFloat ( const T & dbr ) :
    CatSubordCtrl < T > ( dbr ), m_dbr ( dbr )
{
}

template < class T >
void CatSubordCtrlFloat < T > :: 
    traverse ( Introspector & i ) const
{
    CatSubordCtrl < T > :: traverse ( i );
    i.reveal ( pi_precision, m_dbr.precision );
}

template < class T >
void CatSubordCtrlFloat < T > :: 
    find ( const Ident & id, Viewer & v ) const
{
    if ( id == pi_precision ) {
        v.reveal ( m_dbr.precision );
    }
    else {
        CatSubordCtrl < DBR > :: find ( id, v );
    }
}

template < class T >
inline MutSubordCtrlFloat < T > :: 
    MutSubordCtrlFloat ( T & dbr ) :
    MutSubordCtrl < T > ( dbr ), 
    m_dbr ( dbr )
{
}

template < class T >
inline void MutSubordCtrlFloat < T > :: 
    assignCatalog ( const Catalog & src )
{
    MutSubordCtrl < DBR > :: assignCatalog ( src );
    Clerk ( src ).get ( pi_precision, m_dbr.precision );
}

} // end of namespace dbr
} // end of namespace ca

#endif // ifdef cadbrIntfCtrl_h
