
//
// cadbrIntfCtrlLimits.h,v 1.2 2009/11/24 01:34:28 jhill Exp
//
// Authors: Jeff Hill, johill@lanl.gov
//          Ralph Lange, Ralph.Lange@aps.anl.gov

#ifndef cadbrIntfCtrlLimits_h
#define cadbrIntfCtrlLimits_h

#ifdef epicsExportSharedSymbols
#undef epicsExportSharedSymbols
#define epicsExportSharedSymbolsRestore_cadbrIntfCtrlLimits
#endif

#include "daCatalog.h"

#ifdef epicsExportSharedSymbolsRestore_cadbrIntfCtrlLimits
#define epicsExportSharedSymbols
#endif

#include "cadbrIntfControlLimits.h"
#include "cadbrIntfDisplayLimits.h"
#include "cadbrIntfAlarmLimits.h"

#include "shareLib.h"

namespace ca {
namespace dbr {

template < class T >
class CatCtrlLimits : 
    public Catalog
{
public:
    CatCtrlLimits ( const T & dbr );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
private:
    CatAlarmLimits < T > m_alarm;
    CatDisplayLimits < T > m_display;
    CatControlLimits < T > m_ctrl;
};

template < class T >
class MutCtrlLimits : 
    public MutatorCatalog
{
public:
    MutCtrlLimits ( T & dbr );
    void assignCatalog ( const Catalog & src );
private:
    MutAlarmLimits < T > m_alarm;
    MutDisplayLimits < T > m_display;
    MutControlLimits < T > m_ctrl;
};

template < class T >
inline CatCtrlLimits < T > :: 
    CatCtrlLimits ( const T & dbr ) : 
    m_alarm ( dbr ), m_display ( dbr ), 
        m_ctrl ( dbr )
{
}

template < class T >
void CatCtrlLimits < T > :: 
    traverse ( Introspector & i ) const
{
    i.reveal ( pi_alarm, m_alarm );
    i.reveal ( pi_display, m_display );
    i.reveal ( pi_control, m_ctrl );
}

template < class T >
void CatCtrlLimits < T > :: 
    find ( const Ident & id, Viewer & v ) const
{
    if ( id == pi_alarm ) {
        v.reveal ( m_alarm );
    }
    else if ( id == pi_display ) {
        v.reveal ( m_display );
    }
    else if ( id == pi_control ) {
        v.reveal ( m_ctrl );
    }
}

template < class T >
inline MutCtrlLimits < T > :: 
    MutCtrlLimits ( T & dbr ) : 
    m_alarm ( dbr ), m_display ( dbr ), 
        m_ctrl ( dbr )
{
}

template < class T >
void MutCtrlLimits < T > :: 
    assignCatalog ( const Catalog & src )
{
    Clerk clerk ( src );
    clerk.get ( pi_alarm, m_alarm );
    clerk.get ( pi_display, m_display );
    clerk.get ( pi_control, m_ctrl );
}

} // end of namespace dbr
} // end of namespace ca

#endif // cadbrIntfCtrlLimits_h

