
//
// cadbrIntfDisplayLimits.h,v 1.2 2009/11/24 01:34:28 jhill Exp
//
// Authors: Jeff Hill, johill@lanl.gov
//          Ralph Lange, Ralph.Lange@aps.anl.gov

#ifndef cadbrIntfDisplayLimits_h
#define cadbrIntfDisplayLimits_h

#ifdef epicsExportSharedSymbols
#undef epicsExportSharedSymbols
#define epicsExportSharedSymbolsRestore_cadbrIntfDisplayLimits
#endif

#include "daCatalog.h"

#ifdef epicsExportSharedSymbolsRestore_cadbrIntfDisplayLimits
#define epicsExportSharedSymbols
#endif

#include "shareLib.h"

namespace ca {
namespace dbr {

template < class T >
class CatDisplayLimits : 
    public Catalog
{
public:
    CatDisplayLimits ( const T & dbr );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
private:
    const T & m_dbr;
};

template < class T >
class MutDisplayLimits : 
    public MutatorCatalog
{
public:
    MutDisplayLimits ( T & dbr );
    void assignCatalog (  const Catalog & src );
private:
    T & m_dbr;
};

template < class T >
inline CatDisplayLimits < T > :: 
    CatDisplayLimits ( const T & dbr ) : 
    m_dbr ( dbr )
{
}

template < class T >
void CatDisplayLimits < T > :: traverse ( 
    Introspector & introspector ) const
{
    introspector.reveal ( pi_upper, m_dbr.upper_disp_limit ); 
    introspector.reveal ( pi_lower, m_dbr.lower_disp_limit ); 
}

template < class T >
void CatDisplayLimits < T > :: find ( 
    const Ident & id, Viewer & viewer ) const
{
    if ( id == pi_upper ) {
        viewer.reveal ( m_dbr.upper_disp_limit ); 
    }
    else if ( id == pi_lower ) {
        viewer.reveal ( m_dbr.lower_disp_limit ); 
    }
}

template < class T >
inline MutDisplayLimits < T > :: 
    MutDisplayLimits ( T & dbr ) : 
    m_dbr ( dbr )
{
}

template < class T >
void MutDisplayLimits < T > :: 
    assignCatalog ( const Catalog & src )
{
    Clerk clerk ( src );
    clerk.get ( pi_lower, m_dbr.lower_disp_limit );
    clerk.get ( pi_upper, m_dbr.upper_disp_limit );
}

} // end of namespace dbr
} // end of namespace ca

#endif // cadbrIntfDisplayLimits_h

