
//
// cadbrIntfEnum.h,v 1.2 2009/11/24 01:34:28 jhill Exp
//
// Authors: Jeff Hill, johill@lanl.gov
//          Ralph Lange, Ralph.Lange@aps.anl.gov

#ifndef cadbrIntfEnum_h
#define cadbrIntfEnum_h

#include <algorithm>

#ifdef epicsExportSharedSymbols
#undef epicsExportSharedSymbols
#define epicsExportSharedSymbolsRestore_cadbrIntfEnum
#endif

#include "daCatalog.h"
#include "daMutatorAtomicFaceted.h"
#include "daStringInFixedLenCharArray.h"

#ifdef epicsExportSharedSymbolsRestore_cadbrIntfEnum
#define epicsExportSharedSymbols
#endif

#include "shareLib.h"

#include "cadbrIntfNoElem.h"

namespace ca {
namespace dbr {

template < class T >
class CatSubordEnum :
    public Catalog
{
public:
    typedef T DBR;
    CatSubordEnum ( const T & );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
private:
    class CatStateSet :
        public Catalog
    {
    public:
        typedef T DBR;
        CatStateSet ( const T & );
        void traverse ( Introspector & ) const;
        void find ( const Ident & id, Viewer & ) const;
    private:
        const T & m_dbr;
        typedef char MenuFixedLenStr [ MAX_ENUM_STRING_SIZE ];
        ArrayConstVector < MenuFixedLenStr > m_arrayIntfMenu;
        CatNoElem < T > m_noElems;
    };
    typedef CatAlarmCond < T > CatAlarmCond;
    CatalogSingle < pi_condition, CatAlarmCond > m_catAlarm;
    CatStateSet m_catStateSet;
};

template < class T >
class MutSubordEnum :
    public MutatorCatalog
{
public:
    typedef T DBR;
    MutSubordEnum ( T & );
    void assignCatalog ( const Catalog & );
private:
    class MutStateSet :
        public MutatorCatalog
    {
    public:
        typedef T DBR;
        MutStateSet ( T & );
        void assignCatalog ( const Catalog & );
    private:
        T & m_dbr;
        typedef char MenuFixedLenStr [ MAX_ENUM_STRING_SIZE ];
        ArrayVector < MenuFixedLenStr > m_arrayIntfMenu;
        MutNoElem < T > m_noElems;
    };
    typedef MutAlarmCond < T > MutAlarmCond;
    MutatorCatalogSingle < pi_condition, MutAlarmCond > m_mutAlarm;
    MutStateSet m_mutStateSet;
};


template < class T >
struct IntfTraitsEnum 
{
    typedef T DBR;
    typedef CatSubordEnum < DBR > CatSubord;
    typedef CatCmpdScalar < CatSubord > CatScalar;
    typedef CatCmpdVector < CatSubord > CatVector;
    typedef MutSubordEnum < DBR > MutSubord;
    typedef MutCmpdScalar < MutSubord > MutScalar;
    typedef MutCmpdVector < MutSubord > MutVector;
};

template <>
struct TypeCodeTraits < DBR_GR_ENUM > :
    public IntfTraitsEnum < dbr_gr_enum > 
{
};

template <>
struct TypeCodeTraits < DBR_CTRL_ENUM > :
    public IntfTraitsEnum < dbr_ctrl_enum > 
{
};

template < class T >
inline CatSubordEnum < T > :: 
    CatSubordEnum ( const T & dbr ) : 
    m_catAlarm ( dbr ), m_catStateSet ( dbr )
{
}

template < class T >
void CatSubordEnum < T > ::
    traverse ( Introspector & i ) const
{
    i.reveal ( pi_alarm, m_catAlarm );
    i.reveal ( pi_stateSet, m_catStateSet );
}

template < class T >
void CatSubordEnum < T > ::
    find ( const Ident & id, Viewer & v ) const
{
    if ( id == pi_alarm ) {
        v.reveal ( m_catAlarm );
    }
    else if ( id == pi_stateSet ) {
        v.reveal ( m_catStateSet );
    }
}

template < class T >
inline CatSubordEnum < T > :: CatStateSet :: 
    CatStateSet ( const T & dbr ) :
    m_dbr ( dbr ),
    m_arrayIntfMenu ( dbr.strs, dbr.no_str ), 
    m_noElems ( dbr )
{
}

template < class T >
void CatSubordEnum < T > :: CatStateSet :: 
    traverse ( Introspector & i ) const
{
    i.reveal ( pi_labels, m_arrayIntfMenu, m_noElems );
}

template < class T >
void CatSubordEnum < T > :: CatStateSet :: 
    find ( const Ident & id, Viewer & v ) const
{
    if ( id == pi_labels ) {
        v.reveal ( m_arrayIntfMenu, m_noElems );
    }
}

template < class T >
inline MutSubordEnum < T > :: 
    MutSubordEnum ( T & dbr ) : 
    m_mutAlarm ( dbr ), m_mutStateSet ( dbr )
{
}

template < class T >
void MutSubordEnum < T > ::
    assignCatalog ( const Catalog & src )
{
    Clerk clerk ( src );
    clerk.get ( pi_alarm, m_mutAlarm );
    clerk.get ( pi_stateSet, m_mutStateSet );
}

template < class T >
inline MutSubordEnum < T > :: MutStateSet :: 
    MutStateSet ( T & dbr ) :
    m_dbr ( dbr ),
    m_arrayIntfMenu ( dbr.strs, dbr.no_str ), 
    m_noElems ( dbr )
{
}

template < class T >
void MutSubordEnum < T > :: MutStateSet :: 
    assignCatalog ( const Catalog & src )
{
    // PropertyNodeAssignerRef < T > implementation ensures that 
    // number of elements subordinate is written before writing 
    // the labels (so they can safely change the number of 
    // labels)
    MutatorAtomicFaceted < Array &, Mutator & > 
        mutatorAtomicFaceted ( m_arrayIntfMenu, m_noElems );
    Clerk ( src ).get ( pi_labels, mutatorAtomicFaceted );
    // force unused strings to zero
    memset ( &m_dbr.strs[ m_dbr.no_str ], '\0',  
        sizeof ( m_dbr.strs[0] ) * 
            ( MAX_ENUM_STATES - m_dbr.no_str ) );
}

} // end of namespace dbr
} // end of namespace ca

#endif // cadbrIntfEnum_h

