
//
// Author: Jeff Hill, johill@lanl.gov
//

#ifndef cadbrIntfGr_h
#define cadbrIntfGr_h

#ifdef epicsExportSharedSymbols
#undef epicsExportSharedSymbols
#define epicsExportSharedSymbolsRestore_cadbrIntfGr
#endif

#include "daStringInFixedLenCharArray.h"

#ifdef epicsExportSharedSymbolsRestore_cadbrIntfGr
#define epicsExportSharedSymbols
#endif

#include "cadbrIntfAlarmCond.h"
#include "cadbrIntfEnum.h"
#include "cadbrIntfGrLimits.h"

#include "shareLib.h"

namespace ca {
namespace dbr {

template < class T >
class CatSubordGr : 
    public Catalog
{
public:
    typedef T DBR;
    CatSubordGr ( const T & );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
private:
    typedef CatAlarmCond < T > CatAlarmCond;
    CatalogSingle < pi_condition, CatAlarmCond > m_catAlarm;
    CatGrLimits < T > m_catGrLimits;
    StringInFixedLenCharArray :: 
        ScalarIntfConst < MAX_UNITS_SIZE > m_stringReaderUnits;
};

template < class T >
class MutSubordGr : 
    public MutatorCatalog
{
public:
    typedef T DBR;
    MutSubordGr ( T & );
    void assignCatalog ( const Catalog & src );
private:
    typedef MutAlarmCond < T > MutAlarmCond;
    MutatorCatalogSingle < pi_condition, MutAlarmCond > m_mutAlarm;
    MutGrLimits < T > m_mutGrLimits;
    StringInFixedLenCharArray :: 
        ScalarIntf < MAX_UNITS_SIZE > m_stringEditorUnits;
};

template < class T >
class CatSubordGrFloat : 
    public CatSubordGr < T >
{
public:
    typedef T DBR;
    CatSubordGrFloat ( const T & );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
    void assign ( const Catalog & );
private:
    const T & m_dbr;
};

template < class T >
class MutSubordGrFloat : 
    public MutSubordGr < T >
{
public:
    typedef T DBR;
    MutSubordGrFloat ( T & );
    void assignCatalog ( const Catalog & src );
private:
    T & m_dbr;
};

template < class T >
struct IntfTraitsGrInt  
{
    typedef T DBR;
    typedef CatSubordGr < DBR > Subord;
    typedef CatCmpdScalar < Subord > CatScalar;
    typedef CatCmpdVector < Subord > CatVector;
    typedef MutSubordGr < DBR > MutSubord;
    typedef MutCmpdScalar < MutSubord > MutScalar;
    typedef MutCmpdVector < MutSubord > MutVector;
};

template < class T >
struct IntfTraitsGrFloat  
{
    typedef T DBR;
    typedef CatSubordGrFloat < DBR > Subord;
    typedef CatCmpdScalar < Subord > CatScalar;
    typedef CatCmpdVector < Subord > CatVector;
    typedef MutSubordGrFloat < DBR > MutSubord;
    typedef MutCmpdScalar < MutSubord > MutScalar;
    typedef MutCmpdVector < MutSubord > MutVector;
};

template < unsigned DBR_TYPE > class TypeCodeTraits;

template <>
struct TypeCodeTraits < DBR_GR_SHORT > :
    public IntfTraitsGrInt < dbr_gr_short > 
{
};

template <>
struct TypeCodeTraits < DBR_GR_CHAR > :
    public IntfTraitsGrInt < dbr_gr_char >  
{
};

template <>
struct TypeCodeTraits < DBR_GR_LONG > :
    public IntfTraitsGrInt < dbr_gr_long > 
{
};

template <>
struct TypeCodeTraits < DBR_GR_FLOAT > :
    public IntfTraitsGrFloat < dbr_gr_float > 
{
};

template <>
struct TypeCodeTraits < DBR_GR_DOUBLE > :
    public IntfTraitsGrFloat < dbr_gr_double > 
{
};

template < class T >
inline CatSubordGr < T > :: 
    CatSubordGr ( const T & dbr ) :
    m_catAlarm ( dbr ),
    m_catGrLimits ( dbr ),
    m_stringReaderUnits ( dbr.units )
{
}

template < class T >
void CatSubordGr < T > :: 
    traverse ( Introspector & i ) const
{
    i.reveal ( pi_alarm, m_catAlarm );
    i.reveal ( pi_limits, m_catGrLimits );
    i.reveal ( pi_units, m_stringReaderUnits );
}

template < class T >
void CatSubordGr < T > :: 
    find ( const Ident & id, Viewer & v ) const
{
    if ( id == pi_alarm ) {
        v.reveal ( m_catAlarm );
    }
    else if ( id == pi_limits ) {
        v.reveal ( m_catGrLimits );
    }
    else if ( id == pi_units ) {
        v.reveal ( m_stringReaderUnits );
    }
}

template < class T >
inline MutSubordGr < T > :: 
    MutSubordGr ( T & dbr ) :
    m_mutAlarm ( dbr ),
    m_mutGrLimits ( dbr ),
    m_stringEditorUnits ( dbr.units )
{
}

template < class T >
void MutSubordGr < T > :: 
    assignCatalog ( const Catalog & src )
{
    Clerk clerk ( src );
    clerk.get ( pi_alarm, m_mutAlarm );
    clerk.get ( pi_limits, m_mutGrLimits );
    clerk.get ( pi_units, m_stringEditorUnits );
}

template < class T >
inline CatSubordGrFloat < T > :: 
    CatSubordGrFloat ( const T & dbr ) :
    CatSubordGr < T > ( dbr ), m_dbr ( dbr )
{
}

template < class T >
void CatSubordGrFloat < T > :: 
    traverse ( Introspector & i ) const
{
    CatSubordGr < T > :: traverse ( i );
    i.reveal ( pi_precision, m_dbr.precision );
}

template < class T >
void CatSubordGrFloat < T > :: find ( 
    const Ident & id, Viewer & v ) const
{
    if ( id == pi_precision ) {
        v.reveal ( m_dbr.precision );
    }
    else {
        CatSubordGr < T > :: find ( id, v );
    }
}

template < class T >
inline MutSubordGrFloat < T > :: 
    MutSubordGrFloat ( T & dbr ) :
    MutSubordGr < T > ( dbr ), 
    m_dbr ( dbr )
{
}

template < class T >
void MutSubordGrFloat < T > :: 
    assignCatalog ( const Catalog & src )
{
    MutSubordGr < T > :: assign ( src );
    Clerk ( src ).get ( pi_precision, m_dbr.precision );
}

} // end of namespace dbr
} // end of namespace ca

#endif // ifdef cadbrIntfGr_h
