
//
// cadbrIntfGrLimits.h,v 1.2 2009/11/24 01:34:28 jhill Exp
//
// Authors: Jeff Hill, johill@lanl.gov
//          Ralph Lange, Ralph.Lange@aps.anl.gov

#ifndef cadbrIntfGrLimits_h
#define cadbrIntfGrLimits_h

#ifdef epicsExportSharedSymbols
#undef epicsExportSharedSymbols
#define epicsExportSharedSymbolsRestore_cadbrIntfGrLimits
#endif

#include "daCatalog.h"

#ifdef epicsExportSharedSymbolsRestore_cadbrIntfGrLimits
#define epicsExportSharedSymbols
#endif

#include "cadbrIntfDisplayLimits.h"
#include "cadbrIntfAlarmLimits.h"

#include "shareLib.h"

namespace ca {
namespace dbr {

template < class T >
class CatGrLimits : 
    public Catalog
{
public:
    CatGrLimits ( const T & dbr );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
private:
    CatDisplayLimits < T >  m_display;
    CatAlarmLimits < T > m_alarm;
};

template < class T >
class MutGrLimits : 
    public MutatorCatalog
{
public:
    MutGrLimits ( T & dbr );
    void assignCatalog ( const Catalog & src );
private:
    MutDisplayLimits < T >  m_display;
    MutAlarmLimits < T > m_alarm;
};

template < class T >
inline CatGrLimits < T > :: 
    CatGrLimits ( const T & dbr ) : 
    m_display ( dbr ), 
    m_alarm ( dbr )
{
}

template < class T >
void CatGrLimits < T > :: 
    traverse ( Introspector & introspector ) const
{
    introspector.reveal ( pi_display, m_display );
    introspector.reveal ( pi_alarm, m_alarm );
}

template < class T >
void CatGrLimits < T > :: 
    find ( const Ident & id, Viewer & viewer ) const
{
    if ( id == pi_display ) {
        viewer.reveal ( m_display );
    }
    else if ( id == pi_alarm ) {
        viewer.reveal ( m_alarm );
    }
}

template < class T >
inline MutGrLimits < T > :: 
    MutGrLimits ( T & dbr ) : 
    m_display ( dbr ), 
    m_alarm ( dbr )
{
}

template < class T >
void MutGrLimits < T > :: 
    assignCatalog ( const Catalog & src )
{
    Clerk clerk ( src );
    clerk.get ( pi_display, m_display );
    clerk.get ( pi_alarm, m_alarm );
}

} // end of namespace dbr
} // end of namespace ca

#endif // cadbrIntfGrLimits_h

