
//
// cadbrIntfNoElem.h,v 1.2 2009/11/24 01:34:28 jhill Exp
//
// Authors: Jeff Hill, johill@lanl.gov
//          Ralph Lange, Ralph.Lange@aps.anl.gov

#ifndef cadbrIntfNoElem_h
#define cadbrIntfNoElem_h

#include <stdexcept>

#ifdef epicsExportSharedSymbols
#undef epicsExportSharedSymbols
#define epicsExportSharedSymbolsRestore_cadbrIntfNoElem
#endif

#include "daCatalog.h"

#ifdef epicsExportSharedSymbolsRestore_cadbrIntfNoElem
#define epicsExportSharedSymbols
#endif

#include "shareLib.h"

namespace ca {
namespace dbr {

template < class T >
class CatNoElem : 
    public Catalog
{
public:
    CatNoElem ( const T & dbr );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
private:
    const T & m_dbr;
};

template < class T >
class MutNoElem : 
    public MutatorCatalog
{
public:
    MutNoElem ( T & dbr );
    void assignCatalog ( const Catalog & src );
private:
    T & m_dbr;
};

template < class T >
inline CatNoElem < T > :: 
    CatNoElem ( const T & dbr ) : 
    m_dbr ( dbr )
{
}

template < class T >
void CatNoElem < T > :: 
    traverse ( Introspector & introspector ) const
{
    unsigned char nElem;
    if ( m_dbr.no_str > 0 ) {
        nElem = static_cast 
            < unsigned char > ( m_dbr.no_str );
    }
    else {
        nElem = 0u;
    }
    introspector.reveal ( pi_size, nElem ); 
}

template < class T >
void CatNoElem < T > :: 
    find ( const Ident & id, Viewer & viewer ) const
{
    if ( id == pi_size ) {
        unsigned char nElem;
        if ( m_dbr.no_str > 0 ) {
            nElem = static_cast 
                < unsigned char > ( m_dbr.no_str );
        }
        else {
            nElem = 0;
        }
        viewer.reveal ( nElem ); 
    }
}

template < class T >
inline MutNoElem < T > :: 
    MutNoElem ( T & dbr ) : 
    m_dbr ( dbr )
{
}

template < class T >
void MutNoElem < T > :: 
    assignCatalog ( const Catalog & src )
{
    unsigned noStr;
    Clerk ( src ).get ( pi_size, noStr );
    if ( noStr > MAX_ENUM_STATES ) {
        throw std :: out_of_range ( 
            "number of dbr enum strings exceeded "
            "MAX_ENUM_STATES" );
    }
    m_dbr.no_str = 
        static_cast < dbr_short_t > ( noStr );
}

} // end of namespace dbr
} // end of namespace ca

#endif // cadbrIntfNoElem_h

