
//
// Author: Jeff Hill, johill@lanl.gov
//

#ifndef cadbrIntfSts_h
#define cadbrIntfSts_h

#ifdef epicsExportSharedSymbols
#undef epicsExportSharedSymbols
#define epicsExportSharedSymbolsRestore_cadbrIntfSts
#endif

#include "daCatalogSingle.h"
#include "daMutatorCatalogSingle.h"
#include "daStringInFixedLenCharArray.h"

#ifdef epicsExportSharedSymbolsRestore_cadbrIntfSts
#define epicsExportSharedSymbols
#endif

#include "cadbrIntfAlarmCond.h"
#include "cadbrIntfAlarmAck.h"

#include "shareLib.h"

namespace ca {
namespace dbr {

template < class T >
class CatSubordSts : 
    public Catalog {
public:
    typedef T DBR;
    CatSubordSts ( const DBR & );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
private:
    typedef CatAlarmCond < DBR > CatAlarmCond;
    typedef CatalogSingle < pi_condition, CatAlarmCond > CatAlarm;
    CatAlarm m_catAlarm;
};

template < class T >
class MutSubordSts : 
    public MutatorCatalog {
public:
    typedef T DBR;
    MutSubordSts ( DBR & );
    void assignCatalog ( const Catalog & src );
private:
    typedef MutAlarmCond < DBR > MutAlarmCond;
    typedef MutatorCatalogSingle < pi_condition, MutAlarmCond > MutAlarm;
    MutAlarm m_mutAlarm;
};

template < class T >
class CatSubordStsAck : 
    public Catalog {
public:
    typedef T DBR;
    CatSubordStsAck ( const DBR & );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
private:
    class CatAlarm :
        public Catalog
    {
    public:
        CatAlarm ( const T & );
        void traverse ( Introspector & ) const;
        void find ( const Ident & id, Viewer & ) const;
    private:
        CatAlarmCond < T > m_catAlarmCond;
        CatAlarmAck < T > m_catAlarmAck;
    };
    CatAlarm m_catAlarm;
};

template < class T >
class MutSubordStsAck : 
    public MutatorCatalog {
public:
    typedef T DBR;
    MutSubordStsAck ( DBR & );
    void assignCatalog ( const Catalog & src );
private:
    class MutAlarm :
        public MutatorCatalog
    {
    public:
        MutAlarm ( T & );
        void assignCatalog ( const Catalog & src );
    private:
        MutAlarmCond < T > m_mutAlarmCond;
        MutAlarmAck < T > m_mutAlarmAck;
    };
    MutAlarm m_mutAlarm;
};

template < class T >
struct IntfTraitsSts  
{
    typedef T DBR;
    typedef CatSubordSts < DBR > CatSubord;
    typedef CatCmpdScalar < CatSubord > CatScalar;
    typedef CatCmpdVector < CatSubord > CatVector;
    typedef MutSubordSts < DBR > MutSubord;
    typedef MutCmpdScalar < MutSubord > MutScalar;
    typedef MutCmpdVector < MutSubord > MutVector;
};

template < unsigned DBR_TYPE > class TypeCodeTraits;

template <>
struct TypeCodeTraits < DBR_STS_STRING > :
    public IntfTraitsSts < dbr_sts_string >
{
};

template <>
struct TypeCodeTraits < DBR_STS_SHORT > :
    public IntfTraitsSts < dbr_sts_short > 
{
};

template <>
struct TypeCodeTraits < DBR_STS_FLOAT > :
    public IntfTraitsSts < dbr_sts_float > 
{
};

template <>
struct TypeCodeTraits < DBR_STS_ENUM > :
    public IntfTraitsSts < dbr_sts_enum > 
{
};

template <>
struct TypeCodeTraits < DBR_STS_CHAR > :
    public IntfTraitsSts < dbr_sts_char > 
{
};

template <>
struct TypeCodeTraits < DBR_STS_LONG > :
    public IntfTraitsSts < dbr_sts_long > 
{
};

template <>
struct TypeCodeTraits < DBR_STS_DOUBLE > :
    public IntfTraitsSts < dbr_sts_double > 
{
};

template <>
struct TypeCodeTraits < DBR_STSACK_STRING >
{
    typedef dbr_stsack_string DBR;
    typedef CatSubordStsAck < DBR > CatSubord;
    typedef CatCmpdScalar < CatSubord > CatScalar;
    typedef CatCmpdVector < CatSubord > CatVector;
    typedef MutSubordStsAck < DBR > MutSubord;
    typedef MutCmpdScalar < MutSubord > MutScalar;
    typedef MutCmpdVector < MutSubord > MutVector;
};

template < class T >
inline CatSubordSts < T > ::
    CatSubordSts ( const T & dbr ) : 
    m_catAlarm ( dbr )
{
}

template < class T >
void CatSubordSts < T > ::  
    traverse ( Introspector & i ) const
{
    i.reveal ( pi_alarm, m_catAlarm );
}

template < class T >
void CatSubordSts < T > :: 
    find ( const Ident & id, Viewer & v ) const
{
    if ( id == pi_alarm ) {
        v.reveal ( m_catAlarm );
    }
}

template < class T >
inline MutSubordSts < T > ::
    MutSubordSts ( T & dbr ) : 
    m_mutAlarm ( dbr )
{
}

template < class T >
inline void MutSubordSts < T > :: 
    assignCatalog ( const Catalog & src )
{
    Clerk ( src ).get ( pi_alarm, m_mutAlarm );
}

template < class T >
inline CatSubordStsAck < T > ::
    CatSubordStsAck ( const T & dbr ) : 
    m_catAlarm ( dbr )
{
}

template < class T >
void CatSubordStsAck < T > ::  
    traverse ( Introspector & i ) const
{
    i.reveal ( pi_alarm, m_catAlarm );
}

template < class T >
void CatSubordStsAck < T > :: 
    find ( const Ident & id, Viewer & v ) const
{
    if ( id == pi_alarm ) {
        v.reveal ( m_catAlarm );
    }
}

template < class T >
inline CatSubordStsAck < T > :: CatAlarm ::
    CatAlarm ( const T & dbr ) : 
    m_catAlarmCond ( dbr ), 
    m_catAlarmAck ( dbr ) 
{
}

template < class T >
void CatSubordStsAck < T > :: CatAlarm ::
    traverse ( Introspector & i ) const
{
    i.reveal ( pi_condition, m_catAlarmCond ); 
    i.reveal ( pi_acknowledge, m_catAlarmAck );
}

template < class T >
void CatSubordStsAck < T > :: CatAlarm ::
    find ( const Ident & id, Viewer & v ) const
{
    if ( id == pi_condition ) {
        v.reveal ( m_catAlarmCond );
    }
    else if ( id == pi_acknowledge ) {
        v.reveal ( m_catAlarmAck );
    }
}

template < class T >
inline MutSubordStsAck < T > ::
    MutSubordStsAck ( T & dbr ) : 
    m_mutAlarm ( dbr )
{
}

template < class T >
inline void MutSubordStsAck < T > :: 
    assignCatalog ( const Catalog & src )
{
    Clerk ( src ).get ( pi_alarm, m_mutAlarm );
}

template < class T >
inline MutSubordStsAck < T > :: MutAlarm ::
    MutAlarm ( T & dbr ) : 
    m_mutAlarmCond ( dbr ), 
    m_mutAlarmAck ( dbr ) 
{
}

template < class T >
void MutSubordStsAck < T > :: MutAlarm ::
    assignCatalog ( const Catalog & src )
{
    Clerk clerk ( src );
    clerk.get ( pi_condition, m_mutAlarmCond );
    clerk.get ( pi_acknowledge, m_mutAlarmAck );
}

} // end of namespace dbr
} // end of namespace ca

#endif // ifdef cadbrIntfSts_h
