
//
// Author: Jeff Hill, johill@lanl.gov
//

#ifndef cadbrIntfTime_h
#define cadbrIntfTime_h

#include "daMutatorCatalogSingle.h"
#include "cadbrIntfAlarmCond.h"

#include "shareLib.h"

namespace ca {
namespace dbr {

template < class T >
class CatSubordTime : 
    public Catalog
{
public:
    typedef T DBR;
    CatSubordTime ( const T & );
    void traverse ( Introspector & ) const;
    void find ( const Ident & id, Viewer & ) const;
private:
    const T & m_dbr;
    typedef CatAlarmCond < DBR > CatAlarm;
    CatalogSingle < pi_condition, CatAlarm > m_catAlarm;
};

template < class T >
class MutSubordTime : 
    public MutatorCatalog
{
public:
    typedef T DBR;
    MutSubordTime ( T & );
    void assignCatalog ( const Catalog & src );
private:
    T & m_dbr;
    typedef MutAlarmCond < DBR > MutAlarm;
    MutatorCatalogSingle < pi_condition, MutAlarm > m_mutAlarm;
};

template < class T >
struct IntfTraitsTime  
{
    typedef T DBR;
    typedef CatSubordTime < DBR > CatSubord;
    typedef CatCmpdScalar < CatSubord > CatScalar;
    typedef CatCmpdVector < CatSubord > CatVector;
    typedef MutSubordTime < DBR > MutSubord;
    typedef MutCmpdScalar < MutSubord > MutScalar;
    typedef MutCmpdVector < MutSubord > MutVector;
};

template < unsigned DBR_TYPE > class TypeCodeTraits;

template <>
struct TypeCodeTraits < DBR_TIME_STRING > :
    public IntfTraitsTime < dbr_time_string > 

{
};

template <>
struct TypeCodeTraits < DBR_TIME_SHORT > :
    public IntfTraitsTime < dbr_time_short > 
{
};

template <>
struct TypeCodeTraits < DBR_TIME_FLOAT > :
    public IntfTraitsTime < dbr_time_float > 
{
};

template <>
struct TypeCodeTraits < DBR_TIME_ENUM > :
    public IntfTraitsTime < dbr_time_enum > 
{
};

template <>
struct TypeCodeTraits < DBR_TIME_CHAR > :
    public IntfTraitsTime < dbr_time_char >
{
};

template <>
struct TypeCodeTraits < DBR_TIME_LONG > :
    public IntfTraitsTime < dbr_time_long > 
{
};

template <>
struct TypeCodeTraits < DBR_TIME_DOUBLE > :
    public IntfTraitsTime < dbr_time_double > 
{
};

template < class T >
inline CatSubordTime < T > :: 
    CatSubordTime ( const T & dbr ) :
    m_dbr ( dbr ), m_catAlarm ( dbr )
{
}

template < class T >
void CatSubordTime < T > :: 
    traverse ( Introspector & i ) const
{
    i.reveal ( pi_alarm, m_catAlarm );
    const epicsTime ts ( m_dbr.stamp );
    i.reveal ( pi_time, ts );
}

template < class T >
void CatSubordTime < T > :: 
    find ( const Ident & id, Viewer & v ) const
{
    if ( id == pi_alarm ) {
        v.reveal ( m_catAlarm );
    }
    else if ( id == pi_time ) {
        const epicsTime ts ( m_dbr.stamp );
        v.reveal ( ts );
    }
}

template < class T >
inline MutSubordTime < T > :: 
    MutSubordTime ( T & dbr ) :
    m_dbr ( dbr ), m_mutAlarm ( dbr )
{
}

template < class T >
inline void MutSubordTime < T > :: 
    assignCatalog ( const Catalog & src )
{
    Clerk clerk ( src );
    clerk.get ( pi_alarm, m_mutAlarm );
    epicsTime ts;
    clerk.get ( pi_time, ts );
    m_dbr.stamp = ts;
}

} // end of namespace dbr
} // end of namespace ca

#endif // ifdef cadbrIntfTime_h
