
//
// Author: Jeff Hill, johill@lanl.gov
//

#ifndef cadbrMutCmpdScalar_h
#define cadbrMutCmpdScalar_h

namespace ca {
namespace dbr {

template < class DBR > class dbrProp;

template < class DBR >
class MutCmpdScalar : 
    public Mutator {
public:
    typedef typename dbrProp < DBR > :: ValType ValType;
    typedef typename dbrProp < DBR > :: MutFacet MutFacets;
    // This constructor is very unusual in that it saves a copy of 
    // the Guard reference passed in. This implies that the
    // Guard must persit for the lifespan of this object.
    MutCmpdScalar ( Guard &, const Iterator & );
    void assignDefault ();
    void assignNexus ( const Catalog & facets );
    void assignAtomic ( const Atomic & val, 
                        const Catalog & facets );
    void assignCatalog ( const Catalog & val );
private:
    const Iterator m_iterator;
    MutFacets m_mutFacets;
    Guard & m_guard;
};

template < class T, class M >
inline MutCmpdScalar < T, M > ::
    MutCmpdScalar (  Guard & guard, const Iterator & iterator ) :
    m_iterator ( iterator ), 
    m_mutFacets ( *this ),
    m_guard ( guard )
{
}

template < class T, class M >
void MutCmpdScalar < M > ::
    assignDefault ()
{
    throw UndefinedProperty ();
}

template < class T, class M >
void MutCmpdScalar < T, M > ::
    assignNexus ( const Catalog & )
{
    throw UndefinedProperty ();
}

template < class T, class M >
void MutCmpdScalar < T, M > ::
    assignAtomic ( const Atomic & src, 
        const Catalog & srcFacets )
{
    Iterator tmpIter = m_iterator;
    MutFacets :: pushFacets ( m_guard, tmpIter, srcFacets );
    ValType tmp;
    get ( src, tmp );
    tmpIter.push ( m_guard, tmp );
}

template < class T, class M >
void MutCmpdScalar < T, M > ::
    assignCatalog ( const Catalog & src )
{
    throw IncompatibleTypes ( typeid ( DBR ), typeid ( src ) );
}

} // end of namespace dbr
} // end of namespace ca

#endif // ifdef cadbrMutCmpdScalar_h
