


//
// dbrIntfTest.cpp,v 1.1 2009/10/01 00:29:43 jhill Exp
//
// Authors: Jeff Hill, johill@lanl.gov
//         

#include <typeinfo>

#include "cadbr.h"

using namespace ca;
using namespace dbr;

template < unsigned t1, unsigned t2 >
inline void testCatDbrAssign ( 
    typename TypeCodeTraits < t1 > :: DBR & tst1, 
    typename TypeCodeTraits < t2 > :: DBR & tst2 )
{
    typename TypeCodeTraits < t1 > :: MutScalar t1Mut ( tst1 );
    const typename TypeCodeTraits < t2 > :: CatScalar t2Cat ( tst2 );
    Clerk ( t2Cat ).get ( pi_value, t1Mut );
}

template < unsigned t1, unsigned t2 >
void testCatDbr ()
{
#if _DEBUG
    static const size_t N = 100;
#else
    static const size_t N = 1000000;
#endif
    typename TypeCodeTraits < t1 > :: DBR tst1;
    memset ( &tst1, '\0', sizeof ( tst1 ) );
    typename TypeCodeTraits < t2 > :: DBR tst2;
    memset ( &tst2, '\0', sizeof ( tst2 ) );
    clock_t begin = clock ();
    try {
        for ( size_t i = 0; i < N; i++ ) {
            testCatDbrAssign < t1, t2 > ( tst1, tst2 );
            testCatDbrAssign < t1, t2 > ( tst1, tst2 );
            testCatDbrAssign < t1, t2 > ( tst1, tst2 );
            testCatDbrAssign < t1, t2 > ( tst1, tst2 );
            testCatDbrAssign < t1, t2 > ( tst1, tst2 );

            testCatDbrAssign < t1, t2 > ( tst1, tst2 );
            testCatDbrAssign < t1, t2 > ( tst1, tst2 );
            testCatDbrAssign < t1, t2 > ( tst1, tst2 );
            testCatDbrAssign < t1, t2 > ( tst1, tst2 );
            testCatDbrAssign < t1, t2 > ( tst1, tst2 );
        }
    }
    catch ( std :: exception & except ) {
#if 0
        printf ( 
            "\"%s\" = \"%s\" caused an exception\n",
                dbr_text [ t1 ], dbr_text [ t2 ] ); 
#else
        printf ( 
            "\"%u\" = \"%u\" caused an exception\n",
                t1, t2 ); 
#endif
        printf ( 
            "detail, \"%s\"\n", except.what () ); 

        return;
    }
    clock_t end = clock ();
    double delay = end - begin;
    delay /= CLOCKS_PER_SEC;
    delay /= N;
    delay /= 10;
#if 0
    printf ( 
        "\"%s\" = \"%s\" requires %g sec\n",
        dbr_text [ t1 ],
        dbr_text [ t2 ],
        delay ); 
#else
    printf ( 
        "\"%u\" = \"%u\" requires %g sec\n",
        t1, t2, delay ); 
#endif
}

int main ()
{
    testCatDbr < DBR_INT, DBR_DOUBLE > ();
    testCatDbr < DBR_STS_STRING, DBR_TIME_SHORT > ();
    testCatDbr < DBR_STS_DOUBLE, DBR_STS_DOUBLE > ();
    testCatDbr < DBR_STSACK_STRING, DBR_STSACK_STRING > ();
    testCatDbr < DBR_TIME_DOUBLE, DBR_TIME_DOUBLE > ();
    testCatDbr < DBR_TIME_DOUBLE, DBR_TIME_FLOAT > ();
    testCatDbr < DBR_TIME_DOUBLE, DBR_TIME_INT > ();
    testCatDbr < DBR_TIME_STRING, DBR_TIME_SHORT > ();
    testCatDbr < DBR_GR_SHORT, DBR_GR_FLOAT > ();
    testCatDbr < DBR_CTRL_SHORT, DBR_CTRL_FLOAT > ();
    testCatDbr < DBR_CTRL_ENUM, DBR_GR_ENUM > ();
    testCatDbr < DBR_CTRL_ENUM, DBR_CTRL_FLOAT > ();
}
