/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *      inBuf.cc,v 1.16 2009/11/24 01:34:27 jhill Exp
 *
 *      Author  Jeffrey O. Hill
 *              johill@lanl.gov
 *              505 665 1831
 */

#error defunct

#include <stdio.h>
#include <string.h>

#define epicsExportSharedSymbols
#include "inBuf.h"

//
// inBuf :: inBuf()
//
inBuf :: inBuf ( 
    casStreamReadIO & clientIn, 
    clientBufMemoryManager & memMgrIn, 
    size_t ioMinSizeIn ) :
        client ( clientIn ), memMgr ( memMgrIn ), 
        pBuf ( 0 ), bufSize ( 0u ), bytesInBuffer ( 0u ),
        nextReadIndex ( 0u ), ioMinSize ( ioMinSizeIn ),
        ctxRecursCount ( 0u )
{
    if ( this->ioMinSize == 0 ) {
        this->ioMinSize = 1;
    }
    casBufferParm bufParm = this->memMgr.allocate ( this->ioMinSize );
    this->pBuf = bufParm.pBuf;
    this->bufSize = bufParm.bufSize;
}

//
// inBuf :: ~inBuf()
// (virtual destructor)
//
inBuf :: ~inBuf ()
{
    assert ( this->ctxRecursCount == 0 );
    this->memMgr.release ( this->pBuf, this->bufSize );
}

//
// inBuf :: show()
//
void inBuf :: show (unsigned level) const
{
    if ( level > 1u ) {
        printf (
            "\tUnprocessed request bytes = %d\n",
            this->bytesAvailable());
    }
}

//
// inBuf :: fill()
//
casStreamReadIO :: fillCondition inBuf :: fill ()
{
	size_t bytesOpen;
	size_t bytesRecv;
	casStreamReadIO::fillCondition stat;

	//
	// move back any prexisting data to the start of the buffer
	//
	if ( this->nextReadIndex > 0 ) {
		size_t unprocessedBytes;
		unprocessedBytes = this->bytesInBuffer - this->nextReadIndex;
		//
		// memmove() handles overlapping buffers
		//
		if (unprocessedBytes>0u) {
			memmove (this->pBuf, this->pBuf+this->nextReadIndex, 
				unprocessedBytes);
		}
		this->bytesInBuffer = unprocessedBytes;
		this->nextReadIndex = 0u;
	}

	//
	// noop if the buffer is full
	//
	bytesOpen = this->bufSize - this->bytesInBuffer;
	if ( bytesOpen < this->ioMinSize ) {
        return casStreamReadIO::casFillNone;
	}

	stat = this->client.xRecv ( &this->pBuf[this->bytesInBuffer], 
			bytesOpen, bytesRecv );
    if ( stat == casStreamReadIO::casFillProgress ) {
	    assert (bytesRecv<=bytesOpen);
	    this->bytesInBuffer += bytesRecv;
	}

	return stat;
}

//
// inBuf :: pushCtx ()
//
const inBufCtx inBuf :: pushCtx ( size_t headerSize, // X aCC 361
                               size_t bodySize )
{
    if ( headerSize + bodySize > ( this->bytesInBuffer - this->nextReadIndex ) || 
        this->ctxRecursCount == UINT_MAX ) {
        return inBufCtx ();
    }
    else {
        inBufCtx result (*this);
        size_t effectiveNextReadIndex = this->nextReadIndex + headerSize;
        this->pBuf = this->pBuf + effectiveNextReadIndex;
        this->bytesInBuffer -= effectiveNextReadIndex;
        this->nextReadIndex = 0;
        this->bytesInBuffer = bodySize;
        this->bufSize = this->bytesInBuffer;
        this->ctxRecursCount++;
        return result;
    }
}

//
// inBuf :: popCtx ()
//
size_t inBuf :: popCtx ( const inBufCtx &ctx ) // X aCC 361
{
    if ( ctx.stat==inBufCtx::pushCtxSuccess ) {
        size_t bytesRemoved = this->nextReadIndex;
        this->pBuf = ctx.pBuf;
        this->bufSize = ctx.bufSize;
        this->bytesInBuffer = ctx.bytesInBuffer;
        this->nextReadIndex = ctx.nextReadIndex;
        assert ( this->ctxRecursCount > 0 );
        this->ctxRecursCount--;
        return bytesRemoved;
    }
    else {
        return 0;
    }
}

void inBuf :: expandBuffer ()
{
    size_t max = this->memMgr.maxSize();
    if ( this->bufSize <  max ) {
        try {
            casBufferParm bufParm = this->memMgr.allocate ( max );
            size_t unprocessedBytes = this->bytesPresent ();
            memcpy ( bufParm.pBuf, &this->pBuf[this->nextReadIndex], unprocessedBytes );
            this->bytesInBuffer = unprocessedBytes;
            this->nextReadIndex = 0u;
            this->memMgr.release ( this->pBuf, this->bufSize );
            this->pBuf = bufParm.pBuf;
            this->bufSize = bufParm.bufSize;
        }
        catch ( std :: bad_alloc & ) {
        }
    }
}

unsigned inBuf :: bufferSize () const
{
    return this->bufSize;
}

size_t inBuf :: bytesAvailable (
    Guard & guard ) const 
{
    guard.assertIdenticalMutex ( m_mutex );
	size_t bp = bytesPresent ( guard );
	bp += m_client.incomingBytesPresent ();
	return bp;
}

