/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

#error defunct

#ifndef inBufh
#define inBufh

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_inBufh
#   undef epicsExportSharedSymbols
#endif

#undef epicsAssertAuthor
#define epicsAssertAuthor "Jeff Hill johill@lanl.gov"
#include "epicsAssert.h"
#include "epicsMutex.h"

#ifdef epicsExportSharedSymbols_inBufh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "clientBufMemoryManager.h"

#if ! defined ( __GNUC__ ) || __GNUC__ > 2 || ( __GNUC__ == 2 && __GNUC_MINOR__ >= 95 )
namespace ca {
#endif

class inBufCtx {
    friend class inBuf;
public:
    enum pushCtxResult 
        { pushCtxNoSpace, pushCtxSuccess };
    inBufCtx ( const class inBuf & ); // success
    inBufCtx (); // failure
    pushCtxResult pushResult () const;
private:
    pushCtxResult m_stat;
    char * m_pBuf;
    size_t m_bufSize;
    size_t m_bytesInBuffer;
    size_t m_nextReadIndex;
};

struct casStreamReadIO;
typedef epics :: Mutex :: Guard Guard;

class inBuf {
public:
    inBuf ( epics :: Mutex &, casStreamReadIO &, 
        class clientBufMemoryManager &, size_t ioMinSizeIn );
    ~inBuf ();
    size_t bytesPresent ( Guard & ) const;
    size_t bytesAvailable ( Guard & ) const;
    bool full ( Guard & ) const;
    void fill ( Guard & );  
    void show ( Guard &, unsigned level ) const;
    void removeMsg ( Guard &, size_t nBytes );
    char * msgPtr ( Guard & ) const;
    //
    // This is used to create recursive protocol stacks. A subsegment
    // of the buffer of max size "maxSize" is assigned to the next
    // layer down in the protocol stack by pushCtx () until popCtx ()
    // is called. The roiutine popCtx () returns the actual number
    // of bytes used by the next layer down.
    //
    // pushCtx() returns an outBufCtx to be restored by popCtx()
    //
    const inBufCtx pushCtx ( 
        Guard &, size_t headerSize, size_t bodySize );
	size_t popCtx ( 
        Guard &, const inBufCtx & ); // returns actual size
    unsigned bufferSize ( Guard & ) const;
    void expandBuffer ( Guard & );
private:
	epics :: Mutex & m_mutex;
    casStreamReadIO & m_client;
    class clientBufMemoryManager & m_memMgr;
    char * m_pBuf;
    size_t m_bufSize;
    size_t m_bytesInBuffer;
    size_t m_nextReadIndex;
    size_t m_ioMinSize;
    friend class inBufCtx;
    unsigned m_ctxRecursCount;
	inBuf ( const inBuf & );
	inBuf & operator = ( const inBuf & );
};

//
// inBuf::bytesPresent()
//
inline size_t inBuf :: bytesPresent (
    Guard & guard ) const 
{
    guard.assertIdenticalMutex ( m_mutex );
	return m_bytesInBuffer - m_nextReadIndex;
}

//
// inBuf :: full()
//
inline bool inBuf :: full (
    Guard & guard ) const
{
    guard.assertIdenticalMutex ( m_mutex );
	if ( m_bufSize - bytesPresent ( guard ) < m_ioMinSize) {
		return true;
	}
	return false;
}

//
// inBuf :: msgPtr()
//
inline char *inBuf :: msgPtr (
    Guard & guard ) const 
{
    guard.assertIdenticalMutex ( m_mutex );
	return & m_pBuf[m_nextReadIndex];
}

//
// inBuf :: removeMsg()
//
inline void inBuf :: removeMsg ( 
    Guard & guard, size_t nBytes ) 
{
    guard.assertIdenticalMutex ( m_mutex );
	m_nextReadIndex += nBytes;
	assert ( m_nextReadIndex <= m_bytesInBuffer );
}

//
// inBufCtx :: inBufCtx ()
//
inline inBufCtx :: inBufCtx () :
    m_stat ( pushCtxNoSpace ),
    m_pBuf ( 0 ), 
    m_bufSize ( 0 ), 
    m_bytesInBuffer ( 0 ),
    m_nextReadIndex ( 0 ) {}

//
// inBufCtx :: inBufCtx ()
//
inline inBufCtx :: inBufCtx ( const inBuf & inBufIn ) :
    m_stat ( pushCtxSuccess ), 
    m_pBuf ( inBufIn.m_pBuf ), 
    m_bufSize ( inBufIn.m_bufSize ), 
    m_bytesInBuffer ( inBufIn.m_bytesInBuffer ),
    m_nextReadIndex ( inBufIn.m_nextReadIndex ) {}

//
// inBufCtx :: pushResult
// 
inline inBufCtx :: pushCtxResult 
    inBufCtx :: pushResult () const
{
    return m_stat;
}

#if ! defined ( __GNUC__ ) || __GNUC__ > 2 || ( __GNUC__ == 2 && __GNUC_MINOR__ >= 95 )
} // end of namespace ca
#endif

#endif // inBufh

