/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

#error defunct

#ifndef inBufIIh
#define inBufIIh

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_inBufh
#   undef epicsExportSharedSymbols
#endif

#include "epicsAssert.h"
#include "tsFreeList.h"

#ifdef epicsExportSharedSymbols_inBufh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "clientBufMemoryManager.h"

#if ! defined ( __GNUC__ ) || __GNUC__ > 2 || ( __GNUC__ == 2 && __GNUC_MINOR__ >= 95 )
namespace ca {
#endif

struct casStreamReadIO;
struct caHdrLargeArray;

class InBuf {
public:
    class Buffer :
        public tsDLNode < Buffer > {
    public:
        class Msg {
        public:
            Msg ();
            Msg ( 
                Guard &, 
                Ref < Buffer > &, 
                const caHdrLargeArray &, 
                const void * pPayload );
            Msg ( Guard &, const Msg & );
            const caHdrLargeArray & hdr () const;
            const Catalog & payload () const;
        private:
            Ptr < Ref < Buffer > > m_pBuf;             
            const caHdrLargeArray * m_pMsg;
            const void * m_pPayload;

            // If the header or the first payload element
            // (the compound type) spans buffers then we will
            // copy the partial protocol to the next buffer with
            // the header alligned to start at the beggining of 
            // the next buffer, and shorten the length of the 
            // previous buffer.
            //
            // If the array elements cross over multiple buffers
            // we will access them by traversing from one buffer
            // to the next using the doubly linked list until they 
            // see the entire array. This will not require storage 
            // of an array index because we will always start at 
            // the beginning of the array when traversing. Since
            // the protocol is 8 byte aligned we will not need
            // to worry about an array element spanning buffers.
            //
            // We will need to have a specialized Catalog that 
            // knows how to deal with protocol buffers.
            //
#error this design requires a dbr interfacing catalog that knows 
#error how to traverse arrays that span buffers, and knows
#error how to use the type and element count in the protocol header
            class Cat :
                public da :: Catalog 
            {
            };
            Msg ( const Msg & );
            Msg & operator = ( const Msg & );
        };

        class Chunk {
        public:
            Chunk ( Guard &, Ref < Buffer > &, 
                const char * pRawBuf, size_t bufSize );
            Chunk ( Guard &, Chunk &, size_t extractSize );
            const char * pRawBuf () const;
            size_t bufSize () const;
        private:
            Ptr < Ref < Buffer > > m_ptr;
            const char * m_pRaw;
            size_t m_rawSize;
        };
        Buffer ( epics :: Mutex &, casStreamReadIO & );
        ~Buffer ();
        bool full ( Guard & ) const;
        void fill ( Guard & );  
        // size_t bytesPresent ( Guard & ) const;
        // size_t bytesAvailable ( Guard & ) const;
        Chunk reference ( Guard & );
        void show ( Guard &, unsigned level ) const;
    private:
        // size here must currently be an even multiple of 8,
        // but its best to stick with an even power of two to 
        // remain future proof
        //
        // o A smaller buffer size avoids wasted space when only one
        // asynchronous request is still referencing the buffer.
        // o A larger buffer size improves performance when filling
        // buffers using socket system calls.
        unsigned char m_rawBytes [ 1 << 10 ];
        Ref < Buffer > m_ref; 
        InBuf & m_inBuf;
	    epics :: Mutex & m_mutex;
        size_t m_bytesInBuffer;
        size_t m_nextReadIndex;
    };

    InBuf ( epics :: Mutex &, casStreamReadIO &, 
        tsFreeList < Buffer, 128 > &, size_t ioMinSizeIn );
    ~InBuf ();
    void show ( Guard &, unsigned level ) const;
    void getNextMsg ( Guard &, Buffer :: Msg & );

private:
    tsDLList < Buffer > m_curMsgBufList;
    tsFreeList < Buffer, 128 > & m_bufferFreeList;
	epics :: Mutex & m_mutex;
    casStreamReadIO & m_client;
    size_t m_ioMinSize;
	InBuf ( const InBuf & );
	InBuf & operator = ( const InBuf & );
};


#if ! defined ( __GNUC__ ) || __GNUC__ > 2 || ( __GNUC__ == 2 && __GNUC_MINOR__ >= 95 )
} // end of namespace ca
#endif

#endif // inBufIIh

