
#include "casdef2.h"

void initiateScalarReq ( 
    Guard & guard,
    unsigned dbrType, const void * pSrc, 
    Request & req );

void initiateVectorReq ( 
    Guard & guard,
    unsigned dbrType, size_t nElemOfSrc, 
    const void * pSrc, Request & req );

#define nElements( A ) ( sizeof ( A) / sizeof ( A [ 0 ] ) )

template < class T >
static void initiateScalarReqUntyped ( 
    Guard & guard,
    const void * pSrc, Request & req )
{
    const T & src ( * reinterpretCast < const T * > ( pSrc ) );
    Cat < const T, Scalar > cat ( src );
    req.initiate ( guard, cat );
}

static PInitiateScalarReqFunc initiateScalarReqTbl [] = {
    initiateScalarReqUntyped < DBR_STRING >,
    initiateScalarReqUntyped < DBR_INT >,
    initiateScalarReqUntyped < DBR_SHORT >,	
    initiateScalarReqUntyped < DBR_FLOAT >,	
    initiateScalarReqUntyped < DBR_ENUM >,
    initiateScalarReqUntyped < DBR_CHAR >,
    initiateScalarReqUntyped < DBR_LONG >,
    initiateScalarReqUntyped < DBR_DOUBLE >,
    initiateScalarReqUntyped < DBR_STS_STRING >,
    initiateScalarReqUntyped < DBR_STS_SHORT >,
    initiateScalarReqUntyped < DBR_STS_INT >,	
    initiateScalarReqUntyped < DBR_STS_FLOAT >,
    initiateScalarReqUntyped < DBR_STS_ENUM >,
    initiateScalarReqUntyped < DBR_STS_CHAR >,
    initiateScalarReqUntyped < DBR_STS_LONG >,
    initiateScalarReqUntyped < DBR_STS_DOUBLE >,
    initiateScalarReqUntyped < DBR_TIME_STRING >,
    initiateScalarReqUntyped < DBR_TIME_INT >,
    initiateScalarReqUntyped < DBR_TIME_SHORT >,
    initiateScalarReqUntyped < DBR_TIME_FLOAT >,
    initiateScalarReqUntyped < DBR_TIME_ENUM >,
    initiateScalarReqUntyped < DBR_TIME_CHAR >,
    initiateScalarReqUntyped < DBR_TIME_LONG >,
    initiateScalarReqUntyped < DBR_TIME_DOUBLE >,
    initiateScalarReqUntyped < DBR_GR_STRING >,
    initiateScalarReqUntyped < DBR_GR_SHORT >,
    initiateScalarReqUntyped < DBR_GR_INT >,
    initiateScalarReqUntyped < DBR_GR_FLOAT >,
    initiateScalarReqUntyped < DBR_GR_ENUM >,
    initiateScalarReqUntyped < DBR_GR_CHAR >,
    initiateScalarReqUntyped < DBR_GR_LONG	>,
    initiateScalarReqUntyped < DBR_GR_DOUBLE >,
    initiateScalarReqUntyped < DBR_CTRL_STRING >,
    initiateScalarReqUntyped < DBR_CTRL_SHORT >,
    initiateScalarReqUntyped < DBR_CTRL_INT >,
    initiateScalarReqUntyped < DBR_CTRL_FLOAT >,
    initiateScalarReqUntyped < DBR_CTRL_ENUM >,
    initiateScalarReqUntyped < DBR_CTRL_CHAR >,
    initiateScalarReqUntyped < DBR_CTRL_LONG >,
    initiateScalarReqUntyped < DBR_CTRL_DOUBLE >,
    initiateScalarReqUntyped < DBR_PUT_ACKT >,
    initiateScalarReqUntyped < DBR_PUT_ACKS >,
    initiateScalarReqUntyped < DBR_STSACK_STRING >,
    initiateScalarReqUntyped < DBR_CLASS_NAME >
};

void initiateScalarReq ( Guard & guard,
    unsigned dbrType, const void * pSrc, Request & req )
{
    if ( dbrType > nElements ( initiateScalarReqTbl ) ) {
        throw xxx;
    }
    ( * initiateScalarReqTbl [ dbrType ] ) ( pSerc, req );
}

template < class T >
static void initiateVectorReqUntyped ( 
    Guard & guard,
    size_t nElemOfSrc, const void * pSrc, 
    Request & req )
{
    const T & src ( * reinterpretCast < const T * > ( pSrc ) );
    Cat < const T, Vector > cat ( src, nElemOfSrc );
    req.initiate ( guard, cat );
}

static PInitiateVectorReqFunc initiateVectorReq [] = {
    initiateVectorReqUntyped < DBR_STRING >,
    initiateVectorReqUntyped < DBR_INT >,
    initiateVectorReqUntyped < DBR_SHORT >,	
    initiateVectorReqUntyped < DBR_FLOAT >,	
    initiateVectorReqUntyped < DBR_ENUM >,
    initiateVectorReqUntyped < DBR_CHAR >,
    initiateVectorReqUntyped < DBR_LONG >,
    initiateVectorReqUntyped < DBR_DOUBLE >,
    initiateVectorReqUntyped < DBR_STS_STRING >,
    initiateVectorReqUntyped < DBR_STS_SHORT >,
    initiateVectorReqUntyped < DBR_STS_INT >,	
    initiateVectorReqUntyped < DBR_STS_FLOAT >,
    initiateVectorReqUntyped < DBR_STS_ENUM >,
    initiateVectorReqUntyped < DBR_STS_CHAR >,
    initiateVectorReqUntyped < DBR_STS_LONG >,
    initiateVectorReqUntyped < DBR_STS_DOUBLE >,
    initiateVectorReqUntyped < DBR_TIME_STRING >,
    initiateVectorReqUntyped < DBR_TIME_INT >,
    initiateVectorReqUntyped < DBR_TIME_SHORT >,
    initiateVectorReqUntyped < DBR_TIME_FLOAT >,
    initiateVectorReqUntyped < DBR_TIME_ENUM >,
    initiateVectorReqUntyped < DBR_TIME_CHAR >,
    initiateVectorReqUntyped < DBR_TIME_LONG >,
    initiateVectorReqUntyped < DBR_TIME_DOUBLE >,
    initiateVectorReqUntyped < DBR_GR_STRING >,
    initiateVectorReqUntyped < DBR_GR_SHORT >,
    initiateVectorReqUntyped < DBR_GR_INT >,
    initiateVectorReqUntyped < DBR_GR_FLOAT >,
    initiateVectorReqUntyped < DBR_GR_ENUM >,
    initiateVectorReqUntyped < DBR_GR_CHAR >,
    initiateVectorReqUntyped < DBR_GR_LONG	>,
    initiateVectorReqUntyped < DBR_GR_DOUBLE >,
    initiateVectorReqUntyped < DBR_CTRL_STRING >,
    initiateVectorReqUntyped < DBR_CTRL_SHORT >,
    initiateVectorReqUntyped < DBR_CTRL_INT >,
    initiateVectorReqUntyped < DBR_CTRL_FLOAT >,
    initiateVectorReqUntyped < DBR_CTRL_ENUM >,
    initiateVectorReqUntyped < DBR_CTRL_CHAR >,
    initiateVectorReqUntyped < DBR_CTRL_LONG >,
    initiateVectorReqUntyped < DBR_CTRL_DOUBLE >,
    initiateVectorReqUntyped < DBR_PUT_ACKT >,
    initiateVectorReqUntyped < DBR_PUT_ACKS >,
    initiateVectorReqUntyped < DBR_STSACK_STRING >,
    initiateVectorReqUntyped < DBR_CLASS_NAME >
};

void initiateVectorReq ( Guard & guard,
    unsigned dbrType, size_t nElemOfSrc, const void * pSrc, 
    Request & req )
{
    if ( dbrType > nElements ( initiateVectorReqTbl ) ) {
        throw xxx;
    }
    ( * initiateVectorReqTbl [ dbrType ] ) ( nElemOfSrc, pSrc, req );
}
