
#include <cstdio>

#include "errlog.h"
#include "FixedString.h"

#define epicsExportSharedSymbols
#include "caHdrLargeArray.h"
#include "protoDiagnostics.h"
#include "caNetAddr.h"

namespace ca {

void errlogDumpMsg ( 
    const StringReader & hostName, 
    const StringReader & userName,
    const HdrLargeArray & msg, 
    const char * pFormat, ... )
{
    va_list theArgs;
    va_start ( theArgs, pFormat );
    errlogDumpMsgVA ( hostName, userName, msg, pFormat, theArgs );
    va_end ( theArgs );
}

void errlogDumpMsgVA ( 
    const StringReader & hostName, 
    const StringReader & userName,
    const HdrLargeArray & msg, 
    const char * pFormat, 
    va_list & theArgs )
{
    if ( pFormat ) {
        errlogPrintf ( "CAS: " );
        errlogVprintf ( pFormat, theArgs );
    }
    FixedString < 128 > hostNameStr = hostName;
    FixedString < 128 > userNameStr = userName;
    errlogPrintf ( 
        "CAS Request: %s on %s: cmd=%u cid=%u typ=%u "
            "cnt=%u psz=%u avail=%x\n",
        hostNameStr.c_str (),
        userNameStr.c_str (),
        msg.m_cmmd,
        msg.m_cid,
        msg.m_dataType,
        msg.m_count,
        msg.m_postsize,
        msg.m_available );
}

void errlogDumpMsg ( 
    const NetAddr & addr,
    const HdrLargeArray & msg, 
    const char * pFormat, ... )
{
    va_list theArgs;
    va_start ( theArgs, pFormat );
    errlogDumpMsgVA ( addr, msg, pFormat, theArgs);
    va_end ( theArgs );
}

void errlogDumpMsgVA ( 
    const NetAddr  & addr,
    const HdrLargeArray & msg, 
    const char * pFormat, 
    va_list & theArgs )
{
    if ( pFormat ) {
        errlogPrintf ( "CAS: " );
        errlogVprintf ( pFormat, theArgs );
    }
    char pHostName[64u];
    addr.stringConvert ( pHostName, sizeof ( pHostName ) );
    errlogPrintf ( 
        "CAS Request: %s: cmd=%u cid=%u typ=%u "
            "cnt=%u psz=%u avail=%x\n",
        pHostName,
        msg.m_cmmd,
        msg.m_cid,
        msg.m_dataType,
        msg.m_count,
        msg.m_postsize,
        msg.m_available);
}

void dumpMsg ( const HdrLargeArray & msg )
{
    printf ( 
        "CAS Request: cmd=%u cid=%u typ=%u "
            "cnt=%u psz=%u avail=%x\n",
        msg.m_cmmd,
        msg.m_cid,
        msg.m_dataType,
        msg.m_count,
        msg.m_postsize,
        msg.m_available );
}

} // end of namespace ca
