
// Author: Jeff HIll

#ifndef v3Protoh
#define v3Protoh

#ifdef epicsExportSharedSymbols
#   define epicsExportSharedSymbols_v3Protoh
#   undef epicsExportSharedSymbols
#endif

#include "db_access.h"
#include "caProto.h"
#include "daCatalog.h"

#ifdef epicsExportSharedSymbols_v3Protoh
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "ByteSeqOut.h"

namespace ca {

using namespace da;

struct HdrLargeArray;

class MsgMinderV3 {
public:
    MsgMinderV3 ( ByteSeqOut &, unsigned protoRev );
    void pushResp ( const HdrLargeArray & );
    void pushResp ( const HdrLargeArray &, const Catalog & );
    void errorResponse ( 
        const HdrLargeArray & msg, 
        epicsUInt32 cid, const epicsUInt32 legacyStatus, 
        const char * const pformat, ... );
    void errorResponse ( 
        const HdrLargeArray & msg, 
        ca_uint32_t cid, ca_uint32_t legacyStatus,
        const Catalog & diag );
    void errorResponse ( 
        const HdrLargeArray & msg, 
        ca_uint32_t cid, ca_uint32_t legacyStatus, 
        const StringReader & diag );
    void commit ();
private:
    typedef ByteSeqOut :: Iter Iter;
    Iter m_iter;
    unsigned m_protoRev;
    size_t m_pushHdr ( const HdrLargeArray &,
                    size_t payloadSize );
    template < unsigned typeCode >
    void m_pushScalarPayload ( 
            const size_t payloadSize, 
            const Catalog & payloadSource );
    template < unsigned typeCode >
    void m_pushCmpdScalarPayload ( 
            const size_t payloadSize, 
            const Catalog & payloadSource );
    typedef void ( MsgMinderV3 :: * PPushScalarPayload )
            ( const size_t payloadSize, const Catalog & );
    static PPushScalarPayload m_pPushScalarPayload[];
    template < unsigned typeCode >
    void m_pushVectorPayload ( const size_t payloadSize, 
                const size_t nElem, const Catalog & src );
    template < unsigned typeCode >
    void m_pushCmpdVectorPayload ( const size_t payloadSize, 
                const size_t nElem, const Catalog & src );
    typedef void ( MsgMinderV3 :: * PPushVectorPayload )
            ( const size_t payloadSize, 
                const size_t nElem, const Catalog & );
    static PPushVectorPayload m_pPushVectorPayload[];
    void m_pushVal ( const dbr_char_t & );
    void m_pushVal ( const dbr_short_t & );
    void m_pushVal ( const dbr_ushort_t & );
    void m_pushVal ( const dbr_long_t & );
    void m_pushVal ( const dbr_ulong_t & );
    void m_pushVal ( const dbr_float_t & );
    void m_pushVal ( const dbr_double_t & );
    template < size_t N >
    void m_pushVal ( const char ( & pStrBuf ) [ N ] );
    // we prefer for the implementation of m_pushOldString
    // to optimize the length of a scaler string, and not 
    // transmit on the wire the entire string as has been 
    // the behavior in the past when the string is at the
    // end of an atomic payload, but this improvement
    // hasnt been installed, at this time. This is partly
    // because in dbr structure embedded situations the 
    // optimization would not be appropriate.
    template < size_t N >
    void m_pushOldString ( const char ( & pStrBuf ) [ N ] );

    template < size_t M, size_t N >
    void m_pushOldStringArray ( 
        const char ( & pStrBuf ) [ M ][ N ], size_t nElem );
    void m_pushFacetsDBR ( const dbr_sts_string & );
    void m_pushFacetsDBR ( const dbr_stsack_string & );
    void m_pushFacetsDBR ( const dbr_sts_int & );
    void m_pushFacetsDBR ( const dbr_sts_short & );
    void m_pushFacetsDBR ( const dbr_sts_float & );
    void m_pushFacetsDBR ( const dbr_sts_enum & );
    void m_pushFacetsDBR ( const dbr_sts_char & );
    void m_pushFacetsDBR ( const dbr_sts_long & );
    void m_pushFacetsDBR ( const dbr_sts_double & );
    void m_pushFacetsDBR ( const dbr_time_string & );
    void m_pushFacetsDBR ( const dbr_time_short & );
    void m_pushFacetsDBR ( const dbr_time_float & );
    void m_pushFacetsDBR ( const dbr_time_enum & );
    void m_pushFacetsDBR ( const dbr_time_char & );
    void m_pushFacetsDBR ( const dbr_time_long & );
    void m_pushFacetsDBR ( const dbr_time_double & );
    void m_pushFacetsDBR ( const dbr_gr_int & );
    void m_pushFacetsDBR ( const dbr_gr_short & );
    void m_pushFacetsDBR ( const dbr_gr_float & );
    void m_pushFacetsDBR ( const dbr_gr_enum & );
    void m_pushFacetsDBR ( const dbr_gr_char & );
    void m_pushFacetsDBR ( const dbr_gr_long & );
    void m_pushFacetsDBR ( const dbr_gr_double & );
    void m_pushFacetsDBR ( const dbr_ctrl_int & );
    void m_pushFacetsDBR ( const dbr_ctrl_short & );
    void m_pushFacetsDBR ( const dbr_ctrl_float & );
    void m_pushFacetsDBR ( const dbr_ctrl_enum & );
    void m_pushFacetsDBR ( const dbr_ctrl_char & );
    void m_pushFacetsDBR ( const dbr_ctrl_long & );
    void m_pushFacetsDBR ( const dbr_ctrl_double & );
    template < class T > class MutatorVec;
    template < class T, class E > class MutatorVecFaceted;
};

inline MsgMinderV3 :: 
    MsgMinderV3 ( ByteSeqOut & byteSeqOut, unsigned protoRev ) :
    m_iter ( byteSeqOut ),
    m_protoRev ( protoRev )
{
}

inline void MsgMinderV3 :: 
    commit ()
{
    m_iter.commit ();
}

} // end of namespace ca

#endif // ifndef v3Protoh

