

//
// Author: Jeff Hill
//

#ifndef caChannel_h
#define caChannel_h

#if ! defined ( __GNUC__ ) || __GNUC__ > 2 || ( __GNUC__ == 2 && __GNUC_MINOR__ >= 95 )
namespace ca {
#endif

struct Channel :
    public SurveyRequest::Factory,
    public WriteRequest::Factory,
    public ReadRequest::Factory,
    public SubscribeRequest::Factory
{
    struct StateNotify {
        virtual void connectNotify () = 0;
        virtual void reposnsiveNotify () = 0;
        virtual void unreposnsiveNotify () = 0;
        virtual void disconnectNotify () = 0;
    };

    struct PropertyPath {
        virtual unsigned length () const = 0;
        virtual PropertyId operator [] ( unsigned ) const = 0;
    };

    typedef unsigned priLevel_t;
    static const priLevel_t priorityMax;
    static const priLevel_t priorityMin;

    struct Factory {
        virtual Channel & createChannel (
            epicsGuard &, const StringReader & name, StateNotify &,
            priLevel_t = priorityMin, const PropertyPath * = 0,
            ArrayBounds * = 0, xxx timeUnits ) = 0;
        virtual void destroyChannel (
            epicsGuard &, Channel & ) = 0;
        // exceptions
        class PriorityOutOfBounds {} : public Abberation {};
    };

    virtual void cannonicalName (
        epicsGuard &, StringEditor & ) const = 0;
    virtual void className (
        epicsGuard &, StringEditor & ) const = 0;
    virtual unsigned serviceName (
        epicsGuard &, StringEditor & ) const = 0;
    virtual bool isConnected (
        epicsGuard & ) const = 0;
    virtual void show (
        epicsGuard &, unsigned level ) const = 0;

    // exceptions
    class badType;
    class propertyValueOutOfBounds;
    class invalidEventSelection;
    class noWriteAccess; 
    class noReadAccess;
    class unsupportedByService;
    class requestTimedOut;
    class unresponsive;
    class disconnect;
};

#if ! defined ( __GNUC__ ) || __GNUC__ > 2 || ( __GNUC__ == 2 && __GNUC_MINOR__ >= 95 )
} // end of namespace ca
#endif

#endif // ifndef caChannel_h
