/*************************************************************************\
 * Copyright (c) 2002 The University of Chicago, as Operator of Argonne
 *     National Laboratory.
 * Copyright (c) 2002 The Regents of the University of California, as
 *     Operator of Los Alamos National Laboratory.
 * EPICS BASE Versions 3.13.7
 * and higher are distributed subject to a Software License Agreement found
 * in file LICENSE that is included with this distribution. 
\*************************************************************************/

//
// SockCir.h,v 1.6 2009/12/04 23:28:17 jhill Exp
//

#ifndef SockCirh
#define SockCirh

#include "caPtr.h"
#include "caIntfIO.h"

namespace ca {

class SockCir : public Circuit {
public:
    //static Ptr < SockCir > factory ( const Ptr < VecAllocator > &, SOCKET sock );
    SockCir ( SOCKET sock );
    ~SockCir ();
    void show ( unsigned level ) const;
    std :: auto_ptr < CircuitIn > createInputStream ( const Ptr < VecAllocator > & );
    std :: auto_ptr < CircuitOut > createOutStream ();
private:
    class M_CircuitIn : public CircuitIn { 
    public:
        M_CircuitIn ( SockCir &, const Ptr < VecAllocator > & );
        Vec < const epicsUInt8 > read ( size_t nBytesMax );
        void shutdown ();
        void show ( unsigned level ) const;
    private:
        SockCir & m_sockCir;
        Ptr < VecAllocator > m_pVecAllocator;
    };
    class M_CircuitOut : public CircuitOut { 
    public:
        M_CircuitOut ( SockCir & );
        void write ( const Vec < const epicsUInt8 > & );
        void shutdown ();
        void flush ();
        void show ( unsigned level ) const;
    private:
        SockCir & m_sockCir;
    };
    SOCKET m_sock;
    epics :: osi :: AtomicWord m_sockCloseHasCompleted;
    bool m_circuitInHasExisted;
    bool m_circuitOutHasExisted;
    void m_throwDisconnect ();
    SockCir ( const SockCir & ); // disabled
    SockCir & operator = ( const SockCir & ); // disabled
};

} // end of namespace ca

#endif // SockCirh

