/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE is distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
//
//
// verify connection state prior to doing anything in this file
//
//

#include <ccstring>
#include <cstdexcept>
#include <cctype>

#include "epicsSignal.h"
#include "envDefs.h"
#include "caProto.h"
<<<<<<< caServerIO.cc
#include "casStreamIO.h"
=======
#include "errlog.h"
>>>>>>> 1.25

#define epicsExportSharedSymbols
#include "caServerIO.h"

static char * getToken ( 
                const char **ppString, 
                char *pBuf, unsigned bufSIze );

class caSocketBasedServer : 
    public ServerIO {
public:
    ~caSocketBasedServer ();
	void show ( 
		Guard &, unsigned level ) const;
    void sendBeacon ( 
		Guard &, ca_uint32_t beaconNo );
private:
#error doees this need to remain a virtual interface
    tsDLList < IntfIO > m_intfList;
    void m_locateInterfaces ();
	void m_attachInterface ( const caNetAddr & addr, 
        bool autoBeaconAddr, bool addConfigAddr );
};

static epicsThreadOnceId 
    caSocketBasedServerOnce = EPICS_THREAD_ONCE_INIT;

void caSocketBasedServerFunc ( void * ) 
{
    epicsSignalInstallSigPipeIgnore ();
}

ServerIO & ServerIO :: factory ()
{

#error
    return * new caSocketBasedServer ();
}

//
// caSocketBasedServer :: caSocketBasedServer ()
//
caSocketBasedServer :: caSocketBasedServer ()
{
    epicsThreadOnce ( 
        caSocketBasedServerOnce, 
        caSocketBasedServerFunc, 0 );

	if ( ! osiSockAttach () ) {
        throw std::runtime_error ( 
            "caSocketBasedServer: socket library init fail" );
	}
    m_locateInterfaces ();
	if ( m_intfList.count () == 0u ) {
        throw std::runtime_error ( 
            "caSocketBasedServer: no network interfaces" );
	}
}

//
// caSocketBasedServer :: ~caSocketBasedServer ()
//
caSocketBasedServer :: ~caSocketBasedServer ()
{
    // delete all interfaces
	while ( IntfIO * pIF = m_intfList.get() ) {
		delete pIF;
	}
	osiSockRelease ();
}

void caSocketBasedServer :: sendBeacon ( ca_uint32_t beaconNo )
{
	tsDLIter < IntfIO > iter = m_intfList.firstIter ();
	while ( iter.valid () ) {
		iter->sendBeacon ( beaconNo );
		iter++;
	}
}

//
// caSocketBasedServer :: show()
//
void caSocketBasedServer :: show ( unsigned level ) const
{
	printf ( "caSocketBasedServer at %p\n", 
        static_cast <const void *> ( this ) );
    if ( level > 0 ) {
        tsDLIterConst < IntfIO > iterIF = 
            m_intfList.firstIter ();
        while ( iterIF.valid () ) {
            iterIF->show ( level - 1 );
            ++iterIF;
        }
    }
}

void caSocketBasedServer :: attachInterface ( 
    const caNetAddr & addrIn, 
    bool autoBeaconAddr, 
    bool addConfigBeaconAddr )
{    
#error
    IntfIO ( addrIn ),
	casDGIntfIO ( addrIn, autoBeaconAddr, addConfigBeaconAddr ),

    casDGIntfOS ( casIn, memMgrIn, addrIn, autoBeaconAddr, 
        addConfigBeaconAddr ),
#error
    IntfIO * pIntf = new casIntfOS ( 
        *this, m_clientBufMemMgr, addrIn, 
        autoBeaconAddr, addConfigBeaconAddr );
    Guard guard ( m_mutex );
    m_intfList.add ( *pIntf );
}

//
// caSocketBasedServer :: m_locateInterfaces()
//
void caSocketBasedServer :: m_locateInterfaces ()
{
	//
	// first try for the server's private port number env var.
	// If this available use the CA server port number (used by
	// clients to find the server). If this also isnt available
	// then use a hard coded default - CA_SERVER_PORT.
	//
	unsigned short port;
	if ( envGetConfigParamPtr ( & EPICS_CAS_SERVER_PORT ) ) {
		port = envGetInetPortConfigParam (
                    &EPICS_CAS_SERVER_PORT,
                    static_cast <unsigned short> ( CA_SERVER_PORT ) );
	}
	else {
		port = envGetInetPortConfigParam (
                    & EPICS_CA_SERVER_PORT,
                    static_cast <unsigned short> ( CA_SERVER_PORT ) );
	}

	char buf[64u]; 
	bool autoBeaconAddr;
    const char * pStr = envGetConfigParam ( 
        &EPICS_CAS_AUTO_BEACON_ADDR_LIST, sizeof(buf), buf );
    if ( ! pStr ) {
	    pStr = envGetConfigParam ( &EPICS_CA_AUTO_ADDR_LIST, sizeof(buf), buf );
    }
	if (pStr) {
		if (strstr(pStr,"no")||strstr(pStr,"NO")) {
			autoBeaconAddr = false;
		}
		else if (strstr(pStr,"yes")||strstr(pStr,"YES")) {
			autoBeaconAddr = true;
		}
		else {
			fprintf(stderr, 
		"CAS: EPICS_CA(S)_AUTO_ADDR_LIST = \"%s\"? Assuming \"YES\"\n", pStr);
			autoBeaconAddr = true;
		}
	}
	else {
		autoBeaconAddr = true;
	}

	//
	// bind to the the interfaces specified - otherwise wildcard
	// with INADDR_ANY and allow clients to attach from any interface
	//
	struct sockaddr_in saddr;
	memset ((char *)&saddr,0,sizeof(saddr));
	pStr = envGetConfigParamPtr ( & EPICS_CAS_INTF_ADDR_LIST );
	if ( pStr ) {
		bool configAddrOnceFlag = true;
		caStatus stat = S_cas_noInterface; 
		while ( char * pToken = 
            getToken ( & pStr, buf, sizeof ( buf ) ) ) {
			int status;

			status = aToIPAddr (pToken, port, &saddr);
			if (status) {
				errlogPrintf(
					"%s: Parsing '%s'\n",
					__FILE__,
					EPICS_CAS_INTF_ADDR_LIST.name);
				errlogPrintf(
					"\tBad internet address or host name: '%s'\n",
					pToken);
				continue;
			}
			m_attachInterface (
                caNetAddr(saddr), autoBeaconAddr, 
                configAddrOnceFlag );
			configAddrOnceFlag = false;
		}
	}
	else {
        saddr.sin_family = AF_INET;
        saddr.sin_port =  ntohs (port);
		saddr.sin_addr.s_addr = htonl(INADDR_ANY);
		m_attachInterface ( caNetAddr(saddr), autoBeaconAddr, true );
	}
}

//
// getToken()
//
static char * getToken ( const char **ppString, // X aCC 361
                      char *pBuf, unsigned bufSIze )
{
    const char * pToken = *ppString;
    while ( isspace ( *pToken ) && *pToken ) {
        pToken++;
    }
    
    unsigned i;
    for ( i = 0u; i < bufSIze; i++ ) {
        if ( isspace ( pToken[i] ) || pToken[i]=='\0' ) {
            pBuf[i] = '\0';
            break;
        }
        pBuf[i] = pToken[i];
    }
    
    *ppString = &pToken[i];
    
    if ( *pToken ) {
        return pBuf;
    }
    else{
        return NULL;
    }
}

