/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 *  dbCAC.h,v 1.45 2009/11/24 19:50:17 jhill Exp
 *
 *
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Copyright, 1986, The Regents of the University of California.
 *
 *
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 *	505 665 1831
 *
 * NOTES:
 * 1) This interface is preliminary and will change in the future
 */

#ifndef dbCACh
#define dbCACh

#ifdef epicsExportSharedSymbols
#   define dbCACh_restore_epicsExportSharedSymbols
#   undef epicsExportSharedSymbols
#endif

#include "tsDLList.h"
#include "tsFreeList.h"
#include "resourceLib.h"
#include "cacIO.h"
#include "compilerDependencies.h"
#include "epicsMemory.h"
#include "resourceLib.h"

#ifdef dbCACh_restore_epicsExportSharedSymbols
#   define epicsExportSharedSymbols
#   include "shareLib.h"
#endif

#include "db_access.h"
#include "dbNotify.h"
#include "dbEvent.h"
#include "dbAddr.h"
#include "dbLock.h"
#include "dbCommon.h"
#include "db_convert.h"
#include "dbChannelIO.h"

extern "C" void putNotifyCompletion ( putNotify *ppn );

class dbContext;
class dbChannelIO;
class dbPutNotifyBlocker;
class dbSubscriptionIO;

typedef epics :: Mutex :: Guard Guard;

class dbBaseIO                  //  X aCC 655
    : public chronIntIdRes < dbBaseIO > {
public:
    virtual dbSubscriptionIO * isSubscription () = 0;
    virtual void show ( Guard &, unsigned level ) const = 0;
    virtual void show ( unsigned level ) const = 0;
    dbBaseIO ();
    dbBaseIO ( const dbBaseIO & );
    dbBaseIO & operator = ( const dbBaseIO & );
protected:
    virtual ~dbBaseIO() {}
};

class dbPutNotifyBlocker : 
    public dbBaseIO {
public:
    dbPutNotifyBlocker ( Guard & );
    ~dbPutNotifyBlocker ();
    void cleanup ( Guard & );
    void initiatePutNotify ( Guard &, 
            cacWriteNotify &, struct dbAddr &, 
            unsigned type, unsigned long count, const void * pValue );
    void cancel ( Guard & );
    void show ( Guard &, unsigned level ) const;
    void show ( unsigned level ) const;
private:
    putNotify pn;
    //
    // Include a union of all scalar types 
    // including fixed length strings so
    // that in many cases we can avoid 
    // allocating another buffer
    // 
    union {
        dbr_string_t strval;
        dbr_short_t shrtval;
        dbr_short_t intval;
        dbr_float_t fltval;
        dbr_enum_t enmval;
        dbr_char_t charval;
        dbr_long_t longval;
        dbr_double_t doubleval;
    } dbrScalarValue;
    epicsEvent block;
    mutable Mutex mutex;
    cacWriteNotify * pNotify;
    unsigned long maxValueSize;
    bool _cleanupHasRun;
    dbSubscriptionIO * isSubscription ();
    void expandValueBuf ( 
        Guard &, unsigned long newSize );
    friend void putNotifyCompletion ( putNotify * ppn );
	dbPutNotifyBlocker ( const dbPutNotifyBlocker & );
	dbPutNotifyBlocker & operator = ( const dbPutNotifyBlocker & );
};

extern "C" void dbSubscriptionEventCallback ( void *pPrivate, struct dbAddr *paddr,
    int eventsRemaining, struct db_field_log *pfl );

class dbSubscriptionIO : 
    public tsDLNode < dbSubscriptionIO >, 
    public dbBaseIO {
public:
    dbSubscriptionIO ( 
        Guard &, dbContext &, dbChannelIO &, struct dbAddr &, cacStateNotify &, 
        unsigned type, unsigned long count, unsigned mask, dbEventCtx );
    ~dbSubscriptionIO ();
    void cleanup ( Guard & );
    void unsubscribe ( Guard & );
    void channelDeleteException ( Guard & );
    void show ( Guard &, unsigned level ) const;
    void show ( unsigned level ) const;
private:
    mutable Mutex mutex;
    unsigned long count;
    cacStateNotify & notify;
    dbChannelIO & chan;
    dbEventSubscription es;
    unsigned type;
    unsigned id;
    dbSubscriptionIO * isSubscription ();
    friend void dbSubscriptionEventCallback ( 
        void * pPrivate, struct dbAddr * paddr,
        int eventsRemaining, struct db_field_log * pfl );
    dbSubscriptionIO ( const dbSubscriptionIO & );
    dbSubscriptionIO & operator = ( const dbSubscriptionIO & );
};

class dbContext;

#if 0
class dbContextPrivateListOfIO {
public:
    dbContextPrivateListOfIO ();
    ~dbContextPrivateListOfIO ();
private:
    tsDLList < dbSubscriptionIO > eventq;
    dbPutNotifyBlocker * pBlocker;
    friend class dbContext;
    dbContextPrivateListOfIO ( const dbContextPrivateListOfIO & );
    dbContextPrivateListOfIO & operator = ( const dbContextPrivateListOfIO & );
};
#endif

class dbContextReadNotifyCacheAllocator  {
public:
    dbContextReadNotifyCacheAllocator ();
    ~dbContextReadNotifyCacheAllocator ();
    char * alloc ( unsigned long size );
    void free ( char * pFree );
    void show ( unsigned level ) const;
private:
    struct cacheElem_t {
        struct cacheElem_t * pNext;
    };
    unsigned long _readNotifyCacheSize;
    cacheElem_t * _pReadNotifyCache;
    void reclaimAllCacheEntries ();
    dbContextReadNotifyCacheAllocator ( const dbContextReadNotifyCacheAllocator & );
    dbContextReadNotifyCacheAllocator & operator = ( const dbContextReadNotifyCacheAllocator & );
};

class dbContextReadNotifyCache  {
public:
    dbContextReadNotifyCache ( Mutex & );
    void callReadNotify ( Guard &, 
        struct dbAddr & addr, unsigned type, unsigned long count, 
            cacReadNotify & notify );
    void show ( Guard &, unsigned level ) const;
private:
    dbContextReadNotifyCacheAllocator _allocator;
    Mutex & _mutex;
	dbContextReadNotifyCache ( const dbContextReadNotifyCache & );
	dbContextReadNotifyCache & 
        operator = ( const dbContextReadNotifyCache & );
};

class dbContext : public cacContext {
public:
    dbContext ( Mutex & cbMutex, Mutex & mutex, 
        cacContextNotify & notify );
    virtual ~dbContext ();
    cacChannel & createChannel ( 
        Guard &, const char * pChannelName, 
        cacChannelNotify &, cacChannel::priLev );
    void destroyChannel ( Guard &, cacChannel & );
    void callReadNotify ( Guard &, 
            struct dbAddr & addr, unsigned type, unsigned long count, 
            cacReadNotify & notify );
    void callStateNotify ( struct dbAddr &addr, unsigned type, unsigned long count, 
            const struct db_field_log * pfl, cacStateNotify & notify );
    void subscribe ( 
            Guard &,
            struct dbAddr & addr, dbChannelIO & chan,
            unsigned type, unsigned long count, unsigned mask, 
            cacStateNotify & notify, cacChannel::ioid * pId );
    void initiatePutNotify ( 
            Guard &, dbChannelIO &, struct dbAddr &, 
            unsigned type, unsigned long count, const void * pValue, 
            cacWriteNotify & notify, cacChannel::ioid * pId ); 
    void show ( unsigned level ) const;
    void showAllIO ( const dbChannelIO & chan, unsigned level ) const;
    void destroyAllIO ( 
        Guard &, dbChannelIO & chan );
    void ioCancel ( Guard &, 
        dbChannelIO & chan, const cacChannel::ioid &id );
    void ioShow ( Guard &,
        const cacChannel::ioid & id, unsigned level ) const;
private:
    typedef tsFreeList < dbPutNotifyBlocker, 64 > 
                            DBPutNotifyBlockerFreeList;
    typedef tsFreeList < dbSubscriptionIO, 256 > 
                            DBSubscriptionIOFreeList;
    typedef tsFreeList < dbChannelIO, 256 > 
                            DBChannelIOFreeList;
    DBPutNotifyBlockerFreeList dbPutNotifyBlockerFreeList;
    DBSubscriptionIOFreeList dbSubscriptionIOFreeList;
    DBChannelIOFreeList dbChannelIOFreeList;
    chronIntIdResTable < dbBaseIO > ioTable;
    dbContextReadNotifyCache readNotifyCache;
    dbEventCtx ctx;
    unsigned long stateNotifyCacheSize;
    mutable Mutex & mutex;
    mutable Mutex & cbMutex;
    cacContextNotify & notify;
    epics_auto_ptr < cacContext > pNetContext;
    char * pStateNotifyCache;
    void flush ( Guard & );
    unsigned circuitCount ( Guard & ) const;
    void selfTest ( Guard & ) const;
    unsigned beaconAnomaliesSinceProgramStart ( Guard & ) const;
    void show ( Guard &, unsigned level ) const;

    dbContext ( const dbContext & );
    dbContext & operator = ( const dbContext & );
};

inline void dbContext::callReadNotify ( 
    Guard & guard, struct dbAddr &addr, 
    unsigned type, unsigned long count, cacReadNotify & notifyIn )
{
    guard.assertIdenticalMutex ( this-> mutex );
    this->readNotifyCache.callReadNotify ( guard, addr, type, count, notifyIn );
}

#endif // dbCACh

