/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

/*
 *
 *
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Copyright, 1986, The Regents of the University of California.
 *
 *
 *	Author Jeffrey O. Hill
 *	johill@lanl.gov
 *	505 665 1831
 *
 * NOTES:
 * 1) This interface is preliminary and will change in the future
 */

#ifndef dbChannelIOh
#define dbChannelIOh

#ifdef epicsExportSharedSymbols
#   define dbChannelIOh_restore_epicsExportSharedSymbols
#   undef epicsExportSharedSymbols
#endif

#include "compilerDependencies.h"
#include "epicsMemory.h"

#ifdef dbChannelIOh_restore_epicsExportSharedSymbols
#   define epicsExportSharedSymbols
#endif

typedef epics :: Mutex Mutex;
typedef Mutex :: Guard Guard;

class dbContext;
class dbSubscriptionIO;
class dbPutNotifyBlocker;

class dbContextPrivateListOfIO {
public:
    dbContextPrivateListOfIO ();
    ~dbContextPrivateListOfIO ();
private:
    tsDLList < dbSubscriptionIO > eventq;
    dbPutNotifyBlocker * pBlocker;
    friend class dbContext;
	dbContextPrivateListOfIO ( const dbContextPrivateListOfIO & );
	dbContextPrivateListOfIO & operator = ( const dbContextPrivateListOfIO & );
};

class dbChannelIO : 
    public cacChannel, 
    public dbContextPrivateListOfIO {
public:
    dbChannelIO ( 
        Guard &, cacChannelNotify &, 
        const dbAddr &, dbContext & );
    ~dbChannelIO ();
    void cleanup ( Guard & );
    void callReadNotify ( 
        Guard &, unsigned type, unsigned long count, 
        cacReadNotify & notify );
    void callStateNotify ( 
        unsigned type, unsigned long count, 
        const struct db_field_log * pfl, cacStateNotify & notify );
    void show ( Guard &, unsigned level ) const;
    unsigned getName ( Guard &, char * pBuf, 
        unsigned bufLen ) const throw ();
    const char * pName ( Guard & ) const throw ();
private:
    Mutex _mutex;
    dbContext & serviceIO;
    dbAddr addr;
    epics_auto_ptr < char, eapt_array > pNameStr;
    bool _cleanUpHasRun;

    void initiateConnect ( Guard & );
    unsigned requestMessageBytesPending ( Guard & );
    void flush ( Guard & );
    ioStatus read ( 
        Guard &, unsigned type, unsigned long count, 
        cacReadNotify &, ioid * );
    void write ( 
        Guard &, unsigned type, unsigned long count, 
        const void * pvalue );
    ioStatus write ( 
        Guard &, unsigned type, unsigned long count, 
        const void * pvalue, cacWriteNotify &, ioid * );
    void subscribe ( 
        Guard &, unsigned type, unsigned long count, 
        unsigned mask, cacStateNotify &notify, ioid * );
    void ioCancel ( Guard & mutualExclusionGuard,
        const ioid & );
    void ioShow ( 
        Guard &, const ioid &, unsigned level ) const;
    short nativeType ( Guard & ) const;
    unsigned long nativeElementCount ( Guard & ) const;
	dbChannelIO ( const dbChannelIO & );
	dbChannelIO & operator = ( const dbChannelIO & );
};

inline dbContextPrivateListOfIO::dbContextPrivateListOfIO () :
    pBlocker ( 0 )
{
}

inline dbContextPrivateListOfIO::~dbContextPrivateListOfIO () 
{
    assert ( ! this->pBlocker );
}

#endif // dbChannelIOh

