/*************************************************************************\
* Copyright (c) 2009 UChicago Argonne LLC, as Operator of Argonne
*     National Laboratory.
* EPICS BASE is distributed subject to the Software License Agreement
* in the file LICENSE that is included with this distribution.
\*************************************************************************/
//	Author: Andrew Johnson

// ChunkedString.h

#ifndef INC_ChunkedString_H
#define INC_ChunkedString_H

#include "SegString.h"
#include "SegBuffer.h"

// This implementation of a segmented string uses a SegBuffer
// for storage, which manages data segments using a freelist.

template class epicsShareClass SegBuffer < 16, 64 >;

class epicsShareClass ChunkedString :
    public SegStringEditor {
private:

    // Data
    SegBuffer<16, 64> buf_;	// Storage
    size_t len_;		// String length

// Virtual methods for base class to use
    virtual bool getSegment(const char *&buf, size_t &len, size_t pos) const;
    virtual bool getSegment(char *&buf, size_t &len, size_t pos);
    virtual void setSize(size_t len);

public:
    // Constructors
    ChunkedString();
    ChunkedString(const StringReader &rhs);
    ChunkedString(const char *buf, size_t len = 0);

    // Destructor
    ~ChunkedString();

// StringReader virtual functions
    // String length
    virtual size_t size(void) const;

// StringEditor virtual functions
    // Buffer storage management
    virtual size_t capacity() const;
    virtual void reserve(size_t cap);

    // Swap storage
    void swap(ChunkedString & src);

    // Overloaded operators
    ChunkedString& operator = (const StringReader &rhs);
};


// Inline member functions

// Constructors
inline ChunkedString::ChunkedString() :
    buf_(), len_(0) {}
inline ChunkedString::ChunkedString(const StringReader &rhs) :
    buf_(rhs.size()), len_(0) {
    assign(rhs);
}

// Destructor
inline ChunkedString::~ChunkedString() {}

// String length
inline size_t ChunkedString::size(void) const {
    return len_;
}

// Buffer storage management
inline size_t ChunkedString::capacity() const {
    return buf_.capacity();
}
inline void ChunkedString::reserve(size_t cap) {
    buf_.reserve(cap);
}

// Swap storage
inline void ChunkedString::swap(ChunkedString & src) {
    buf_.swap(src.buf_);
    size_t len = len_;
    len_ = src.len_;
    src.len_ = len;
}

// Overloaded operators
inline ChunkedString& ChunkedString::operator = (const StringReader &rhs) {
    assign(rhs);
    return *this;
}

#endif /* INC_ChunkedString_H */
