/*************************************************************************\
* Copyright (c) 2007 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* EPICS BASE Versions 3.13.7 and higher are distributed subject to the
* Software License Agreement in the file LICENSE that is included with
* this distribution.
\*************************************************************************/
//	Author: Andrew Johnson

// ConString.h

#ifndef INC_ConString_H
#define INC_ConString_H

#include "BaseString.h"

// ConStringReader, ConStringEditor : Abstract derived classes for use by
// implementations with a single contiguous buffer.  They implement most of
// the StringReader and StringEditor API functions to simplify concrete classes.

class epicsShareClass ConStringReader :
    public StringReader {
protected:
    virtual ~ConStringReader() = 0;
private:
    // Buffer access for the ConStringReader methods
    virtual const char * getBuffer() const = 0;
    // This method returns a const pointer to the string buffer

public:
    ConStringReader ();

    // Comparison functions
    virtual bool equals(const StringReader & rhs) const;
    virtual bool contains(size_t pos, const StringReader & rhs) const;
    virtual bool contains(size_t pos, const char *str, size_t len) const;

    // Indexed byte access
    virtual char at(size_t pos) const;

    // Copy out to buffer
    virtual size_t extract(size_t pos, char *buf, size_t cap) const;
};

class epicsShareClass ConStringEditor :
    public StringEditor {
protected:
    virtual ~ConStringEditor() = 0;
private:
    // Buffer access for the ConStringEditor methods
    virtual const char * getBuffer() const = 0;
    virtual char * getBuffer() = 0;
    // These methods must return a pointer to the string buffer
    // The base methods may read/write up to capacity() chars.

    // Size adjustment and optional null termination
    virtual void setSize(size_t len) = 0;
    // This method must set the member variable returned by size()
    // A \0 terminator would also be added here if desired.

public:
    ConStringEditor ();

    // Comparison functions
    virtual bool equals(const StringReader & rhs) const;
    virtual bool contains(size_t pos, const StringReader & rhs) const;
    virtual bool contains(size_t pos, const char *str, size_t len) const;

    // Indexed byte access
    virtual char at(size_t pos) const;

    // Copy out to buffer
    virtual size_t extract(size_t pos, char *buf, size_t cap) const;

    // Length adjustment
    virtual void resize(size_t len, char fill = '\0');

    // Assignment
    virtual void assign(const StringReader & src, size_t off = 0);
    virtual void assign(const char *str, size_t len);

    // Append
    virtual void append(char chr);
    virtual void append(const StringReader & src, size_t off = 0);
    virtual void append(const char *str, size_t len);
};


#endif
