/*************************************************************************\
* Copyright (c) 2006 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* EPICS BASE Versions 3.13.7 and higher are distributed subject to the
* Software License Agreement in the file LICENSE that is included with
* this distribution.
\*************************************************************************/
//	Author: Andrew Johnson

// ConstString.h

#ifndef INC_ConstString_H
#define INC_ConstString_H

#include <string.h>
#include <stdexcept>

#include "ConString.h"

// This implementation is a wrapper for const char* literals.

class ConstString :
    public ConStringReader {
private:
    const char *str_;
    size_t len_;

    // Private methods for base class use
    const char * getBuffer() const;
public:
    // Constructors
    ConstString(const ConstString &rhs);
    ConstString(const char *str, size_t len = 0);

    // String length
    size_t size(void) const;

    // Access for C compatibility
    const char *c_str() const;
    
private: // Not implemented
    ConstString& operator = (const ConstString &rhs);
};


// Inline member functions

// Methods for base class use
inline const char * ConstString::getBuffer() const {
    return str_;
}

// Constructors
inline ConstString::ConstString(const ConstString &rhs) :
    str_(rhs.str_), len_(rhs.len_) {}

inline ConstString::ConstString(const char *str, size_t len) :
    str_(str), len_(len) {
    if (!str) throw std::invalid_argument("FIXME");
    if (!len) len_ = strlen(str);
}

// String length
inline size_t ConstString::size(void) const {
    return len_;
}

// Access for C compatibility
inline const char * ConstString::c_str() const {
    // Check the buffer hasn't obviously changed since construction:
    if (str_[len_] != '\0') throw std::length_error("FIXME");
    return str_;
}


#endif
