/*************************************************************************\
* Copyright (c) 2009 UChicago Argonne LLC, as Operator of Argonne
*     National Laboratory.
* EPICS BASE is distributed subject to the Software License Agreement
* in the file LICENSE that is included with this distribution.
\*************************************************************************/
//	Author: Andrew Johnson

// SegBuffer.h

#ifndef INC_SegBuffer_H
#define INC_SegBuffer_H

#include <stddef.h>
#include "SegTree.h"

// A Segmented Buffer template

template <int N, int M>
class SegBuffer {
private:
    SegTree<N, M> *tree_;	// Storage
public:
    SegBuffer(size_t capacity = 0);

    ~SegBuffer();

    size_t capacity() const;
    void reserve(size_t capacity);

    bool getSegment(const char *&buf, size_t &len, size_t idx) const;
    bool getSegment(char *&buf, size_t &len, size_t idx);

    void swap(SegBuffer<N, M> & src);
};


// Inline member functions

// Constructor
template <int N, int M>
inline SegBuffer<N, M>::SegBuffer(size_t cap) :
    tree_(NULL) {
    reserve(cap);
}

// Destructor
template <int N, int M>
inline SegBuffer<N, M>::~SegBuffer() {
    if (tree_)
        tree_->recycle();
}

// Buffer storage management
template <int N, int M>
inline size_t SegBuffer<N, M>::capacity() const {
    return tree_ ? tree_->capacity() : 0;
}

template <int N, int M>
inline void SegBuffer<N, M>::reserve(size_t cap) {
    if (cap)
        SegTree<N, M>::Factory(tree_, cap);
}


// Data Accessors
template <int N, int M>
inline bool SegBuffer<N, M>::getSegment(const char *&buf, size_t &len, size_t idx) const {
    if (idx >= capacity()) return false;
    tree_->getSegment(buf, len, idx);
    return true;
}

template <int N, int M>
inline bool SegBuffer<N, M>::getSegment(char *&buf, size_t &len, size_t idx) {
    if (idx >= capacity()) return false;
    tree_->getSegment(buf, len, idx);
    return true;
}


// Swap buffers
template <int N, int M>
inline void SegBuffer<N, M>::swap(SegBuffer<N, M> &src) {
    SegTree<N, M> *tree = tree_;
    tree_ = src.tree_;
    src.tree_ = tree;
}

#endif /* INC_SegBuffer_H */
