/*************************************************************************\
* Copyright (c) 2009 UChicago Argonne LLC, as Operator of Argonne
*     National Laboratory.
* EPICS BASE is distributed subject to the Software License Agreement
* in the file LICENSE that is included with this distribution.
\*************************************************************************/
//	Author: Andrew Johnson

// SegString.h

#ifndef INC_SegString_H
#define INC_SegString_H

#include "BaseString.h"

// SegStringReader, SegStringEditor : Abstract derived classes for use by
// implementations with a series of segmented buffers.  They implement most of
// the StringReader and StringEditor API functions to simplify concrete classes.

class epicsShareClass SegStringReader :
    public StringReader {
private:
    // Segment accessor, for use by SegStringReader methods
    virtual bool getSegment(const char *&buf, size_t &len, size_t pos) const = 0;
    // This method finds the segment containing the byte at offset pos, then
    // sets buf to point to this character and len to the number of characters
    // found there in this segment (len = 1 means that *buf is the last
    // character in the segment). If pos points beyond the end of the string
    // getSegment() should return false, otherwise it returns true.

public:
    // Comparison functions
    virtual bool equals(const StringReader & rhs) const;
    virtual bool contains(size_t pos, const StringReader & rhs) const;
    virtual bool contains(size_t pos, const char *buf, size_t len) const;

    // Indexed byte access
    virtual char at(size_t pos) const;

    // Copy out to buffer
    virtual size_t extract(size_t pos, char *buf, size_t cap) const;
};

class epicsShareClass SegStringEditor :
    public StringEditor {
private:
    // Segment access for the SegStringEditor methods
    virtual bool getSegment(const char *&buf, size_t &len, size_t pos) const = 0;
    virtual bool getSegment(char *&buf, size_t &len, size_t pos) = 0;
    // These methods find the segment containing the byte at offset pos, then
    // set buf to point to this character and len to the number of characters
    // remaining in this segment (len = 1 means that *buf is the last
    // character in the segment). If pos points beyond the end of the string
    // getSegment() should return false, otherwise it returns true.

    // Size adjustment and optional null termination
    virtual void setSize(size_t len) = 0;
    // This method must set the member variable returned by size()

public:
    virtual ~SegStringEditor () {}
    // Comparison functions
    virtual bool equals(const StringReader & rhs) const;
    virtual bool contains(size_t pos, const StringReader & rhs) const;
    virtual bool contains(size_t pos, const char *buf, size_t len) const;

    // Indexed byte access
    virtual char at(size_t pos) const;

    // Copy out to buffer
    virtual size_t extract(size_t pos, char *buf, size_t cap) const;

    // Length adjustment
    virtual void resize(size_t len, char fill = '\0');

    // Assignment
    virtual void assign(const StringReader & src, size_t off = 0);
    virtual void assign(const char *buf, size_t len);

    // Append
    virtual void append(char chr);
    virtual void append(const StringReader & src, size_t off = 0);
    virtual void append(const char *buf, size_t len);
};

#endif
