/*************************************************************************\
* Copyright (c) 2009 UChicago Argonne LLC, as Operator of Argonne
*     National Laboratory.
* EPICS BASE is distributed subject to the Software License Agreement
* in the file LICENSE that is included with this distribution.
\*************************************************************************/
//      Author: Andrew Johnson

// SegTree.h

#ifndef INC_SegTree_H
#define INC_SegTree_H

#include <stddef.h>
#include "shareLib.h"

// SegTree is the base class for the segments used by a SegBuffer.
// It also contains the static Factory() method that allocates segments.

template <int N, int M>
class epicsShareClass SegTree {
protected:
    // Factory, for internal use
    static SegTree<N, M>* Factory(size_t limit);

    // Destructor
    virtual ~SegTree() {};

public:
    // Public factory
    static void Factory(SegTree<N, M>* &tree, size_t cap);

    // Buffer storage management
    virtual void recycle() = 0;
    virtual size_t capacity() const = 0;
    virtual size_t limit() const = 0;
    virtual size_t grow(size_t capacity) = 0;

    // Data access
    virtual void getSegment(const char *&buf, size_t &len, size_t idx) const = 0;
    virtual void getSegment(char *&buf, size_t &len, size_t idx) = 0;
};

#endif /* INC_SegTree_H */
