/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

/*  
 *  epicsOnce.cpp,v 1.9 2009/11/23 19:14:08 jhill Exp
 *
 *                              
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *                                  
 *  Copyright, 1986, The Regents of the University of California.
 *                                  
 *           
 *	Author Jeff Hill
 *	johill@lanl.gov
 *	505 665 1831
 */

#define epicsExportSharedSymbols
#include "epicsMutex.h"
#include "epicsSingleton.h"
#include "epicsOnce.h"
#include "tsFreeList.h"

typedef epics :: Mutex :: Guard Guard;

class epicsOnceImpl : public epicsOnce {
public:
    epicsOnceImpl ( epicsOnceNotify & notifyIn );
private:
    epicsSingleton < struct StaticData > :: reference _staticRef;
    epicsOnceNotify & _notify;
    bool _onceFlag;
    void destroy ();
    void once ();
    epicsOnceImpl ( epicsOnceImpl & ); // disabled
    epicsOnceImpl & operator = ( epicsOnceImpl & ); // disabled
	static epicsSingleton < epics :: Mutex > mutex;
    static epicsSingleton < tsFreeList < class epicsOnceImpl, 16 > > freeList;
};

struct StaticData {
    StaticData ();
    epics :: Mutex _mutex;
    tsFreeList < class epicsOnceImpl, 16 > _freeList;
};

#ifdef _MSC_VER
#   pragma warning ( push )
#   pragma warning ( disable:4660 )
#endif

template class epicsSingleton < StaticData >;

#ifdef _MSC_VER
#   pragma warning ( pop )
#endif

static epicsSingleton < StaticData > _staticData;

inline StaticData :: StaticData () :
    _freeList ()
{
}

inline epicsOnceImpl :: epicsOnceImpl ( epicsOnceNotify & notify ) : 
    _staticRef ( _staticData.getReference() ), 
        _notify ( notify ), _onceFlag ( false )
{
}

void epicsOnceImpl :: once ()
{
    Guard guard ( _staticRef->_mutex );
    if ( ! _onceFlag ) {
        _notify.initialize ();
        _onceFlag = true;
    }
}

epicsOnce & epicsOnce :: create ( epicsOnceNotify & notifyIn )
{
    epicsSingleton < StaticData > :: reference 
        staticMemDataRef = _staticData.getReference ();
    Guard guard ( staticMemDataRef->_mutex );
    return * new ( staticMemDataRef->_freeList ) epicsOnceImpl ( notifyIn );
}

void epicsOnceImpl :: destroy ()
{
    epicsSingleton < StaticData > :: reference staticMemDataRef = 
                _staticData.getReference ();
    Guard guard ( staticMemDataRef->_mutex );
    staticMemDataRef->_freeList.deleteProduct ( *this );
}
