/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

/*
 *  epicsSingletonMutex.cpp,v 1.5 2009/11/23 19:14:08 jhill Exp
 *
 *  Author: Jeff O. Hill
 *
 */

#include <climits>

#define epicsExportSharedSymbols
#include "epicsMutex.h"
#include "epicsThread.h"
#include "epicsSingleton.h"

#if 0
extern "C" void epicsSingletonMutexOnce ( void * pParm );

class epicsShareClass epicsSingletonMutex {
public: 
    epicsSingletonMutex ();
    ~epicsSingletonMutex ();
    epics :: Mutex & get ();
private:
    epicsThreadOnceId onceFlag;
    epics :: Mutex * pMutex;
    static void once ( void * );
    friend void epicsSingletonMutexOnce ( void * pParm );
};

epicsSingletonMutex::epicsSingletonMutex () :
    onceFlag ( EPICS_THREAD_ONCE_INIT ), pMutex ( 0 )
{
}

epicsSingletonMutex::~epicsSingletonMutex ()
{
    delete this->pMutex;
}

void epicsSingletonMutex::once ( void * pParm )
{
    epicsSingletonMutex *pSM = static_cast < epicsSingletonMutex * > ( pParm );
    pSM->pMutex = new epics :: Mutex;
}

extern "C" void epicsSingletonMutexOnce ( void * pParm )
{
    epicsSingletonMutex::once ( pParm );
}

epics :: Mutex & epicsSingletonMutex::get ()
{
    epicsThreadOnce ( & this->onceFlag, epicsSingletonMutexOnce, this );
    return * this->pMutex;
}

epics :: Mutex & epicsSingletonPrivateMutex ()
{
    static epicsSingletonMutex mutex;
    return mutex.get ();
#endif

#ifndef SIZE_MAX
#   define SIZE_MAX UINT_MAX
#endif

static epicsThreadOnceId epicsSigletonOnceFlag ( EPICS_THREAD_ONCE_INIT );
static epics :: Mutex * pEPICSSigletonMutex = 0;

extern "C" void SingletonMutexOnce ( void * /* pParm */ )
{
    // This class exists for the purpose of avoiding file scope
    // object chicken and egg problems. Therefore, pEPICSSigletonMutex 
    // is never destroyed.
    pEPICSSigletonMutex = new epics :: Mutex;
}

void SingletonUntyped :: incrRefCount ( PBuild pBuild )
{
    epicsThreadOnce ( & epicsSigletonOnceFlag, SingletonMutexOnce, 0 );
    epics :: Mutex :: Guard 
        guard ( *pEPICSSigletonMutex );
    assert ( _refCount < SIZE_MAX );
    if ( _refCount == 0 ) {
        _pInstance = ( * pBuild ) ();
    }
    _refCount++;
}

void SingletonUntyped :: decrRefCount ( PDestroy pDestroy )
{
    assert ( _refCount > 0 );
    epics :: Mutex :: Guard 
        guard ( *pEPICSSigletonMutex );
    _refCount--;
    if ( _refCount == 0 ) {
        ( *pDestroy ) ( _pInstance );
        _pInstance = 0;
    }
}
