

//
//      Author  Jeffrey O. Hill
//              johill@lanl.gov
//              505 665 1831
//

#ifndef EnclLnkh
#define EnclLnkh

#include <cstddef>

namespace epics {

//
// EnclLnk - returns a pointer to the enclosing class 
// given a pointer to the enclosed member.
// T - type of target (the enclosing class)
// M - type of member (the enclosed member data)
//
//
template < class T, class M >
class EnclLnk {
public:
    // pDataMemeber - pointer to enclosed data member
    EnclLnk ( M T :: * pDataMemeber );
    T & enclosure ( M & ) const;
    const T & enclosure ( const M & ) const;
private:
    // we are careful to allow only POD member
    // data so that we dont have modifiable
    // shared data and so that we dont end up
    // with static data startup ordering issues
    M T :: * m_pDM;
    static std :: ptrdiff_t m_computeOffset ( M T :: * );
};

template < class T, class M >
inline EnclLnk < T, M > :: EnclLnk ( M T :: * pDataMemeber ) :
    m_pDM ( pDataMemeber )
{
}

template < class T, class M >
inline std :: ptrdiff_t EnclLnk < T, M > :: 
    m_computeOffset ( M T :: * pDM )
{
    typedef unsigned char Octet;
    T * const pT = 0;
    Octet * const pTM = reinterpret_cast < Octet * > ( & ( pT ->* pDM ) );
    return pTM - reinterpret_cast < Octet * > ( pT );
}

template < class T, class M >
inline T & EnclLnk < T, M > ::
    enclosure ( M & member ) const
{
    typedef unsigned char Octet;
    Octet * const p = reinterpret_cast < Octet * > ( & member );
    return * reinterpret_cast < T * > ( p - m_computeOffset ( m_pDM ) ); 
}

template < class T, class M >
inline const T & EnclLnk < T, M > ::
    enclosure ( const M & member ) const
{
    typedef const unsigned char Octet;
    Octet * const p = reinterpret_cast < Octet * > ( & member );
    return * reinterpret_cast < const T * > ( p - m_computeOffset ( m_pDM ) ); 
}

#if 0
template < class T >
struct Types {
    template < class X >
    struct ConstAlign {
        typedef X Result;
    };
};

template < class T >
struct Types < const T > {
    template < class X >
    struct ConstAlign {
        typedef const X Result;
    };
};
#endif

} // end of namespace epics

#endif // ifdef EnclLnkh
