/*************************************************************************\
* Copyright (c) 2007 UChicago Argonne LLC, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE is distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

/*
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Author Jeffrey O. Hill
 *  johill@lanl.gov
 */

#ifndef osdAtomich
#define osdAtomich

#include "shareLib.h"

#if defined ( _MSC_VER ) 

// the intrinsic version appears to run slightly faster
#include <intrin.h>

namespace epics {
namespace osi {

typedef long AtomicWord;

epicsShareFunc inline AtomicWord atomicIncr ( AtomicWord & target )
{
    return _InterlockedIncrement ( & target );
}

epicsShareFunc inline AtomicWord atomicDecr ( AtomicWord & target )
{
    return _InterlockedDecrement ( & target );
}

epicsShareFunc inline void atomicSet ( AtomicWord & target, AtomicWord newVal )
{
    _InterlockedExchange ( &target, newVal );
}

epicsShareFunc inline AtomicWord atomicFetch ( const AtomicWord & target )
{
    return _InterlockedExchangeAdd ( const_cast < AtomicWord * > ( & target ), 0 );
}

epicsShareFunc inline bool atomicTestAndSet ( AtomicWord & target )
{
    AtomicWord prev = _InterlockedCompareExchange ( & target, 1, 0 );
    return prev == 0;
}

} // end of namespace osi
} // end of namespace epics

#else

#include "osdAtomicGCC.h"

#if !defined ( gccStdLibCxxAtomicIsPresent )

#include "windows.h"

namespace epics {
namespace osi {

typedef LONG AtomicWord;

epicsShareFunc inline AtomicWord atomicIncr ( AtomicWord & target )
{
    return InterlockedIncrement ( & target );
}

epicsShareFunc inline AtomicWord atomicDecr ( AtomicWord & target )
{
    return InterlockedDecrement ( & target );
}

epicsShareFunc inline void atomicSet ( AtomicWord & target, AtomicWord newVal )
{
    InterlockedExchange ( &target, newVal );
}

epicsShareFunc inline AtomicWord atomicFetch ( const AtomicWord & target )
{
    return InterlockedExchangeAdd ( const_cast < AtomicWord * > ( & target ), 0 );
}

epicsShareFunc inline bool atomicTestAndSet ( AtomicWord & target )
{
    AtomicWord prev = InterlockedCompareExchange ( & target, 1, 0 );
    return prev == 0;
}

} // end of namespace osi
} // end of namespace epics

#endif // if !defined ( gccStdLibCxxAtomicIsPresent 
#endif // if defined ( _MSC_VER )


#endif // ifndef osdAtomich
