/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/
/* osdMutex.c */
/*
 *      WIN32 version
 *
 *      Author  Jeffrey O. Hill
 *              johill@lanl.gov
 *              505 665 1831
 *
 */

#include <stdio.h>
#include <limits.h>

#define VC_EXTRALEAN
#define STRICT
/* 
 * Defining this allows the *much* faster critical
 * section mutex primitive to be used. Unfortunately,
 * using certain of these functions drops support for 
 * W95\W98\WME. 
 *
 */
#define _WIN32_WINNT 0x0400 
#include <windows.h>

#define epicsExportSharedSymbols
#include "shareLib.h"
#include "epicsMutex.h"
#include "epicsAssert.h"
#include "epicsStdioRedirect.h"

typedef struct epicsMutexOSD { 
    CRITICAL_SECTION critSect;
} epicsMutexOSD;

/*
 * epicsMutexCreate ()
 */
epicsMutexOSD * epicsMutexOsdCreate ( void ) 
{
    epicsMutexOSD * pSem;
    pSem = malloc ( sizeof (*pSem) );
    if ( pSem ) {
        InitializeCriticalSection ( &pSem->critSect );
    }    
    return pSem;
}

/*
 * epicsMutexOsdDestroy ()
 */
void epicsMutexOsdDestroy ( epicsMutexOSD * pSem ) 
{    
    DeleteCriticalSection  ( &pSem->critSect );
    free ( pSem );
}

/*
 * epicsMutexOsdUnlock ()
 */
void epicsMutexOsdUnlock ( epicsMutexOSD * pSem ) 
{
    LeaveCriticalSection ( &pSem->critSect );
}

/*
 * epicsMutexOsdLock ()
 */
epicsMutexLockStatus epicsMutexOsdLock ( epicsMutexOSD * pSem ) 
{
    EnterCriticalSection ( &pSem->critSect );
    return epicsMutexLockOK;
}

/*
 * epicsMutexOsdTryLock ()
 */
epicsMutexLockStatus epicsMutexOsdTryLock ( epicsMutexOSD * pSem ) 
{ 
    if ( TryEnterCriticalSection ( &pSem->critSect ) ) {
        return epicsMutexLockOK;
    }
    else {
        return epicsMutexLockTimeout;
    }
    return epicsMutexLockOK;
}

/*
 * epicsMutexOsdAssertOwnership ()
 */
void epicsMutexOsdAssertOwnership ( epicsMutexOSD * pSem ) 
{ 
    assert ( pSem->critSect.OwningThread == GetCurrentThread () );
}

/*
 * epicsMutexOsdShow ()
 */
void epicsMutexOsdShow ( epicsMutexOSD * pSem, unsigned level ) 
{ 
    printf ("epicsMutex: win32 critical section at %p\n",
        (void * ) & pSem->critSect );
}

