/*************************************************************************\
* Copyright (c) 2007 UChicago Argonne LLC, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE is distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

/*
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Author Jeffrey O. Hill
 *  johill@lanl.gov
 */

#ifndef osdAtomicGCC_h
#define osdAtomicGCC_h

// use gcc 4 compiler builtins if possible
#if __GNUC__ * 100 + __GNUC_MINOR__ >= 402

#include <ext/atomicity.h> 

#define gccStdLibCxxAtomicIsPresent

namespace epics {
namespace osi {

typedef _Atomic_word AtomicWord;

inline AtomicWord atomicIncr ( AtomicWord & target )
{
    using __gnu_cxx::__exchange_and_add_dispatch;
    return __exchange_and_add_dispatch ( & target, 1 ) + 1;
}

inline AtomicWord atomicDecr ( AtomicWord & target )
{
    using __gnu_cxx::__exchange_and_add_dispatch;
    return __exchange_and_add_dispatch ( & target, -1 ) - 1;
}

inline void atomicSet ( AtomicWord & target, AtomicWord newValue )
{
    target = newValue;
    _GLIBCXX_WRITE_MEM_BARRIER;
}

inline AtomicWord atomicFetch ( const AtomicWord & target )
{
    using __gnu_cxx::__exchange_and_add_dispatch;
    return __exchange_and_add_dispatch ( 
        const_cast < AtomicWord *> ( & target ), 0 );
}

// glibc doesnt yet have a compare / exchange functionality
// so we use __exchange_and_add to make a relable, but
// slower version which could set the value to be true, but
// not necessarily one.
inline bool atomicTestAndSet ( AtomicWord & target )
{
    using __gnu_cxx::__exchange_and_add_dispatch;
    if ( __exchange_and_add_dispatch ( & target, 0 ) != 0 ) {
        return false;
    }
    else {
        return __exchange_and_add_dispatch ( & target, 1 ) == 0;
    }
}

} // end of namespace osi
} // endof namespace epics

#endif // __GNUC__ * 100 + __GNUC_MINOR__ >= 402

#endif // osdAtomicGCC_h
