

/*************************************************************************\
* Copyright (c) 2007 UChicago Argonne LLC, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE is distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

/*
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Author Jeffrey O. Hill
 *  johill@lanl.gov
 */

#include <climits>
#include <cstdlib>
#include "pthread.h"

#define epicsExportSharedSymbols
#include "osiAtomic.h"
#include "epicsAssert.h"

#ifndef SIZE_MAX 
#   define SIZE_MAX ( static_cast < size_t > ( -1 ) )
#endif

namespace epics {

namespace osi {

#ifndef gccStdLibCxxAtomicIsPresent

// otherwise revert to global mutex

using std :: atexit;

class AtomicGuard {
public:
    AtomicGuard ();
    ~AtomicGuard ();
private:
    static pthread_mutex_t _mutex;
    static pthread_once_t _onceFlag;
    static void _atExit ();
    static void _once ();
};

pthread_once_t AtomicGuard :: _onceFlag = PTHREAD_ONCE_INIT;
pthread_mutex_t AtomicGuard :: _mutex;

void AtomicGuard :: _atExit ()
{
    pthread_mutex_destroy ( & _mutex );
}

void AtomicGuard :: _once ()
{
    pthread_mutex_init ( & _mutex, 0 );
    assert ( atexit ( _atExit ) == 0);
}

inline AtomicGuard :: AtomicGuard ()
{
    pthread_once ( & _onceFlag, _once ); 
    pthread_mutex_lock ( & _mutex );
}

inline AtomicGuard :: ~AtomicGuard ()
{
    pthread_mutex_unlock ( & _mutex );
}

AtomicWord atomicIncr ( AtomicWord & target )
{
    AtomicGuard atomicGuard;
    return ++target;
}

AtomicWord atomicDecr ( AtomicWord & target )
{
    AtomicGuard atomicGuard;
    return --target;
}

void atomicSet ( const AtomicWord & target, AtomicWord newVal )
{
    AtomicGuard atomicGuard;
    target = newVal;
}

AtomicWord atomicFetch ( const AtomicWord & target )
{
    AtomicGuard atomicGuard;
    return target;
}

bool atomicTestAndSet ( AtomicWord & target )
{
    AtomicGuard atomicGuard;
    if ( target != 0 ) {
        return false;
    }
    else {
        target = 1;
        return true;
    }
}

#endif

} // end of namespace osi

} // end of namespace epics


