/*************************************************************************\
* Copyright (c) 2007 UChicago Argonne LLC, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE is distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

/*
 *                    L O S  A L A M O S
 *              Los Alamos National Laboratory
 *               Los Alamos, New Mexico 87545
 *
 *  Author Jeffrey O. Hill
 *  johill@lanl.gov
 */

#ifndef osdAtomic
#define osdAtomic

#include "osdAtomicGCC.h"

#ifndef gccStdLibCxxAtomicIsPresent
#ifdef EPICS_USE_VXATOMICLIB

#include "vxLib.h"
#include "vxAtomicLib.h"

namespace epics {
namespace osi {

typedef atomic_t AtomicWord;

inline AtomicWord atomicIncr ( AtomicWord & target )
{
    return vxAtomicInc ( & target ) + 1;
}

inline AtomicWord atomicDecr ( AtomicWord & target )
{
    return vxAtomicDec ( & target ) - 1;
}

inline void atomicSet ( AtomicWord & target, AtomicWord newVal )
{
    vxAtomicSet ( & target, newVal ); 
}

inline AtomicWord atomicFetch ( const AtomicWord & target )
{
    return vxAtomicAdd ( const_cast < atomic_t * > ( & target ), 0 ); 
}

inline bool atomicTestAndSet ( AtomicWord & target )
{
    return vxCas ( & target, 0, 1 );
}

} // end of namespace osi
} // end of namespace epics


#else // ifdef EPICS_USE_VXATOMICLIB

#include "intLib.h"

namespace epics {
namespace osi {

typedef size_t AtomicWord;

inline AtomicWord atomicIncr ( AtomicWord & target )
{
    int key = intLock ();
    size_t result = ++target;
    intUnlock ( key );
    return result;
}

inline AtomicWord atomicDecr ( AtomicWord & target )
{
    int key = intLock ();
    size_t result = --target;
    intUnlock ( key );
    return result;
}

inline void atomicSet ( AtomicWord & target, AtomicWord newVal )
{
    target = newVal; 
}

inline AtomicWord atomicFetch ( const AtomicWord & target )
{
    return target;
}

inline bool atomicTestAndSet ( AtomicWord & target )
{
    int key = intLock ();
    bool match = ( target == 0 );
    if ( match ) {
        target = 1;
    }
    intUnlock ( key );
    return match;
}

} // end of namespace osi
} // end of namespace epics

#endif // ifdef EPICS_USE_VXATOMICLIB
#endif // ifndef gccStdLibCxxAtomicIsPresent

#endif // osdAtomic
