/*************************************************************************\
* Copyright (c) 2009 UChicago Argonne LLC, as Operator of Argonne
*     National Laboratory.
* EPICS BASE is distributed subject to the Software License Agreement
* in the file LICENSE that is included with this distribution.
\*************************************************************************/
//      Author: Andrew Johnson

#include "SegBuffer.h"
#include "epicsUnitTest.h"
#include "testMain.h"

// These match the templates instantiated in SegTree.cpp
#define LEAF 64
#define TWIG 16

typedef SegBuffer<TWIG, LEAF> TestBuffer;

int segcount(const TestBuffer &b) {
    int c = 0;
    size_t i = 0;
    size_t l;
    const char *p;
    while (b.getSegment(p, l, i)) {
        i += l;
        c++;
    }
    return c;
}

MAIN(SegBufferTest)
{
    testPlan(19);

    TestBuffer b1;
    testOk(b1.capacity() == 0, "Initial capacity = %zu", b1.capacity());
    testOk(segcount(b1) == 0, "Data segments = %d", segcount(b1));

    b1.reserve(1);
    testOk(b1.capacity() == LEAF, "Created leaf");
    testOk(segcount(b1) == 1, "One data segment");

    TestBuffer b2(LEAF * 2 + 1);
    testOk(b2.capacity() == LEAF * 3, "Created 1-layer twig");
    testOk(segcount(b2) == 3, "Three data segments");

    b1.swap(b2);
    testOk(b1.capacity() == LEAF * 3 && b2.capacity() == LEAF, "Swap moves capacity");

    b2.reserve(LEAF * 4);
    testOk(b2.capacity() == LEAF * 4, "Grew leaf to 1-layer twig");
    testOk(segcount(b2) == 4, "Four data segments");

    b1.reserve(LEAF * TWIG);
    testOk(b1.capacity() == LEAF * TWIG, "Grew layer to 1-layer limit");
    testOk(segcount(b1) == TWIG, "Full 1-layer");
    b1.reserve(LEAF * TWIG + 1);
    testOk(b1.capacity() == LEAF * (TWIG + 1), "Grew to 2 layers");
    testOk(segcount(b1) == TWIG + 1, "More than 1-layer");

    b1.reserve(LEAF * TWIG * TWIG);
    testOk(b1.capacity() == LEAF * TWIG * TWIG, "Grew to 2-layer limit");
    testOk(segcount(b1) == TWIG * TWIG, "Full 2-layer");
    b1.reserve(LEAF * TWIG * TWIG + 1);
    testOk(b1.capacity() == LEAF * (TWIG * TWIG + 1), "Grew to 3 layers");
    testOk(segcount(b1) == TWIG * TWIG + 1, "More than 2 layers");

    {
        TestBuffer mort;
        b1.swap(mort);
    }
    testOk(b1.capacity() == 0, "Empty capacity = %zu", b1.capacity());
    testOk(segcount(b1) == 0, "Data segments = %d", segcount(b1));

    return testDone();
}
