
#include <cstdlib>
#include <cstring>
#include <cassert>

#include "osiAtomic.h"
#include "epicsTime.h"
#include "epicsUnitTest.h"
#include "testMain.h"

using std :: size_t;

class RefCtr {
public:
    RefCtr ();
    ~RefCtr ();
    void reference ();
    void unreference ();
private:
    epics :: osi :: AtomicWord _cnt;
};

class Ownership {
public:
    Ownership ();
    Ownership ( RefCtr & refCtr );
    Ownership ( const Ownership & );
    ~Ownership ();
    Ownership & operator = ( const Ownership & );
private:
    RefCtr * _pRefCtr;
};

inline RefCtr :: RefCtr () : 
    _cnt ( 0 ) 
{
}

inline RefCtr :: ~RefCtr () 
{ 
    assert ( _cnt == 0u ); 
}

inline void RefCtr :: 
    reference () 
{ 
    using epics :: osi :: atomicIncr;
    atomicIncr ( _cnt ); 
}

inline void RefCtr :: 
    unreference () 
{ 
    using epics :: osi :: atomicDecr;
    atomicDecr ( _cnt ); 
}

inline Ownership ::
    Ownership () : 
    _pRefCtr ( 0 ) 
{
}

inline Ownership ::
    Ownership ( RefCtr & refCtr ) : 
    _pRefCtr ( & refCtr ) 
{ 
    refCtr.reference (); 
} 

inline Ownership ::
    Ownership ( const Ownership & ownership ) : 
    _pRefCtr ( ownership._pRefCtr ) 
{ 
    if ( _pRefCtr ) _pRefCtr->reference (); 
} 

inline Ownership ::
    ~Ownership () 
{ 
    if ( _pRefCtr ) _pRefCtr->unreference (); 
} 

inline Ownership & Ownership ::
    operator = ( const Ownership & ownership )
{
    if ( _pRefCtr != ownership._pRefCtr ) {
        if ( _pRefCtr ) _pRefCtr->unreference (); 
        _pRefCtr = ownership._pRefCtr;
        if ( _pRefCtr ) _pRefCtr->reference (); 
    }
    return *this;
}

inline Ownership retOwnership ( const Ownership & ownership )
{
    return Ownership ( ownership );
}

inline Ownership recurRetOwner10 ( const Ownership & ownershipIn )
{
    Ownership ownership = retOwnership ( retOwnership ( retOwnership ( retOwnership ( retOwnership ( ownershipIn ) ) ) ) );
    return retOwnership ( retOwnership ( retOwnership ( retOwnership ( retOwnership ( ownership ) ) ) ) );
}

inline Ownership recurRetOwner100 ( const Ownership & ownershipIn )
{
    Ownership ownership = recurRetOwner10 ( recurRetOwner10 ( recurRetOwner10 ( recurRetOwner10 ( recurRetOwner10 ( ownershipIn ) ) ) ) );
    return recurRetOwner10 ( recurRetOwner10 ( recurRetOwner10 ( recurRetOwner10 ( recurRetOwner10 ( ownership ) ) ) ) );
}

inline Ownership recurRetOwner1000 ( const Ownership & ownershipIn )
{
    Ownership ownership = recurRetOwner100 ( recurRetOwner100 ( recurRetOwner100 ( recurRetOwner100 ( recurRetOwner100 ( ownershipIn ) ) ) ) );
    return recurRetOwner100 ( recurRetOwner100 ( recurRetOwner100 ( recurRetOwner100 ( recurRetOwner100 ( ownership ) ) ) ) );
}

inline void passRefOwnership ( const Ownership & ownershipIn, Ownership & ownershipOut )
{
    ownershipOut = ownershipIn;
}

inline void passRefOwnership10 ( const Ownership & ownershipIn, Ownership & ownershipOut )
{
    Ownership ownershipTmp0;
    passRefOwnership ( ownershipIn,  ownershipTmp0 );
    Ownership ownershipTmp1;
    passRefOwnership ( ownershipTmp0, ownershipTmp1 );
    Ownership ownershipTmp2;
    passRefOwnership ( ownershipTmp1, ownershipTmp2 );
    Ownership ownershipTmp3; 
    passRefOwnership ( ownershipTmp2, ownershipTmp3 );
    Ownership ownershipTmp4;
    passRefOwnership ( ownershipTmp3, ownershipTmp4 );
    Ownership ownershipTmp5;
    passRefOwnership ( ownershipTmp4, ownershipTmp5 );
    Ownership ownershipTmp6; 
    passRefOwnership ( ownershipTmp5, ownershipTmp6 );
    Ownership ownershipTmp7;
    passRefOwnership ( ownershipTmp6, ownershipTmp7 );
    Ownership ownershipTmp8;
    passRefOwnership ( ownershipTmp7, ownershipTmp8 );
    passRefOwnership ( ownershipTmp8, ownershipOut );
}

inline void passRefOwnership100 ( const Ownership & ownershipIn, Ownership & ownershipOut )
{
    Ownership ownershipTmp0;
    passRefOwnership10 ( ownershipIn,  ownershipTmp0 );
    Ownership ownershipTmp1;
    passRefOwnership10 ( ownershipTmp0, ownershipTmp1 );
    Ownership ownershipTmp2;
    passRefOwnership10 ( ownershipTmp1, ownershipTmp2 );
    Ownership ownershipTmp3; 
    passRefOwnership10 ( ownershipTmp2, ownershipTmp3 );
    Ownership ownershipTmp4;
    passRefOwnership10 ( ownershipTmp3, ownershipTmp4 );
    Ownership ownershipTmp5;
    passRefOwnership10 ( ownershipTmp4, ownershipTmp5 );
    Ownership ownershipTmp6; 
    passRefOwnership10 ( ownershipTmp5, ownershipTmp6 );
    Ownership ownershipTmp7;
    passRefOwnership10 ( ownershipTmp6, ownershipTmp7 );
    Ownership ownershipTmp8;
    passRefOwnership10 ( ownershipTmp7, ownershipTmp8 );
    passRefOwnership10 ( ownershipTmp8, ownershipOut );
}

inline void passRefOwnership1000 ( const Ownership & ownershipIn, Ownership & ownershipOut )
{
    Ownership ownershipTmp0;
    passRefOwnership100 ( ownershipIn,  ownershipTmp0 );
    Ownership ownershipTmp1;
    passRefOwnership100 ( ownershipTmp0, ownershipTmp1 );
    Ownership ownershipTmp2;
    passRefOwnership100 ( ownershipTmp1, ownershipTmp2 );
    Ownership ownershipTmp3; 
    passRefOwnership100 ( ownershipTmp2, ownershipTmp3 );
    Ownership ownershipTmp4;
    passRefOwnership100 ( ownershipTmp3, ownershipTmp4 );
    Ownership ownershipTmp5;
    passRefOwnership100 ( ownershipTmp4, ownershipTmp5 );
    Ownership ownershipTmp6; 
    passRefOwnership100 ( ownershipTmp5, ownershipTmp6 );
    Ownership ownershipTmp7;
    passRefOwnership100 ( ownershipTmp6, ownershipTmp7 );
    Ownership ownershipTmp8;
    passRefOwnership100 ( ownershipTmp7, ownershipTmp8 );
    passRefOwnership100 ( ownershipTmp8, ownershipOut );
}

const char * pExtStr = "";

// tests the time it takes to do an extenal 
// function call and an increment. The strlen
// of an empty string is added so that individual
// increments cant be optimized away, and so
// that we compare fairly with osiAtomic
// which sometimes requires an external function 
// call.
inline void tenOrdinaryIncr ( epics :: osi :: AtomicWord & target )
{
    target += std :: strlen ( pExtStr );
    target += std :: strlen ( pExtStr );
    target += std :: strlen ( pExtStr );
    target += std :: strlen ( pExtStr );
    target += std :: strlen ( pExtStr );
    target += std :: strlen ( pExtStr );
    target += std :: strlen ( pExtStr );
    target += std :: strlen ( pExtStr );
    target += std :: strlen ( pExtStr );
    target += std :: strlen ( pExtStr );
}

inline void oneHundredOrdinaryIncr ( epics :: osi :: AtomicWord & target )
{
    tenOrdinaryIncr ( target );
    tenOrdinaryIncr ( target );
    tenOrdinaryIncr ( target );
    tenOrdinaryIncr ( target );
    tenOrdinaryIncr ( target );
    tenOrdinaryIncr ( target );
    tenOrdinaryIncr ( target );
    tenOrdinaryIncr ( target );
    tenOrdinaryIncr ( target );
    tenOrdinaryIncr ( target );
}

inline void oneThousandOrdinaryIncr ( epics :: osi :: AtomicWord & target )
{
    oneHundredOrdinaryIncr ( target );
    oneHundredOrdinaryIncr ( target );
    oneHundredOrdinaryIncr ( target );
    oneHundredOrdinaryIncr ( target );
    oneHundredOrdinaryIncr ( target );
    oneHundredOrdinaryIncr ( target );
    oneHundredOrdinaryIncr ( target );
    oneHundredOrdinaryIncr ( target );
    oneHundredOrdinaryIncr ( target );
    oneHundredOrdinaryIncr ( target );
}

inline void tenAtomicIncr ( epics :: osi :: AtomicWord & target )
{
    using epics :: osi :: atomicIncr;
    atomicIncr ( target );
    atomicIncr ( target );
    atomicIncr ( target );
    atomicIncr ( target );
    atomicIncr ( target );
    atomicIncr ( target );
    atomicIncr ( target );
    atomicIncr ( target );
    atomicIncr ( target );
    atomicIncr ( target );
}

inline void oneHundredAtomicIncr ( epics :: osi :: AtomicWord & target )
{
    tenAtomicIncr ( target );
    tenAtomicIncr ( target );
    tenAtomicIncr ( target );
    tenAtomicIncr ( target );
    tenAtomicIncr ( target );
    tenAtomicIncr ( target );
    tenAtomicIncr ( target );
    tenAtomicIncr ( target );
    tenAtomicIncr ( target );
    tenAtomicIncr ( target );
}

inline void oneThousandAtomicIncr ( epics :: osi :: AtomicWord & target )
{
    oneHundredAtomicIncr ( target );
    oneHundredAtomicIncr ( target );
    oneHundredAtomicIncr ( target );
    oneHundredAtomicIncr ( target );
    oneHundredAtomicIncr ( target );
    oneHundredAtomicIncr ( target );
    oneHundredAtomicIncr ( target );
    oneHundredAtomicIncr ( target );
    oneHundredAtomicIncr ( target );
    oneHundredAtomicIncr ( target );
}

inline void tenAtomicTestAndSet ( epics :: osi :: AtomicWord & target )
{
    using epics :: osi :: atomicTestAndSet;
    atomicTestAndSet ( target );
    atomicTestAndSet ( target );
    atomicTestAndSet ( target );
    atomicTestAndSet ( target );
    atomicTestAndSet ( target );
    atomicTestAndSet ( target );
    atomicTestAndSet ( target );
    atomicTestAndSet ( target );
    atomicTestAndSet ( target );
    atomicTestAndSet ( target );
}

inline void oneHundredAtomicTestAndSet ( epics :: osi :: AtomicWord & target )
{
    tenAtomicTestAndSet ( target );
    tenAtomicTestAndSet ( target );
    tenAtomicTestAndSet ( target );
    tenAtomicTestAndSet ( target );
    tenAtomicTestAndSet ( target );
    tenAtomicTestAndSet ( target );
    tenAtomicTestAndSet ( target );
    tenAtomicTestAndSet ( target );
    tenAtomicTestAndSet ( target );
    tenAtomicTestAndSet ( target );
}

inline void oneThousandAtomicTestAndSet ( epics :: osi :: AtomicWord & target )
{
    oneHundredAtomicTestAndSet ( target );
    oneHundredAtomicTestAndSet ( target );
    oneHundredAtomicTestAndSet ( target );
    oneHundredAtomicTestAndSet ( target );
    oneHundredAtomicTestAndSet ( target );
    oneHundredAtomicTestAndSet ( target );
    oneHundredAtomicTestAndSet ( target );
    oneHundredAtomicTestAndSet ( target );
    oneHundredAtomicTestAndSet ( target );
    oneHundredAtomicTestAndSet ( target );
}

void ordinaryIncrPerformance ()
{
    epicsTime begin = epicsTime::getCurrent ();
    static const unsigned N = 1000;
    epics :: osi :: AtomicWord target;
    epics :: osi :: atomicSet ( target, 0 );
    for ( unsigned i = 0; i < N; i++ ) {
        oneThousandOrdinaryIncr ( target );
    }
    double delay = epicsTime::getCurrent () -  begin;
    testOk1 ( target == 0u );
    delay /= N * 1000u; // convert to delay per lock pair
    delay *= 1e6; // convert to micro seconds
    testDiag ( "raw incr and strlen(\"\") takes %f microseconds", delay );
}

void osiAtomicPerformance ()
{
    epicsTime begin = epicsTime::getCurrent ();
    static const unsigned N = 1000;
    epics :: osi :: AtomicWord target;
    epics :: osi :: atomicSet ( target, 0 );
    for ( unsigned i = 0; i < N; i++ ) {
        oneThousandAtomicIncr ( target );
    }
    double delay = epicsTime::getCurrent () -  begin;
    testOk1 ( target == N * 1000u );
    delay /= N * 1000u; // convert to delay per lock pair
    delay *= 1e6; // convert to micro seconds
    testDiag ( "osiAtomicIncr() takes %f microseconds", delay );
}

void atomicCompareAndSetPerformance ()
{
    epicsTime begin = epicsTime::getCurrent ();
    static const unsigned N = 1000;
    epics :: osi :: AtomicWord target;
    epics :: osi :: atomicSet ( target, 0 );
    testOk1 ( ! target );
    for ( unsigned i = 0; i < N; i++ ) {
        oneThousandAtomicTestAndSet ( target );
    }
    double delay = epicsTime::getCurrent () -  begin;
    testOk1 ( target );
    delay /= N * 1000u; // convert to delay per lock pair
    delay *= 1e6; // convert to micro seconds
    testDiag ( "atomicCompareAndSet() takes %f microseconds", delay );
}

void recursiveOwnershipRetPerformance ()
{
    RefCtr refCtr;
    epicsTime begin = epicsTime::getCurrent ();
    static const unsigned N = 1000;
    for ( unsigned i = 0; i < N; i++ ) {
        Ownership ownership ( refCtr );
        recurRetOwner1000 ( ownership );
    }
    double delay = epicsTime::getCurrent () -  begin;
    delay /= N * 1000u; // convert to delay per lock pair
    delay *= 1e6; // convert to micro seconds
    testDiag ( "retOwnership() takes %f microseconds", delay );
}

void ownershipPassRefPerformance ()
{
    RefCtr refCtr;
    epicsTime begin = epicsTime::getCurrent ();
    static const unsigned N = 1000;
    for ( unsigned i = 0; i < N; i++ ) {
        Ownership ownershipSrc ( refCtr );
        Ownership ownershipDest;
        passRefOwnership1000 ( ownershipSrc, ownershipDest );
    }
    double delay = epicsTime::getCurrent () -  begin;
    delay /= N * 1000u; // convert to delay per lock pair
    delay *= 1e6; // convert to micro seconds
    testDiag ( "passRefOwnership() takes %f microseconds", delay );
}

MAIN(osiAtomicTest)
{
    testPlan(4);
    //
    // The tests running here are measuring fast
    // functions so they tend to be impacted
    // by where the cache lines are wrt to the
    // virtual pages perhaps
    //
    ordinaryIncrPerformance ();
    osiAtomicPerformance ();
    recursiveOwnershipRetPerformance ();
    ownershipPassRefPerformance ();
    atomicCompareAndSetPerformance ();
    return testDone();
}
